@@ -1,7 +1,7 @@
 /*
  * applications.h - the Qucs application list
  *
- * Copyright (C) 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2006 Gunther Kraut <gn.kraut@t-online.de>
  *
  * This is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: applications.h,v 1.9 2006/05/23 09:48:32 raimi Exp $
+ * $Id: applications.h,v 1.18 2007/05/24 16:40:30 ela Exp $
  *
  */
 
@@ -44,12 +44,22 @@ struct application_t eqn::applications[] = {
   { "+", TAG_VECTOR,  evaluate::plus_c_v, 2, { TAG_COMPLEX, TAG_VECTOR  } },
   { "+", TAG_VECTOR,  evaluate::plus_v_v, 2, { TAG_VECTOR,  TAG_VECTOR  } },
   { "+", TAG_MATRIX,  evaluate::plus_m_m, 2, { TAG_MATRIX,  TAG_MATRIX  } },
+  { "+", TAG_MATRIX,  evaluate::plus_m_d, 2, { TAG_MATRIX,  TAG_DOUBLE  } },
+  { "+", TAG_MATRIX,  evaluate::plus_d_m, 2, { TAG_DOUBLE,  TAG_MATRIX  } },
+  { "+", TAG_MATRIX,  evaluate::plus_m_c, 2, { TAG_MATRIX,  TAG_COMPLEX } },
+  { "+", TAG_MATRIX,  evaluate::plus_c_m, 2, { TAG_COMPLEX, TAG_MATRIX  } },
   { "+", TAG_STRING,  evaluate::plus_s_s, 2, { TAG_STRING,  TAG_STRING  } },
   { "+", TAG_STRING,  evaluate::plus_c_s, 2, { TAG_CHAR,    TAG_STRING  } },
   { "+", TAG_STRING,  evaluate::plus_s_c, 2, { TAG_STRING,  TAG_CHAR    } },
-  { "+", TAG_MATVEC,  evaluate::plus_mv_mv, 2, { TAG_MATVEC, TAG_MATVEC } },
+  { "+", TAG_MATVEC,  evaluate::plus_mv_mv, 2, { TAG_MATVEC,TAG_MATVEC  } },
   { "+", TAG_MATVEC,  evaluate::plus_mv_m, 2, { TAG_MATVEC, TAG_MATRIX  } },
   { "+", TAG_MATVEC,  evaluate::plus_m_mv, 2, { TAG_MATRIX, TAG_MATVEC  } },
+  { "+", TAG_MATVEC,  evaluate::plus_mv_d, 2, { TAG_MATVEC, TAG_DOUBLE  } },
+  { "+", TAG_MATVEC,  evaluate::plus_d_mv, 2, { TAG_DOUBLE, TAG_MATVEC  } },
+  { "+", TAG_MATVEC,  evaluate::plus_mv_c, 2, { TAG_MATVEC, TAG_COMPLEX } },
+  { "+", TAG_MATVEC,  evaluate::plus_c_mv, 2, { TAG_COMPLEX,TAG_MATVEC  } },
+  { "+", TAG_MATVEC,  evaluate::plus_mv_v, 2, { TAG_MATVEC, TAG_VECTOR  } },
+  { "+", TAG_MATVEC,  evaluate::plus_v_mv, 2, { TAG_VECTOR, TAG_MATVEC  } },
 
   { "-", TAG_DOUBLE,  evaluate::minus_d, 1, { TAG_DOUBLE  } },
   { "-", TAG_COMPLEX, evaluate::minus_c, 1, { TAG_COMPLEX } },
@@ -67,9 +77,19 @@ struct application_t eqn::applications[] = {
   { "-", TAG_VECTOR,  evaluate::minus_c_v, 2, { TAG_COMPLEX, TAG_VECTOR  } },
   { "-", TAG_VECTOR,  evaluate::minus_v_v, 2, { TAG_VECTOR,  TAG_VECTOR  } },
   { "-", TAG_MATRIX,  evaluate::minus_m_m, 2, { TAG_MATRIX,  TAG_MATRIX  } },
-  { "-", TAG_MATVEC,  evaluate::minus_mv_mv, 2, { TAG_MATVEC, TAG_MATVEC } },
+  { "-", TAG_MATRIX,  evaluate::minus_m_d, 2, { TAG_MATRIX,  TAG_DOUBLE  } },
+  { "-", TAG_MATRIX,  evaluate::minus_d_m, 2, { TAG_DOUBLE,  TAG_MATRIX  } },
+  { "-", TAG_MATRIX,  evaluate::minus_m_c, 2, { TAG_MATRIX,  TAG_COMPLEX } },
+  { "-", TAG_MATRIX,  evaluate::minus_c_m, 2, { TAG_COMPLEX, TAG_MATRIX  } },
+  { "-", TAG_MATVEC,  evaluate::minus_mv_mv, 2, { TAG_MATVEC,TAG_MATVEC  } },
   { "-", TAG_MATVEC,  evaluate::minus_mv_m, 2, { TAG_MATVEC, TAG_MATRIX  } },
   { "-", TAG_MATVEC,  evaluate::minus_m_mv, 2, { TAG_MATRIX, TAG_MATVEC  } },
+  { "-", TAG_MATVEC,  evaluate::minus_mv_d, 2, { TAG_MATVEC, TAG_DOUBLE  } },
+  { "-", TAG_MATVEC,  evaluate::minus_d_mv, 2, { TAG_DOUBLE, TAG_MATVEC  } },
+  { "-", TAG_MATVEC,  evaluate::minus_mv_c, 2, { TAG_MATVEC, TAG_COMPLEX } },
+  { "-", TAG_MATVEC,  evaluate::minus_c_mv, 2, { TAG_COMPLEX,TAG_MATVEC  } },
+  { "-", TAG_MATVEC,  evaluate::minus_mv_v, 2, { TAG_MATVEC, TAG_VECTOR  } },
+  { "-", TAG_MATVEC,  evaluate::minus_v_mv, 2, { TAG_VECTOR, TAG_MATVEC  } },
 
   { "*", TAG_DOUBLE,  evaluate::times_d_d, 2, { TAG_DOUBLE,  TAG_DOUBLE  } },
   { "*", TAG_COMPLEX, evaluate::times_c_c, 2, { TAG_COMPLEX, TAG_COMPLEX } },
@@ -86,6 +106,8 @@ struct application_t eqn::applications[] = {
   { "*", TAG_MATRIX,  evaluate::times_m_d, 2, { TAG_MATRIX,  TAG_DOUBLE  } },
   { "*", TAG_MATRIX,  evaluate::times_d_m, 2, { TAG_DOUBLE,  TAG_MATRIX  } },
   { "*", TAG_MATVEC,  evaluate::times_mv_mv, 2, { TAG_MATVEC,TAG_MATVEC  } },
+  { "*", TAG_MATVEC,  evaluate::times_mv_m, 2, { TAG_MATVEC, TAG_MATRIX  } },
+  { "*", TAG_MATVEC,  evaluate::times_m_mv, 2, { TAG_MATRIX, TAG_MATVEC  } },
   { "*", TAG_MATVEC,  evaluate::times_mv_c, 2, { TAG_MATVEC, TAG_COMPLEX } },
   { "*", TAG_MATVEC,  evaluate::times_c_mv, 2, { TAG_COMPLEX,TAG_MATVEC  } },
   { "*", TAG_MATVEC,  evaluate::times_mv_d, 2, { TAG_MATVEC, TAG_DOUBLE  } },
@@ -128,6 +150,25 @@ struct application_t eqn::applications[] = {
   { "^", TAG_VECTOR,  evaluate::power_c_v, 2, { TAG_COMPLEX, TAG_VECTOR  } },
   { "^", TAG_VECTOR,  evaluate::power_v_v, 2, { TAG_VECTOR,  TAG_VECTOR  } },
 
+  { "hypot", TAG_DOUBLE,  evaluate::xhypot_d_d, 2,
+    { TAG_DOUBLE,  TAG_DOUBLE  } },
+  { "hypot", TAG_DOUBLE, evaluate::xhypot_c_c, 2,
+    { TAG_COMPLEX, TAG_COMPLEX } },
+  { "hypot", TAG_DOUBLE, evaluate::xhypot_c_d, 2,
+    { TAG_COMPLEX, TAG_DOUBLE  } },
+  { "hypot", TAG_DOUBLE, evaluate::xhypot_d_c, 2,
+    { TAG_DOUBLE,  TAG_COMPLEX } },
+  { "hypot", TAG_VECTOR,  evaluate::xhypot_v_d, 2,
+    { TAG_VECTOR,  TAG_DOUBLE  } },
+  { "hypot", TAG_VECTOR,  evaluate::xhypot_d_v, 2,
+    { TAG_DOUBLE,  TAG_VECTOR  } },
+  { "hypot", TAG_VECTOR,  evaluate::xhypot_v_c, 2,
+    { TAG_VECTOR,  TAG_COMPLEX } },
+  { "hypot", TAG_VECTOR,  evaluate::xhypot_c_v, 2,
+    { TAG_COMPLEX, TAG_VECTOR  } },
+  { "hypot", TAG_VECTOR,  evaluate::xhypot_v_v, 2,
+    { TAG_VECTOR,  TAG_VECTOR  } },
+
   { "real", TAG_DOUBLE,  evaluate::real_d, 1, { TAG_DOUBLE  } },
   { "real", TAG_DOUBLE,  evaluate::real_c, 1, { TAG_COMPLEX } },
   { "real", TAG_VECTOR,  evaluate::real_v, 1, { TAG_VECTOR  } },
@@ -203,6 +244,10 @@ struct application_t eqn::applications[] = {
   { "exp", TAG_COMPLEX, evaluate::exp_c, 1, { TAG_COMPLEX } },
   { "exp", TAG_VECTOR,  evaluate::exp_v, 1, { TAG_VECTOR  } },
 
+  { "limexp", TAG_DOUBLE,  evaluate::limexp_d, 1, { TAG_DOUBLE  } },
+  { "limexp", TAG_COMPLEX, evaluate::limexp_c, 1, { TAG_COMPLEX } },
+  { "limexp", TAG_VECTOR,  evaluate::limexp_v, 1, { TAG_VECTOR  } },
+
   { "ln", TAG_COMPLEX, evaluate::ln_d, 1, { TAG_DOUBLE  } },
   { "ln", TAG_COMPLEX, evaluate::ln_c, 1, { TAG_COMPLEX } },
   { "ln", TAG_VECTOR,  evaluate::ln_v, 1, { TAG_VECTOR  } },
@@ -251,10 +296,18 @@ struct application_t eqn::applications[] = {
   { "sec", TAG_COMPLEX, evaluate::sec_c, 1, { TAG_COMPLEX } },
   { "sec", TAG_VECTOR,  evaluate::sec_v, 1, { TAG_VECTOR  } },
 
+  { "arcsec", TAG_DOUBLE,  evaluate::arcsec_d, 1, { TAG_DOUBLE  } },
+  { "arcsec", TAG_COMPLEX, evaluate::arcsec_c, 1, { TAG_COMPLEX } },
+  { "arcsec", TAG_VECTOR,  evaluate::arcsec_v, 1, { TAG_VECTOR  } },
+
   { "cosec", TAG_DOUBLE,  evaluate::cosec_d, 1, { TAG_DOUBLE  } },
   { "cosec", TAG_COMPLEX, evaluate::cosec_c, 1, { TAG_COMPLEX } },
   { "cosec", TAG_VECTOR,  evaluate::cosec_v, 1, { TAG_VECTOR  } },
 
+  { "arccosec", TAG_DOUBLE,  evaluate::arccosec_d, 1, { TAG_DOUBLE  } },
+  { "arccosec", TAG_COMPLEX, evaluate::arccosec_c, 1, { TAG_COMPLEX } },
+  { "arccosec", TAG_VECTOR,  evaluate::arccosec_v, 1, { TAG_VECTOR  } },
+
   { "sinh", TAG_DOUBLE,  evaluate::sinh_d, 1, { TAG_DOUBLE  } },
   { "sinh", TAG_COMPLEX, evaluate::sinh_c, 1, { TAG_COMPLEX } },
   { "sinh", TAG_VECTOR,  evaluate::sinh_v, 1, { TAG_VECTOR  } },
@@ -267,7 +320,7 @@ struct application_t eqn::applications[] = {
   { "cosh", TAG_COMPLEX, evaluate::cosh_c, 1, { TAG_COMPLEX } },
   { "cosh", TAG_VECTOR,  evaluate::cosh_v, 1, { TAG_VECTOR  } },
 
-  { "arcosh", TAG_DOUBLE,  evaluate::arcosh_d, 1, { TAG_DOUBLE  } },
+  { "arcosh", TAG_COMPLEX, evaluate::arcosh_d, 1, { TAG_DOUBLE  } },
   { "arcosh", TAG_COMPLEX, evaluate::arcosh_c, 1, { TAG_COMPLEX } },
   { "arcosh", TAG_VECTOR,  evaluate::arcosh_v, 1, { TAG_VECTOR  } },
 
@@ -283,17 +336,25 @@ struct application_t eqn::applications[] = {
   { "coth", TAG_COMPLEX, evaluate::coth_c, 1, { TAG_COMPLEX } },
   { "coth", TAG_VECTOR,  evaluate::coth_v, 1, { TAG_VECTOR  } },
 
+  { "arcoth", TAG_DOUBLE,  evaluate::arcoth_d, 1, { TAG_DOUBLE  } },
+  { "arcoth", TAG_COMPLEX, evaluate::arcoth_c, 1, { TAG_COMPLEX } },
+  { "arcoth", TAG_VECTOR,  evaluate::arcoth_v, 1, { TAG_VECTOR  } },
+
   { "sech", TAG_DOUBLE,  evaluate::sech_d, 1, { TAG_DOUBLE  } },
   { "sech", TAG_COMPLEX, evaluate::sech_c, 1, { TAG_COMPLEX } },
   { "sech", TAG_VECTOR,  evaluate::sech_v, 1, { TAG_VECTOR  } },
 
+  { "arsech", TAG_COMPLEX, evaluate::arsech_d, 1, { TAG_DOUBLE  } },
+  { "arsech", TAG_COMPLEX, evaluate::arsech_c, 1, { TAG_COMPLEX } },
+  { "arsech", TAG_VECTOR,  evaluate::arsech_v, 1, { TAG_VECTOR  } },
+
   { "cosech", TAG_DOUBLE,  evaluate::cosech_d, 1, { TAG_DOUBLE  } },
   { "cosech", TAG_COMPLEX, evaluate::cosech_c, 1, { TAG_COMPLEX } },
   { "cosech", TAG_VECTOR,  evaluate::cosech_v, 1, { TAG_VECTOR  } },
 
-  { "arcoth", TAG_DOUBLE,  evaluate::arcoth_d, 1, { TAG_DOUBLE  } },
-  { "arcoth", TAG_COMPLEX, evaluate::arcoth_c, 1, { TAG_COMPLEX } },
-  { "arcoth", TAG_VECTOR,  evaluate::arcoth_v, 1, { TAG_VECTOR  } },
+  { "arcosech", TAG_DOUBLE,  evaluate::arcosech_d, 1, { TAG_DOUBLE  } },
+  { "arcosech", TAG_COMPLEX, evaluate::arcosech_c, 1, { TAG_COMPLEX } },
+  { "arcosech", TAG_VECTOR,  evaluate::arcosech_v, 1, { TAG_VECTOR  } },
 
   { "ztor", TAG_DOUBLE,  evaluate::ztor_d, 1, { TAG_DOUBLE } },
   { "ztor", TAG_DOUBLE,  evaluate::ztor_d_d, 2, { TAG_DOUBLE, TAG_DOUBLE } },
@@ -376,6 +437,12 @@ struct application_t eqn::applications[] = {
   { "avg", TAG_COMPLEX, evaluate::avg_v, 1, { TAG_VECTOR  } },
   { "avg", TAG_COMPLEX, evaluate::avg_r, 2, { TAG_VECTOR, TAG_RANGE } },
 
+  { "length", TAG_DOUBLE, evaluate::length_d,  1, { TAG_DOUBLE  } },
+  { "length", TAG_DOUBLE, evaluate::length_c,  1, { TAG_COMPLEX } },
+  { "length", TAG_DOUBLE, evaluate::length_v,  1, { TAG_VECTOR  } },
+  { "length", TAG_DOUBLE, evaluate::length_m,  1, { TAG_MATRIX  } },
+  { "length", TAG_DOUBLE, evaluate::length_mv, 1, { TAG_MATVEC  } },
+
   { "array", TAG_VECTOR, evaluate::index_mv_2, 3,
     { TAG_MATVEC, TAG_DOUBLE, TAG_DOUBLE } },
   { "array", TAG_MATRIX, evaluate::index_mv_1, 2,
@@ -489,6 +556,10 @@ struct application_t eqn::applications[] = {
   { "adjoint",   TAG_MATRIX,  evaluate::adjoint_m,    1, { TAG_MATRIX } },
   { "adjoint",   TAG_MATVEC,  evaluate::adjoint_mv,   1, { TAG_MATVEC } },
 
+  { "signum", TAG_DOUBLE,  evaluate::signum_d, 1, { TAG_DOUBLE  } },
+  { "signum", TAG_COMPLEX, evaluate::signum_c, 1, { TAG_COMPLEX } },
+  { "signum", TAG_VECTOR,  evaluate::signum_v, 1, { TAG_VECTOR  } },
+
   { "sign", TAG_DOUBLE,  evaluate::sign_d, 1, { TAG_DOUBLE  } },
   { "sign", TAG_COMPLEX, evaluate::sign_c, 1, { TAG_COMPLEX } },
   { "sign", TAG_VECTOR,  evaluate::sign_v, 1, { TAG_VECTOR  } },
@@ -727,9 +798,31 @@ struct application_t eqn::applications[] = {
   { "runavg", TAG_VECTOR, evaluate::runavg_v_d, 2,
     { TAG_VECTOR,  TAG_DOUBLE } },
 
+  { "vt", TAG_DOUBLE,  evaluate::vt_d, 1, { TAG_DOUBLE  } },
+  { "vt", TAG_COMPLEX, evaluate::vt_c, 1, { TAG_COMPLEX } },
+  { "vt", TAG_VECTOR,  evaluate::vt_v, 1, { TAG_VECTOR  } },
+
   { "kbd", TAG_VECTOR, evaluate::kbd_d_d, 2, { TAG_DOUBLE, TAG_DOUBLE } },
   { "kbd", TAG_VECTOR, evaluate::kbd_d,   1, { TAG_DOUBLE } },
 
+  { "?:", TAG_DOUBLE, evaluate::ifthenelse_d_d, 3,
+    { TAG_BOOLEAN, TAG_DOUBLE | TAG_COMPLEX, TAG_DOUBLE | TAG_COMPLEX } },
+  { "<=", TAG_BOOLEAN, evaluate::lessorequal_d_d, 2,
+    { TAG_DOUBLE, TAG_DOUBLE } },
+  { ">=", TAG_BOOLEAN, evaluate::greaterorequal_d_d, 2,
+    { TAG_DOUBLE, TAG_DOUBLE } },
+  { "<",  TAG_BOOLEAN, evaluate::less_d_d, 2,     { TAG_DOUBLE, TAG_DOUBLE } },
+  { ">",  TAG_BOOLEAN, evaluate::greater_d_d, 2,  { TAG_DOUBLE, TAG_DOUBLE } },
+  { "==", TAG_BOOLEAN, evaluate::equal_d_d, 2,    { TAG_DOUBLE, TAG_DOUBLE } },
+  { "!=", TAG_BOOLEAN, evaluate::notequal_d_d, 2, { TAG_DOUBLE, TAG_DOUBLE } },
+  { "!",  TAG_BOOLEAN, evaluate::not_b, 1,   { TAG_BOOLEAN } },
+  { "||", TAG_BOOLEAN, evaluate::or_b_b, 2,  { TAG_BOOLEAN, TAG_BOOLEAN } },
+  { "&&", TAG_BOOLEAN, evaluate::and_b_b, 2, { TAG_BOOLEAN, TAG_BOOLEAN } },
+  { "==", TAG_BOOLEAN, evaluate::equal_b_b, 2,
+    { TAG_BOOLEAN, TAG_BOOLEAN } },
+  { "!=", TAG_BOOLEAN, evaluate::notequal_b_b, 2,
+    { TAG_BOOLEAN, TAG_BOOLEAN } },
+
   { NULL, 0, NULL, 0, { 0 } /* end of list */ }
 };
 
@@ -743,6 +836,10 @@ char * checker::tag2key (int tag) {
     key = "U"; break;
   case TAG_DOUBLE:
     key = "D"; break;
+  case TAG_DOUBLE|TAG_COMPLEX:
+    key = "D"; break;
+  case TAG_BOOLEAN:
+    key = "B"; break;
   case TAG_COMPLEX:
     key = "C"; break;
   case TAG_VECTOR:
