@@ -1,7 +1,7 @@
 /*
  * check_netlist.cpp - checker for the Qucs netlist
  *
- * Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: check_netlist.cpp,v 1.92 2006/05/15 06:45:42 raimi Exp $
+ * $Id: check_netlist.cpp,v 1.112 2007/05/27 17:50:14 ela Exp $
  *
  */
 
@@ -35,15 +35,17 @@
 
 #include "logging.h"
 #include "strlist.h"
+#include "netdefs.h"
 #include "equation.h"
 #include "check_netlist.h"
 #include "constants.h"
+#include "environment.h"
+#include "variable.h"
 
 /* Global definitions for parser and checker. */
 struct definition_t * definition_root = NULL;
 struct definition_t * subcircuit_root = NULL;
-struct node_t * node_root = NULL;
-struct pair_t * pair_root = NULL;
+environment * env_root = NULL;
 
 // Include list of available components.
 #include "qucsdefs.h"
@@ -51,7 +53,7 @@ struct pair_t * pair_root = NULL;
 // List of available microstrip components.
 static char * strip_available[] = {
   "MLIN", "MCORN", "MMBEND", "MSTEP", "MOPEN", "MGAP", "MCOUPLED", "MTEE",
-  "MCROSS", "MVIA", "CLIN", "COPEN", "CSHORT", "CGAP", "CSTEP", NULL };
+  "MCROSS", "MVIA", "CLIN", "COPEN", "CSHORT", "CGAP", "CSTEP", "BOND", NULL };
 
 /* The function counts the nodes in a definition line. */
 static int checker_count_nodes (struct definition_t * def) {
@@ -151,7 +153,7 @@ static struct value_t * checker_find_reference (struct definition_t * def,
 
 /* The function looks for the given property key within the properties
    of the given definition line and returns its value if the property
-   is not an identifier.  Otherwise the the function returns NULL. */
+   is not an identifier.  Otherwise the function returns NULL. */
 static struct value_t * checker_find_prop_value (struct definition_t * def,
 						 char * key) {
   struct pair_t * pair;
@@ -163,6 +165,12 @@ static struct value_t * checker_find_prop_value (struct definition_t * def,
   return NULL;
 }
 
+/* The function returns the number of properties in a definition line
+   specified by the given key. */
+static int checker_find_property (struct definition_t * def, char * key) {
+  return checker_find_property (key, def->pairs);
+}
+
 /* This function looks for the specified property 'key' in the given
    definition and return its value (a string reference) if it is a
    reference.  Otherwise the function returns NULL and emits an
@@ -202,13 +210,14 @@ static struct special_t checker_specials[] = {
 			       "LineSearch", "Attenuation", "SteepestDescent",
 			       NULL } },
   { "DC",     "Solver",      { "CroutLU", "DoolittleLU", "HouseholderQR",
-			       "HouseholderLQ", NULL } },
+			       "HouseholderLQ", "GolubSVD", NULL } },
   { "TR",     "Type",        { "lin", "log", NULL } },
   { "TR",     "IntegrationMethod",
     { "Euler", "Trapezoidal", "Gear", "AdamsMoulton", NULL } },
   { "TR",     "Solver",      { "CroutLU", "DoolittleLU", "HouseholderQR",
-			       "HouseholderLQ", NULL } },
+			       "HouseholderLQ", "GolubSVD", NULL } },
   { "TR",     "relaxTSR",    { "yes", "no", NULL } },
+  { "TR",     "initialDC",   { "yes", "no", NULL } },
   { "MLIN",   "DispModel",   { "Kirschning", "Kobayashi", "Yamashita",
 			       "Getsinger", "Schneider", "Pramanick",
 			       "Hammerstad", NULL } },
@@ -245,6 +254,12 @@ static struct special_t checker_specials[] = {
 			       "Getsinger", "Schneider", "Pramanick",
 			       "Hammerstad", NULL } },
   { "MTEE",   "MSModel",     { "Wheeler", "Schneider", "Hammerstad", NULL } },
+  { "MCROSS", "MSDispModel", { "Kirschning", "Kobayashi", "Yamashita",
+			       "Getsinger", "Schneider", "Pramanick",
+			       "Hammerstad", NULL } },
+  { "MCROSS", "MSModel",     { "Wheeler", "Schneider", "Hammerstad", NULL } },
+  { "hicumL2p1", "Type",     { "npn", "pnp", NULL } },
+  { "BOND",   "Model",       { "FREESPACE", "MIRROR", "DESCHARLES", NULL } },
   { NULL, NULL, { NULL } }
 };
 
@@ -334,7 +349,20 @@ static int checker_resolve_variable (struct definition_t * root,
     if ((val = checker_find_variable (root, "SPfile", "File", value->ident))) {
       found++;
     }
-    /* TODO: find variable in equations */
+    /* 7. find variable in equation */
+    if (root->env->getChecker()->containsVariable (value->ident)) {
+      variable * v;
+      value->var = eqn::TAG_DOUBLE;
+      if ((v = root->env->getVariable (value->ident)) == NULL) {
+	// put variable into the environment
+	v = new variable (value->ident);
+	eqn::constant * c = new eqn::constant (eqn::TAG_DOUBLE);
+	v->setConstant (c);
+	root->env->addVariable (v);
+      }
+      found++;
+    }
+    /* not found */
     if (!found) {
       logprint (LOG_ERROR, "line %d: checker error, no such variable `%s' "
 		"used in a `%s:%s' property\n", def->line, value->ident,
@@ -615,21 +643,21 @@ static int checker_validate_lists (struct definition_t * root) {
 	  }
 	}
 	// property 'Start' is invalid
-	if ((val = checker_find_prop_value (def, "Start")) != NULL) {
+	if (checker_find_property (def, "Start") > 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, extraneous property "
 		    "`%s' is invalid in `%s:%s'\n", def->line, "Start",
 		    def->type, def->instance);
 	  errors++;
 	}
 	// property 'Stop' is invalid
-	if ((val = checker_find_prop_value (def, "Stop")) != NULL) {
+	if (checker_find_property (def, "Stop") > 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, extraneous property "
 		    "`%s' is invalid in `%s:%s'\n", def->line, "Stop",
 		    def->type, def->instance);
 	  errors++;
 	}
 	// property 'Points' is also invalid
-	if ((val = checker_find_prop_value (def, "Points")) != NULL) {
+	if (checker_find_property (def, "Points") > 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, extraneous property "
 		    "`%s' is invalid in `%s:%s'\n", def->line, "Points",
 		    def->type, def->instance);
@@ -639,28 +667,28 @@ static int checker_validate_lists (struct definition_t * root) {
       // linearly and logarithmically stepped sweeps
       else if (type && (!strcmp (type, "lin") || !strcmp (type, "log"))) {
 	// property 'Start' required
-	if ((val = checker_find_prop_value (def, "Start")) == NULL) {
+	if (checker_find_property (def, "Start") <= 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, required property "
 		    "`%s' not found in `%s:%s'\n", def->line, "Start",
 		    def->type, def->instance);
 	  errors++;
 	}
 	// property 'Stop' required
-	if ((val = checker_find_prop_value (def, "Stop")) == NULL) {
+	if (checker_find_property (def, "Stop") <= 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, required property "
 		    "`%s' not found in `%s:%s'\n", def->line, "Stop",
 		    def->type, def->instance);
 	  errors++;
 	}
 	// property 'Points' is also required
-	if ((val = checker_find_prop_value (def, "Points")) == NULL) {
+	if (checker_find_property (def, "Points") <= 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, required property "
 		    "`%s' not found in `%s:%s'\n", def->line, "Points",
 		    def->type, def->instance);
 	  errors++;
 	}
 	// property 'Values' is invalid
-	if ((val = checker_find_prop_value (def, "Values")) != NULL) {
+	if (checker_find_property (def, "Values") > 0) {
 	  logprint (LOG_ERROR, "line %d: checker error, extraneous property "
 		    "`%s' is invalid in `%s:%s'\n", def->line, "Values",
 		    def->type, def->instance);
@@ -699,7 +727,7 @@ static int checker_validate_actions (struct definition_t * root) {
 		"single or none required\n", n);
       errors++;
     }
-    if (a >=1 && c >= 1 && n < 1) {
+    if (a >= 1 && c >= 1 && n < 1) {
       logprint (LOG_ERROR, "checker error, a .DC action is required for this "
 		"circuit definition (accounted %d non-linearities)\n", c);
       errors++;
@@ -805,10 +833,11 @@ static int checker_validate_nodesets (struct definition_t * root) {
    list have been checked.  It verifies that parameter sweep
    definitions and equation variable identifiers are unique.  The
    function returns zero on success and non-zero otherwise. */
-static int netlist_checker_variables_intern (struct definition_t * root) {
+static int netlist_checker_variables_intern (struct definition_t * root,
+					     environment * env) {
   int errors = 0, pos;
   struct value_t * para, * ref;
-  strlist * eqnvars = equation_variables ();
+  strlist * eqnvars = env->getChecker()->variables ();
   strlist * instances = new strlist ();
   strlist * vars = new strlist ();
   strlist * refs = new strlist ();
@@ -864,8 +893,8 @@ static int netlist_checker_variables_intern (struct definition_t * root) {
 
 /* This is the overall variable checker for the parsed netlist.  See
    the above function for details. */
-int netlist_checker_variables (void) {
-  return netlist_checker_variables_intern (definition_root);
+int netlist_checker_variables (environment * env) {
+  return netlist_checker_variables_intern (definition_root, env);
 }
 
 /* The function checks whether the given key-value combination is
@@ -1210,12 +1239,33 @@ static char * checker_subcircuit_instance (char * type, char * instances,
    circuit element list in reverse order. */
 static struct definition_t *
 checker_copy_subcircuits (struct definition_t * type,
-			  struct definition_t * inst, strlist * * instances) {
+			  struct definition_t * inst, strlist * * instances,
+			  environment * parent) {
   struct definition_t * def, * copy;
   struct definition_t * root = NULL;
   strlist * instcopy;
   char * list;
 
+  // create environment for subcircuit instance
+  environment * child = new environment (*(type->env));
+  parent->addChild (child);
+
+  // put instance properties into subcircuit environment
+  for (struct pair_t * pair = inst->pairs; pair != NULL; pair = pair->next) {
+    // anything else than the 'Type'
+    if (strcmp (pair->key, "Type")) {
+      if (pair->value->ident == NULL) {
+	// simple value
+	child->setDoubleConstant (pair->key, pair->value->value);
+	child->setDouble (pair->key, pair->value->value);
+      }
+      else {
+	// reference to variable in upper environment
+	child->setDoubleReference (pair->key, pair->value->ident);
+      }
+    }
+  }
+
   // go through element list of subcircuit
   for (def = type->sub; def != NULL; def = def->next) {
 
@@ -1231,7 +1281,7 @@ checker_copy_subcircuits (struct definition_t * type,
       instcopy = new strlist (*(*instances));
       // append instance name to recursive instance list
       (*instances)->append (inst->instance);
-      copy = checker_copy_subcircuits (sub, def, instances);
+      copy = checker_copy_subcircuits (sub, def, instances, child);
       // put the expanded definitions into the sublist
       if (copy) {
 	list = checker_subcircuit_instance_list (instcopy);
@@ -1259,6 +1309,8 @@ checker_copy_subcircuits (struct definition_t * type,
       copy->subcircuit = strdup (type->instance);
       // assign node list
       checker_copy_subcircuit_nodes (type, inst, def, copy, list);
+      // apply environment
+      copy->env = child;
       // chain definition (circuit) list
       copy->next = root;
       root = copy;
@@ -1267,6 +1319,15 @@ checker_copy_subcircuits (struct definition_t * type,
     // cleanup translated nodes
     checker_cleanup_xlat_nodes (def);
   }
+
+  // try giving child environment a unique name
+  strlist * icopy = new strlist ();
+  icopy->append (type->instance);
+  icopy->append (*(instances));
+  icopy->append (inst->instance);
+  child->setName (icopy->toString ("."));
+  delete icopy;
+
   return root;
 }
 
@@ -1306,8 +1367,11 @@ static int checker_validate_sub_cycles (struct definition_t * root,
 	  strlist * copy = new strlist (*(*deps));
 	  // validate subcircuit
 	  sub = checker_find_subcircuit (val->ident);
-	  error = checker_validate_sub_cycles (sub, sub->instance,
-					       instance, deps);
+	  if (sub != NULL) // if possible
+	    error = checker_validate_sub_cycles (sub, sub->instance,
+						 instance, deps);
+	  else
+	    error = 1;
 	  // on errors: go on
 	  if (error) {
 	    errors += error;
@@ -1326,6 +1390,131 @@ static int checker_validate_sub_cycles (struct definition_t * root,
   return errors;
 }
 
+/* This function dynamically creates a circuit definition based on a
+   given subcircuit definition including type, number of nodes and the
+   properties. */
+static struct define_t * netlist_create_define (struct definition_t * def) {
+  int o, r;
+  struct pair_t * p;
+  struct define_t * d =
+    (struct define_t *) calloc (sizeof (struct define_t), 1);
+  d->type = strdup (def->instance);
+  d->nodes = checker_count_nodes (def);
+  d->action = PROP_COMPONENT;
+  for (o = r = 0, p = def->pairs; p != NULL; p = p->next) {
+    if (p->value == NULL) {
+      // required
+      d->required[r].key = strdup (p->key);
+      d->required[r].type = PROP_REAL;
+      d->required[r].defaultval.d = PROP_NO_VAL;
+      d->required[r].defaultval.s = PROP_NO_STR;
+      d->required[r].range.il = '.';
+      d->required[r].range.l = 0;
+      d->required[r].range.h = 0;
+      d->required[r].range.ih = '.';
+      r++;
+    } else {
+      // optional
+      d->optional[o].key = strdup (p->key);
+      d->optional[o].type = PROP_REAL;
+      d->optional[o].defaultval.d = p->value->value;
+      d->optional[o].defaultval.s = PROP_NO_STR;
+      d->optional[o].range.il = '.';
+      d->optional[o].range.l = 0;
+      d->optional[o].range.h = 0;
+      d->optional[o].range.ih = '.';
+      o++;
+    }
+  }
+
+  // extra required
+  d->required[r].key = strdup ("Type");
+  d->required[r].type = PROP_STR;
+  d->required[r].defaultval.d = PROP_NO_VAL;
+  d->required[r].defaultval.s = PROP_NO_STR;
+  d->required[r].range.il = '.';
+  d->required[r].range.l = 0;
+  d->required[r].range.h = 0;
+  d->required[r].range.ih = '.';
+  return d;
+}
+
+/* The function destroys the given circuit definition which must have
+   been dynamically created before. */
+static void netlist_free_define (struct define_t * d) {
+  int i;
+  struct property_t * p;
+  free (d->type);
+  // free required properties
+  for (i = 0, p = d->required; p[i].key != NULL; i++) {
+    free (p[i].key);
+  }
+  // free optional properties
+  for (i = 0, p = d->optional; p[i].key != NULL; i++) {
+    free (p[i].key);
+  }
+  free (d);
+}
+
+/* The function checks the presence of required and optional
+   properties as well as their content in the given definition.  It
+   returns zero on success and non-zero otherwise. */
+static int checker_validate_properties (struct definition_t * root,
+					struct definition_t * def,
+					struct define_t * available) {
+  struct pair_t * pair;
+  int i, n, errors = 0;
+
+  /* check whether the required properties are given */
+  for (i = 0; PROP_IS_PROP (available->required[i]); i++) {
+    n = checker_find_property (available->required[i].key, def->pairs);
+    if (n != 1) {
+      logprint (LOG_ERROR, "line %d: checker error, required property "
+		"`%s' occurred %dx in `%s:%s'\n", def->line,
+		available->required[i].key, n, def->type, def->instance);
+      errors++;
+    }
+  }
+  /* check whether the optional properties are given zero/once */
+  for (i = 0; PROP_IS_PROP (available->optional[i]); i++) {
+    n = checker_find_property (available->optional[i].key, def->pairs);
+    if (n >= 2) {
+      logprint (LOG_ERROR, "line %d: checker error, optional property "
+		"`%s' occurred %dx in `%s:%s'\n", def->line,
+		available->optional[i].key, n, def->type, def->instance);
+      errors++;
+    }
+  }
+
+  /* check the property content */
+  for (pair = def->pairs; pair != NULL; pair = pair->next) {
+    /* check whether properties are either required or optional */
+    if (!checker_is_property (available, pair->key)) {
+      if (strcmp (def->type, "Def")) {
+	logprint (LOG_ERROR, 
+		  "line %d: checker error, extraneous property `%s' is "
+		  "invalid in `%s:%s'\n", def->line,
+		  pair->key, def->type, def->instance);
+	errors++;
+      }
+    }
+    // do not check zero-length lists
+    if (pair->value != NULL) {
+      /* check and evaluate the unit scale in a property */
+      if (!checker_evaluate_scale (pair->value))
+	errors++;
+      /* check whether properties are in range */
+      if (!checker_value_in_range (def->instance, available, pair)) {
+	errors++;
+      }
+      /* check variables in properties */
+      if (!checker_resolve_variable (root, def, pair->value))
+	errors++;
+    }
+  }
+  return errors;
+}
+
 /* This function is used by the netlist checker to validate the
    subcircuits.  It returns zero with no errors and non-zero on
    errors. */
@@ -1360,6 +1549,10 @@ static int checker_validate_subcircuits (struct definition_t * root) {
 		      def->type, def->instance, n1);
 	    errors++;
 	  }
+	  // check the subcircuit instance properties
+	  struct define_t * available = netlist_create_define (sub);
+	  errors += checker_validate_properties (root, def, available);
+	  netlist_free_define (available);
 	  // and finally check for cyclic definitions
 	  strlist * deps = new strlist ();
 	  int err = checker_validate_sub_cycles (sub, sub->instance,
@@ -1384,16 +1577,23 @@ static void netlist_free_nodes (struct node_t * node) {
   }
 }
 
+/* The following function free()'s the given value. */
+static void netlist_free_value (struct value_t * value) {
+  if (value->ident) free (value->ident);
+  if (value->unit)  free (value->unit);
+  if (value->scale) free (value->scale);
+  free (value);
+}
+
 /* Deletes pair list of a definition. */
 static void netlist_free_pairs (struct pair_t * pair) {
-  struct pair_t * n;
-  for (; pair != NULL; pair = n) {
-    n = pair->next;
-    if (pair->value) {
-      if (pair->value->ident) free (pair->value->ident);
-      if (pair->value->unit)  free (pair->value->unit);
-      if (pair->value->scale) free (pair->value->scale);
-      free (pair->value);
+  struct pair_t * np;
+  for (; pair != NULL; pair = np) {
+    np = pair->next;
+    struct value_t * nv, * value;
+    for (value = pair->value; value != NULL; value = nv) {
+      nv = value->next;
+      netlist_free_value (value);
     }
     free (pair->key);
     free (pair);
@@ -1435,7 +1635,7 @@ netlist_unchain_definition (struct definition_t * root,
    list and returns the expanded list with the subcircuit definitions
    removed. */
 static struct definition_t *
-checker_expand_subcircuits (struct definition_t * root) {
+checker_expand_subcircuits (struct definition_t * root, environment * parent) {
   struct definition_t * def, * sub, * copy, * next, * prev;
   strlist * instances = NULL;
 
@@ -1444,9 +1644,10 @@ checker_expand_subcircuits (struct definition_t * root) {
     next = def->next;
     // is this a subcircuit instance definition ?
     if (!strcmp (def->type, "Sub")) {
-      // get the subcircuit type definition and make a copy of it
+      // get the subcircuit type definition
       sub = checker_get_subcircuit (def);
-      copy = checker_copy_subcircuits (sub, def, &instances);
+      // and make a copy of it
+      copy = checker_copy_subcircuits (sub, def, &instances, parent);
       if (instances) { delete instances; instances = NULL; }
       // remove the subcircuit instance from the original list
       if (prev) {
@@ -1459,12 +1660,15 @@ checker_expand_subcircuits (struct definition_t * root) {
       if (copy) {
 	struct definition_t * last = checker_find_last_definition (copy);
 	last->next = root;
-	prev = last;
-	next = root;
+	if (!prev) prev = last;
 	root = copy;
       }
     }
-    else prev = def;
+    // component in the root environment
+    else {
+      prev = def;
+      def->env = parent;
+    }
   }
   return root;
 }
@@ -1474,9 +1678,8 @@ checker_expand_subcircuits (struct definition_t * root) {
    errors. */
 static int netlist_checker_intern (struct definition_t * root) {
   struct definition_t * def;
-  struct pair_t * pair;
   struct define_t * available;
-  int i, n, errors = 0;
+  int n, errors = 0;
 
   /* go through all definitions */
   for (def = root; def != NULL; def = def->next) {
@@ -1516,49 +1719,9 @@ static int netlist_checker_intern (struct definition_t * root) {
 		  available->nodes, def->type, def->instance, n);
 	errors++;
       }
-      /* check whether the required properties are given */
-      for (i = 0; PROP_IS_PROP (available->required[i]); i++) {
-	n = checker_find_property (available->required[i].key, def->pairs);
-	if (n != 1) {
-	  logprint (LOG_ERROR, "line %d: checker error, required property "
-		    "`%s' occurred %dx in `%s:%s'\n", def->line,
-		    available->required[i].key, n, def->type, def->instance);
-	  errors++;
-	}
-      }
-      /* check whether the optional properties are given zero/once */
-      for (i = 0; PROP_IS_PROP (available->optional[i]); i++) {
-	n = checker_find_property (available->optional[i].key, def->pairs);
-	if (n >= 2) {
-	  logprint (LOG_ERROR, "line %d: checker error, optional property "
-		    "`%s' occurred %dx in `%s:%s'\n", def->line,
-		    available->optional[i].key, n, def->type, def->instance);
-	  errors++;
-	}
-      }
-      /* check the properties */
-      for (pair = def->pairs; pair != NULL; pair = pair->next) {
-	/* check whether properties are either required or optional */
-	if (!checker_is_property (available, pair->key)) {
-	  logprint (LOG_ERROR, 
-		    "line %d: checker error, extraneous property `%s' is "
-		    "invalid in `%s:%s'\n", def->line,
-		    pair->key, def->type, def->instance);
-	  errors++;
-	}
-	// do not check zero-length lists
-	if (pair->value != NULL) {
-	  /* check and evaluate the unit scale in a property */
-	  if (!checker_evaluate_scale (pair->value))
-	    errors++;
-	  /* check whether properties are in range */
-	  if (!checker_value_in_range (def->instance, available, pair)) {
-	    errors++;
-	  }
-	  /* check variables in properties */
-	  if (!checker_resolve_variable (root, def, pair->value))
-	    errors++;
-	}
+      /* check the properties except for subcircuits */
+      if (strcmp (def->type, "Sub")) {
+	errors += checker_validate_properties (root, def, available);
       }
     }
     /* check the number of definitions */
@@ -1647,27 +1810,130 @@ void netlist_status (void) {
   }
 }
 
+/* The function builds the equation list for a given list of
+   definition and removes the definition containing the equations from
+   the list. */
+static struct definition_t *
+checker_build_equations (struct definition_t * root, eqn::node ** eroot) {
+  struct definition_t * def, * next, * prev;
+  eqn::node * eqns, * last;
+  *eroot = NULL;
+  // go through list of definitions
+  for (prev = NULL, def = root; def != NULL; def = next) {
+    next = def->next;
+    if (!strcmp (def->type, "Eqn")) {
+      // rechain definition list
+      if (prev) {
+	prev->next = next;
+      } else {
+	root = next;
+      }
+      // append equations
+      eqns = (eqn::node *) def->eqns;
+      last = eqn::checker::lastEquation (eqns);
+      last->setNext (*eroot);
+      *eroot = eqns;
+      // free this definition
+      netlist_free_definition (def);
+    }
+    else prev = def;
+  }
+  return root;
+}
+
+/* The function creates an environment for the given definition root
+   including equation checker and solver. */
+static void checker_setup_env (struct definition_t * root,
+			       environment * env, eqn::node * eqns) {
+  // create equation checker
+  eqn::checker * checkee = new eqn::checker ();
+  // pass equations to the checker
+  checkee->setEquations (eqns);
+  // add constants to the list of equations
+  checkee->constants ();
+  // pass checker
+  env->setChecker (checkee);
+  // create equation solver
+  eqn::solver * solvee = new eqn::solver (checkee);
+  // pass solver
+  env->setSolver (solvee);
+  // apply environment to the netlist root
+  if (root) root->env = env;
+}
+
+/* Adds the arguments of a subcircuit into the equation checker of the
+   given environment. */
+static void checker_subcircuit_args (struct definition_t * def,
+				     environment * env) {
+  for (struct pair_t * pair = def->pairs; pair != NULL; pair = pair->next) {
+    // anything else than the 'Type'
+    if (strcmp (pair->key, "Type")) {
+      // put it into the equation checker
+      env->getChecker()->addDouble ("#subcircuit",
+				    pair->key, pair->value->value);
+      // also put it into the environment
+      variable * v = new variable (pair->key);
+      eqn::constant * c = new eqn::constant (eqn::TAG_DOUBLE);
+      c->d = pair->value->value;
+      v->setConstant (c);
+      env->addVariable (v);
+    }
+  }
+}
+
 /* This is the global netlist checker.  It returns zero on success and
    non-zero on errors. */
-int netlist_checker (void) {
+int netlist_checker (environment * env) {
   int errors = 0;
+  eqn::node * eqns;
   struct definition_t * def;
-  // first create the subcircuit list
+
+  // create top-level environment
+  env_root = new environment (env->getName ());
+  // create the subcircuit list
   definition_root = checker_build_subcircuits (definition_root);
-  // check global netlist
-  errors += netlist_checker_intern (definition_root);
+  // get equation list
+  definition_root = checker_build_equations (definition_root, &eqns);
+  // setup the root environment
+  checker_setup_env (definition_root, env_root, eqns);
   // check list of subcircuits
   errors += netlist_checker_intern (subcircuit_root);
+  // check global netlist
+  errors += netlist_checker_intern (definition_root);
+  // check equations in root
+  env_root->setDefinitions (definition_root);
+  errors += env_root->equationChecker (0);
+  env_root->setDefinitions (NULL);
+
   // then check each subcircuit list
   for (def = subcircuit_root; def != NULL; def = def->next) {
+    // get equation list
+    def->sub = checker_build_equations (def->sub, &eqns);
+    // setup the subcircuit environment
+    environment * subenv = new environment (def->instance);
+    env_root->addChild (subenv);
+    checker_setup_env (def, subenv, eqns);
+    if (def->sub) def->sub->env = subenv;
+    // add subcircuit parameters to equations
+    checker_subcircuit_args (def, subenv);
+    // check subcircuit netlist
     errors += netlist_checker_intern (def->sub);
+    // check equations in subcircuit
+    subenv->setDefinitions (def->sub);
+    errors += subenv->equationChecker (0);
+    subenv->setDefinitions (NULL);
   }
+
   // check actions
   errors += checker_validate_actions (definition_root);
+
   if (!errors) {
+    // create actual root environment
+    env->copy (*env_root);
     // and finally expand the subcircuits into the global netlist
-    definition_root = checker_expand_subcircuits (definition_root);
+    definition_root = checker_expand_subcircuits (definition_root, env);
   }
+
   return errors ? -1 : 0;
 }
 
@@ -1688,7 +1954,13 @@ void netlist_destroy (void) {
   }
   netlist_destroy_intern (subcircuit_root);
   definition_root = subcircuit_root = NULL;
-  node_root = NULL;
-  pair_root = NULL;
   netlist_lex_destroy ();
 }
+
+/* Delete root environment(s) if necessary. */
+void netlist_destroy_env (void) {
+  if (env_root != NULL) {
+    delete env_root;
+    env_root = NULL;
+  }
+}
