@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: capacitor.cpp,v 1.18 2006/03/27 09:55:50 raimi Exp $
+ * $Id: capacitor.cpp,v 1.20 2006/07/24 08:07:42 raimi Exp $
  *
  */
 
@@ -85,6 +85,11 @@ void capacitor::calcTR (nr_double_t) {
   nr_double_t g, i;
   nr_double_t v = real (getV (NODE_1) - getV (NODE_2));
 
+  /* apply initial condition if requested */
+  if (getMode () == MODE_INIT && isPropertyGiven ("V")) {
+    v = getPropertyDouble ("V");
+  }
+
   setState (qState, c * v);
   integrate (qState, c, g, i);
   setY (NODE_1, NODE_1, +g); setY (NODE_2, NODE_2, +g);
