@@ -1,7 +1,7 @@
 /*
  * check_spice.cpp - checker for a Spice netlist
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: check_spice.cpp,v 1.20 2006/05/23 09:48:32 raimi Exp $
+ * $Id: check_spice.cpp,v 1.38 2007/05/17 09:28:19 ela Exp $
  *
  */
 
@@ -41,6 +41,7 @@
 #include "constants.h"
 #include "check_spice.h"
 #include "qucs_producer.h"
+#include "hash.h"
 
 /* Global definitions for parser and checker. */
 struct definition_t * definition_root = NULL;
@@ -131,6 +132,11 @@ struct define_t spice_definition_available[] =
     { PROP_NO_PROP },
     { PROP_NO_PROP }
   },
+  /* lossless transmission line */
+  { "T", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "Z0", PROP_REAL, { 50, PROP_NO_STR }, PROP_POS_RANGE }, PROP_NO_PROP },
+    { PROP_NO_PROP }
+  },
   /* transient analysis */
   { "TRAN", 0, PROP_ACTION, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { PROP_NO_PROP },
@@ -385,6 +391,7 @@ static struct pair_t * spice_get_pairs (struct definition_t * def) {
   // if there is a value given and no description (key), it is required
   foreach_value (def->values, val) {
     prop = &def->define->required[i];
+    // a float given ?
     if (val->hint & HINT_NUMBER && prop->key) {
       p = create_pair ();
       p->key = strdup (prop->key);
@@ -394,7 +401,14 @@ static struct pair_t * spice_get_pairs (struct definition_t * def) {
       spice_value_done (val);
       i++;
     }
-    else break;
+    // skip identifier if next is a float again (F and H sources)
+    else if ((def->type[0] == 'F' || def->type[0] == 'H') &&
+	     val->hint & HINT_NAME &&
+	     val->next && val->next->hint & HINT_NUMBER)
+      continue;
+    // break it here
+    else
+      break;
   }
   // other key/value pairs on that line
   foreach_value (def->values, val) {
@@ -443,14 +457,18 @@ struct spice_device_t {
   char * trans_type_prop; // value of 'Type' in Qucs
 }
 spice_devices[] = {
-  { "NPN",  "BJT",    "npn"  },
-  { "PNP",  "BJT",    "pnp"  },
-  { "NJF",  "JFET",   "nfet" },
-  { "PJF",  "JFET",   "pfet" },
-  { "NMOS", "MOSFET", "nfet" },
-  { "PMOS", "MOSFET", "pfet" },
-  { "D",    "Diode",  NULL   },
-  { "SW",   "Relais", NULL   },
+  { "NPN",     "BJT",    "npn"  },
+  { "PNP",     "BJT",    "pnp"  },
+  { "NJF",     "JFET",   "nfet" },
+  { "PJF",     "JFET",   "pfet" },
+  { "NMOS",    "MOSFET", "nfet" },
+  { "PMOS",    "MOSFET", "pfet" },
+  { "D",       "Diode",  NULL   },
+  { "SW",      "Relais", NULL   },
+  { "VSWITCH", "Relais", NULL   },
+  { "RES",     "R",      NULL   },
+  { "R",       "R",      NULL   },
+  { "C",       "C",      NULL   },
   { NULL, NULL, NULL }
 };
 
@@ -487,6 +505,28 @@ spice_find_property (struct definition_t * def, const char * prop) {
   return NULL;
 }
 
+/* This function is the case-insensitive version of the above. */
+static struct pair_t *
+spice_find_property_nocase (struct definition_t * def, const char * prop) {
+  struct pair_t * pair;
+  for (pair = def->pairs; pair != NULL; pair = pair->next) {
+    if (!strcasecmp (pair->key, prop))
+      return pair;
+  }
+  return NULL;
+}
+
+/* The function starts at a given property pair and is otherwise
+   similar to the above function. */
+static struct pair_t *
+spice_find_property_nocase (struct pair_t * pair, const char * prop) {
+  for (; pair != NULL; pair = pair->next) {
+    if (!strcasecmp (pair->key, prop))
+      return pair;
+  }
+  return NULL;
+}
+
 /* The function replaces or appends the given key/value pair to the
    list of properties of the given definition. */
 static void
@@ -547,6 +587,10 @@ static void spice_adjust_device (struct definition_t * def,
 static void spice_translate_device (struct definition_t * root,
 				    struct definition_t * def) {
   struct value_t * inst = spice_find_device_instance (def);
+
+  // return if no Model given
+  if (inst == NULL) return;
+
   /* first look for the Model in the local definition root, then in
      the global definition root */
   struct definition_t * tran;
@@ -592,6 +636,7 @@ spice_device_table[] = {
   { "H", "CCVS"   },
   { "K", "Tr"     },
   { "S", "Relais" },
+  { "T", "TLIN4P" },
   { NULL, NULL }
 };
 
@@ -636,6 +681,14 @@ node_translations[] = {
     { 2, 1, -1 },
     { 1, 2, -1 }
   },
+  { "Iac", 0,
+    { 2, 1, -1 },
+    { 1, 2, -1 }
+  },
+  { "Idc", 0,
+    { 2, 1, -1 },
+    { 1, 2, -1 }
+  },
   { "VCCS", 0,
     { 3, 1, 2, 4, -1 },
     { 1, 2, 3, 4, -1 }
@@ -664,6 +717,10 @@ node_translations[] = {
     { 3, 1, 2, 4, -1 },
     { 1, 2, 3, 4, -1 }
   },
+  { "TLIN4P", 0,
+    { 1, 3, 4, 2, -1 },
+    { 1, 2, 3, 4, -1 }
+  },
   { NULL, 0, { -1 }, { -1 } }
 };
 
@@ -704,7 +761,7 @@ static void netlist_free_values (struct value_t * value) {
 
 /* Deletes the given key/value pair. */
 void netlist_free_pair (struct pair_t * pair) {
-  if (pair->value) netlist_free_value (pair->value);
+  if (pair->value) netlist_free_values (pair->value);
   free (pair->key);
   free (pair);
 }
@@ -866,37 +923,47 @@ static void spice_adjust_optional_properties (struct definition_t * def) {
 
 // Helper structure for property translations and aliases in devices.
 struct property_translation_t {
+  char * type;
   char * key;
   char * trans;
 }
 property_translations[] = {
   /* BJT device */
-  { "CCS", "Cjs" },
-  { "VA",  "Vaf" },
-  { "VB",  "Var" },
-  { "IK",  "Ikf" },
-  { "PE",  "Vje" },
-  { "ME",  "Mje" },
-  { "PC",  "Vjc" },
-  { "MC",  "Mjc" },
-  { "PS",  "Vjs" },
-  { "MS",  "Mjs" },
+  { NULL, "CCS", "Cjs" },
+  { NULL, "VA",  "Vaf" },
+  { NULL, "VB",  "Var" },
+  { NULL, "IK",  "Ikf" },
+  { NULL, "PE",  "Vje" },
+  { NULL, "ME",  "Mje" },
+  { NULL, "PC",  "Vjc" },
+  { NULL, "MC",  "Mjc" },
+  { NULL, "PS",  "Vjs" },
+  { NULL, "MS",  "Mjs" },
   /* MOSFET device */
-  { "VTO", "Vt0" },
-  { "U0",  "Uo"  },
+  { NULL, "VTO", "Vt0" },
+  { NULL, "U0",  "Uo"  },
   /* DIODE device */
-  { "CJO", "Cj0" },
-  { NULL, NULL }
+  { NULL, "CJO", "Cj0" },
+  /* OTHER devices */
+  { "C",  "IC",  "V"   },
+  { "L",  "IC",  "I"   },
+  { NULL, "Z0",  "Z"   },
+  { "R",  "TC",  "Tc1" },
+  /* END of list */
+  { NULL, NULL,  NULL  }
 };
 
 /* The function translates property and aliases of devices in the list
    of key/value pairs of the given definition. */
-void spice_adjust_alias_properties (struct pair_t * pair) {
+void spice_adjust_alias_properties (struct definition_t * def,
+				    struct pair_t * pair) {
   struct property_translation_t * prop = property_translations;
   for (; prop->key != NULL; prop++) {
-    if (!strcasecmp (prop->key, pair->key)) {
-      free (pair->key);
-      pair->key = strdup (prop->trans);
+    if (!prop->type || !strcmp (prop->type, def->type)) {
+      if (!strcasecmp (prop->key, pair->key)) {
+	free (pair->key);
+	pair->key = strdup (prop->trans);
+      }
     }
   }
 }
@@ -929,7 +996,7 @@ void spice_adjust_properties (struct definition_t * def) {
 	}
       }
       // some other direct translations
-      if (!found) spice_adjust_alias_properties (pair);
+      if (!found) spice_adjust_alias_properties (def, pair);
     }
   }
 }
@@ -1070,6 +1137,25 @@ spice_del_definition (struct definition_t * root, struct definition_t * def) {
   return root;
 }
 
+/* Unchains a property pair and deletes it. */
+struct pair_t *
+spice_del_property (struct pair_t * root, struct pair_t * pair) {
+  struct pair_t * prev;
+  if (pair == root) {
+    root = pair->next;
+    netlist_free_pair (pair);
+  }
+  else {
+    // find previous to the candidate to be deleted
+    for (prev = root; prev != NULL && prev->next != pair; prev = prev->next);
+    if (prev != NULL) {
+      prev->next = pair->next;
+      netlist_free_pair (pair);
+    }
+  }
+  return root;
+}
+
 /* This function creates an internal node name used during the
    conversion process.  The returned string is static and must be
    copied by the caller.  Successive calls result in different node
@@ -1248,6 +1334,20 @@ static char * spice_untranslated_text (struct definition_t * def) {
 #define VAL_IS_DONE(val) \
   ((val) == NULL || (val)->hint & HINT_DONE)
 
+/* The function counts values in a property list up to a stop
+   value. */
+static int spice_count_real_values (struct value_t * values) {
+  int ret = 0;
+  struct value_t * val;
+  // go through each not yet processed value
+  foreach_value (values, val) {
+    if (val->hint & (HINT_NODE | HINT_NUMBER))
+      ret++;
+    if (val->hint & HINT_MSTOP) break; // stop here if necessary
+  }
+  return ret;
+}
+
 /* This function is the independent source translator.  If necessary
    new kinds of sources are created.  This must be done since Qucs has
    separate sources for each type of excitation and Spice summarizes
@@ -1255,15 +1355,18 @@ static char * spice_untranslated_text (struct definition_t * def) {
 static struct definition_t *
 spice_translate_source (struct definition_t * root,
 			struct definition_t * def, char type) {
-  struct definition_t * ac = NULL, * dc = def, * pulse = NULL;
+  struct definition_t * ac = NULL, * dc = def, * pulse = NULL, * expo = NULL;
   struct value_t * prop;
+  char * ui = (char *) ((type == 'U') ? "U" : "I");
 
   // adjust the DC part of the source
   if ((prop = spice_find_property (dc->values, "DC")) != NULL) {
     spice_value_done (prop);
     prop = prop->next;
-    spice_append_pair (def, type == 'U' ? "U" : "I", prop->ident, 1);
-    spice_value_done (prop);
+    if (VAL_IS_NUMBER (prop)) {
+      spice_append_pair (def, ui, prop->ident, 1);
+      spice_value_done (prop);
+    }
   }
   free (dc->type);
   dc->type = strdup (type == 'U' ? "Vdc" : "Idc");
@@ -1274,11 +1377,17 @@ spice_translate_source (struct definition_t * root,
     spice_value_done (prop);
     prop = prop->next;
     if (VAL_IS_NUMBER (prop)) {
-      spice_append_pair (dc, type == 'U' ? "U" : "I", prop->ident, 0);
+      // adjust DC offset
+      nr_double_t off;
+      off = spice_get_property_value (dc, ui);
+      spice_append_pair (dc, ui, prop->ident, 1);
+      off += spice_get_property_value (dc, ui);
+      spice_set_property_value (dc, ui, off);
       spice_value_done (prop);
+      // extract frequency, phase and theta
       prop = prop->next;
       struct property_field_t field =
-      { { type == 'U' ? "U" : "I", "f", "Phase", "Theta", NULL } };
+      { { ui, "f", "Phase", "Theta", NULL } };
       spice_extract_properties (ac, prop, &field);
     }
     double v, f;
@@ -1286,7 +1395,7 @@ spice_translate_source (struct definition_t * root,
     v = spice_get_property_value (ac, "Phase") * f * -360.0;
     spice_set_property_value (ac, "Phase", v);
     v = spice_get_property_value (ac, "Theta") / f;
-    spice_set_property_value (ac, "Theta", v);
+    spice_set_property_value (ac, "Theta", f > 0 ? v : 0);
   }
 
   // adjust the AC part of the source
@@ -1307,11 +1416,11 @@ spice_translate_source (struct definition_t * root,
       ac = spice_create_definition (dc, type == 'U' ? "Vac" : "Iac");
     }
     if (Mag) {
-      spice_append_pair (ac, type == 'U' ? "U" : "I", Mag, 1);
+      spice_append_pair (ac, ui, Mag, 1);
       free (Mag);
     }
     else {
-      spice_append_pair (ac, type == 'U' ? "U" : "I", "0", 0);
+      spice_append_pair (ac, ui, "0", 0);
     }
     if (Phase) {
       spice_append_pair (ac, "Phase", Phase, 1);
@@ -1324,21 +1433,73 @@ spice_translate_source (struct definition_t * root,
     pulse = spice_create_definition (dc, type == 'U' ? "Vpulse" : "Ipulse");
     spice_value_done (prop);
     prop = prop->next;
-    if (VAL_IS_NUMBER (prop)) {
-      spice_append_pair (pulse, type == 'U' ? "U1" : "I1", prop->ident, 0);
-      spice_append_pair (dc, type == 'U' ? "U" : "I", "0", 1);
-      spice_value_done (prop);
-      prop = prop->next;
-      struct property_field_t field =
-      { { type == 'U' ? "U2" : "I2", "T1", "Tr", "Tf", "T2", NULL } };
-      spice_extract_properties (pulse, prop, &field);
+
+    // periodic
+    if (spice_count_real_values (prop) > 6) {
+      free (pulse->type);
+      pulse->type = type == 'U' ? strdup ("Vrect") : strdup ("Irect");
+      double add, off = 0;
+      if (VAL_IS_NUMBER (prop)) {
+	add = spice_get_property_value (dc, ui);
+	spice_append_pair (dc, ui, prop->ident, 1);
+	off = spice_get_property_value (dc, ui);
+	add += off;
+	spice_set_property_value (dc, ui, add);
+	prop = prop->next;
+      }
+      if (VAL_IS_NUMBER (prop)) {
+	struct property_field_t field =
+	  { { ui, "Td", "Tr", "Tf", "TH", "TL", NULL } };
+	spice_extract_properties (pulse, prop, &field);
+      }
+      double v;
+      v  = spice_get_property_value (pulse, ui);   // V2
+      v -= off;
+      spice_set_property_value (pulse, ui, v);
+      v  = spice_get_property_value (pulse, "TH"); // PW
+      v += spice_get_property_value (pulse, "Tr");
+      v += spice_get_property_value (pulse, "Tf");
+      spice_set_property_value (pulse, "TH", v);
+      v  = spice_get_property_value (pulse, "TL"); // PER
+      v -= spice_get_property_value (pulse, "TH");
+      v -= spice_get_property_value (pulse, "Td");
+      spice_set_property_value (pulse, "TL", v);
     }
-    double v;
-    v  = spice_get_property_value (pulse, "T1");
-    v += spice_get_property_value (pulse, "Tr");
-    v += spice_get_property_value (pulse, "Tf");
-    v += spice_get_property_value (pulse, "T2");
-    spice_set_property_value (pulse, "T2", v);
+    // single pulse
+    else {
+      if (VAL_IS_NUMBER (prop)) {
+	spice_append_pair (pulse, type == 'U' ? "U1" : "I1", prop->ident, 0);
+	spice_append_pair (dc, ui, "0", 0);
+	spice_value_done (prop);
+	prop = prop->next;
+	struct property_field_t field =
+	  { { type == 'U' ? "U2" : "I2", "T1", "Tr", "Tf", "T2", NULL } };
+	spice_extract_properties (pulse, prop, &field);
+      }
+      double v;
+      v  = spice_get_property_value (pulse, "T1");
+      v += spice_get_property_value (pulse, "Tr");
+      v += spice_get_property_value (pulse, "Tf");
+      v += spice_get_property_value (pulse, "T2");
+      spice_set_property_value (pulse, "T2", v);
+    }
+  }
+
+  // adjust the exponential part of the source
+  if ((prop = spice_find_property (dc->values, "EXP")) != NULL) {
+    expo = spice_create_definition (dc, type == 'U' ? "Vexp" : "Iexp");
+    spice_value_done (prop);
+    prop = prop->next;
+    // extract pulse values, rise and fall constants
+    struct property_field_t field =
+      { { type == 'U' ? "U1" : "I1", type == 'U' ? "U2" : "I2", "T1", "Tr",
+	  "T2", "Tf", NULL } };
+    spice_extract_properties (expo, prop, &field);
+  }
+
+  // set DC value to zero if necessary
+  if (spice_find_property (dc, ui) == NULL) {
+    spice_append_pair (def, ui, "0", 1);
   }
 
   // finally add sources to list of definitions
@@ -1356,6 +1517,13 @@ spice_translate_source (struct definition_t * root,
       spice_adjust_isource_nodes (pulse, ac ? ac : dc);
     root = spice_add_definition (root, pulse);
   }
+  if (expo) {
+    if (type == 'U')
+      spice_adjust_vsource_nodes (expo, pulse ? pulse : ac ? ac : dc);
+    else
+      spice_adjust_isource_nodes (expo, pulse ? pulse : ac ? ac : dc);
+    root = spice_add_definition (root, expo);
+  }
   return root;
 }
 
@@ -1438,6 +1606,18 @@ spice_find_definition (struct definition_t * root, char * type, char * inst) {
   return NULL;
 }
 
+/* The function looks through the list of definitions if there is a
+   component with the given type and returns it.  Otherwise the
+   function returns NULL. */
+static struct definition_t *
+spice_find_definition (struct definition_t * root, char * type) {
+  for (struct definition_t * def = root; def != NULL; def = def->next) {
+    if (!strcasecmp (def->type, type))
+      return def;
+  }
+  return NULL;
+}
+
 /* The function appends the given key/value pair to the properties of
    any definition of the given type. */
 static void spice_add_property (struct definition_t * root, char * type,
@@ -1538,6 +1718,25 @@ spice_find_coupled (struct definition_t * root, struct definition_t * coupled,
   return NULL;
 }
 
+/* Looks for a mutual inductor instance referencing the two given
+   inductors and returns it. */
+static struct definition_t *
+spice_find_coupled (struct definition_t * root, char * type,
+		    char * inst1, char * inst2) {
+  for (struct definition_t * def = root; def != NULL; def = def->next) {
+    if (!strcmp (def->type, type) && !def->action) {
+      if (VAL_IS_DONE (def->values) || VAL_IS_DONE (def->values->next))
+	continue;
+      char * linst1 = def->values->ident;
+      char * linst2 = def->values->next->ident;
+      if (!strcasecmp (linst1, inst1) && !strcasecmp (linst2, inst2) ||
+	  !strcasecmp (linst1, inst2) && !strcasecmp (linst2, inst1))
+	return def;
+    }
+  }
+  return NULL;
+}
+
 /* Looks for the inductor definition used in a coupled inductor.
    Emits an error message if there is no such inductor. */
 static struct definition_t *
@@ -1724,6 +1923,164 @@ spice_translate_coupled (struct definition_t * root,
   return root;
 }
 
+/* Finds an additional mutual inductor definition using the same
+   inductors as the original one if there is any. */
+static struct definition_t *
+spice_find_coupled (struct definition_t * root,
+		    qucs::hash<struct definition_t> * coupled,
+		    char * type, char * inst) {
+  for (struct definition_t * def = root; def != NULL; def = def->next) {
+    if (!strcmp (def->type, type) && !def->action) {
+      if (VAL_IS_DONE (def->values) || VAL_IS_DONE (def->values->next))
+	continue;
+      if (!coupled->get (def->instance)) {
+	char * linst1 = def->values->ident;
+	char * linst2 = def->values->next->ident;
+	if (!strcasecmp (linst1, inst) || !strcasecmp (linst2, inst))
+	  return def;
+      }
+    }
+  }
+  return NULL;
+}
+
+/* Looks recursively for mutual inductors.  In the end the two hash
+   maps contain all the inductors coupled by those mutual inductors in
+   the other hash map. */
+static struct definition_t *
+spice_search_coupled (struct definition_t * root,
+		      qucs::hash<struct definition_t> * K_hash,
+		      qucs::hash<struct definition_t> * L_hash,
+		      char * type, char * inst) {
+  char * linst;
+  struct definition_t * l, * k;
+
+  // find mutual inductors referencing the given inductor instance
+  while ((k = spice_find_coupled (root, K_hash, type, inst)) != NULL) {
+    // already had this one?
+    if (K_hash->get (k->instance))
+      continue;
+    K_hash->put (k->instance, k);
+
+    // first referenced inductor
+    linst = k->values->ident;
+    if (!L_hash->get (linst)) {
+      if ((l = spice_find_coupled_inductor (root, k, "L", linst)) != NULL) {
+	L_hash->put (linst, l);
+	// recurse
+	root = spice_search_coupled (root, K_hash, L_hash, type, linst);
+      }
+    }
+
+    // second referenced inductor
+    linst = k->values->next->ident;
+    if (!L_hash->get (linst)) {
+      if ((l = spice_find_coupled_inductor (root, k, "L", linst)) != NULL) {
+	L_hash->put (linst, l);
+	// recurse
+	root = spice_search_coupled (root, K_hash, L_hash, type, linst);
+      }
+    }
+  }
+  return root;
+}
+
+/* Post translation function for multiple coupled inductors. */
+static struct definition_t *
+spice_translate_coupled_x (struct definition_t * root,
+			   struct definition_t * def) {
+  char * linst1, * linst2;
+  qucs::hash<struct definition_t> K_hash;
+  qucs::hash<struct definition_t> L_hash;
+  qucs::hashiterator<struct definition_t> it;
+  struct definition_t * l;
+
+  // save first 2 referenced inductors and the mutual inductor
+  linst1 = def->values->ident;
+  if ((l = spice_find_coupled_inductor (root, def, "L", linst1)) != NULL)
+    L_hash.put (linst1, l);
+  linst2 = def->values->next->ident;
+  if ((l = spice_find_coupled_inductor (root, def, "L", linst2)) != NULL)
+    L_hash.put (linst2, l);
+  K_hash.put (def->instance, def);
+
+  // look for more mutual inductors involving the first 2 inductors
+  root = spice_search_coupled (root, &K_hash, &L_hash, "Tr", linst1);
+  root = spice_search_coupled (root, &K_hash, &L_hash, "Tr", linst2);
+
+  // create coupling coefficient matrix
+  int i, o, s = L_hash.count ();
+  nr_double_t * kval = new nr_double_t[s * s];
+  struct definition_t * k;
+  struct value_t * val;
+  qucs::hashiterator<struct definition_t> iti, ito;
+  // outer loop
+  for (ito = qucs::hashiterator<struct definition_t> (L_hash), o = 0;
+       *ito; ++ito, ++o) {
+    // inner loop
+    for (iti = qucs::hashiterator<struct definition_t> (L_hash), i = 0;
+	 *iti; ++iti, ++i) {
+      if (i > o) {
+	// cross-coupling coefficients
+	nr_double_t kvalue = 0;
+	k = spice_find_coupled (root, "Tr",
+				iti.currentKey (), ito.currentKey ());
+	if (k != NULL) {
+	  if ((val = spice_get_value_coupled (k)) != NULL) {
+	    kvalue = spice_evaluate_value (val);
+	  }
+	}
+	kval[s * o + i] = kvalue;
+	kval[s * i + o] = kvalue;
+      }
+      else if (i == o) {
+	// self-coupling coefficient
+	kval[s * o + i] = 1;
+      }
+    }
+  }
+
+  spice_value_done (def->values);
+  spice_value_done (def->values->next);
+
+  // adjust MUTX instance
+  free (def->type); def->type = strdup ("MUTX");
+  netlist_free_pairs (def->pairs); def->pairs = NULL;
+
+  // create L property vector
+  struct pair_t * pair = create_pair ();
+  pair->key = strdup ("L");
+  def->pairs = netlist_append_pairs (def->pairs, pair);
+  for (it = qucs::hashiterator<struct definition_t> (L_hash); *it; ++it) {
+    spice_append_node (def, spice_get_node(it.currentVal (), 1)->node);
+    spice_append_node (def, spice_get_node(it.currentVal (), 2)->node);
+    val = create_value ();
+    val->value = spice_get_property_value (it.currentVal (), "L");
+    pair->value = netlist_append_values (pair->value, val);
+    root = spice_del_definition (root, it.currentVal ());
+  }
+
+  // create k property vector
+  pair = create_pair ();
+  pair->key = strdup ("k");
+  def->pairs = netlist_append_pairs (def->pairs, pair);
+  for (i = 0; i < s * s; i++) {
+    val = create_value ();
+    val->value = kval[i];
+    pair->value = netlist_append_values (pair->value, val);
+  }
+
+  // remove remaining coupled inductors from definition list
+  for (it = qucs::hashiterator<struct definition_t> (K_hash); *it; ++it) {
+    if (def != it.currentVal ()) {
+      root = spice_del_definition (root, it.currentVal ());
+    }
+  }
+
+  delete[] kval;
+  return root;
+}
+
 /* This function must be called after the actual Spice netlist
    translator in order to adjust some references or whatever in the
    resulting netlist. */
@@ -1759,7 +2116,6 @@ spice_post_translator (struct definition_t * root) {
     if (!def->action && (!strcmp (def->type, "CCCS") ||
 			 !strcmp (def->type, "CCVS"))) {
       struct definition_t * target;
-      struct pair_t * prop;
       struct value_t * val = spice_find_device_instance (def);
       char * key = val->ident;
       target = spice_find_definition (root, "Vdc", key);
@@ -1767,10 +2123,6 @@ spice_post_translator (struct definition_t * root) {
 	// adjust the controlling nodes of the source
 	spice_adjust_vsource_nodes (def, target);      
 	spice_translate_nodes (def, 1);
-	// and the properties of the source
-	prop = spice_find_property (def, "G");
-	double v = spice_get_property_value (def, "G");
-	spice_set_property_value (def, "G", 1 / v);
       }
       else {
 	fprintf (stderr, "spice error, no such voltage source `%s' found as "
@@ -1779,9 +2131,195 @@ spice_post_translator (struct definition_t * root) {
 	spice_errors++;
       }
     }
+    // post-process switches
+    if (!def->action && !strcmp (def->type, "Relais")) {
+      struct pair_t * pon = spice_find_property_nocase (def, "VON");
+      struct pair_t * pof = spice_find_property_nocase (def, "VOFF");
+      if (pon != NULL && pof != NULL) {
+	nr_double_t von = spice_evaluate_value (pon->value);
+	nr_double_t vof = spice_evaluate_value (pof->value);
+	def->pairs = spice_del_property (def->pairs, pon);
+	def->pairs = spice_del_property (def->pairs, pof);
+	nr_double_t vh = (von - vof) / 2;
+	nr_double_t vt = (von + vof) / 2;
+	spice_set_property_value (def, "Vt", vt);
+	spice_set_property_value (def, "Vh", fabs (vh));
+      }
+    }
+    // post-process pulse and rectangular sources
+    if (!def->action && (!strcmp (def->type, "Vpulse") ||
+			 !strcmp (def->type, "Ipulse") ||
+			 !strcmp (def->type, "Vrect") ||
+			 !strcmp (def->type, "Irect"))) {
+      struct definition_t * tran;
+      struct pair_t * tr = spice_find_property (def, "Tr");
+      struct pair_t * tf = spice_find_property (def, "Tf");
+      if (tr == NULL || tf == NULL) {
+	if ((tran = spice_find_definition (definition_root, "TR")) != NULL) {
+	  nr_double_t start = spice_get_property_value (tran, "Start");
+	  nr_double_t stop = spice_get_property_value (tran, "Stop");
+	  nr_double_t points = spice_get_property_value (tran, "Points");
+	  nr_double_t tstep = (stop - start) / (points - 1);
+	  nr_double_t add = 0;
+	  if (!tf) {
+	    spice_set_property_value (def, "Tf", tstep);
+	    add += tstep;
+	  }
+	  if (!tr) {
+	    spice_set_property_value (def, "Tr", tstep);
+	    add += tstep;
+	  }
+	  if (!strcmp (&def->type[1], "pulse")) {
+	    nr_double_t t2 = spice_get_property_value (def, "T2");
+	    spice_set_property_value (def, "T2", t2 + add);
+	  }
+	}
+      }
+    }
+    // post-process sinusoidal sources
+    if (!def->action && (!strcmp (def->type, "Vac") ||
+			 !strcmp (def->type, "Iac"))) {
+      struct definition_t * tran;
+      struct pair_t * f = spice_find_property (def, "f");
+      if (f == NULL) {
+	if ((tran = spice_find_definition (definition_root, "TR")) != NULL) {
+	  nr_double_t stop = spice_get_property_value (tran, "Stop");
+	  spice_set_property_value (def, "f", 1 / stop);
+	}
+      }
+    }
+    // post-process resistors
+    if (!def->action && !strcmp (def->type, "R")) {
+      // drop the second "R" given in Model
+      struct pair_t * r1 = spice_find_property_nocase (def, "R");
+      struct pair_t * r2 = spice_find_property_nocase (r1->next, "R");
+      if (r2 != NULL) {
+	def->pairs = spice_del_property (def->pairs, r2);
+      }
+      // calculate R value
+      struct pair_t * L = spice_find_property_nocase (def, "L");
+      struct pair_t * W = spice_find_property_nocase (def, "W");
+      struct pair_t * R = spice_find_property_nocase (def, "RSH");
+      struct pair_t * D = spice_find_property_nocase (def, "DEFW");
+      struct pair_t * N = spice_find_property_nocase (def, "NARROW");
+      nr_double_t l = 0, w = 0, r = 0, d = 0, n = 0;
+      if (L) {
+	l = spice_evaluate_value (L->value);
+	def->pairs = spice_del_property (def->pairs, L);
+      }
+      if (W) {
+	w = spice_evaluate_value (W->value);
+	def->pairs = spice_del_property (def->pairs, W);
+      }
+      if (R) {
+	r = spice_evaluate_value (R->value);
+	def->pairs = spice_del_property (def->pairs, R);
+      }
+      if (D) {
+	d = spice_evaluate_value (D->value);
+	def->pairs = spice_del_property (def->pairs, D);
+      }
+      if (N) {
+	n = spice_evaluate_value (N->value);
+	def->pairs = spice_del_property (def->pairs, N);
+      }
+      if (d == 0) d = 1e-6;
+      if (w == 0) w = d;
+      if (l != 0 && w != 0 && r != 0) {
+	r = r * (l - n) / (w - n);
+	spice_set_property_value (def, "R", r);
+      }
+      // handle Spice 2g6 syntax
+      struct value_t * val;
+      foreach_value (def->values, val) {
+	if (!(val->hint & HINT_DONE) && (val->hint & HINT_NUMBER)) {
+	  spice_append_pair (def, "Tc2", val->ident, 0);
+	  spice_value_done (val);
+	  break;
+	}
+      }
+    }
+    // post-process capacitor
+    if (!def->action && !strcmp (def->type, "C")) {
+      // calculate C value
+      struct pair_t * L = spice_find_property_nocase (def, "L");
+      struct pair_t * W = spice_find_property_nocase (def, "W");
+      struct pair_t * C = spice_find_property_nocase (def, "CJ");
+      struct pair_t * S = spice_find_property_nocase (def, "CJSW");
+      struct pair_t * D = spice_find_property_nocase (def, "DEFW");
+      struct pair_t * N = spice_find_property_nocase (def, "NARROW");
+      nr_double_t l = 0, w = 0, c = 0, d = 0, n = 0, s = 0;
+      if (L) {
+	l = spice_evaluate_value (L->value);
+	def->pairs = spice_del_property (def->pairs, L);
+      }
+      if (W) {
+	w = spice_evaluate_value (W->value);
+	def->pairs = spice_del_property (def->pairs, W);
+      }
+      if (C) {
+	c = spice_evaluate_value (C->value);
+	def->pairs = spice_del_property (def->pairs, C);
+      }
+      if (S) {
+	s = spice_evaluate_value (S->value);
+	def->pairs = spice_del_property (def->pairs, S);
+      }
+      if (D) {
+	d = spice_evaluate_value (D->value);
+	def->pairs = spice_del_property (def->pairs, D);
+      }
+      if (N) {
+	n = spice_evaluate_value (N->value);
+	def->pairs = spice_del_property (def->pairs, N);
+      }
+      if (d == 0) d = 1e-6;
+      if (w == 0) w = d;
+      if (l != 0 && w != 0 && c != 0) {
+	c = c * (l - n) * (w - n) + 2 * s * (l + w - 2 * n);
+	spice_set_property_value (def, "C", c);
+      }
+    }
+    // post-process lossless transmission line
+    if (!def->action && !strcmp (def->type, "TLIN4P")) {
+      struct pair_t * pt = spice_find_property (def, "TD");
+      struct pair_t * pf = spice_find_property (def, "F");
+      struct pair_t * pl = spice_find_property (def, "NL");
+      nr_double_t len = 1e-3;
+      if (pt != NULL) {
+	// delay given
+	len = spice_evaluate_value (pt->value) * C0;
+	def->pairs = spice_del_property (def->pairs, pt);
+	spice_set_property_value (def, "L", len);
+      }
+      else if (pf != NULL && pl != NULL) {
+	// frequency and normalized length given
+	nr_double_t f = spice_evaluate_value (pf->value);
+	nr_double_t l = spice_evaluate_value (pl->value);
+	def->pairs = spice_del_property (def->pairs, pf);
+	def->pairs = spice_del_property (def->pairs, pl);
+	len = C0 / f * l;
+	spice_set_property_value (def, "L", len);
+      }
+      else if (pf != NULL) {
+	// only frequency given, default normalized length
+	nr_double_t f = spice_evaluate_value (pf->value);
+	def->pairs = spice_del_property (def->pairs, pf);
+	len = C0 / f * 0.25;
+	spice_set_property_value (def, "L", len);
+      }
+      else {
+	fprintf (stderr, "spice error, either TD or F required in "
+		 "lossless `%s' line \n", def->instance);
+	spice_errors++;
+      }
+    }
     // post-process mutual inductors (transformer)
     if (!def->action && !strcmp (def->type, "Tr")) {
-      root = spice_translate_coupled (root, def);
+      if (1)
+	root = spice_translate_coupled_x (root, def);
+      else
+	root = spice_translate_coupled (root, def);
     }
     // post-process general analysis options
     if (def->action && strstr (def->type, "OPT")) {
@@ -1980,7 +2518,8 @@ static struct definition_t * spice_translator (struct definition_t * root) {
 	// devices
 	if (!strcasecmp (def->type, "Q") || !strcasecmp (def->type, "M") ||
 	    !strcasecmp (def->type, "J") || !strcasecmp (def->type, "D") ||
-	    !strcasecmp (def->type, "S")) {
+	    !strcasecmp (def->type, "S") || !strcasecmp (def->type, "R") ||
+	    !strcasecmp (def->type, "C")) {
 	  spice_translate_device (root, def);
 	}
 	// voltage sources
@@ -2025,7 +2564,7 @@ static struct definition_t * spice_translator (struct definition_t * root) {
        - current sources
        - subcircuits
    - file includes
-   - transmission line (won't be possible)
+   - transmission lines
        - voltage dependent sources
        - current dependent sources
        - initial conditions
@@ -2034,6 +2573,12 @@ static struct definition_t * spice_translator (struct definition_t * root) {
    - mesfet if available in Qucs
    - other mosfet models if available in Qucs
        - three mutual inductors
+   - current controlled switch (gyrator + voltage controlled switch)
+   - single-frequency FM (using pm-modulator)
+   - analog behavioural B, E, G, F, and H sources
+   - piece-wise linear (PWL) voltage and current sources
+   - temperature analysis (.TEMP)
+   - temperature option (.OPTIONS TNOM=27)
 */
 
 #if 0
