@@ -1,7 +1,7 @@
 /*
  * qucs_producer.cpp - the Qucs netlist producer
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: qucs_producer.cpp,v 1.16 2006/01/09 09:11:07 raimi Exp $
+ * $Id: qucs_producer.cpp,v 1.23 2007/05/17 09:28:19 ela Exp $
  *
  */
 
@@ -30,7 +30,12 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
+#include <ctype.h>
 
+#include "object.h"
+#include "complex.h"
+#include "vector.h"
+#include "dataset.h"
 #include "netdefs.h"
 #include "check_spice.h"
 #include "check_vcd.h"
@@ -88,9 +93,16 @@ static void qucs_collect_nodes (struct definition_t * root) {
 
 /* Prints value representation. */
 static void netlist_list_value (struct value_t * value) {
-  if (value->ident)
+  if (value == NULL)
+    fprintf (qucs_out, "[]");
+  else if (value->ident)
     fprintf (qucs_out, "%s", value->ident);
-  else {
+  else if (value->next) {
+    fprintf (qucs_out, "[");
+    for (; value != NULL; value = value->next)
+      fprintf (qucs_out, "%g%s", value->value, value->next ? ";" : "");
+    fprintf (qucs_out, "]");
+  } else {
     fprintf (qucs_out, "%g", value->value);
     if (value->scale)
       fprintf (qucs_out, "%s", value->scale);
@@ -112,8 +124,8 @@ static void netlist_list_def (struct definition_t * def, char * prefix) {
       // skip specific actions if required
       if (def->action && strcmp (def->type, "Def")) return;
     }
-    fprintf (qucs_out, "%s%s%s:%s", prefix, def->action ? "." : "",
-	     def->type, def->instance);
+    fprintf (qucs_out, "%s%s%s:%s%s", prefix, def->action ? "." : "",
+	     def->type, isdigit (def->instance[0]) ? "X" : "", def->instance);
     for (node = def->nodes; node != NULL; node = node->next)
       fprintf (qucs_out, " %s", node->node);
     for (pair = def->pairs; pair != NULL; pair = pair->next) {
@@ -178,6 +190,22 @@ static void netlist_reference_ground (void) {
   }
 }
 
+/* Look for a single subcircuit definition in a netlist. */
+static struct definition_t *
+netlist_get_single_subcircuit (struct definition_t * root) {
+  int count = 0;
+  struct definition_t * ret = NULL;
+  for (struct definition_t * def = root; def != NULL; def = def->next) {
+    if (!def->action) {
+      count++;
+    } else if (def->sub) {
+      count++;
+      ret = def;
+    }
+  }
+  return (count == 1) ? ret : NULL;
+}
+
 /* Prints the overall netlist representation. */
 static void netlist_list (void) {
   struct node_t * n;
@@ -189,10 +217,20 @@ static void netlist_list (void) {
   }
   netlist_lister (definition_root, "");
   for (def = subcircuit_root; def != NULL; def = def->next) {
-    fprintf (qucs_out, ".Def:%s\n", def->instance);
+    fprintf (qucs_out, ".Def:%s%s\n", isdigit (def->instance[0]) ? "X" : "",
+	     def->instance);
     netlist_lister (def->sub, "  ");
     fprintf (qucs_out, ".Def:End\n");
   }
+  /* Instantiate subcircuit if there is just a single definition. */
+  if (definition_root != NULL && subcircuit_root == NULL &&
+      (def = netlist_get_single_subcircuit (definition_root)) != NULL) {
+    fprintf (qucs_out, "\n# no instance of subcircuit \"%s\" found, "
+	     "creating it\n", def->instance);
+    fprintf (qucs_out, "Sub:X1");
+    for (n = def->nodes; n; n = n->next) fprintf (qucs_out, " %s", n->node);
+    fprintf (qucs_out, " Type=\"%s\"\n", def->instance);
+  }
   /* Print overall (toplevel only) node list. */
   qucs_collect_nodes (definition_root);
   fprintf (qucs_out, "\n### TOPLEVEL NODELIST BEGIN\n");
@@ -232,9 +270,9 @@ struct device_t {
 qucs_devices[] = {
   /* diode */
   { "Diode", "Diode", "D", 2,
-    {"Is", "N", "Cj0", "M", "Vj", "Fc", "Cp", "Isr", "Nr", "Rs", "Tt", "Temp",
-     "Kf", "Af", "Ffe", "Bv", "Ibv", "Xti", "Eg", "Tbv", "Trs", "Ttt1", "Ttt2",
-     "Tm1", "Tm2", "Tnom", "Area", NULL },
+    {"Is", "N", "Cj0", "M", "Vj", "Fc", "Cp", "Isr", "Nr", "Rs", "Tt", "Ikf",
+     "Kf", "Af", "Ffe", "Bv", "Ibv",  "Temp", "Xti", "Eg", "Tbv", "Trs",
+     "Ttt1", "Ttt2", "Tm1", "Tm2", "Tnom", "Area", NULL },
     "  <.PortSym -30 0 1>\n"
     "  <.PortSym 30 0 2>\n"
     "  <Line -10 -10 0 20 #000080 2 1>\n"
@@ -389,8 +427,8 @@ void qucslib_producer (void) {
   }
 }
 
-/* This function is the overall Qucs dataset producer. */
-void qucsdata_producer (void) {
+/* This function is the Qucs dataset producer for VCD files. */
+void qucsdata_producer_vcd (void) {
   struct dataset_variable * ds;
   struct dataset_value * dv;
   fprintf (qucs_out, "<Qucs Dataset " PACKAGE_VERSION ">\n");
@@ -400,7 +438,8 @@ void qucsdata_producer (void) {
     if (ds->type == DATA_INDEPENDENT)
       fprintf (qucs_out, "<indep %s %d>\n", ds->ident, ds->size);
     else if (ds->type == DATA_DEPENDENT)
-      fprintf (qucs_out, "<dep %s.X %s>\n", ds->ident, ds->dependencies);
+      fprintf (qucs_out, "<dep %s.%s %s>\n", ds->ident, ds->isreal ? "R" : "X",
+	       ds->dependencies);
     for (dv = ds->values; dv; dv = dv->next) {
       fprintf (qucs_out, "  %s\n", dv->value);
     }
@@ -410,3 +449,8 @@ void qucsdata_producer (void) {
       fprintf (qucs_out, "</dep>\n");
   }
 }
+
+/* This function is the Qucs dataset producer. */
+void qucsdata_producer (dataset * data) {
+  data->print ();
+}
