@@ -1,7 +1,7 @@
 /*
  * qucs_producer.h - the Qucs producer definitions
  *
- * Copyright (C) 2004, 2005 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: qucs_producer.h,v 1.7 2006/01/09 09:11:07 raimi Exp $
+ * $Id: qucs_producer.h,v 1.8 2006/06/06 07:45:51 raimi Exp $
  *
  */
 
 #ifndef __QUCS_PRODUCER_H__
 #define __QUCS_PRODUCER_H__
 
+class dataset;
+
 /* Externalize variables. */
 extern FILE * qucs_out;
 extern int    qucs_actions;
@@ -34,6 +36,7 @@ extern char * qucs_gnd;
 void qucs_producer (void);
 int  qucs_find_node (struct node_t *, char *);
 void qucslib_producer (void);
-void qucsdata_producer (void);
+void qucsdata_producer_vcd (void);
+void qucsdata_producer (dataset *);
 
 #endif /* __QUCS_PRODUCER_H__ */
