@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: dataset.cpp,v 1.17 2006/03/27 09:55:49 raimi Exp $
+ * $Id: dataset.cpp,v 1.22 2006/09/04 08:05:39 raimi Exp $
  *
  */
 
@@ -41,6 +41,9 @@
 #include "dataset.h"
 #include "check_dataset.h"
 #include "check_touchstone.h"
+#include "check_citi.h"
+#include "check_zvr.h"
+#include "check_mdl.h"
 
 // Constructor creates an unnamed instance of the dataset class.
 dataset::dataset () : object () {
@@ -93,6 +96,21 @@ void dataset::addDependency (vector * v) {
   dependencies = v;
 }
 
+// This function removes a dependency vector from the current dataset.
+void dataset::delDependency (vector * v) {
+  if (dependencies == v) {
+    dependencies = (vector *) v->getNext ();
+    if (dependencies) dependencies->setPrev (NULL);
+  }
+  else {
+    vector * next = (vector *) v->getNext ();
+    vector * prev = (vector *) v->getPrev ();
+    prev->setNext (next);
+    if (next) next->setPrev (prev);
+  }
+  delete v;
+}
+
 /* The function adds the given list of vectors to the dependency set
    of the current dataset. */
 void dataset::addDependencies (vector * v) {
@@ -136,6 +154,21 @@ void dataset::addVariable (vector * v) {
   variables = v;
 }
 
+// This function removes a variable vector from the current dataset.
+void dataset::delVariable (vector * v) {
+  if (variables == v) {
+    variables = (vector *) v->getNext ();
+    if (variables) variables->setPrev (NULL);
+  }
+  else {
+    vector * next = (vector *) v->getNext ();
+    vector * prev = (vector *) v->getPrev ();
+    prev->setNext (next);
+    if (next) next->setPrev (prev);
+  }
+  delete v;
+}
+
 /* The function adds the given list of vectors to the variable set of
    the current dataset. */
 void dataset::addVariables (vector * v) {
@@ -422,3 +455,78 @@ dataset * dataset::load_touchstone (const char * file) {
   touchstone_result->setFile (file);
   return touchstone_result;
 }
+
+/* The function read a full dataset from the given CITIfile and
+   returns it.  On failure the function emits appropriate error
+   messages and returns NULL. */
+dataset * dataset::load_citi (const char * file) {
+  FILE * f;
+  if ((f = fopen (file, "r")) == NULL) {
+    logprint (LOG_ERROR, "error loading `%s': %s\n", file, strerror (errno));
+    return NULL;
+  }
+  citi_in = f;
+  citi_restart (citi_in);
+  if (citi_parse () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  if (citi_check () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  fclose (f);
+  citi_lex_destroy ();
+  citi_result->setFile (file);
+  return citi_result;
+}
+
+/* The function read a full dataset from the given ZVR file and
+   returns it.  On failure the function emits appropriate error
+   messages and returns NULL. */
+dataset * dataset::load_zvr (const char * file) {
+  FILE * f;
+  if ((f = fopen (file, "r")) == NULL) {
+    logprint (LOG_ERROR, "error loading `%s': %s\n", file, strerror (errno));
+    return NULL;
+  }
+  zvr_in = f;
+  zvr_restart (zvr_in);
+  if (zvr_parse () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  if (zvr_check () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  fclose (f);
+  zvr_lex_destroy ();
+  if (zvr_result) zvr_result->setFile (file);
+  return zvr_result;
+}
+
+/* The function read a full dataset from the given MDL file and
+   returns it.  On failure the function emits appropriate error
+   messages and returns NULL. */
+dataset * dataset::load_mdl (const char * file) {
+  FILE * f;
+  if ((f = fopen (file, "r")) == NULL) {
+    logprint (LOG_ERROR, "error loading `%s': %s\n", file, strerror (errno));
+    return NULL;
+  }
+  mdl_in = f;
+  mdl_restart (mdl_in);
+  if (mdl_parse () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  if (mdl_check () != 0) {
+    fclose (f);
+    return NULL;
+  }
+  fclose (f);
+  mdl_lex_destroy ();
+  if (mdl_result) mdl_result->setFile (file);
+  return mdl_result;
+}
