@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: dataset.h,v 1.9 2006/03/06 10:13:23 raimi Exp $
+ * $Id: dataset.h,v 1.13 2006/08/21 08:10:30 raimi Exp $
  *
  */
 
@@ -43,6 +43,8 @@ class dataset : public object
   void appendVariable (vector *);
   void appendVariables (vector *);
   void applyDependencies (vector * v);
+  void delDependency (vector *);
+  void delVariable (vector *);
 
   void assignDependency (char *, char *);
   char * getFile (void);
@@ -60,6 +62,9 @@ class dataset : public object
   vector * findOrigin (char *);
   static dataset * load (const char *);
   static dataset * load_touchstone (const char *);
+  static dataset * load_citi (const char *);
+  static dataset * load_zvr (const char *);
+  static dataset * load_mdl (const char *);
 
   int countDependencies (void);
   int countVariables (void);
