@@ -1,7 +1,7 @@
 /*
  * equation.cpp - checker for the Qucs equations
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: equation.cpp,v 1.41 2006/03/24 14:30:05 raimi Exp $
+ * $Id: equation.cpp,v 1.58 2007/05/27 17:50:14 ela Exp $
  *
  */
 
@@ -40,8 +40,10 @@
 #include "matvec.h"
 #include "dataset.h"
 #include "strlist.h"
+#include "netdefs.h"
 #include "equation.h"
 #include "evaluate.h"
+#include "differentiate.h"
 #include "constants.h"
 #include "range.h"
 #include "exception.h"
@@ -50,11 +52,6 @@
 using namespace eqn;
 using namespace qucs;
 
-/* The global list of equations and expression lists. */
-node   * eqn::equations = NULL;
-node   * eqn::expressions = NULL;
-solver * eqn::solve = NULL;
-
 #define A(a) ((assignment *) (a))
 #define N(n) ((node *) (n))
 #define C(c) ((constant *) (c))
@@ -64,6 +61,7 @@ solver * eqn::solve = NULL;
 constant::constant () : node (CONSTANT) {
   type = TAG_UNKNOWN;
   dataref = false;
+  d = 0.0;
   setType (type);
 }
 
@@ -71,9 +69,53 @@ constant::constant () : node (CONSTANT) {
 constant::constant (int tag) : node (CONSTANT) {
   type = tag;
   dataref = false;
+  d = 0.0;
   setType (type);
 }
 
+/* This copy constructor creates a instance of the constant class
+   based on the given constant. */
+constant::constant (const constant & o) : node (o) {
+  type = o.type;
+  dataref = o.dataref;
+  d = 0.0;
+  setType (type);
+  switch (type) {
+  case TAG_BOOLEAN:
+    b = o.b;
+    break;
+  case TAG_DOUBLE:
+    d = o.d;
+    break;
+  case TAG_COMPLEX:
+    c = dataref ? o.c : new complex (*o.c);
+    break;
+  case TAG_VECTOR:
+    v = dataref ? o.v : new vector (*o.v);
+    break;
+  case TAG_MATRIX:
+    m = dataref ? o.m : new matrix (*o.m);
+    break;
+  case TAG_MATVEC:
+    mv = dataref ? o.mv : new matvec (*o.mv);
+    break;
+  case TAG_STRING:
+    s = dataref ? o.s : strdup (s);
+    break;
+  case TAG_CHAR:
+    chr = o.chr;
+    break;
+  case TAG_RANGE:
+    r = dataref ? o.r : new range (*o.r);
+    break;
+  }
+}
+
+// Re-creates the given instance.
+node * constant::recreate (void) {
+  return new constant (*this);
+}
+
 // Destructor deletes an instance of the constant class.
 constant::~constant () {
   if (!dataref) {
@@ -125,6 +167,10 @@ char * constant::toString (void) {
   char str[256];
   if (txt != NULL) free (txt);
   switch (type) {
+  case TAG_BOOLEAN:
+    sprintf (str, "%d", b ? 1 : 0);
+    txt = strdup (str);
+    break;
   case TAG_DOUBLE:
     sprintf (str, "%g", (double) d);
     txt = strdup (str);
@@ -197,12 +243,39 @@ constant * constant::evaluate (void) {
   return getResult ();
 }
 
+// Returns the derivative of a constant.
+node * constant::differentiate (char *) {
+  constant * res = new constant (TAG_DOUBLE);
+  res->d = 0;
+  return res;
+}
+
 // Constructor creates an instance of the reference class.
 reference::reference () : node (REFERENCE) {
   n = NULL;
   ref = NULL;
 }
 
+/* This copy constructor creates a instance of the reference class
+   based on the given reference. */
+reference::reference (const reference & o) : node (o) {
+  n = o.n ? strdup (o.n) : NULL;
+  ref = o.ref;
+}
+
+// Re-creates the given instance.
+node * reference::recreate (void) {
+  return new reference (*this);
+}
+
+// Replaces reference name by the new given name.
+void reference::replace (char * src, char * dst) {
+  if (!strcmp (src, n)) {
+    free (n);
+    n = dst ? strdup (dst) : NULL;
+  }
+}
+
 // Destructor deletes an instance of the reference class.
 reference::~reference () {
   if (n) free (n);
@@ -228,16 +301,19 @@ void reference::addDependencies (strlist * depends) {
 
 // Find and save the actual equation reference.
 void reference::findVariable (void) {
+  ref = NULL; // force reference to be updated
   if (!ref) {
     node * eqn;
-    for (eqn = eqn::equations; eqn; eqn = eqn->getNext ()) {
-      if (!strcmp (n, A(eqn)->result)) {
-	ref = eqn;
-	break;
+    if (checkee != NULL) {
+      for (eqn = checkee->getEquations (); eqn; eqn = eqn->getNext ()) {
+	if (!strcmp (n, A(eqn)->result)) {
+	  ref = eqn;
+	  break;
+	}
       }
     }
-    if (eqn::solve && !ref) {
-      for (eqn = eqn::solve->getEquations (); eqn; eqn = eqn->getNext ()) {
+    if (solvee != NULL && !ref) {
+      for (eqn = solvee->getEquations (); eqn; eqn = eqn->getNext ()) {
 	if (!strcmp (n, A(eqn)->result)) {
 	  ref = eqn;
 	  break;
@@ -267,12 +343,45 @@ constant * reference::evaluate (void) {
   return getResult ();
 }
 
+// Returns the derivative of a reference.
+node * reference::differentiate (char * derivative) {
+  constant * res = new constant (TAG_DOUBLE);
+  if (n != NULL && !strcmp (n, derivative))
+    res->d = 1;
+  else
+    res->d = 0;
+  return res;
+}
+
 // Constructor creates an instance of the assignment class.
 assignment::assignment () : node (ASSIGNMENT) {
   body = NULL;
   result = NULL;
 }
 
+/* This copy constructor creates a instance of the assignment class
+   based on the given assignment. */
+assignment::assignment (const assignment & o) : node (o) {
+  body = o.body->recreate ();
+  result = o.result ? strdup (o.result) : NULL;
+}
+
+// Re-creates the given instance.
+node * assignment::recreate (void) {
+  return new assignment (*this);
+}
+
+// Replaces reference name by the new given name.
+void assignment::replace (char * src, char * dst) {
+  body->replace (src, dst);
+}
+
+// Renames the left hand side of the assignment.
+void assignment::rename (char * n) {
+  if (result) free (result);
+  result = n ? strdup (n) : NULL;
+}
+
 // Destructor deletes an instance of the assignment class.
 assignment::~assignment () {
   delete body;
@@ -295,6 +404,7 @@ char * assignment::toString (void) {
 
 // Adds the right hand side of the assignment to the list of dependencies.
 void assignment::addDependencies (strlist * depends) {
+  body->checkee = checkee;
   body->addDependencies (depends);
 }
 
@@ -317,12 +427,129 @@ constant * assignment::evaluate (void) {
   return getResult ();
 }
 
+// Returns the derivative of an assignment.
+node * assignment::differentiate (char * derivative) {
+  char * txt = (char *) malloc (strlen (result) + strlen (derivative) + 4);
+  sprintf (txt, "d%s_d%s", result, derivative);
+  assignment * res = new assignment ();
+  res->result = txt;
+  res->body = body->differentiate (derivative);
+  return res;
+}
+
+// Some small helpers.
+#define D(con) (C(con)->d)
+#define isConst(n) ((n)->getTag()==CONSTANT && C(n)->getType()==TAG_DOUBLE)
+#define isZero(n)  (isConst(n) && D(n) == 0.0)
+#define isOne(n)   (isConst(n) && D(n) == 1.0)
+#define defCon(res,val) res = new constant (TAG_DOUBLE); C(res)->d = val;
+
+/* Multiply two assignments. */
+void assignment::mul (assignment * f) {
+  node * factor = f->body->recreate ();
+  if (isZero (body) || isZero (factor)) {
+    delete body; delete factor;
+    defCon (body, 0);
+  } else if (isOne (body)) {
+    delete body;
+    body = factor;
+  } else if (isOne (factor)) {
+    delete factor;
+    body = body;
+  } else {
+    application * mul = new application ("*", 2);
+    mul->args = body;
+    mul->args->append (factor);
+    body = mul;
+  }
+}
+
+/* Multiply two assignments by reference. */
+void assignment::mulref (assignment * f) {
+  node * factor = f->body->recreate ();
+  reference * r = new reference ();
+  r->n = strdup (f->result);
+  if (isZero (body) || isZero (factor)) {
+    delete body;
+    defCon (body, 0);
+  } else if (isOne (body)) {
+    body = r;
+  } else if (isOne (factor)) {
+    body = body;
+  } else {
+    application * mul = new application ("*", 2);
+    mul->args = body;
+    mul->args->append (r);
+    body = mul;
+  }
+}
+
+/* Add two assignments. */
+void assignment::add (assignment * f) {
+  node * factor = f->body->recreate ();
+  if (isZero (body) && isZero (factor)) {
+    delete body; delete factor;
+    defCon (body, 0);
+  } else if (isZero (body)) {
+    delete body;
+    body = factor;
+  } else if (isZero (factor)) {
+    delete factor;
+    body = body;
+  } else {
+    application * add = new application ("+", 2);
+    add->args = body;
+    add->args->append (factor);
+    body = add;
+  }
+}
+
 // Constructor creates an instance of the application class.
 application::application () : node (APPLICATION) {
   n = NULL;
   nargs = 0;
   args = NULL;
   eval = NULL;
+  derive = NULL;
+}
+
+/* Constructor creates an instance of the application class with a
+   given function name and the number of arguments. */
+application::application (char * func, int a) : node (APPLICATION) {
+  n = func ? strdup (func) : NULL;
+  nargs = a;
+  args = NULL;
+  eval = NULL;
+  derive = NULL;
+}
+
+/* This copy constructor creates a instance of the application class
+   based on the given application. */
+application::application (const application & o) : node (o) {
+  n = o.n ? strdup (o.n) : NULL;
+  nargs = o.nargs;
+  if (o.args != NULL) {
+    node * arg = o.args;
+    args = arg->recreate ();
+    for (arg = arg->getNext (); arg != NULL; arg = arg->getNext ()) {
+      args->append (arg->recreate ());
+    }
+  }
+  else args = NULL;
+  eval = o.eval;
+  derive = o.derive;
+}
+
+// Re-creates the given instance.
+node * application::recreate (void) {
+  return new application (*this);
+}
+
+// Replaces reference name by the new given name.
+void application::replace (char * src, char * dst) {
+  for (node * arg = args; arg != NULL; arg = arg->getNext ()) {
+    arg->replace (src, dst);
+  }
 }
 
 // Destructor deletes an instance of the application class.
@@ -345,14 +572,25 @@ void application::print (void) {
 char * application::toString (void) {
   if (txt) free (txt);
   // binary operations
-  if ((!strcmp (n, "+") || !strcmp (n, "-") || !strcmp (n, "*") ||
-       !strcmp (n, "/") || !strcmp (n, "^") || !strcmp (n, "%"))
+  if ((!strcmp (n, "+")  || !strcmp (n, "-")  || !strcmp (n, "*") ||
+       !strcmp (n, "/")  || !strcmp (n, "^")  || !strcmp (n, "%") ||
+       !strcmp (n, "<")  || !strcmp (n, ">")  || !strcmp (n, "<=") ||
+       !strcmp (n, ">=") || !strcmp (n, "&&") || !strcmp (n, "||") ||
+       !strcmp (n, "==") || !strcmp (n, "!="))
       && nargs == 2) {
     char * arg1 = args->toString ();
     char * arg2 = args->getNext()->toString ();
     txt = (char *) malloc (strlen (n) + strlen (arg1) + strlen (arg2) + 3);
     sprintf (txt, "(%s%s%s)", arg1, n, arg2);
   }
+  // ternary ?: operator
+  else if (!strcmp (n, "?:")) {
+    char * arg1 = args->toString ();
+    char * arg2 = args->getNext()->toString ();
+    char * arg3 = args->getNext()->getNext()->toString ();
+    txt = (char *) malloc (strlen (arg3) + strlen (arg1) + strlen (arg2) + 5);
+    sprintf (txt, "(%s?%s:%s)", arg1, arg2, arg3);
+  }
   // array indices
   else if (!strcmp (n, "array")) {
     int len = strlen (args->toString ()) + 3 + nargs - 1;
@@ -385,6 +623,7 @@ char * application::toString (void) {
 // Adds the arguments of the application to the list of dependencies.
 void application::addDependencies (strlist * depends) {
   for (node * arg = args; arg != NULL; arg = arg->getNext ()) {
+    arg->checkee = checkee;
     arg->addDependencies (depends);
   }  
 }
@@ -438,6 +677,8 @@ int application::evalType (void) {
   setType (TAG_UNKNOWN);
   // Evaluate type of arguments.
   evalTypeArgs ();
+  // Find an appropriate differentiator.
+  findDifferentiator ();
   // Try the fast method.
   if (evalTypeFast () != TAG_UNKNOWN) return getType ();
 
@@ -475,13 +716,27 @@ int application::evalType (void) {
   return getType ();
 }
 
+/* This function returns zero if the applications differentiation
+   function could be found and otherwise non-zero. */
+int application::findDifferentiator (void) {
+  for (int i = 0; differentiations[i].application != NULL; i++) {
+    if (!strcmp (n, differentiations[i].application) &&
+	nargs == differentiations[i].nargs) {
+      derive = differentiations[i].derive;
+      return 0;
+    }
+  }
+  return -1;
+}
+
 /* This function runs the actual evaluation function and the returns
    the result. */
 constant * application::evaluate (void) {
   int errors = 0;
   // first evaluate each argument
   for (node * arg = args; arg != NULL; arg = arg->getNext ()) {
-    if (arg->evaluated == 0) {
+    // FIXME: Can save evaluation of already evaluated equations?
+    if (arg->evaluated == 0 || 1) {
       arg->solvee = solvee;
       arg->evaluate ();
       if (arg->getResult () == NULL) {
@@ -507,6 +762,9 @@ constant * application::evaluate (void) {
     }
   }
   if (!errors) {
+    node * res;
+    // delete previous result if necessary
+    if ((res = getResult ()) != NULL) delete res;
     // then evaluate the application
     setResult (eval (C (args)));
     // check the returned type once again
@@ -518,6 +776,13 @@ constant * application::evaluate (void) {
   return getResult ();
 }
 
+// Returns the derivative of an application.
+node * application::differentiate (char * derivative) {
+  if (derive)
+    return derive (this, derivative);
+  return recreate ();
+}
+
 // Constructor creates an untyped instance of the equation node class.
 node::node () {
   tag = UNKNOWN;
@@ -531,6 +796,7 @@ node::node () {
   res = NULL;
   instance = NULL;
   solvee = NULL;
+  checkee = NULL;
 }
 
 // This constructor creates an typed instance of the equation node class.
@@ -545,6 +811,25 @@ node::node (int type) {
   txt = NULL;
   res = NULL;
   instance = NULL;
+  solvee = NULL;
+  checkee = NULL;
+}
+
+/* This copy constructor creates a instance of the node class based on
+   the given node. */
+node::node (const node & o) {
+  tag = o.tag;
+  dropdeps = output = evaluated = evalPossible = cycle = duplicate = 0;
+  next = NULL;
+  dependencies = NULL;
+  dataDependencies = NULL;
+  dropDependencies = NULL;
+  prepDependencies = NULL;
+  txt = NULL;
+  res = NULL;
+  instance = NULL;
+  solvee = o.solvee;
+  checkee = o.checkee;
 }
 
 // Destructor deletes an instance of the equation node class.
@@ -603,6 +888,11 @@ node * node::get (int pos) {
   return n;
 }
 
+// Sets the constant equation node result.
+void node::setResult (constant * r) {
+  res = r;
+}
+
 // Returns the constant equation node at the given argument position.
 constant * node::getResult (int pos) {
   node * n = this;
@@ -660,7 +950,7 @@ strlist * node::recurseDependencies (checker * check, strlist * deps) {
 
   /* Recurse once again if the child equations revealed any more
      dependencies. */
-  if (sub && sub->length () > 0) {
+  if (cycle && sub && sub->length () > 0) {
     res = recurseDependencies (check, sub);
     delete sub;
     sub = res;
@@ -738,7 +1028,9 @@ strlist * node::collectDataDependencies (void) {
 
 // Constructor creates an instance of the checker class.
 checker::checker () {
+  defs = NULL;
   equations = NULL;
+  consts = false;
 }
 
 // Destructor deletes an instance of the checker class.
@@ -759,12 +1051,17 @@ checker::~checker () {
    checker and applies the dependency list. */
 void checker::collectDependencies (void) {
   foreach_equation (eqn) {
-    strlist * depends = new strlist ();
-    eqn->addDependencies (depends);
-    eqn->setDependencies (depends);
+    collectDependencies (eqn);
   }
 }
 
+// Creates dependency list of given equation node.
+void checker::collectDependencies (node * eqn) {
+  strlist * depends = new strlist ();
+  eqn->addDependencies (depends);
+  eqn->setDependencies (depends);
+}
+
 /* The following function goes through the list of equations and
    checks whether there is any kind of 'Export="yes|no"' assignment in
    it.  Depending on the value the referred equation results are saved
@@ -810,7 +1107,6 @@ int checker::checkExport (void) {
       }
     }
   }
-  eqn::equations = equations;
   return errors;
 }
 
@@ -821,6 +1117,7 @@ void checker::list (void) {
     logprint (LOG_STATUS, "%s", eqn->evalPossible ? 
 	      (eqn->getType () == TAG_UNKNOWN ? "U!" :
 	       eqn->getType () == TAG_DOUBLE  ? "D!" :
+	       eqn->getType () == TAG_BOOLEAN ? "B!" :
 	       eqn->getType () == TAG_COMPLEX ? "C!" :
 	       eqn->getType () == TAG_VECTOR  ? "V!" :
 	       eqn->getType () == TAG_CHAR    ? "CHR!" :
@@ -858,12 +1155,24 @@ int checker::findUndefined (int noundefined) {
     for (int i = 0; i < depends->length (); i++) {
       char * var = depends->get (i);
       if (idents->contains (var) <= 0) {
+	// check if this is a circuit property
+	if (defs) {
+	  node * eqn = findProperty (var);
+	  if (eqn) {
+	    idents->append (var);
+	    eqn->collectDependencies ();
+	    continue;
+	  }
+	} 
+	// give an error
 	if (noundefined) {
-	  if (isGenerated (var)) continue;
+	  if (isGenerated (var)) // skip probably generated variables
+	    continue;
 	  logprint (LOG_ERROR, "checker error, undefined variable `%s' in "
 		    "equation `%s'\n", var, eqn->result);
 	  err++;
 	}
+	// give a notice only
 	else {
 	  logprint (LOG_STATUS, "checker notice, variable `%s' in "
 		    "equation `%s' not yet defined\n", var, eqn->result);
@@ -875,6 +1184,54 @@ int checker::findUndefined (int noundefined) {
   return err;
 }
 
+/* This function tries to find the given variable name which occurred
+   in an equation dependency in the netlist.  If there is such a
+   circuit property it returns a new assignment equation. */
+node * checker::findProperty (char * var) {
+
+  node * eqn = NULL;
+  int found = 0;
+
+  // split into instance and property name
+  char * ret, * inst, * prop;
+  if ((ret = strchr (var, '.')) != NULL) {
+    int len = ret - var;
+    inst = (char *) calloc (1, len + 1);
+    memcpy (inst, var, len);
+    prop = &var[len + 1];
+  }
+  else return NULL;
+
+  // go through list of circuit elements
+  for (struct definition_t * def = defs; def; def = def->next) {
+    if (!strcmp (def->instance, inst)) {
+      for (struct pair_t * pair = def->pairs; pair; pair = pair->next) {
+	if (!strcmp (pair->key, prop)) {
+	  if (++found == 1) {
+	    if (pair->value->ident != NULL) {
+	      // reference
+	      eqn = createReference ("#property", var, pair->value->ident);
+	    }
+	    else {
+	      // value
+	      eqn = createDouble ("#property", var, pair->value->value);
+	    }
+	  }
+	}
+      }
+    }
+  }
+  if (found > 1) {
+    logprint (LOG_ERROR, "checker error, desired property variable `%s' found "
+	      "%dx, is not unique'\n", var, found);
+    delete eqn; eqn = NULL;
+  }
+  else if (found == 1)
+    appendEquation (eqn);
+  free (inst);
+  return eqn;
+}
+
 /* Go through the list of equations and store the left hand side in
    a string list. */
 strlist * checker::getVariables (void) {
@@ -916,8 +1273,9 @@ int checker::findDuplicate (void) {
   return err;
 }
 
-/* The function returns the equations resulting in the passed variable
-   or NULL if there is no such equation. */
+/* The function returns the equation resulting in the passed variable
+   or NULL if there is no such equation.  The function looks through
+   the passed equation root. */
 node * checker::findEquation (node * root, char * n) {
   for (node * eqn = root; eqn != NULL; eqn = eqn->getNext ()) {
     if (!strcmp (A(eqn)->result, n)) return eqn;
@@ -925,6 +1283,15 @@ node * checker::findEquation (node * root, char * n) {
   return NULL;
 }
 
+/* The function returns the equation resulting in the passed variable
+   or NULL if there is no such equation. */
+node * checker::findEquation (char * n) {
+  foreach_equation (eqn) {
+    if (!strcmp (A(eqn)->result, n)) return eqn;
+  }
+  return NULL;
+}
+
 /* This function display the error messages due to equation cycles and
    returns zero if there are no such cycles. */
 int checker::detectCycles (void) {
@@ -1027,14 +1394,25 @@ void checker::reorderEquations (void) {
   }
 }
 
+
+/* The function passes a list of equations to the checker and also
+   passes the checker instance to each equation. */
+void checker::setEquations (node * eqns) {
+  equations = eqns;
+  foreach_equation (eqn) { eqn->checkee = this; }
+}
+
 /* The function evaluates the types for each equation and recursively
    checks the availability of the appropriate function. */
 int checker::applyTypes (void) {
   int err = 0;
   foreach_equation (eqn) {
     if (eqn->evalPossible) {
-      if (eqn->evalType () == TAG_UNKNOWN)
+      if (eqn->evalType () == TAG_UNKNOWN) {
+	logprint (LOG_ERROR, "checker error, type of equation `%s' "
+		  "undefined\n", eqn->result);
 	err++;
+      }
     }
     else break;
   }
@@ -1044,31 +1422,27 @@ int checker::applyTypes (void) {
 /* This function is the checker routine for a parsed equations.  It
    returns zero on success or non-zero if the parsed equations
    contained errors. */
-int equation_checker (int noundefined) {
+int checker::check (int noundefined) {
   int err = 0;
-  eqn::checker * check = new eqn::checker ();
-  check->setEquations (eqn::equations);
-  err += check->checkExport ();
-  check->collectDependencies ();
-  err += check->findUndefined (noundefined);
-  err += check->findDuplicate ();
-  err += check->detectCycles ();
-  check->reorderEquations ();
-  err += check->applyTypes ();
+  err += checkExport ();
+  collectDependencies ();
+  err += findUndefined (noundefined);
+  err += findDuplicate ();
+  err += detectCycles ();
+  reorderEquations ();
+  err += applyTypes ();
 #if DEBUG && 0
-  check->list ();
+  list ();
 #endif /* DEBUG */
-  eqn::equations = check->getEquations ();
-  check->setEquations (NULL);
-  delete check;
   return err;
 }
 
 // Constructor creates an instance of the solver class.
-solver::solver () {
+solver::solver (checker * c) {
   equations = NULL;
   data = NULL;
   generated = 0;
+  checkee = c;
 }
 
 // Destructor deletes an instance of the solver class.
@@ -1081,9 +1455,10 @@ solver::~solver () {
 }
 
 // The function finally evaluates each equation passed to the solver.
-void solver::solve (void) {
+void solver::evaluate (void) {
   foreach_equation (eqn) {
-    if (eqn->evalPossible && eqn->evaluated == 0) {
+    // FIXME: Can save evaluation of already evaluated equations?
+    if (eqn->evalPossible /* && eqn->evaluated == 0 */) {
       // exception handling around evaluation
       try_running () {
 	eqn->solvee = this;
@@ -1097,8 +1472,15 @@ void solver::solve (void) {
       }
       eqn->evaluated++;
 #if DEBUG
+      // print equation results
       logprint (LOG_STATUS, "%s = %s\n", A(eqn)->result, 
 		eqn->getResult () ? eqn->getResult()->toString () : "error");
+#if TESTING_DERIVATIVE || 0
+      // print equation
+      logprint (LOG_STATUS, "%s\n", eqn->toString ());
+      // print derivations
+      logprint (LOG_STATUS, "%s\n", eqn->differentiate("x")->toString ());
+#endif
 #endif
     }
   }
@@ -1155,6 +1537,10 @@ vector * solver::dataVector (node * eqn) {
     v = new vector ();
     v->add (eqn->getResult()->d);
     break;
+  case TAG_BOOLEAN: // boolean value
+    v = new vector ();
+    v->add (eqn->getResult()->b ? 1 : 0);
+    break;
   case TAG_COMPLEX: // complex value
     v = new vector ();
     v->add (* (eqn->getResult()->c));
@@ -1336,7 +1722,7 @@ int solver::dataSize (constant * eqn) {
    variable name.  It must be ensured that the variable actually
    exists and is already evaluated. */
 int solver::getDataSize (char * var) {
-  node * eqn = checker::findEquation (eqn::equations, var);
+  node * eqn = checker::findEquation (equations, var);
   return dataSize (C (eqn));
 }
 
@@ -1383,16 +1769,26 @@ vector * solver::getDataVector (char * str) {
    returns NULL if there are no such dependencies. */
 strlist * solver::collectDataDependencies (node * eqn) {
   strlist * sub = NULL, * datadeps = NULL;
+  // should all data dependencies be dropped?
   if (!eqn->getResult()->dropdeps) {
     strlist * deps = eqn->getDependencies ();
     datadeps = eqn->getDataDependencies ();
     datadeps = datadeps ? new strlist (*datadeps) : NULL;
+    // go through equation dependencies
     for (int i = 0; deps && i < deps->length (); i++) {
       char * var = deps->get (i);
-      node * n = checker::findEquation (eqn::equations, var);
+      // find equation node for the dependency
+      node * n = checker::findEquation (equations, var);
+      // try again in the solver equations
       if (n == NULL && eqn->solvee != NULL)
 	n = checker::findEquation (eqn->solvee->getEquations (), var);
+      // if finally founf the equation node
       if (n != NULL) {
+	// pass resulting data dependencies up
+	strlist * resdeps;
+	if ((resdeps = n->getResult()->getDataDependencies ()) != NULL)
+	  n->setDataDependencies (resdeps);
+	// add data dependencies
 	sub = strlist::join (datadeps, n->getDataDependencies ());
 	sub->del (n->getResult()->getDropDependencies ());
 	sub->add (n->getResult()->getPrepDependencies ());
@@ -1401,6 +1797,7 @@ strlist * solver::collectDataDependencies (node * eqn) {
       datadeps = sub;
     }
   }
+  // prepend dependencies if necessary
   strlist * preps = eqn->getResult()->getPrepDependencies ();
   if (datadeps) {
     if (preps) datadeps->add (preps->last ());
@@ -1408,8 +1805,11 @@ strlist * solver::collectDataDependencies (node * eqn) {
     datadeps = new strlist ();
     if (preps) datadeps->add (preps->last ());
   }
+  // drop duplicate entries
   datadeps = checker::foldDependencies (datadeps);
+  // delete the dependencies to be dropped intentionally
   datadeps->del (eqn->getResult()->getDropDependencies ());
+  // finally return the correct data dependencies
   if (datadeps->length () == 0) {
     delete datadeps;
     datadeps = NULL;
@@ -1491,27 +1891,27 @@ int solver::findEquationResult (node * eqn) {
 /* This function is called in order to run the equation checker and
    the solver.  The optional dataset passed to the function receives
    the results of the calculations. */
-int equation_solver (dataset * data) {
-  if (!solve) solve = new solver ();
-  solve->setEquations (eqn::equations);
-  solve->setData (data);
-  solve->checkinDataset ();
-  eqn::equations = solve->getEquations ();
-  if (equation_checker (data ? 1 : 0) != 0) {
-    solve->setEquations (NULL);
+int solver::solve (dataset * data) {
+  // load additional dataset equations
+  setData (data);
+  checkinDataset ();
+  // put these into the checker
+  checkee->setEquations (equations);
+  // and check
+  if (checkee->check (data ? 1 : 0) != 0) {
     return -1;
   }
-  solve->setEquations (eqn::equations);
-  solve->solve ();
-  solve->checkoutDataset ();
-  eqn::equations = solve->getEquations ();
-  solve->setEquations (NULL);
+  equations = checkee->getEquations ();
+  // finally evaluate equations
+  evaluate ();
+  // put results into the dataset
+  checkoutDataset ();
   return 0;
 }
 
 /* Go through the list of equations and store the left hand side in
    a string list. */
-strlist * equation_variables (void) {
+strlist * checker::variables (void) {
   strlist * idents = new strlist ();
   foreach_equation (eqn) {
     idents->add (eqn->result);
@@ -1519,6 +1919,15 @@ strlist * equation_variables (void) {
   return idents;
 }
 
+// Checks if the given variable name is an equation.
+bool checker::containsVariable (char * ident) {
+  foreach_equation (eqn) {
+    if (!strcmp (ident, eqn->result))
+      return true;
+  }
+  return false;
+}
+
 // Structure defining a predefined constant.
 struct pconstant {
   char * ident;
@@ -1528,42 +1937,113 @@ struct pconstant {
 // List of global constant variables.
 static struct pconstant pconstants[] = {
   { "pi", M_PI },
-  { "e",  M_E },
-  { "kB", kB },
-  { NULL, 0 }
+  { "e",  M_E  },
+  { "kB", kB   },
+  { "q",  Q    },
+  { NULL, 0    }
 };
 
 /* The function should be called before parsing the netlist.  It
    appends the predefined constants to the list of equations. */
-void equation_constants (void) {
+void checker::constants (void) {
+
+  // return if nothing to do
+  if (consts) return;
+
+  // go through constants and add these to the equations
   for (int i = 0; pconstants[i].ident != NULL; i++) {
-    // create constant double value
-    constant * c = new constant (eqn::TAG_DOUBLE);
-    c->d = pconstants[i].value;
-    // create the appropriate assignment
-    assignment * a = new assignment ();
-    a->result = strdup (pconstants[i].ident);
-    a->body = c;
-    a->output = 0;
-    a->setInstance ("#predefined");
-    // append the assignment to equations
-    a->setNext (eqn::equations);
-    eqn::equations = a;
+    addDouble ("#predefined", pconstants[i].ident, pconstants[i].value);
   }
+
+  // indicate that constants have been added
+  consts = true;
 }
 
-/* This is the final destructor for the equation solver. */
-void equation_destructor (void) {
-  // delete solver
-  if (solve) {
-    delete solve;
-    solve = NULL;
+/* The function adds a new equation to the equation checker consisting
+   of an assignment of a double variable. */
+node * checker::addDouble (char * type, char * ident, nr_double_t value) {
+  node * eqn = createDouble (type, ident, value);
+  addEquation (eqn);
+  return eqn;
+}
+
+// Adds given equation to the equation list.
+void checker::addEquation (node * eqn) {
+  eqn->setNext (equations);
+  equations = eqn;
+}
+
+// Appends the given equation to the equation list.
+void checker::appendEquation (node * eqn) {
+  eqn->setNext (NULL);
+  node * last = lastEquation (equations);
+  if (last != NULL)
+    last->setNext (eqn);
+  else
+    equations = eqn;
+}
+
+/* This function creates a equation consisting of an assignment of a
+   double variable. */
+node * checker::createDouble (char * type, char * ident, nr_double_t value) {
+  // create constant double value
+  constant * c = new constant (TAG_DOUBLE);
+  c->checkee = this;
+  c->d = value;
+  // create the appropriate assignment
+  assignment * a = new assignment ();
+  a->checkee = this;
+  a->result = strdup (ident);
+  a->body = c;
+  a->output = 0;
+  a->setInstance (type);
+  return a;
+}
+
+/* This function creates a equation consisting of an assignment of a
+   reference. */
+node * checker::createReference (char * type, char * ident, char * value) {
+  // create reference value
+  reference * r = new reference ();
+  r->checkee = this;
+  r->n = strdup (value);
+  // create the appropriate assignment
+  assignment * a = new assignment ();
+  a->checkee = this;
+  a->result = strdup (ident);
+  a->body = r;
+  a->output = 0;
+  a->setInstance (type);
+  return a;
+}
+
+/* The functions looks through the set of equations for a real valued
+   result and returns it.  If there is no such assignment, zero is
+   returned. */ 
+nr_double_t checker::getDouble (char * ident) {
+  foreach_equation (eqn) {
+    if (!strcmp (ident, eqn->result)) {
+      constant * c = C (eqn->getResult ());
+      if (c != NULL) {
+	if (c->type == TAG_DOUBLE)
+	  return c->d;
+	else if (c->type == TAG_COMPLEX)
+	  return real (*(c->c));
+      }
+    }
   }
-  // delete equations
-  node * next, * eqn;
-  for (eqn = eqn::equations; eqn != NULL; eqn = next) {
-    next = eqn->getNext ();
-    delete eqn;
+  return 0.0;
+}
+
+/* The function goes through the equation set and looks for the
+   specified assignment.  If found the given value is set. */
+void checker::setDouble (char * ident, nr_double_t val) {
+  foreach_equation (eqn) {
+    if (!strcmp (ident, eqn->result)) {
+      if (eqn->body->getTag () == CONSTANT) {
+	constant * c = C (eqn->body);
+	if (c->type == TAG_DOUBLE) c->d = val;
+      }
+    }
   }
-  equations = NULL;
 }
