@@ -1,7 +1,7 @@
 /*
  * evaluate.cpp - the Qucs equation evaluator implementations
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2006 Gunther Kraut <gn.kraut@t-online.de>
  *
  * This is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: evaluate.cpp,v 1.50 2006/05/26 12:11:40 raimi Exp $
+ * $Id: evaluate.cpp,v 1.61 2007/05/24 16:40:31 ela Exp $
  *
  */
 
@@ -66,6 +66,7 @@ using namespace fspecial;
 #define CHR(con) ((constant *) (con))->chr
 #define INT(con) ((int) D (con))
 #define RNG(con) ((constant *) (con))->r
+#define B(con)   ((constant *) (con))->b
 
 #define A(a) ((assignment *) (a))
 #define R(r) ((reference *) (r))
@@ -75,6 +76,7 @@ using namespace fspecial;
 #define _ARG(idx) args->get(idx)
 
 #define _D(var,idx) nr_double_t (var) = D (_ARES (idx));
+#define _B(var,idx) bool (var) = B (_ARES (idx));
 #define _CX(var,idx) complex * (var) = C (_ARES (idx));
 #define _V(var,idx) vector * (var) = V (_ARES (idx));
 #define _M(var,idx) matrix * (var) = M (_ARES (idx));
@@ -91,6 +93,9 @@ using namespace fspecial;
 #define _ARD0(var) _D (var,0)
 #define _ARD1(var) _D (var,1)
 #define _ARD2(var) _D (var,2)
+#define _ARB0(var) _B (var,0)
+#define _ARB1(var) _B (var,1)
+#define _ARB2(var) _B (var,2)
 #define _ARC0(var) _CX (var,0)
 #define _ARC1(var) _CX (var,1)
 #define _ARC2(var) _CX (var,2)
@@ -104,20 +109,30 @@ using namespace fspecial;
 #define _ARMV1(var) _MV (var,1)
 #define _ARMV2(var) _MV (var,2)
 
-// Return value macros.
+// Return value definition macros.
 #define _DEFD() constant * res = new constant (TAG_DOUBLE);
+#define _DEFB() constant * res = new constant (TAG_BOOLEAN);
 #define _DEFC() constant * res = new constant (TAG_COMPLEX);
 #define _DEFV() constant * res = new constant (TAG_VECTOR);
 #define _DEFM() constant * res = new constant (TAG_MATRIX);
 #define _DEFMV() constant * res = new constant (TAG_MATVEC);
 #define _DEFR() constant * res = new constant (TAG_RANGE);
+
+// Return value macros.
 #define _RETD(var) res->d = (var); return res;
+#define _RETB(var) res->b = (var); return res;
 #define _RETC(var) res->c = new complex (var); return res;
 #define _RETV(var) res->v = new vector (var); return res;
 #define _RETM(var) res->m = new matrix (var); return res;
 #define _RETMV(var) res->mv = new matvec (var); return res;
 #define _RETR(var) res->r = (var); return res;
 
+// Return value macros without arguments.
+#define __RETC() res->c = new complex (); return res;
+#define __RETV() res->v = new vector (); return res;
+#define __RETM() res->m = new matrix (); return res;
+#define __RETMV() res->mv = new matvec (); return res;
+
 #define SOLVEE(idx) args->get(idx)->solvee
 
 // Throws a math exception.
@@ -183,6 +198,7 @@ constant * evaluate:: QUCS_CONCAT2 (cfunc,_mv) (constant * args) {\
 }
 
 MAKE_FUNC_DEFINITION_0 (exp);    // exponential function
+MAKE_FUNC_DEFINITION_0 (limexp); // limited exponential function
 MAKE_FUNC_DEFINITION_0 (sin);    // sine
 MAKE_FUNC_DEFINITION_0 (cos);    // cosine
 MAKE_FUNC_DEFINITION_0 (tan);    // tangent
@@ -192,7 +208,8 @@ MAKE_FUNC_DEFINITION_0 (tanh);   // tangent hyperbolicus
 MAKE_FUNC_DEFINITION_0 (coth);   // cotangent hyperbolicus
 MAKE_FUNC_DEFINITION_0 (sech);   // secans hyperbolicus
 MAKE_FUNC_DEFINITION_0 (cosech); // cosecans hyperbolicus
-MAKE_FUNC_DEFINITION_0 (sign);   // signum function
+MAKE_FUNC_DEFINITION_0 (signum); // signum function
+MAKE_FUNC_DEFINITION_0 (sign);   // sign function
 MAKE_FUNC_DEFINITION_0 (sinc);   // sin(x)/x aka sinc function
 MAKE_FUNC_DEFINITION_0 (sqr);    // square value
 
@@ -313,6 +330,76 @@ constant * evaluate::plus_m_mv (constant * args) {
   _RETMV (*m1 + *v2);
 }
 
+constant * evaluate::plus_m_d (constant * args) {
+  _ARM0 (m1);
+  _ARD1 (d2);
+  _DEFM ();
+  _RETM (*m1 + d2);
+}
+
+constant * evaluate::plus_d_m (constant * args) {
+  _ARD0 (d1);
+  _ARM1 (m2);
+  _DEFM ();
+  _RETM (d1 + *m2);
+}
+
+constant * evaluate::plus_m_c (constant * args) {
+  _ARM0 (m1);
+  _ARC1 (c2);
+  _DEFM ();
+  _RETM (*m1 + *c2);
+}
+
+constant * evaluate::plus_c_m (constant * args) {
+  _ARC0 (c1);
+  _ARM1 (m2);
+  _DEFM ();
+  _RETM (*c1 + *m2);
+}
+
+constant * evaluate::plus_mv_d (constant * args) {
+  _ARMV0 (m1);
+  _ARD1  (d2);
+  _DEFMV ();
+  _RETMV (*m1 + d2);
+}
+
+constant * evaluate::plus_d_mv (constant * args) {
+  _ARD0  (d1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (d1 + *m2);
+}
+
+constant * evaluate::plus_mv_c (constant * args) {
+  _ARMV0 (m1);
+  _ARC1  (c2);
+  _DEFMV ();
+  _RETMV (*m1 + *c2);
+}
+
+constant * evaluate::plus_c_mv (constant * args) {
+  _ARC0  (c1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (*c1 + *m2);
+}
+
+constant * evaluate::plus_mv_v (constant * args) {
+  _ARMV0 (m1);
+  _ARV1  (v2);
+  _DEFMV ();
+  _RETMV (*m1 + *v2);
+}
+
+constant * evaluate::plus_v_mv (constant * args) {
+  _ARV0  (v1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (*v1 + *m2);
+}
+
 constant * evaluate::plus_s_s (constant * args) {
   char * s1 = STR (_ARES(0));
   char * s2 = STR (_ARES(1));
@@ -456,6 +543,76 @@ constant * evaluate::minus_m_mv (constant * args) {
   _RETMV (*m1 - *v2);
 }
 
+constant * evaluate::minus_m_d (constant * args) {
+  _ARM0 (m1);
+  _ARD1 (d2);
+  _DEFM ();
+  _RETM (*m1 - d2);
+}
+
+constant * evaluate::minus_d_m (constant * args) {
+  _ARD0 (d1);
+  _ARM1 (m2);
+  _DEFM ();
+  _RETM (d1 - *m2);
+}
+
+constant * evaluate::minus_m_c (constant * args) {
+  _ARM0 (m1);
+  _ARC1 (c2);
+  _DEFM ();
+  _RETM (*m1 - *c2);
+}
+
+constant * evaluate::minus_c_m (constant * args) {
+  _ARC0 (c1);
+  _ARM1 (m2);
+  _DEFM ();
+  _RETM (*c1 - *m2);
+}
+
+constant * evaluate::minus_mv_d (constant * args) {
+  _ARMV0 (m1);
+  _ARD1  (d2);
+  _DEFMV ();
+  _RETMV (*m1 - d2);
+}
+
+constant * evaluate::minus_d_mv (constant * args) {
+  _ARD0  (d1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (d1 - *m2);
+}
+
+constant * evaluate::minus_mv_c (constant * args) {
+  _ARMV0 (m1);
+  _ARC1  (c2);
+  _DEFMV ();
+  _RETMV (*m1 - *c2);
+}
+
+constant * evaluate::minus_c_mv (constant * args) {
+  _ARC0  (c1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (*c1 - *m2);
+}
+
+constant * evaluate::minus_mv_v (constant * args) {
+  _ARMV0 (m1);
+  _ARV1  (v2);
+  _DEFMV ();
+  _RETMV (*m1 - *v2);
+}
+
+constant * evaluate::minus_v_mv (constant * args) {
+  _ARV0  (v1);
+  _ARMV1 (m2);
+  _DEFMV ();
+  _RETMV (*v1 - *m2);
+}
+
 // ****************** multiplication *************************
 constant * evaluate::times_d_d (constant * args) {
   _ARD0 (d1);
@@ -876,6 +1033,70 @@ constant * evaluate::power_v_v (constant * args) {
   _RETV (pow (*v1, *v2));
 }
 
+// ****************** hypotenuse *************************
+constant * evaluate::xhypot_d_d (constant * args) {
+  _ARD0 (d1);
+  _ARD1 (d2);
+  _DEFD ();
+  _RETD (xhypot (d1, d2));
+}
+
+constant * evaluate::xhypot_c_c (constant * args) {
+  _ARC0 (c1);
+  _ARC1 (c2);
+  _DEFD ();
+  _RETD (xhypot (*c1, *c2));
+}
+
+constant * evaluate::xhypot_c_d (constant * args) {
+  _ARC0 (c1);
+  _ARD1 (d2);
+  _DEFD ();
+  _RETD (xhypot (*c1, d2));
+}
+
+constant * evaluate::xhypot_d_c (constant * args) {
+  _ARD0 (d1);
+  _ARC1 (c2);
+  _DEFD ();
+  _RETD (xhypot (d1, *c2));
+}
+
+constant * evaluate::xhypot_v_d (constant * args) {
+  _ARV0 (v1);
+  _ARD1 (d2);
+  _DEFV ();
+  _RETV (xhypot (*v1, d2));
+}
+
+constant * evaluate::xhypot_d_v (constant * args) {
+  _ARD0 (d1);
+  _ARV1 (v2);
+  _DEFV ();
+  _RETV (xhypot (d1, *v2));
+}
+
+constant * evaluate::xhypot_v_c (constant * args) {
+  _ARV0 (v1);
+  _ARC1 (c2);
+  _DEFV ();
+  _RETV (xhypot (*v1, *c2));
+}
+
+constant * evaluate::xhypot_c_v (constant * args) {
+  _ARC0 (c1);
+  _ARV1 (v2);
+  _DEFV ();
+  _RETV (xhypot (*c1, *v2));
+}
+
+constant * evaluate::xhypot_v_v (constant * args) {
+  _ARV0 (v1);
+  _ARV1 (v2);
+  _DEFV ();
+  _RETV (xhypot (*v1, *v2));
+}
+
 // ************** conjugate complex **********************
 constant * evaluate::conj_d (constant * args) {
   _ARD0 (d1);
@@ -1270,6 +1491,25 @@ constant * evaluate::sec_v (constant * args) {
   _RETV (1.0 / cos (*v1));
 }
 
+// *************** arcus secans *******************
+constant * evaluate::arcsec_d (constant * args) {
+  _ARD0 (d1);
+  _DEFD ();
+  _RETD (acos (1 / d1));
+}
+
+constant * evaluate::arcsec_c (constant * args) {
+  _ARC0 (c1);
+  _DEFC ();
+  _RETC (arccos (1 / *c1));
+}
+
+constant * evaluate::arcsec_v (constant * args) {
+  _ARV0 (v1);
+  _DEFV ();
+  _RETV (arccos (1 / *v1));
+}
+
 // ***************** cosecans *********************
 constant * evaluate::cosec_d (constant * args) {
   _ARD0 (d1);
@@ -1289,6 +1529,25 @@ constant * evaluate::cosec_v (constant * args) {
   _RETV (1.0 / sin (*v1));
 }
 
+// ************* arcus cosecans *******************
+constant * evaluate::arccosec_d (constant * args) {
+  _ARD0 (d1);
+  _DEFD ();
+  _RETD (asin (1 / d1));
+}
+
+constant * evaluate::arccosec_c (constant * args) {
+  _ARC0 (c1);
+  _DEFC ();
+  _RETC (arcsin (1 / *c1));
+}
+
+constant * evaluate::arccosec_v (constant * args) {
+  _ARV0 (v1);
+  _DEFV ();
+  _RETV (arcsin (1 / *v1));
+}
+
 // ********** area sine hyperbolicus **************
 constant * evaluate::arsinh_d (constant * args) {
   _ARD0 (d1);
@@ -1308,11 +1567,31 @@ constant * evaluate::arsinh_v (constant * args) {
   _RETV (arsinh (*v1));
 }
 
+// ********** area cosecans hyperbolicus **************
+constant * evaluate::arcosech_d (constant * args) {
+  _ARD0 (d1);
+  _DEFD ();
+  d1 = 1 / d1;
+  _RETD (log (d1 + sqrt (d1 * d1 + 1)));
+}
+
+constant * evaluate::arcosech_c (constant * args) {
+  _ARC0 (c1);
+  _DEFC ();
+  _RETC (arsinh (1 / *c1));
+}
+
+constant * evaluate::arcosech_v (constant * args) {
+  _ARV0 (v1);
+  _DEFV ();
+  _RETV (arsinh (1 / *v1));
+}
+
 // ********* area cosine hyperbolicus ************
 constant * evaluate::arcosh_d (constant * args) {
   _ARD0 (d1);
-  _DEFD ();
-  _RETD (log (d1 + sqrt (d1 * d1 - 1)));
+  _DEFC ();
+  _RETC (arcosh (complex (d1)));
 }
 
 constant * evaluate::arcosh_c (constant * args) {
@@ -1327,6 +1606,25 @@ constant * evaluate::arcosh_v (constant * args) {
   _RETV (arcosh (*v1));
 }
 
+// ********* area secans hyperbolicus ***********
+constant * evaluate::arsech_d (constant * args) {
+  _ARD0 (d1);
+  _DEFC ();
+  _RETC (arsech (complex (d1)));
+}
+
+constant * evaluate::arsech_c (constant * args) {
+  _ARC0 (c1);
+  _DEFC ();
+  _RETC (arsech (*c1));
+}
+
+constant * evaluate::arsech_v (constant * args) {
+  _ARV0 (v1);
+  _DEFV ();
+  _RETV (arsech (*v1));
+}
+
 // ******* area tangent hyperbolicus **********
 constant * evaluate::artanh_d (constant * args) {
   _ARD0 (d1);
@@ -1639,6 +1937,34 @@ constant * evaluate::avg_v (constant * args) {
   _RETC (avg (*v1));
 }
 
+// ******************* lengths *********************
+constant * evaluate::length_d (constant *) {
+  _DEFD ();
+  _RETD (1);
+}
+
+constant * evaluate::length_c (constant *) {
+  _DEFD ();
+  _RETD (1);
+}
+
+constant * evaluate::length_v (constant * args) {
+  _ARV0 (v1);
+  _DEFD ();
+  _RETD (v1->getSize ());
+}
+
+constant * evaluate::length_m (constant *) {
+  _DEFD ();
+  _RETD (1);
+}
+
+constant * evaluate::length_mv (constant * args) {
+  _ARV0 (mv);
+  _DEFD ();
+  _RETD (mv->getSize ());
+}
+
 // ***************** array indices *****************
 constant * evaluate::index_mv_2 (constant * args) {
   _ARMV0 (mv);
@@ -1687,18 +2013,18 @@ void evaluate::extract_vector (constant * args, int idx, int &skip, int &size,
   vector * vres;
   strlist * deps = _ARES(0)->getDataDependencies ();
   int didx = (deps ? deps->length () : 0) - idx;
-  int dsize = solver::getDependencySize (deps, idx);
+  int dsize = SOLVEE(0)->getDependencySize (deps, idx);
 
   // all of the data vector
   if (type == TAG_RANGE) {
     vres = new vector (*(res->v));
-    skip *= deps ? solver::getDataSize (deps->get (didx - 1)) : 1;
-    size *= deps ? solver::getDataSize (deps->get (didx)) : 1;
+    skip *= deps ? SOLVEE(0)->getDataSize (deps->get (didx - 1)) : 1;
+    size *= deps ? SOLVEE(0)->getDataSize (deps->get (didx)) : 1;
   }
   // a subset
   else {
     vres = new vector (dsize * size);
-    int len = deps ? solver::getDataSize (deps->get (didx)) : v->getSize ();
+    int len = deps ? SOLVEE(0)->getDataSize (deps->get (didx)) : v->getSize ();
     if (i < 0 || i >= len) {
       char txt[256];
       sprintf (txt, "vector index %d (%d) out of bounds [%d,%d]",
@@ -3246,7 +3572,7 @@ constant * evaluate::runavg_d_d (constant * args) {
   if (n < 1) {
     THROW_MATH_EXCEPTION ("runavg: number n to be averaged over must be "
 			  "larger or equal 1");
-    _RETV ();
+    __RETV ();
   }
   _RETV (runavg (rect (x, 0), n));
 }
@@ -3258,7 +3584,7 @@ constant * evaluate::runavg_c_d (constant * args) {
   if (n < 1) {
     THROW_MATH_EXCEPTION ("runavg: number n to be averaged over must be "
 			  "larger or equal 1");
-    _RETV ();
+    __RETV ();
   }
   _RETV (runavg (*x, n));
 }
@@ -3271,11 +3597,33 @@ constant * evaluate::runavg_v_d (constant * args) {
     THROW_MATH_EXCEPTION ("runavg: number n to be averaged over must be "
 			  "larger or equal 1 and less or equal than the "
 			  "number of vector elements");
-    _RETV ();
+    __RETV ();
   }
   _RETV (runavg (*x, n));
 }
 
+// ********************* thermal voltage **********************
+constant * evaluate::vt_d (constant * args) {
+  _ARD0 (d1);
+  _DEFD ();
+  _RETD (d1 * kBoverQ);
+}
+
+constant * evaluate::vt_c (constant * args) {
+  _ARC0 (c1);
+  _DEFC ();
+  _RETC (*c1 * kBoverQ);
+}
+
+constant * evaluate::vt_v (constant * args) {
+  _ARV0 (v1);
+  _DEFV ();
+  vector * v = new vector ();
+  for (int i = 0; i < v1->getSize (); i++) v->add (v1->get (i) * kBoverQ);
+  res->v = v;
+  return res;
+}
+
 // ************** Kaiser-Bessel derived window ****************
 constant * evaluate::kbd_d_d (constant * args) {
   _ARD0 (alpha);
@@ -3285,7 +3633,7 @@ constant * evaluate::kbd_d_d (constant * args) {
   nr_double_t sval = 0.0;
   if (size <= 0) {
     THROW_MATH_EXCEPTION ("kbd: vector length must be greater than zero");
-    _RETV ();
+    __RETV ();
   }
   vector v (size);
   for (i = 0; i < size / 2; i++) {
@@ -3311,5 +3659,108 @@ constant * evaluate::kbd_d (constant * args) {
   return kbd_d_d (args);
 }
 
+// ***************** if-then-else operation ****************
+constant * evaluate::ifthenelse_d_d (constant * args) {
+  _ARB0 (cond);
+  int t1 = _ARG(1)->getType ();
+  int t2 = _ARG(2)->getType ();
+  nr_double_t d1, d2;
+  if (t1 == TAG_DOUBLE)
+    d1 = D(_ARES(1));
+  else
+    d1 = abs (*C(_ARES(1)));
+  if (t2 == TAG_DOUBLE)
+    d2 = D(_ARES(2));
+  else
+    d2 = abs (*C(_ARES(2)));
+  _DEFD ();
+  _RETD (cond ? d1 : d2);
+}
+
+// ************************** less *************************
+constant * evaluate::less_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 < d1);
+}
+
+// ************************* greater ***********************
+constant * evaluate::greater_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 > d1);
+}
+
+// ********************** less or equal ********************
+constant * evaluate::lessorequal_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 <= d1);
+}
+
+// ********************* greater or equal ******************
+constant * evaluate::greaterorequal_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 >= d1);
+}
+
+// ************************** equal ************************
+constant * evaluate::equal_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 == d1);
+}
+
+constant * evaluate::equal_b_b (constant * args) {
+  _ARB0 (b0);
+  _ARB1 (b1);
+  _DEFB ();
+  _RETB (b0 == b1);
+}
+
+// ************************ not equal **********************
+constant * evaluate::notequal_d_d (constant * args) {
+  _ARD0 (d0);
+  _ARD1 (d1);
+  _DEFB ();
+  _RETB (d0 != d1);
+}
+
+constant * evaluate::notequal_b_b (constant * args) {
+  _ARB0 (b0);
+  _ARB1 (b1);
+  _DEFB ();
+  _RETB (b0 != b1);
+}
+
+// *************************** not *************************
+constant * evaluate::not_b (constant * args) {
+  _ARB0 (b0);
+  _DEFB ();
+  _RETB (!b0);
+}
+
+// *************************** or **************************
+constant * evaluate::or_b_b (constant * args) {
+  _ARB0 (b0);
+  _ARB1 (b1);
+  _DEFB ();
+  _RETB (b0 || b1);
+}
+
+// ************************** and **************************
+constant * evaluate::and_b_b (constant * args) {
+  _ARB0 (b0);
+  _ARB1 (b1);
+  _DEFB ();
+  _RETB (b0 && b1);
+}
+
 // Include the application array.
 #include "applications.h"
