@@ -1,7 +1,7 @@
 /*
  * evaluate.h - definitions for Qucs equation evaluations
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2006 Gunther Kraut <gn.kraut@t-online.de>
  *
  * This is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: evaluate.h,v 1.29 2006/05/23 09:48:32 raimi Exp $
+ * $Id: evaluate.h,v 1.36 2007/05/24 16:40:31 ela Exp $
  *
  */
 
@@ -51,12 +51,22 @@ public:
   static constant * plus_c_v (constant *);
   static constant * plus_v_v (constant *);
   static constant * plus_m_m (constant *);
+  static constant * plus_m_d (constant *);
+  static constant * plus_d_m (constant *);
+  static constant * plus_m_c (constant *);
+  static constant * plus_c_m (constant *);
   static constant * plus_s_s (constant *);
   static constant * plus_c_s (constant *);
   static constant * plus_s_c (constant *);
   static constant * plus_mv_mv (constant *);
   static constant * plus_mv_m (constant *);
   static constant * plus_m_mv (constant *);
+  static constant * plus_mv_d (constant *);
+  static constant * plus_d_mv (constant *);
+  static constant * plus_mv_c (constant *);
+  static constant * plus_c_mv (constant *);
+  static constant * plus_mv_v (constant *);
+  static constant * plus_v_mv (constant *);
 
   static constant * minus_d (constant *);
   static constant * minus_c (constant *);
@@ -74,9 +84,19 @@ public:
   static constant * minus_c_v (constant *);
   static constant * minus_v_v (constant *);
   static constant * minus_m_m (constant *);
+  static constant * minus_m_d (constant *);
+  static constant * minus_d_m (constant *);
+  static constant * minus_m_c (constant *);
+  static constant * minus_c_m (constant *);
   static constant * minus_mv_mv (constant *);
   static constant * minus_mv_m (constant *);
   static constant * minus_m_mv (constant *);
+  static constant * minus_mv_d (constant *);
+  static constant * minus_d_mv (constant *);
+  static constant * minus_mv_c (constant *);
+  static constant * minus_c_mv (constant *);
+  static constant * minus_mv_v (constant *);
+  static constant * minus_v_mv (constant *);
 
   static constant * times_d_d (constant *);
   static constant * times_c_c (constant *);
@@ -137,6 +157,16 @@ public:
   static constant * power_c_v (constant *);
   static constant * power_v_v (constant *);
 
+  static constant * xhypot_d_d (constant *);
+  static constant * xhypot_c_c (constant *);
+  static constant * xhypot_c_d (constant *);
+  static constant * xhypot_d_c (constant *);
+  static constant * xhypot_v_d (constant *);
+  static constant * xhypot_d_v (constant *);
+  static constant * xhypot_v_c (constant *);
+  static constant * xhypot_c_v (constant *);
+  static constant * xhypot_v_v (constant *);
+
   static constant * real_d (constant *);
   static constant * real_c (constant *);
   static constant * real_v (constant *);
@@ -199,6 +229,10 @@ public:
   static constant * exp_c (constant *);
   static constant * exp_v (constant *);
 
+  static constant * limexp_d (constant *);
+  static constant * limexp_c (constant *);
+  static constant * limexp_v (constant *);
+
   static constant * ln_d (constant *);
   static constant * ln_c (constant *);
   static constant * ln_v (constant *);
@@ -247,10 +281,18 @@ public:
   static constant * sec_c (constant *);
   static constant * sec_v (constant *);
 
+  static constant * arcsec_d (constant *);
+  static constant * arcsec_c (constant *);
+  static constant * arcsec_v (constant *);
+
   static constant * cosec_d (constant *);
   static constant * cosec_c (constant *);
   static constant * cosec_v (constant *);
 
+  static constant * arccosec_d (constant *);
+  static constant * arccosec_c (constant *);
+  static constant * arccosec_v (constant *);
+
   static constant * sinh_d (constant *);
   static constant * sinh_c (constant *);
   static constant * sinh_v (constant *);
@@ -275,22 +317,30 @@ public:
   static constant * artanh_c (constant *);
   static constant * artanh_v (constant *);
 
-  static constant * arcoth_d (constant *);
-  static constant * arcoth_c (constant *);
-  static constant * arcoth_v (constant *);
-
   static constant * coth_d (constant *);
   static constant * coth_c (constant *);
   static constant * coth_v (constant *);
 
+  static constant * arcoth_d (constant *);
+  static constant * arcoth_c (constant *);
+  static constant * arcoth_v (constant *);
+
   static constant * sech_d (constant *);
   static constant * sech_c (constant *);
   static constant * sech_v (constant *);
 
+  static constant * arsech_d (constant *);
+  static constant * arsech_c (constant *);
+  static constant * arsech_v (constant *);
+
   static constant * cosech_d (constant *);
   static constant * cosech_c (constant *);
   static constant * cosech_v (constant *);
 
+  static constant * arcosech_d (constant *);
+  static constant * arcosech_c (constant *);
+  static constant * arcosech_v (constant *);
+
   static constant * ztor_d   (constant *);
   static constant * ztor_d_d (constant *);
   static constant * ztor_d_c (constant *);
@@ -371,6 +421,10 @@ public:
   static constant * avg_v (constant *);
   static constant * avg_r (constant *);
 
+  static constant * signum_d (constant *);
+  static constant * signum_c (constant *);
+  static constant * signum_v (constant *);
+
   static constant * sign_d (constant *);
   static constant * sign_c (constant *);
   static constant * sign_v (constant *);
@@ -379,6 +433,12 @@ public:
   static constant * sinc_c (constant *);
   static constant * sinc_v (constant *);
 
+  static constant * length_d (constant *);
+  static constant * length_c (constant *);
+  static constant * length_v (constant *);
+  static constant * length_m (constant *);
+  static constant * length_mv (constant *);
+
   static void extract_vector (constant *, int, int &, int &, constant *);
   static constant * index_mv_2 (constant *);
   static constant * index_mv_1 (constant *);
@@ -625,8 +685,25 @@ public:
   static constant * runavg_c_d (constant *);
   static constant * runavg_v_d (constant *);
 
+  static constant * vt_d (constant *);
+  static constant * vt_c (constant *);
+  static constant * vt_v (constant *);
+
   static constant * kbd_d_d (constant *);
   static constant * kbd_d   (constant *);
+
+  static constant * ifthenelse_d_d (constant *);
+  static constant * less_d_d (constant *);
+  static constant * greater_d_d (constant *);
+  static constant * lessorequal_d_d (constant *);
+  static constant * greaterorequal_d_d (constant *);
+  static constant * equal_d_d (constant *);
+  static constant * equal_b_b (constant *);
+  static constant * notequal_d_d (constant *);
+  static constant * notequal_b_b (constant *);
+  static constant * not_b (constant *);
+  static constant * or_b_b (constant *);
+  static constant * and_b_b (constant *);
 };
 
 // Type of application function.
