@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: hbsolver.cpp,v 1.19 2006/05/15 06:45:42 raimi Exp $
+ * $Id: hbsolver.cpp,v 1.22 2007/04/04 19:25:41 ela Exp $
  *
  */
 
@@ -179,7 +179,7 @@ void hbsolver::solve (void) {
       iterations++;
 
 #if HB_DEBUG
-      fprintf (stderr, "   -- iteration step: %d\n", iterations);
+      fprintf (stderr, "\n   -- iteration step: %d\n", iterations);
       fprintf (stderr, "vs -- voltage in t:\n"); vs->print ();
 #endif
 
@@ -205,6 +205,7 @@ void hbsolver::solve (void) {
       fprintf (stderr, "VS -- voltage in f:\n"); VS->print ();
       fprintf (stderr, "FQ -- charge in f:\n"); FQ->print ();
       fprintf (stderr, "IG -- current in f:\n"); IG->print ();
+      fprintf (stderr, "IR -- corrected Jacobi current in f:\n"); IR->print ();
 #endif
       
       // solve HB equation --> FV = IC + [YV] * VS + j[O] * FQ + IG
@@ -214,6 +215,7 @@ void hbsolver::solve (void) {
       fprintf (stderr, "FV -- error vector F(V) in f:\n"); FV->print ();
       fprintf (stderr, "IL -- linear currents in f:\n"); IL->print ();
       fprintf (stderr, "IN -- non-linear currents in f:\n"); IN->print ();
+      fprintf (stderr, "RH -- right-hand side currents in f:\n"); RH->print ();
 #endif
 
       // termination criteria met
@@ -331,7 +333,12 @@ void hbsolver::expandFrequencies (nr_double_t f, int n) {
   posfreqs.clear ();
   if (len > 0) {
     // frequency expansion for full frequency sets
-    for (i = -n; i <= n + 1; i++) {
+    for (i = 0; i <= n + 1; i++) {
+      for (k = 0; k < len; k++) {
+	negfreqs.add (i * f + nfreqs.get (k));
+      }
+    }
+    for (i = -n; i < 0; i++) {
       for (k = 0; k < len; k++) {
 	negfreqs.add (i * f + nfreqs.get (k));
       }
@@ -344,7 +351,8 @@ void hbsolver::expandFrequencies (nr_double_t f, int n) {
   }
   else {
     // first frequency
-    for (i = -n; i <= n + 1; i++) negfreqs.add (i * f);
+    for (i = 0; i <= n + 1; i++) negfreqs.add (i * f);
+    for (i = -n; i < 0; i++) negfreqs.add (i * f);
     for (i = 0; i <= 2 * n + 1; i++) posfreqs.add (i * f);
   }
 }
@@ -396,7 +404,7 @@ void hbsolver::collectFrequencies (void) {
   // build frequency dimension lengths
   ndfreqs = new int[dfreqs.getSize ()];
   for (i = 0; i < dfreqs.getSize (); i++) {
-    ndfreqs[i] = n * 2 + 1;
+    ndfreqs[i] = (n + 1) * 2;
   }
 
 #if DEBUG && 0
@@ -417,9 +425,7 @@ void hbsolver::collectFrequencies (void) {
 
   // pre-calculate the j[O] vector
   OM = new tvector<complex> (nlfreqs);
-  for (i = n = 0; n < lnfreqs; n++, i++)
-    OM_(n) = rect (0, 2 * M_PI * rfreqs (i));
-  for (i = 0; n < nlfreqs; n++, i++)
+  for (n = i = 0; n < nlfreqs; n++, i++)
     OM_(n) = rect (0, 2 * M_PI * negfreqs (i));
 }
 
@@ -1080,10 +1086,11 @@ void hbsolver::loadMatrices (void) {
 void hbsolver::VectorFFT (tvector<complex> * V, int isign) {
   int i, k, r;
   int n = nlfreqs;
+  int nd = dfreqs.getSize ();
   int nodes = V->getSize () / n;
   nr_double_t * d = (nr_double_t *) V->getData ();
 
-  if (dfreqs.getSize () == 1) {
+  if (nd == 1) {
     // for each node a single 1d-FFT
     for (k = i = 0; i < nodes; i++, k += 2 * n) {
       nr_double_t * dst = &d[k];
@@ -1093,6 +1100,11 @@ void hbsolver::VectorFFT (tvector<complex> * V, int isign) {
   }
   else {
     // for each node a single nd-FFT
+    for (k = i = 0; i < nodes; i++, k += 2 * n) {
+      nr_double_t * dst = &d[k];
+      _fft_nd (dst, ndfreqs, nd, isign);
+      if (isign > 0) for (r = 0; r < 2 * n; r++) *dst++ /= ndfreqs[0];
+    }
   }
 }
 			
