@@ -1,7 +1,7 @@
 /*
  * input.cpp - input netlist class implementation
  *
- * Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: input.cpp,v 1.57 2006/03/24 14:30:05 raimi Exp $
+ * $Id: input.cpp,v 1.72 2007/05/27 17:50:15 ela Exp $
  *
  */
 
@@ -34,6 +34,7 @@
 
 #include "logging.h"
 #include "component.h"
+#include "netdefs.h"
 #include "net.h"
 #include "analysis.h"
 #include "spsolver.h"
@@ -43,6 +44,7 @@
 #include "trsolver.h"
 #include "hbsolver.h"
 #include "variable.h"
+#include "property.h"
 #include "environment.h"
 #include "nodeset.h"
 #include "input.h"
@@ -93,15 +95,10 @@ int input::netlist (net * netlist) {
     return -1;
 
   logprint (LOG_STATUS, "checking netlist...\n");
-  if (netlist_checker () != 0)
+  if (netlist_checker (env) != 0)
     return -1;
 
-  // add equation constants and then check the equations
-  equation_constants ();
-  logprint (LOG_STATUS, "checking equations...\n");
-  if (equation_checker (0) != 0)
-    return -1;
-  if (netlist_checker_variables () != 0)
+  if (netlist_checker_variables (env) != 0)
     return -1;
 
 #if DEBUG
@@ -150,14 +147,25 @@ void input::factory (void) {
 	for (pairs = def->pairs; pairs != NULL; pairs = pairs->next)
 	  if (pairs->value->ident) {
 	    if (pairs->value->var) {
-	      // put new parameter sweep variable into environment
-	      variable * v = new variable (pairs->value->ident);
-	      constant * c = new constant (TAG_DOUBLE);
-	      c->d = 0; // initialize the variable
-	      v->setConstant (c);
-	      env->addVariable (v);
+	      variable * v;
+	      if ((v = def->env->getVariable (pairs->value->ident)) != NULL) {
+		// equation variable reference in analysis property
+		a->addProperty (pairs->key, v);
+	      }
+	      else {
+		// put new parameter sweep variable into environment
+		v = new variable (pairs->value->ident);
+		constant * c = new constant (TAG_DOUBLE);
+		c->d = 0; // initialize the variable
+		v->setConstant (c);
+		def->env->addVariable (v);
+		a->addProperty (pairs->key, pairs->value->ident);
+	      }
+	    }
+	    else {
+	      // ususal string property
+	      a->addProperty (pairs->key, pairs->value->ident);
 	    }
-	    a->addProperty (pairs->key, pairs->value->ident);
 	  } else {
 	    if (pairs->value->var) {
 	      // add list sweeps and constants to the properties
@@ -173,7 +181,7 @@ void input::factory (void) {
 	  }
 	// additionally add missing optional properties
 	assignDefaultProperties (a, def->define);
-
+	a->setEnv (def->env);
 	subnet->insertAnalysis (a);
       }
       // remove this definition from the list
@@ -192,7 +200,17 @@ void input::factory (void) {
 	// add the properties to substrate
 	for (pairs = def->pairs; pairs != NULL; pairs = pairs->next)
 	  if (pairs->value->ident) {
-	    s->addProperty (pairs->key, pairs->value->ident);
+	    // a variable
+	    if (pairs->value->var) {
+	      // at this stage it should be ensured that the variable is
+	      // already in the root environment
+	      variable * v = def->env->getVariable (pairs->value->ident);
+	      s->addProperty (pairs->key, v);
+	    }
+	    // a usual string property
+	    else {
+	      s->addProperty (pairs->key, pairs->value->ident);
+	    }
 	  } else {
 	    s->addProperty (pairs->key, pairs->value->value);
 	  }
@@ -200,9 +218,10 @@ void input::factory (void) {
 	assignDefaultProperties (s, def->define);
 
 	// put new substrate definition into environment
-	variable * v = new variable (def->instance);
+	char * n = strrchr (def->instance, '.');
+	variable * v = new variable (n ? n + 1 : def->instance);
 	v->setSubstrate (s);
-	env->addVariable (v);
+	def->env->addVariable (v);
       }
       // remove this definition from the list
       definition_root = netlist_unchain_definition (definition_root, def);
@@ -240,7 +259,7 @@ void input::factory (void) {
 	  c->setNode (i, nodes->node);
 
       // add the properties to circuit
-      for (pairs = def->pairs; pairs != NULL; pairs = pairs->next)
+      for (pairs = def->pairs; pairs != NULL; pairs = pairs->next) {
 	if (pairs->value == NULL) {
 	  // zero-length value lists
 	  variable * v = new variable (pairs->key);
@@ -253,11 +272,11 @@ void input::factory (void) {
 	  if (pairs->value->var) {
 	    // at this stage it should be ensured that the variable is
 	    // already in the root environment
-	    variable * v = env->getVariable (pairs->value->ident);
+	    variable * v = def->env->getVariable (pairs->value->ident);
 	    o->addProperty (pairs->key, v);
 	  } else {
 	    if (pairs->value->subst) {
-	      variable * v = env->getVariable (pairs->value->ident);
+	      variable * v = def->env->getVariable (pairs->value->ident);
 	      c->setSubstrate (v->getSubstrate ());
 	    }
 	    o->addProperty (pairs->key, pairs->value->ident);
@@ -274,6 +293,10 @@ void input::factory (void) {
 	    o->addProperty (pairs->key, pairs->value->value);
 	  }
 	}
+      }
+      // set local circuit environment
+      c->setEnv (def->env);
+
       // additionally add missing optional properties
       assignDefaultProperties (c, def->define);
 
@@ -293,13 +316,18 @@ void input::assignDefaultProperties (object * obj, struct define_t * def) {
   for (int i = 0; PROP_IS_PROP (def->optional[i]); i++) {
     // is the property already assigned ?
     if (!obj->hasProperty (def->optional[i].key)) {
+      property * p;
       if (PROP_IS_VAL (def->optional[i])) {
 	// add double property
-	obj->addProperty (def->optional[i].key, def->optional[i].defaultval.d);
+	p = obj->addProperty (def->optional[i].key,
+			      def->optional[i].defaultval.d);
+	p->setDefault (true);
       }
       else {
 	// add string property
-	obj->addProperty (def->optional[i].key, def->optional[i].defaultval.s);
+	p = obj->addProperty (def->optional[i].key,
+			      def->optional[i].defaultval.s);
+	p->setDefault (true);
       }
     }
   }
@@ -347,6 +375,10 @@ circuit * input::createCircuit (char * type) {
     return new vac ();
   else if (!strcmp (type, "Iac"))
     return new iac ();
+  else if (!strcmp (type, "Iexp"))
+    return new iexp ();
+  else if (!strcmp (type, "Vexp"))
+    return new vexp ();
   else if (!strcmp (type, "AM_Mod"))
     return new vam ();
   else if (!strcmp (type, "PM_Mod"))
@@ -369,12 +401,18 @@ circuit * input::createCircuit (char * type) {
     return new relais ();
   else if (!strcmp (type, "TLIN"))
     return new tline ();
+  else if (!strcmp (type, "TLIN4P"))
+    return new tline4p ();
   else if (!strcmp (type, "COAX"))
     return new coaxline ();
+  else if (!strcmp (type, "TWIST"))
+    return new twistedpair ();
   else if (!strcmp (type, "Coupler"))
     return new coupler ();
   else if (!strcmp (type, "Diode"))
     return new diode ();
+  else if (!strcmp (type, "EDD"))
+    return new eqndefined ();
   else if (!strcmp (type, "MLIN"))
     return new msline ();
   else if (!strcmp (type, "MCORN"))
@@ -395,6 +433,8 @@ circuit * input::createCircuit (char * type) {
     return new mscross ();
   else if (!strcmp (type, "MVIA"))
     return new msvia ();
+  else if (!strcmp (type, "BOND"))
+    return new bondwire ();
   else if (!strcmp (type, "CLIN"))
     return new cpwline ();
   else if (!strcmp (type, "COPEN"))
@@ -431,6 +471,8 @@ circuit * input::createCircuit (char * type) {
     return new mutual ();
   else if (!strcmp (type, "MUT2"))
     return new mutual2 ();
+  else if (!strcmp (type, "MUTX"))
+    return new mutualx ();
   else if (!strcmp (type, "IVnoise"))
     return new ivnoise ();
   else if (!strcmp (type, "VVnoise"))
@@ -451,6 +493,10 @@ circuit * input::createCircuit (char * type) {
     return new logicxor ();
   else if (!strcmp (type, "DigiSource"))
     return new digisource ();
+  else if (!strcmp (type, "hicumL2p1"))
+    return new hicumL2p1 ();
+  else if (!strcmp (type, "HBT_X"))
+    return new HBT_X ();
 
   logprint (LOG_ERROR, "no such circuit type `%s'\n", type);
   return NULL;
