@@ -1,7 +1,7 @@
 /*
  * nasolver.cpp - nodal analysis solver class implementation
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: nasolver.cpp,v 1.43 2006/05/03 09:43:56 raimi Exp $
+ * $Id: nasolver.cpp,v 1.47 2007/04/04 19:25:41 ela Exp $
  *
  */
 
@@ -124,6 +124,7 @@ nasolver<nr_type_t>::nasolver (nasolver & o) : analysis (o) {
   gMin = o.gMin;
   srcFactor = o.srcFactor;
   eqns = new eqnsys<nr_type_t> (*(o.eqns));
+  solution = nasolution<nr_type_t> (o.solution);
 }
 
 /* The function runs the nodal analysis solver once, reports errors if
@@ -341,7 +342,12 @@ int nasolver<nr_type_t>::solve_nonlinear_continuation_Source (void) {
 
     // not yet converged, so decreased the source-step
     if (run >= MaxIterations || error) {
-      sStep /= 2;
+      if (error)
+	sStep *= 0.1;
+      else
+	sStep *= 0.5;
+      restorePreviousIteration ();
+      saveSolution ();
       // here the absolute minimum step checker
       if (sStep < NR_EPSI) {
 	error = 1;
@@ -354,10 +360,13 @@ int nasolver<nr_type_t>::solve_nonlinear_continuation_Source (void) {
       srcFactor = MIN (sPrev + sStep, 1);
     } 
     // converged, increased the source-step
-    else {
+    else if (run < MaxIterations / 4) {
       sPrev = srcFactor;
       srcFactor = MIN (srcFactor + sStep, 1);
-      sStep *= 2;
+      sStep *= 1.5;
+    }
+    else {
+      srcFactor = MIN (srcFactor + sStep, 1);
     }
   }
   // continue until no source factor is necessary
@@ -1018,6 +1027,24 @@ void nasolver<nr_type_t>::savePreviousIteration (void) {
     zprev = new tvector<nr_type_t> (*z);
 }
 
+/* The function restores the solution and right hand vector of the
+   previous (successful) iteration. */
+template <class nr_type_t>
+void nasolver<nr_type_t>::restorePreviousIteration (void) {
+  if (xprev != NULL) *x = *xprev;
+  if (zprev != NULL) *z = *zprev;
+}
+
+/* The function restarts the NR iteration for each non-linear
+   circuit. */
+template <class nr_type_t>
+void nasolver<nr_type_t>::restartNR (void) {
+  circuit * root = subnet->getRoot ();
+  for (circuit * c = root; c != NULL; c = (circuit *) c->getNext ()) {
+    if (c->isNonLinear ()) c->restartDC ();
+  }
+}
+
 /* This function goes through solution (the x vector) and saves the
    node voltages of the last iteration into each non-linear
    circuit. */
@@ -1061,6 +1088,49 @@ void nasolver<nr_type_t>::saveSolution (void) {
   saveBranchCurrents ();
 }
 
+// This function stores the solution (node voltages and branch currents).
+template <class nr_type_t>
+void nasolver<nr_type_t>::storeSolution (void) {
+  // cleanup solution previously
+  solution.clear ();
+  int r;
+  int N = countNodes ();
+  int M = countVoltageSources ();
+  // store all nodes except reference node
+  for (r = 0; r < N; r++) {
+    struct nodelist_t * n = nlist->getNode (r);
+    solution.add (n->name, x->get (r), 0);
+  }
+  // store all branch currents of voltage sources
+  for (r = 0; r < M; r++) {
+    circuit * vs = findVoltageSource (r);
+    int vn = r - vs->getVoltageSource () + 1;
+    solution.add (vs->getName (), x->get (r + N), vn);
+  }
+}
+
+// This function recalls the solution (node voltages and branch currents).
+template <class nr_type_t>
+void nasolver<nr_type_t>::recallSolution (void) {
+  int r;
+  int N = countNodes ();
+  int M = countVoltageSources ();
+  naentry<nr_type_t> * na;
+  // store all nodes except reference node
+  for (r = 0; r < N; r++) {
+    struct nodelist_t * n = nlist->getNode (r);
+    if ((na = solution.find (n->name, 0)) != NULL)
+      x->set (r, na->value);
+  }
+  // store all branch currents of voltage sources
+  for (r = 0; r < M; r++) {
+    circuit * vs = findVoltageSource (r);
+    int vn = r - vs->getVoltageSource () + 1;
+    if ((na = solution.find (vs->getName (), vn)) != NULL)
+      x->set (r + N, na->value);
+  }
+}
+
 /* This function saves the results of a single solve() functionality
    into the output dataset. */
 template <class nr_type_t>
@@ -1098,7 +1168,8 @@ void nasolver<nr_type_t>::saveResults (char * volts, char * amps, int saveOPs,
       if (c->getSubcircuit () && !(saveOPs & SAVE_ALL)) continue;
       c->saveOperatingPoints ();
       n = createOP (c->getName (), volts);
-      saveVariable (n, c->getOperatingPoint ("V"), f);
+      saveVariable (n, rect (c->getOperatingPoint ("Vr"),
+			     c->getOperatingPoint ("Vi")), f);
       free (n);
     }
   }
