@@ -1,7 +1,7 @@
 /*
  * netdefs.h - netlist definitions for arbitrary netlists
  *
- * Copyright (C) 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,15 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: netdefs.h,v 1.7 2005/10/31 16:15:31 ela Exp $
+ * $Id: netdefs.h,v 1.11 2007/02/25 16:57:35 ela Exp $
  *
  */
 
 #ifndef __NETDEFS_H__
 #define __NETDEFS_H__
 
+class environment;
+
 /* Representation of a node list. */
 struct node_t {
   char * node;
@@ -60,6 +62,8 @@ struct definition_t {
   struct pair_t * pairs;
   struct definition_t * next;
   struct definition_t * sub;
+  environment * env;
+  void * eqns;
   int duplicate;
   int action;
   int substrate;
@@ -92,13 +96,13 @@ struct property_t {
 
 // Structure defining an available component type.
 struct define_t {
-  char * type;                    // component name
-  int nodes;                      // number of nodes
-  int action;                     // is that an action?
-  int substrate;                  // is that a substrate?
-  int nonlinear;                  // is the component linear?
-  struct property_t required[64]; // required properties
-  struct property_t optional[64]; // optional proberties
+  char * type;                     // component name
+  int nodes;                       // number of nodes
+  int action;                      // is that an action?
+  int substrate;                   // is that a substrate?
+  int nonlinear;                   // is the component linear?
+  struct property_t required[128]; // required properties
+  struct property_t optional[64];  // optional proberties
 };
 
 // Maximum number of S-parameter ports.
