@@ -1,7 +1,7 @@
 /*
  * qucsdefs.h - netlist definitions for the Qucs netlists
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: qucsdefs.h,v 1.49 2006/05/23 09:48:32 raimi Exp $
+ * $Id: qucsdefs.h,v 1.71 2007/05/17 09:28:18 ela Exp $
  *
  */
 
@@ -40,13 +40,15 @@ struct define_t qucs_definition_available[] =
   /* inductor */
   { "L", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { { "L", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_NO_RANGE }, PROP_NO_PROP },
-    { PROP_NO_PROP }
+    { { "I", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      PROP_NO_PROP }
   },
   /* capacitor */
   { "C", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { { "C", PROP_REAL, { 1e-12, PROP_NO_STR }, PROP_NO_RANGE },
       PROP_NO_PROP },
-    { PROP_NO_PROP }
+    { { "V", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      PROP_NO_PROP }
   },
   /* two mutual inductors */
   { "MUT", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
@@ -67,6 +69,13 @@ struct define_t qucs_definition_available[] =
       PROP_NO_PROP },
     { PROP_NO_PROP }
   },
+  /* multiple mutual inductors */
+  { "MUTX", PROP_NODES, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "L", PROP_LIST, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "k", PROP_LIST, { 0.9, PROP_NO_STR }, { '[', -1, +1, ']' } },
+      PROP_NO_PROP },
+    { PROP_NO_PROP }
+  },
   /* voltage controlled current source */
   { "VCCS", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { { "G", PROP_REAL, { 1, PROP_NO_STR }, PROP_NO_RANGE }, PROP_NO_PROP },
@@ -161,18 +170,40 @@ struct define_t qucs_definition_available[] =
   },
   /* AC voltage source */
   { "Vac", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
-    { { "U", PROP_REAL, { 1, PROP_NO_STR }, PROP_NO_RANGE }, 
-      { "f", PROP_REAL, { 1e9, PROP_NO_STR }, PROP_POS_RANGE }, PROP_NO_PROP },
+    { { "U", PROP_REAL, { 1, PROP_NO_STR }, PROP_NO_RANGE }, PROP_NO_PROP },
     { { "Phase", PROP_REAL, { 0, PROP_NO_STR }, { '[', -360, 360, ']' } },
       { "Theta", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
+      { "f", PROP_REAL, { 1e9, PROP_NO_STR }, PROP_POS_RANGE },
       PROP_NO_PROP }
   },
   /* AC current source */
   { "Iac", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
-    { { "I", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_NO_RANGE }, 
-      { "f", PROP_REAL, { 1e9, PROP_NO_STR }, PROP_POS_RANGE }, PROP_NO_PROP },
+    { { "I", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_NO_RANGE }, PROP_NO_PROP },
     { { "Phase", PROP_REAL, { 0, PROP_NO_STR }, { '[', -360, 360, ']' } },
       { "Theta", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
+      { "f", PROP_REAL, { 1e9, PROP_NO_STR }, PROP_POS_RANGE },
+      PROP_NO_PROP }
+  },
+  /* exponential current source */
+  { "Iexp", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "I1", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "I2", PROP_REAL, { 1, PROP_NO_STR }, PROP_NO_RANGE },
+      { "T1", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
+      { "T2", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
+      PROP_NO_PROP },
+    { { "Tr", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Tf", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      PROP_NO_PROP }
+  },
+  /* exponential voltage source */
+  { "Vexp", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "U1", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "U2", PROP_REAL, { 1, PROP_NO_STR }, PROP_NO_RANGE },
+      { "T1", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
+      { "T2", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
+      PROP_NO_PROP },
+    { { "Tr", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Tf", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
       PROP_NO_PROP }
   },
   /* AM modulated AC voltage source */
@@ -221,7 +252,10 @@ struct define_t qucs_definition_available[] =
       { "TH", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
       { "TL", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
       PROP_NO_PROP },
-    { PROP_NO_PROP }
+    { { "Tr", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Tf", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Td", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      PROP_NO_PROP }
   },
   /* rectangular pulse current source */
   { "Irect", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
@@ -229,7 +263,10 @@ struct define_t qucs_definition_available[] =
       { "TH", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
       { "TL", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
       PROP_NO_PROP },
-    { PROP_NO_PROP }
+    { { "Tr", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Tf", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Td", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
+      PROP_NO_PROP }
   },
   /* phase shifter */
   { "PShift", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
@@ -269,7 +306,16 @@ struct define_t qucs_definition_available[] =
     { { "Z", PROP_REAL, { 50, PROP_NO_STR }, PROP_POS_RANGE }, 
       { "L", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_NO_RANGE },
       PROP_NO_PROP },
-    { { "Alpha", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGEX },
+    { { "Alpha", PROP_REAL, { 1, PROP_NO_STR }, PROP_POS_RANGEX },
+      { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
+      PROP_NO_PROP }
+  },
+  /* ideal 4-terminal transmission line */
+  { "TLIN4P", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "Z", PROP_REAL, { 50, PROP_NO_STR }, PROP_POS_RANGE }, 
+      { "L", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_NO_RANGE },
+      PROP_NO_PROP },
+    { { "Alpha", PROP_REAL, { 1, PROP_NO_STR }, PROP_POS_RANGEX },
       { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
       PROP_NO_PROP }
   },
@@ -286,6 +332,20 @@ struct define_t qucs_definition_available[] =
     { { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
       PROP_NO_PROP }
   },
+  /* twisted pair line */
+  { "TWIST", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "d", PROP_REAL, { 0.5e-3, PROP_NO_STR }, PROP_POS_RANGEX }, 
+      { "D", PROP_REAL, { 0.8e-3, PROP_NO_STR }, PROP_POS_RANGEX },
+      { "L", PROP_REAL, { 1500e-3, PROP_NO_STR }, PROP_NO_RANGE },
+      { "T", PROP_REAL, { 100, PROP_NO_STR }, PROP_POS_RANGE },
+      { "er", PROP_REAL, { 4, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "mur", PROP_REAL, { 1, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "tand", PROP_REAL, { 4e-4, PROP_NO_STR }, PROP_POS_RANGE },
+      { "rho", PROP_REAL, { 0.022e-6, PROP_NO_STR }, PROP_POS_RANGE },
+      PROP_NO_PROP },
+    { { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
+      PROP_NO_PROP }
+  },
   /* ideal coupler */
   { "Coupler", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { { "k", PROP_REAL, { M_SQRT1_2, PROP_NO_STR }, PROP_NO_RANGE },
@@ -384,16 +444,17 @@ struct define_t qucs_definition_available[] =
   /* diode */
   { "Diode", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
     { { "Is", PROP_REAL, { 1e-15, PROP_NO_STR }, PROP_POS_RANGE },
-      { "N", PROP_REAL, { 1, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "N", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "M", PROP_REAL, { 0.5, PROP_NO_STR }, { '[', 0, 2, ']' } },
       { "Cj0", PROP_REAL, { 10e-15, PROP_NO_STR }, PROP_POS_RANGE },
       { "Vj", PROP_REAL, { 0.7, PROP_NO_STR }, { ']', 0, 10, ']' } },
       PROP_NO_PROP },
     { { "Rs", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Isr", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
-      { "Nr", PROP_REAL, { 2, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "Nr", PROP_REAL, { 2, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "Bv", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Ibv", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Ikf", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Tt", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Fc", PROP_REAL, { 0.5, PROP_NO_STR }, { '[', 0, 1, '[' } },
       { "Cp", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
@@ -444,16 +505,16 @@ struct define_t qucs_definition_available[] =
   /* sgp bjt */
   { "BJT", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
     { { "Is", PROP_REAL, { 1e-16, PROP_NO_STR }, PROP_POS_RANGE },
-      { "Nf", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.9, 100, ']' } },
-      { "Nr", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.9, 100, ']' } },
+      { "Nf", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
+      { "Nr", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "Ikf", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Ikr", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Vaf", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Var", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Ise", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
-      { "Ne", PROP_REAL, { 1.5, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "Ne", PROP_REAL, { 1.5, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "Isc", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
-      { "Nc", PROP_REAL, { 2, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "Nc", PROP_REAL, { 2, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "Bf", PROP_REAL, { 100, PROP_NO_STR }, PROP_POS_RANGEX },
       { "Br", PROP_REAL, { 1, PROP_NO_STR }, PROP_POS_RANGEX },
       { "Rbm", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
@@ -497,7 +558,7 @@ struct define_t qucs_definition_available[] =
   /* fet: level 1 */
   { "MOSFET", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
     { { "Is", PROP_REAL, { 1e-14, PROP_NO_STR }, PROP_POS_RANGE },
-      { "N", PROP_REAL, { 1, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "N", PROP_REAL, { 1, PROP_NO_STR }, { '[', 0.1, 100, ']' } },
       { "Vt0", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE },
       { "Lambda", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "Kp", PROP_REAL, { 2e-5, PROP_NO_STR }, PROP_POS_RANGE },
@@ -544,6 +605,223 @@ struct define_t qucs_definition_available[] =
       { "capModel", PROP_INT, { 2, PROP_NO_STR }, { '[', 1 , 2, ']' } },
       PROP_NO_PROP }
   },
+  /* equation defined device */
+  { "EDD", PROP_NODES, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
+    { { "I1", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q1", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      PROP_NO_PROP },
+    { { "I2", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q2", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I3", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q3", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I4", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q4", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I5", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q5", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I6", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q6", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I7", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q7", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "I8", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      { "Q8", PROP_REAL, { 0, PROP_NO_STR }, PROP_NO_RANGE }, 
+      PROP_NO_PROP }
+  },
+
+
+  /* hicumL2p1 verilog device */
+  { "hicumL2p1", 5, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
+    {
+      { "c10", PROP_REAL, { 1.516E-31, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "qp0", PROP_REAL, { 5.939E-15, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "ich", PROP_REAL, { 1.0E11, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "hfe", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "hfc", PROP_REAL, { 0.03999, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "hjei", PROP_REAL, { 0.435, PROP_NO_STR }, { '[', 0, 100, ']' } },
+      { "hjci", PROP_REAL, { 0.09477, PROP_NO_STR }, { '[', 0, 100, ']' } },
+      { "ibeis", PROP_REAL, { 3.47E-20, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "mbei", PROP_REAL, { 1.025, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ireis", PROP_REAL, { 390E-12, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "mrei", PROP_REAL, { 3, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ibeps", PROP_REAL, { 4.18321E-21, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "mbep", PROP_REAL, { 1.045, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ireps", PROP_REAL, { 1.02846E-14, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "mrep", PROP_REAL, { 3, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "mcf", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ibcis", PROP_REAL, { 3.02613E-18, PROP_NO_STR }, { '[', 0, 1.0, ']' } },
+      { "mbci", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ibcxs", PROP_REAL, { 4.576E-29, PROP_NO_STR }, { '[', 0, 1.0, ']' } },
+      { "mbcx", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "ibets", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "abet", PROP_REAL, { 36.74, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "favl", PROP_REAL, { 14.97, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "qavl", PROP_REAL, { 7.2407E-14, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "alfav", PROP_REAL, { 0.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "alqav", PROP_REAL, { 0.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "rbi0", PROP_REAL, { 7.9, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "rbx", PROP_REAL, { 13.15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "fgeo", PROP_REAL, { 0.724, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "fdqr0", PROP_REAL, { 200, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "fcrbi", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "fqi", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "re", PROP_REAL, { 9.77, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "rcx", PROP_REAL, { 10, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "itss", PROP_REAL, { 2.81242E-19, PROP_NO_STR }, { '[', 0, 1.0, ']' } },
+      { "msf", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "iscs", PROP_REAL, { 7.6376E-17, PROP_NO_STR }, { '[', 0, 1.0, ']' } },
+      { "msc", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "tsf", PROP_REAL, { 1.733E-8, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "rsu", PROP_REAL, { 800, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "csu", PROP_REAL, { 1.778E-14, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "cjei0", PROP_REAL, { 5.24382E-14, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vdei", PROP_REAL, { 0.9956, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "zei", PROP_REAL, { 0.4, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "aljei", PROP_REAL, { 2.5, PROP_NO_STR }, { '[', 1, 0, '.' } },
+      { "cjep0", PROP_REAL, { 0, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vdep", PROP_REAL, { 1, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "zep", PROP_REAL, { 0.01, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "aljep", PROP_REAL, { 2.5, PROP_NO_STR }, { '[', 1, 0, '.' } },
+      { "cjci0", PROP_REAL, { 4.46887E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vdci", PROP_REAL, { 0.7, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "zci", PROP_REAL, { 0.38, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "vptci", PROP_REAL, { 100, PROP_NO_STR }, { ']', 0, 100, ']' } },
+      { "cjcx0", PROP_REAL, { 1.55709E-14, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vdcx", PROP_REAL, { 0.733, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "zcx", PROP_REAL, { 0.34, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "vptcx", PROP_REAL, { 100, PROP_NO_STR }, { ']', 0, 100, ']' } },
+      { "fbc", PROP_REAL, { 0.3487, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "cjs0", PROP_REAL, { 17.68E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vds", PROP_REAL, { 0.621625, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "zs", PROP_REAL, { 0.122136, PROP_NO_STR }, { ']', 0, 1, ']' } },
+      { "vpts", PROP_REAL, { 1000, PROP_NO_STR }, { ']', 0, 1000, ']' } },
+      { "t0", PROP_REAL, { 1.28E-12, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "dt0h", PROP_REAL, { 260E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "tbvl", PROP_REAL, { 2.0E-13, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "tef0", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "gtfe", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "thcs", PROP_REAL, { 46E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "alhc", PROP_REAL, { 0.08913, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "fthc", PROP_REAL, { 0.8778, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "rci0", PROP_REAL, { 50.4277, PROP_NO_STR }, { ']', 0, 0, '.' } },
+      { "vlim", PROP_REAL, { 0.9, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "vces", PROP_REAL, { 0.01, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "vpt", PROP_REAL, { 10, PROP_NO_STR }, { '[', 0, 100, ']' } },
+      { "tr", PROP_REAL, { 1.0E-11, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "ceox", PROP_REAL, { 1.71992E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "ccox", PROP_REAL, { 4.9E-15, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "alqf", PROP_REAL, { 0.1288, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "alit", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0, 1, ']' } },
+      { "kf", PROP_REAL, { 2.83667E-9, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "af", PROP_REAL, { 2.0, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "krbi", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0, 0, '.' } },
+      { "latb", PROP_REAL, { 10.479, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "latl", PROP_REAL, { 0.300012, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "vgb", PROP_REAL, { 1.112, PROP_NO_STR }, { ']', 0, 10, ']' } },
+      { "alt0", PROP_REAL, { 0.0017580, PROP_NO_STR }, PROP_NO_RANGE },
+      { "kt0", PROP_REAL, { 4.07E-6, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetaci", PROP_REAL, { 0.7, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetacx", PROP_REAL, { 1.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "alvs", PROP_REAL, { 0.001, PROP_NO_STR }, PROP_NO_RANGE },
+      { "alces", PROP_REAL, { 0.000125, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetarbi", PROP_REAL, { 0.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetarbx", PROP_REAL, { 0.2, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetarcx", PROP_REAL, { 0.21, PROP_NO_STR }, PROP_NO_RANGE },
+      { "zetare", PROP_REAL, { 0.7, PROP_NO_STR }, PROP_NO_RANGE },
+      { "alb", PROP_REAL, { 0.007, PROP_NO_STR }, PROP_NO_RANGE },
+      { "rth", PROP_REAL, { 1293.95, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "cth", PROP_REAL, { 7.22203E-11, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "tnom", PROP_REAL, { 27.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "dt", PROP_REAL, { 0.0, PROP_NO_STR }, PROP_NO_RANGE },
+      { "Type", PROP_STR, { PROP_NO_VAL, "npn" }, PROP_NO_RANGE },
+      { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
+      PROP_NO_PROP },
+    { PROP_NO_PROP }
+  },
+  /* HBT_X verilog device */
+  { "HBT_X", 4, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_NONLINEAR,
+    {
+      { "Mode", PROP_INT, { 1, PROP_NO_STR }, { '[', 0, 4, ']' } },
+      { "Noise", PROP_INT, { 1, PROP_NO_STR }, { '[', 0, 4, ']' } },
+      { "Debug", PROP_INT, { 0, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "DebugPlus", PROP_INT, { 0, PROP_NO_STR }, { '[', 0, 0, '.' } },
+      { "Temp", PROP_REAL, { 25.0, PROP_NO_STR }, { '[', (-273.15), 0, '.' } },
+      { "Rth", PROP_REAL, { 0.1, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cth", PROP_REAL, { (700*1.0e-9), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "N", PROP_INT, { 1, PROP_NO_STR }, { ']', 0, 0, '.' } },
+      { "L", PROP_REAL, { (30*1.0e-6), PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "W", PROP_REAL, { (3*1.0e-6), PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Jsf", PROP_REAL, { 20e-24, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "nf", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Vg", PROP_REAL, { 1.3, PROP_NO_STR }, { '[', (-2.0), 0, '.' } },
+      { "Jse", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "ne", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Rbxx", PROP_REAL, { 1e6, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Vgb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Jsee", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "nee", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Rbbxx", PROP_REAL, { 1e6, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Vgbb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Jsr", PROP_REAL, { 20e-18, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "nr", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Vgr", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "XCjc", PROP_REAL, { 0.5, PROP_NO_STR }, { '[', 0.0, 1.0, '[' } },
+      { "Jsc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "nc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Rcxx", PROP_REAL, { 1e6, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Vgc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Bf", PROP_REAL, { 100.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "kBeta", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Br", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "VAF", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "VAR", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "IKF", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "IKR", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Mc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "BVceo", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "kc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "BVebo", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Tr", PROP_REAL, { (1*1.0e-15), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Trx", PROP_REAL, { (1*1.0e-15), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Tf", PROP_REAL, { (1*1.0e-12), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Tft", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Thcs", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Ahc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cje", PROP_REAL, { (1*1.0e-15), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "mje", PROP_REAL, { 0.5, PROP_NO_STR }, { '[', 0.0, 1, '[' } },
+      { "Vje", PROP_REAL, { 1.3, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cjc", PROP_REAL, { (1*1.0e-15), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "mjc", PROP_REAL, { 0.5, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Vjc", PROP_REAL, { 1.3, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "kjc", PROP_REAL, { 1.0, PROP_NO_STR }, { '.', 0, 0, '.' } },
+      { "Cmin", PROP_REAL, { (0.1*1.0e-15), PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "J0", PROP_REAL, { 1e-3, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "XJ0", PROP_REAL, { 1.0, PROP_NO_STR }, { '[', 0.0, 1.0, ']' } },
+      { "Rci0", PROP_REAL, { 1e-3, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Jk", PROP_REAL, { 4e-4, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "RJk", PROP_REAL, { 1e-3, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Vces", PROP_REAL, { 1e-3, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Rc", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Re", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Rb", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Rb2", PROP_REAL, { 1.0, PROP_NO_STR }, { ']', 0.0, 0, '.' } },
+      { "Lc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Le", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Lb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cq", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cpb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Cpc", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Kfb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Afb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Ffeb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Kb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Ab", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Fb", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Kfe", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Afe", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Ffee", PROP_REAL, { 0.0, PROP_NO_STR }, { '[', 0.0, 0, '.' } },
+      { "Tnom", PROP_REAL, { 20.0, PROP_NO_STR }, { '[', (-273.15), 0, '.' } },
+      PROP_NO_PROP },
+    { PROP_NO_PROP }
+  },
 
   /* microstrip substrate */
   { "SUBST", 0, PROP_COMPONENT, PROP_SUBSTRATE, PROP_LINEAR,
@@ -647,6 +925,9 @@ struct define_t qucs_definition_available[] =
       { "W3", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
       { "W4", PROP_REAL, { 2e-3, PROP_NO_STR }, PROP_POS_RANGE },
       { "Subst", PROP_STR, { PROP_NO_VAL, "Subst1" }, PROP_NO_RANGE },
+      { "MSDispModel", PROP_STR, { PROP_NO_VAL, "Kirschning" },
+	PROP_NO_RANGE },
+      { "MSModel", PROP_STR, { PROP_NO_VAL, "Hammerstad" }, PROP_NO_RANGE },
       PROP_NO_PROP },
     { PROP_NO_PROP }
   },
@@ -658,6 +939,19 @@ struct define_t qucs_definition_available[] =
     { { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
       PROP_NO_PROP }
   },
+  /* bond wire */
+  { "BOND", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
+    { { "D", PROP_REAL, { 25e-6, PROP_NO_STR }, PROP_POS_RANGE },
+      { "L", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
+      { "H", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
+      { "mur", PROP_REAL, { 1, PROP_NO_STR }, { '[', 1, 100, ']' } },
+      { "rho", PROP_REAL, { 0.022e-6, PROP_NO_STR }, PROP_POS_RANGE },
+      { "Model", PROP_STR, { PROP_NO_VAL, "FREESPACE" }, PROP_NO_RANGE },
+      { "Subst", PROP_STR, { PROP_NO_VAL, "Subst1" }, PROP_NO_RANGE },
+      PROP_NO_PROP },
+    { { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
+      PROP_NO_PROP }
+  },
   /* coplanar line */
   { "CLIN", 2, PROP_COMPONENT, PROP_NO_SUBSTRATE, PROP_LINEAR,
     { { "W", PROP_REAL, { 1e-3, PROP_NO_STR }, PROP_POS_RANGE },
@@ -816,6 +1110,7 @@ struct define_t qucs_definition_available[] =
       { "Order", PROP_INT, { 2, PROP_NO_STR }, { '[', 1, 6, ']' } },
       { "InitialStep", PROP_REAL, { 1e-9, PROP_NO_STR }, PROP_POS_RANGE },
       { "MinStep", PROP_REAL, { 1e-16, PROP_NO_STR }, PROP_POS_RANGE },
+      { "MaxStep", PROP_REAL, { 0, PROP_NO_STR }, PROP_POS_RANGE },
       { "MaxIter", PROP_INT, { 150, PROP_NO_STR }, { '[', 2, 10000, ']' } },
       { "abstol", PROP_REAL, { 1e-12, PROP_NO_STR }, PROP_RNG_X01I },
       { "vntol", PROP_REAL, { 1e-6, PROP_NO_STR }, PROP_RNG_X01I },
@@ -826,6 +1121,7 @@ struct define_t qucs_definition_available[] =
       { "Temp", PROP_REAL, { 26.85, PROP_NO_STR }, PROP_MIN_VAL (K) },
       { "Solver", PROP_STR, { PROP_NO_VAL, "CroutLU" }, PROP_NO_RANGE },
       { "relaxTSR", PROP_STR, { PROP_NO_VAL, "no" }, PROP_NO_RANGE },
+      { "initialDC", PROP_STR, { PROP_NO_VAL, "yes" }, PROP_NO_RANGE },
       PROP_NO_PROP }
   },
   /* harmonic balance analysis */
