@@ -1,7 +1,7 @@
 /*
  * trsolver.cpp - transient solver class implementation
  *
- * Copyright (C) 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: trsolver.cpp,v 1.46 2006/04/28 07:08:26 raimi Exp $
+ * $Id: trsolver.cpp,v 1.51 2007/03/13 19:39:22 ela Exp $
  *
  */
 
@@ -62,6 +62,7 @@ trsolver::trsolver ()
   for (int i = 0; i < 8; i++) solution[i] = NULL;
   tHistory = NULL;
   relaxTSR = false;
+  initialDC = true;
 }
 
 // Constructor creates a named instance of the trsolver class.
@@ -73,6 +74,7 @@ trsolver::trsolver (char * n)
   for (int i = 0; i < 8; i++) solution[i] = NULL;
   tHistory = NULL;
   relaxTSR = false;
+  initialDC = true;
 }
 
 // Destructor deletes the trsolver class object.
@@ -90,6 +92,7 @@ trsolver::trsolver (trsolver & o)
   for (int i = 0; i < 8; i++) solution[i] = NULL;
   tHistory = o.tHistory ? new history (*o.tHistory) : NULL;
   relaxTSR = o.relaxTSR;
+  initialDC = o.initialDC;
 }
 
 // This function creates the time sweep if necessary.
@@ -101,23 +104,10 @@ void trsolver::initSteps (void) {
 // Macro for the n-th state of the solution vector history.
 #define SOL(state) (solution[(int) getState (sState, (state))])
 
-/* This is the transient netlist solver.  It prepares the circuit list
-   for each requested time and solves it then. */
-void trsolver::solve (void) {
-  nr_double_t time, saveCurrent;
-  int error = 0, convError = 0;
-  char * solver = getPropertyString ("Solver");
-  relaxTSR = !strcmp (getPropertyString ("relaxTSR"), "yes") ? true : false;
 
-  runs++;
-  saveCurrent = current = 0;
-  stepDelta = -1;
-  converged = 0;
-  fixpoint = 0;
-  statRejected = statSteps = statIterations = statConvergence = 0;
-
-  // Create time sweep if necessary.
-  initSteps ();
+// Performs the initial DC analysis.
+int trsolver::dcAnalysis (void) {
+  int error = 0;
 
   // First calculate a initial state using the non-linear DC analysis.
   setDescription ("initial DC");
@@ -126,16 +116,6 @@ void trsolver::solve (void) {
   solve_pre ();
   applyNodeset ();
 
-  // Choose a solver.
-  if (!strcmp (solver, "CroutLU"))
-    eqnAlgo = ALGO_LU_DECOMPOSITION;
-  else if (!strcmp (solver, "DoolittleLU"))
-    eqnAlgo = ALGO_LU_DECOMPOSITION_DOOLITTLE;
-  else if (!strcmp (solver, "HouseholderQR"))
-    eqnAlgo = ALGO_QR_DECOMPOSITION;
-  else if (!strcmp (solver, "HouseholderLQ"))
-    eqnAlgo = ALGO_QR_DECOMPOSITION_LS;
-
   // Run the DC solver once.
   try_running () {
     error = solve_nonlinear ();
@@ -147,6 +127,8 @@ void trsolver::solve (void) {
     convHelper = CONV_LineSearch;
     logprint (LOG_ERROR, "WARNING: %s: %s analysis failed, using line search "
 	      "fallback\n", getName (), getDescription ());
+    applyNodeset ();
+    restart ();
     error = solve_nonlinear ();
     break;
   default:
@@ -155,13 +137,57 @@ void trsolver::solve (void) {
     error++;
     break;
   }
+
+  // Save the DC solution.
+  storeSolution ();
+
+  // Cleanup nodal analysis solver.
   solve_post ();
 
   // Really failed to find initial DC solution?
   if (error) {
     logprint (LOG_ERROR, "ERROR: %s: %s analysis failed\n",
 	      getName (), getDescription ());
-    return;
+  }
+  return error;
+}
+
+/* This is the transient netlist solver.  It prepares the circuit list
+   for each requested time and solves it then. */
+void trsolver::solve (void) {
+  nr_double_t time, saveCurrent;
+  int error = 0, convError = 0;
+  char * solver = getPropertyString ("Solver");
+  relaxTSR = !strcmp (getPropertyString ("relaxTSR"), "yes") ? true : false;
+  initialDC = !strcmp (getPropertyString ("initialDC"), "yes") ? true : false;
+
+  runs++;
+  saveCurrent = current = 0;
+  stepDelta = -1;
+  converged = 0;
+  fixpoint = 0;
+  statRejected = statSteps = statIterations = statConvergence = 0;
+
+  // Create time sweep if necessary.
+  initSteps ();
+
+  // Choose a solver.
+  if (!strcmp (solver, "CroutLU"))
+    eqnAlgo = ALGO_LU_DECOMPOSITION;
+  else if (!strcmp (solver, "DoolittleLU"))
+    eqnAlgo = ALGO_LU_DECOMPOSITION_DOOLITTLE;
+  else if (!strcmp (solver, "HouseholderQR"))
+    eqnAlgo = ALGO_QR_DECOMPOSITION;
+  else if (!strcmp (solver, "HouseholderLQ"))
+    eqnAlgo = ALGO_QR_DECOMPOSITION_LS;
+  else if (!strcmp (solver, "GolubSVD"))
+    eqnAlgo = ALGO_SV_DECOMPOSITION;
+
+  // Perform initial DC analysis.
+  if (initialDC) {
+    error = dcAnalysis ();
+    if (error)
+      return;
   }
 
   // Initialize transient analysis.
@@ -171,6 +197,9 @@ void trsolver::solve (void) {
   solve_pre ();
   swp->reset ();
 
+  // Recall the DC solution.
+  recallSolution ();
+
   // Apply the nodesets and adjust previous solutions.
   applyNodeset (false);
   fillSolution (x);
@@ -179,6 +208,7 @@ void trsolver::solve (void) {
   setMode (MODE_INIT);
 
   int running = 0;
+  rejected = 0;
   delta /= 10;
   fillState (dState, delta);
   adjustOrder (1);
@@ -202,11 +232,16 @@ void trsolver::solve (void) {
       }
 #endif
       updateCoefficients (delta);
-      rejected = 0;
 
       // Run predictor.
       error += predictor ();
 
+      // restart Newton iteration
+      if (rejected) {
+	restart ();      // restart non-linear devices
+	rejected = 0;
+      }
+
       // Run corrector process.
       try_running () {
 	error += corrector ();
@@ -597,6 +632,15 @@ void trsolver::calcTR (trsolver * self) {
   }
 }
 
+/* Goes through the list of non-linear circuit objects and runs its
+   restartDC() function. */
+void trsolver::restart (void) {
+  circuit * root = subnet->getRoot ();
+  for (circuit * c = root; c != NULL; c = (circuit *) c->getNext ()) {
+    if (c->isNonLinear ()) c->restartDC ();
+  }
+}
+
 /* Goes through the list of circuit objects and runs its initDC()
    function. */
 void trsolver::initDC (void) {
@@ -646,7 +690,7 @@ void trsolver::initTR (void) {
 
   // initialize history of solution vectors (solutions)
   for (int i = 0; i < 8; i++) {
-    solution[i] = new tvector<nr_double_t> (*x);
+    solution[i] = new tvector<nr_double_t>;
     setState (sState, (nr_double_t) i, i);
   }
 
