@@ -1,7 +1,7 @@
 /*
  * ucs.cpp - main program implementation
  *
- * Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: ucs.cpp,v 1.24 2006/01/04 10:40:32 raimi Exp $
+ * $Id: ucs.cpp,v 1.28 2007/02/27 12:05:28 ela Exp $
  *
  */
 
@@ -39,6 +39,7 @@
 #include "equation.h"
 #include "environment.h"
 #include "exceptionstack.h"
+#include "check_netlist.h"
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
@@ -54,6 +55,7 @@ int main (int argc, char ** argv) {
   circuit * gnd;
   dataset * out;
   environment * root;
+  int ret = 0;
 
   loginit ();
   precinit ();
@@ -63,7 +65,10 @@ int main (int argc, char ** argv) {
     if (!strcmp (argv[i], "-v") || !strcmp (argv[i], "--version")) {
       fprintf (stdout,
 	"Qucsator " PACKAGE_VERSION "\n"
-	"Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>\n"
+	"Copyright (C) 2003, 2004, 2005, 2006, 2007 "
+	"Stefan Jahn <stefan@lkcc.org>\n"
+        "Copyright (C) 2006 Helene Parruitte <parruit@enseirb.fr>\n"
+        "Copyright (C) 2006 Bastien Roucaries <roucaries.bastien@gmail.com>\n"
 	"\nThis is free software; see the source for copying "
 	"conditions.  There is NO\n"
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A "
@@ -134,8 +139,7 @@ int main (int argc, char ** argv) {
   out = subnet->runAnalysis ();
 
   // evaluate output dataset
-  equation_solver (out);
-  equation_destructor ();
+  ret |= root->equationSolver (out);
   out->setFile (outfile);
   out->print ();
 
@@ -145,5 +149,6 @@ int main (int argc, char ** argv) {
   delete in;
   delete out;
   delete root;
-  return 0;
+  netlist_destroy_env ();
+  return ret;
 }
