@@ -1,7 +1,7 @@
 /*
  * variable.cpp - generic variable class implementation
  *
- * Copyright (C) 2004 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2004, 2007 Stefan Jahn <stefan@lkcc.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: variable.cpp,v 1.5 2005/06/02 18:17:51 raimi Exp $
+ * $Id: variable.cpp,v 1.7 2007/06/15 21:13:30 ela Exp $
  *
  */
 
@@ -40,6 +40,7 @@
 // Constructor creates an unnamed instance of the variable class.
 variable::variable () {
   name = NULL;
+  text = NULL;
   next = NULL;
   type = VAR_UNKNOWN;
 }
@@ -47,6 +48,7 @@ variable::variable () {
 // This constructor creates a named instance of the variable class.
 variable::variable (char * n) {
   name = n ? strdup (n) : NULL;
+  text = NULL;
   next = NULL;
   type = VAR_UNKNOWN;
 }
@@ -54,7 +56,8 @@ variable::variable (char * n) {
 /* This copy constructor creates a instance of the variable class based
    on the given variable. */
 variable::variable (const variable & o) {
-  if (o.name != NULL) name = o.name ? strdup (o.name) : NULL;
+  name = o.name ? strdup (o.name) : NULL;
+  text = o.text ? strdup (o.text) : NULL;
   type = o.type;
   next = o.next;
   value = o.value;
@@ -63,6 +66,7 @@ variable::variable (const variable & o) {
 /// Destructor deletes an instance of the variable class.
 variable::~variable () {
   if (name) free (name);
+  if (text) free (text);
 }
 
 // Sets the name of the variable.
@@ -75,3 +79,45 @@ void variable::setName (char * n) {
 char * variable::getName (void) {
   return name;
 }
+
+// Creates textual representation of a variable.
+char * variable::toString (void) {
+  char * str = NULL;
+  char * val = NULL;
+  if (text) { free (text); text = NULL; }
+  switch (type) {
+  case VAR_UNKNOWN:
+    text = strdup ("variable");
+    break;
+  case VAR_CONSTANT:
+    str = value.c->toString ();
+    text = (char *) malloc (strlen (str) + 11);
+    sprintf (text, "constant: %s", str);
+    break;
+  case VAR_VALUE:
+    str = value.v->toString ();
+    text = (char *) malloc (strlen (str) + 8);
+    sprintf (text, "value: %s", str);
+    break;
+  case VAR_REFERENCE:
+    str = value.r->toString ();
+    val = value.r->getResult()->toString ();
+    text = (char *) malloc (strlen (str) + strlen (val) + 15);
+    sprintf (text, "reference: %s = %s", str, val);
+    break;
+  case VAR_SUBSTRATE:
+    str = value.s->getName ();
+    text = (char *) malloc (strlen (str) + 12);
+    sprintf (text, "substrate: %s", str);
+    break;
+  case VAR_ANALYSIS:
+    str = value.a->getName ();
+    text = (char *) malloc (strlen (str) + 11);
+    sprintf (text, "analysis: %s", str);
+    break;
+  default:
+    text = strdup ("?variable?");
+    break;
+  }
+  return text;
+}
