@@ -1,8 +1,8 @@
 /*
  * vector.cpp - vector class implementation
  *
- * Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
- * Copyright (C) 2006 Gunther Kraut <gn.kraut@t-online.de>
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2006, 2007 Gunther Kraut <gn.kraut@t-online.de>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: vector.cpp,v 1.28 2006/03/15 16:46:28 raimi Exp $
+ * $Id: vector.cpp,v 1.34 2007/05/03 19:01:05 ela Exp $
  *
  */
 
@@ -260,12 +260,59 @@ complex avg (vector v) {
   return result / v.getSize ();
 }
 
+vector signum (vector v) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (signum (v.get (i)), i);
+  return result;
+}
+
 vector sign (vector v) {
   vector result (v);
   for (int i = 0; i < v.getSize (); i++) result.set (sign (v.get (i)), i);
   return result;
 }
 
+vector xhypot (vector v, const complex z) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (xhypot (v.get(i), z), i);
+  return result;
+}
+
+vector xhypot (vector v, const nr_double_t d) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (xhypot (v.get(i), d), i);
+  return result;
+}
+
+vector xhypot (const complex z, vector v) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (xhypot (z, v.get (i)), i);
+  return result;
+}
+
+vector xhypot (const nr_double_t d, vector v) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (xhypot (d, v.get (i)), i);
+  return result;
+}
+
+vector xhypot (vector v1, vector v2) {
+  int j, i, n, len, len1 = v1.getSize (), len2 = v2.getSize ();
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    len = len1;
+  } else {
+    assert (len2 % len1 == 0);
+    len = len2;
+  }
+  vector res (len);
+  for (j = i = n = 0; n < len; n++) {
+    res (n) = xhypot (v1 (i), v2 (j));
+    if (++i >= len1) i = 0; if (++j >= len2) j = 0;
+  }
+  return res;
+}
+
 vector sinc (vector v) {
   vector result (v);
   for (int i = 0; i < v.getSize (); i++) result.set (sinc (v.get (i)), i);
@@ -327,6 +374,12 @@ vector exp (vector v) {
   return result;
 }
 
+vector limexp (vector v) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (limexp (v.get (i)), i);
+  return result;
+}
+
 vector ln (vector v) {
   vector result (v);
   for (int i = 0; i < v.getSize (); i++) result.set (ln (v.get (i)), i);
@@ -370,11 +423,20 @@ vector pow (const nr_double_t d, vector v) {
 }
 
 vector pow (vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  for (int i = 0; i < v1.getSize (); i++)
-    result.set (pow (v1.get (i), v2.get (i)), i);
-  return result;
+  int j, i, n, len, len1 = v1.getSize (), len2 = v2.getSize ();
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    len = len1;
+  } else {
+    assert (len2 % len1 == 0);
+    len = len2;
+  }
+  vector res (len);
+  for (j = i = n = 0; n < len; n++) {
+    res (n) = pow (v1 (i), v2 (j));
+    if (++i >= len1) i = 0; if (++j >= len2) j = 0;
+  }
+  return res;
 }
 
 vector sin (vector v) {
@@ -449,6 +511,12 @@ vector arcosh (vector v) {
   return result;
 }
 
+vector arsech (vector v) {
+  vector result (v);
+  for (int i = 0; i < v.getSize (); i++) result.set (arsech (v.get (i)), i);
+  return result;
+}
+
 vector tanh (vector v) {
   vector result (v);
   for (int i = 0; i < v.getSize (); i++) result.set (tanh (v.get (i)), i);
@@ -540,35 +608,42 @@ vector diff (vector var, vector dep, int n) {
 }
 
 vector vector::operator=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] = c;
+  for (int i = 0; i < size; i++) data[i] = c;
   return *this;
 }
 
 vector vector::operator=(const nr_double_t d) {
-  for (int i = 0; i < getSize (); i++) data[i] = d;
+  for (int i = 0; i < size; i++) data[i] = d;
   return *this;
 }
 
 vector vector::operator+=(vector v) {
-  if (checkSizes (*this, v))
-    for (int i = 0; i < getSize (); i++) data[i] += v.get (i);
+  int i, n, len = v.getSize ();
+  assert (size % len == 0);
+  for (i = n = 0; i < size; i++) { data[i] += v (n); if (++n >= len) n = 0; }
   return *this;
 }
 
 vector vector::operator+=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] += c;
+  for (int i = 0; i < size; i++) data[i] += c;
   return *this;
 }
 
 vector vector::operator+=(const nr_double_t d) {
-  for (int i = 0; i < getSize (); i++) data[i] += d;
+  for (int i = 0; i < size; i++) data[i] += d;
   return *this;
 }
 
 vector operator+(vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  result += v2;
+  int len1 = v1.getSize (), len2 = v2.getSize ();
+  vector result;
+  if (len1 >= len2) {
+    result  = v1;
+    result += v2;
+  } else {
+    result  = v2;
+    result += v1;
+  }
   return result;
 }
 
@@ -593,31 +668,38 @@ vector operator+(const nr_double_t d, vector v) {
 }
 
 vector vector::operator-() {
-  vector result (*this);
-  for (int i = 0; i < getSize (); i++) result.set (-get (i), i);
+  vector result (size);
+  for (int i = 0; i < size; i++) result (i) = -data[i];
   return result;
 }
 
 vector vector::operator-=(vector v) {
-  if (checkSizes (*this, v))
-    for (int i = 0; i < getSize (); i++) data[i] -= v.get (i);
+  int i, n, len = v.getSize ();
+  assert (size % len == 0);
+  for (i = n = 0; i < size; i++) { data[i] -= v (n); if (++n >= len) n = 0; }
   return *this;
 }
 
 vector vector::operator-=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] -= c;
+  for (int i = 0; i < size; i++) data[i] -= c;
   return *this;
 }
 
 vector vector::operator-=(const nr_double_t d) {
-  for (int i = 0; i < getSize (); i++) data[i] -= d;
+  for (int i = 0; i < size; i++) data[i] -= d;
   return *this;
 }
 
 vector operator-(vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  result -= v2;
+  int len1 = v1.getSize (), len2 = v2.getSize ();
+  vector result;
+  if (len1 >= len2) {
+    result  = v1;
+    result -= v2;
+  } else {
+    result  = -v2;
+    result += v1;
+  }
   return result;
 }
 
@@ -634,39 +716,44 @@ vector operator-(vector v, const nr_double_t d) {
 }
 
 vector operator-(const complex c, vector v) {
-  vector result (v);
-  result *= -1.0;
+  vector result (-v);
   result += c;
   return result;
 }
 
 vector operator-(const nr_double_t d, vector v) {
-  vector result (v);
-  result *= -1.0;
+  vector result (-v);
   result += d;
   return result;
 }
 
 vector vector::operator*=(vector v) {
-  if (checkSizes (*this, v))
-    for (int i = 0; i < getSize (); i++) data[i] *= v.get (i);
+  int i, n, len = v.getSize ();
+  assert (size % len == 0);
+  for (i = n = 0; i < size; i++) { data[i] *= v (n); if (++n >= len) n = 0; }
   return *this;
 }
 
 vector vector::operator*=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] *= c;
+  for (int i = 0; i < size; i++) data[i] *= c;
   return *this;
 }
 
 vector vector::operator*=(const nr_double_t d) {
-  for (int i = 0; i < getSize (); i++) data[i] *= d;
+  for (int i = 0; i < size; i++) data[i] *= d;
   return *this;
 }
 
 vector operator*(vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  result *= v2;
+  int len1 = v1.getSize (), len2 = v2.getSize ();
+  vector result;
+  if (len1 >= len2) {
+    result  = v1;
+    result *= v2;
+  } else {
+    result  = v2;
+    result *= v1;
+  }
   return result;
 }
 
@@ -691,25 +778,34 @@ vector operator*(const nr_double_t d, vector v) {
 }
 
 vector vector::operator/=(vector v) {
-  if (checkSizes (*this, v))
-    for (int i = 0; i < getSize (); i++) data[i] /= v.get (i);
+  int i, n, len = v.getSize ();
+  assert (size % len == 0);
+  for (i = n = 0; i < size; i++) { data[i] /= v (n); if (++n >= len) n = 0; }
   return *this;
 }
 
 vector vector::operator/=(const complex c) {
-  for (int i = 0; i < getSize (); i++) data[i] /= c;
+  for (int i = 0; i < size; i++) data[i] /= c;
   return *this;
 }
 
 vector vector::operator/=(const nr_double_t d) {
-  for (int i = 0; i < getSize (); i++) data[i] /= d;
+  for (int i = 0; i < size; i++) data[i] /= d;
   return *this;
 }
 
 vector operator/(vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  result /= v2;
+  int len1 = v1.getSize (), len2 = v2.getSize ();
+  vector result;
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    result  = v1;
+    result /= v2;
+  } else {
+    assert (len2 % len1 == 0);
+    result  = 1 / v2;
+    result *= v1;
+  }
   return result;
 }
 
@@ -740,35 +836,48 @@ vector operator/(const nr_double_t d, vector v) {
 }
 
 vector operator%(vector v, const complex z) {
-  vector result (v);
-  for (int i = 0; i < v.getSize (); i++) result.set (v.get (i) % z, i);
+  int len = v.getSize ();
+  vector result (len);
+  for (int i = 0; i < len; i++) result (i) = v (i) % z;
   return result;
 }
 
 vector operator%(vector v, const nr_double_t d) {
-  vector result (v);
-  for (int i = 0; i < v.getSize (); i++) result.set (v.get (i) % d, i);
+  int len = v.getSize ();
+  vector result (len);
+  for (int i = 0; i < len; i++) result (i) = v (i) % d;
   return result;
 }
 
 vector operator%(const complex z, vector v) {
-  vector result (v);
-  for (int i = 0; i < v.getSize (); i++) result.set (z % v.get (i), i);
+  int len = v.getSize ();
+  vector result (len);
+  for (int i = 0; i < len; i++) result (i) = z % v (i);
   return result;
 }
 
 vector operator%(const nr_double_t d, vector v) {
-  vector result (v);
-  for (int i = 0; i < v.getSize (); i++) result.set (d % v.get (i), i);
+  int len = v.getSize ();
+  vector result (len);
+  for (int i = 0; i < len; i++) result (i) = d % v (i);
   return result;
 }
 
 vector operator%(vector v1, vector v2) {
-  assert (v1.checkSizes (v1, v2));
-  vector result (v1);
-  for (int i = 0; i < v1.getSize (); i++)
-    result.set (v1.get (i) % v2.get (i), i);
-  return result;
+  int j, i, n, len, len1 = v1.getSize (), len2 = v2.getSize ();
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    len = len1;
+  } else {
+    assert (len2 % len1 == 0);
+    len = len2;
+  }
+  vector res (len);
+  for (j = i = n = 0; n < len; n++) {
+    res (n) = v1 (i) % v2 (j);
+    if (++i >= len1) i = 0;  if (++j >= len2) j = 0;
+  }
+  return res;
 }
 
 /* This function reverses the order of the data list. */
@@ -940,9 +1049,12 @@ nr_double_t vector::rms (void) {
 }
 
 nr_double_t vector::variance (void) { 
-  nr_double_t rms2 = integrate_n (*this) / getSize ();
-  nr_double_t avg2 = norm (avg (*this));
-  return rms2 - avg2;
+  nr_double_t result = 0.0;
+  complex average = avg (*this);
+  for (int i = 0; i < getSize (); i++) result += norm (get (i) - average);
+  if (getSize () > 1)
+    return result / (getSize () - 1);
+  return 0.0;
 }
 
 nr_double_t vector::stddev (void) {
@@ -1004,11 +1116,20 @@ vector polar (vector v, const complex p) {
 }
 
 vector polar (vector a, vector p) {
-  assert (a.checkSizes (a, p));
-  int n = a.getSize () < p.getSize () ? a.getSize () : p.getSize ();
-  vector result (n);
-  for (int i = 0; i < n; i++) result.set (polar (a.get (i), p.get (i)), i);
-  return result;
+  int j, i, n, len, len1 = a.getSize (), len2 = p.getSize ();
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    len = len1;
+  } else {
+    assert (len2 % len1 == 0);
+    len = len2;
+  }
+  vector res (len);
+  for (j = i = n = 0; n < len; n++) {
+    res (n) = polar (a (i), p (j));
+    if (++i >= len1) i = 0;  if (++j >= len2) j = 0;
+  }
+  return res;
 }
 
 vector arctan2 (const nr_double_t y, vector v) {
@@ -1026,12 +1147,20 @@ vector arctan2 (vector v, const nr_double_t x) {
 }
 
 vector arctan2 (vector y, vector x) {
-  assert (y.checkSizes (y, x));
-  int n = y.getSize () < x.getSize () ? y.getSize () : x.getSize ();
-  vector result (n);
-  for (int i = 0; i < n; i++)
-    result.set (arctan2 (y.get (i), x.get (i)), i);
-  return result;
+  int j, i, n, len, len1 = y.getSize (), len2 = x.getSize ();
+  if (len1 >= len2) {
+    assert (len1 % len2 == 0);
+    len = len1;
+  } else {
+    assert (len2 % len1 == 0);
+    len = len2;
+  }
+  vector res (len);
+  for (j = i = n = 0; n < len; n++) {
+    res (n) = arctan2 (y (i), x (j));
+    if (++i >= len1) i = 0; if (++j >= len2) j = 0;
+  }
+  return res;
 }
 
 vector w2dbm (vector v) {
