@@ -1,7 +1,7 @@
 /*
  * vector.h - vector class definitions
  *
- * Copyright (C) 2003, 2004, 2005, 2006 Stefan Jahn <stefan@lkcc.org>
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007 Stefan Jahn <stefan@lkcc.org>
  * Copyright (C) 2006 Gunther Kraut <gn.kraut@t-online.de>
  *
  * This is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  * Boston, MA 02110-1301, USA.  
  *
- * $Id: vector.h,v 1.24 2006/03/15 16:46:28 raimi Exp $
+ * $Id: vector.h,v 1.27 2007/05/03 19:01:07 ela Exp $
  *
  */
 
@@ -115,11 +115,17 @@ class vector : public object
   friend vector arctan2 (vector, vector);
   friend vector dbm2w   (vector);
   friend vector w2dbm   (vector);
+  friend vector xhypot  (vector, vector);
+  friend vector xhypot  (vector, const complex);
+  friend vector xhypot  (vector, const nr_double_t);
+  friend vector xhypot  (const complex, vector);
+  friend vector xhypot  (const nr_double_t, vector);
 
   // overloaded math functions
   friend vector abs     (vector);
   friend vector log10   (vector);
   friend vector exp     (vector);
+  friend vector limexp  (vector);
   friend vector sqrt    (vector);
   friend vector sin     (vector);
   friend vector arcsin  (vector);
@@ -133,10 +139,12 @@ class vector : public object
   friend vector arsinh  (vector);
   friend vector cosh    (vector);
   friend vector arcosh  (vector);
+  friend vector arsech  (vector);
   friend vector tanh    (vector);
   friend vector artanh  (vector);
   friend vector coth    (vector);
   friend vector arcoth  (vector);
+  friend vector signum  (vector);
   friend vector sign    (vector);
   friend vector sinc    (vector);
   friend vector ceil    (vector);
