@@ -32,10 +32,10 @@
 #include <qmessagebox.h>
 #include <qtextedit.h>
 #include <qlistbox.h>
-#include <qdir.h>
 #include <qregexp.h>
 
 #include "qucslib.h"
+#include "librarydialog.h"
 #include "displaydialog.h"
 #include "symbolwidget.h"
 #include "searchdialog.h"
@@ -52,8 +52,15 @@ QucsLib::QucsLib()
 
   // create file menu
   QPopupMenu * fileMenu = new QPopupMenu ();
+  QAction * manageLib =
+    new QAction ("Manage User Libraries...", tr("Manage User &Libraries..."), CTRL+Key_M, this);
+  manageLib->addTo (fileMenu);
+  connect(manageLib, SIGNAL(activated()), SLOT(slotManageLib()));
+
+  fileMenu->insertSeparator();
+
   QAction * fileQuit =
-    new QAction (tr("Quit"), tr("&Quit"), CTRL+Key_Q, this);
+    new QAction ("Quit", tr("&Quit"), CTRL+Key_Q, this);
   fileQuit->addTo (fileMenu);
   connect(fileQuit, SIGNAL(activated()), SLOT(slotQuit()));
 
@@ -119,15 +126,7 @@ QucsLib::QucsLib()
   connect(ShowButton, SIGNAL(clicked()), SLOT(slotShowModel()));
 
   // ......................................................
-  // Put all available libraries into ComboBox.
-  QDir LibDir(QucsSettings.LibDir);
-  QStringList LibFiles = LibDir.entryList("*.lib", QDir::Files, QDir::Name);
-
-  QStringList::iterator it;
-  for(it = LibFiles.begin(); it != LibFiles.end(); it++)
-    Library->insertItem((*it).left((*it).length()-4));
-
-  slotSelectLibrary(0);
+  putLibrariesIntoCombobox();
 }
 
 /* Destructor destroys the application. */
@@ -135,6 +134,33 @@ QucsLib::~QucsLib()
 {
 }
 
+// ----------------------------------------------------
+// Put all available libraries into ComboBox.
+void QucsLib::putLibrariesIntoCombobox()
+{
+  Library->clear();
+
+  UserLibCount = 0;
+  QStringList LibFiles;
+  QStringList::iterator it;
+  if(UserLibDir.cd(".")) { // user library directory exists ?
+    LibFiles = UserLibDir.entryList("*.lib", QDir::Files, QDir::Name);
+    UserLibCount = LibFiles.count();
+
+    for(it = LibFiles.begin(); it != LibFiles.end(); it++)
+      Library->insertItem((*it).left((*it).length()-4));
+  }
+
+  QDir LibDir(QucsSettings.LibDir);
+  LibFiles = LibDir.entryList("*.lib", QDir::Files, QDir::Name);
+
+  for(it = LibFiles.begin(); it != LibFiles.end(); it++)
+    Library->insertItem((*it).left((*it).length()-4));
+
+  slotSelectLibrary(0);
+}
+
+// ----------------------------------------------------
 void QucsLib::slotAbout()
 {
   QMessageBox::about(this, tr("About..."),
@@ -146,6 +172,7 @@ void QucsLib::slotAbout()
     "\nFITNESS FOR A PARTICULAR PURPOSE.");
 }
 
+// ----------------------------------------------------
 void QucsLib::slotQuit()
 {
   int tmp;
@@ -157,6 +184,7 @@ void QucsLib::slotQuit()
   accept();
 }
 
+// ----------------------------------------------------
 void QucsLib::closeEvent(QCloseEvent *Event)
 {
   int tmp;
@@ -168,11 +196,19 @@ void QucsLib::closeEvent(QCloseEvent *Event)
   Event->accept();
 }
 
+// ----------------------------------------------------
+void QucsLib::slotManageLib()
+{
+  (new LibraryDialog(this))->exec();
+  putLibrariesIntoCombobox();
+}
+
+// ----------------------------------------------------
 void QucsLib::slotHelp()
 {
   DisplayDialog *d = new DisplayDialog(this);
-  d->setCaption("QucsLib Help");
-  d->resize(250, 300);
+  d->setCaption(tr("QucsLib Help"));
+  d->resize(250, 325);
   d->Text->setText(
      tr("QucsLib is a program to manage Qucs component libraries. "
 	"On the left side of the application window the available "
@@ -182,38 +218,41 @@ void QucsLib::slotHelp()
 	"transported to the Qucs application by clicking on the "
 	"button \"Copy to Clipboard\". Being back in the schematic "
 	"window the component can be inserted by pressing CTRL-V "
-	" (paste from clipboard)."));
+	" (paste from clipboard).") + "\n" +
+     tr("A more comfortable way: The component can also be placed "
+        "onto the schematic by using Drag n'Drop."));
   d->show();
 }
 
+// ----------------------------------------------------
 void QucsLib::slotCopyToClipBoard()
 {
   QString s = "<Qucs Schematic " PACKAGE_VERSION ">\n";
-  s += "<Components>\n  ";
-  if(ModelString.contains('\n') < 2)
-    s += ModelString;
-  else
-    s += "<Lib " + Symbol->Prefix + " 1 0 0 " +
-         QString::number(Symbol->Text_x) + " " +
-         QString::number(Symbol->Text_y) + " 0 0 \"" +
-         Symbol->LibraryName + "\" 0 \"" + Symbol->ComponentName + "\" 0>\n";
-  s += "</Components>\n";
+  s += "<Components>\n  " +
+       Symbol->theModel() +
+       "\n</Components>\n";
 
   // put resulting schematic into clipboard
   QClipboard *cb = QApplication::clipboard();
   cb->setText(s);
 }
 
+// ----------------------------------------------------
 void QucsLib::slotShowModel()
 {
   DisplayDialog *d = new DisplayDialog(this);
-  d->setCaption("Model");
+  d->setCaption(tr("Model"));
   d->resize(500, 150);
-  d->Text->setText(ModelString);
+  d->Text->setText(Symbol->ModelString);
   d->Text->setWordWrap(QTextEdit::NoWrap);
+  d->VHDLText->setText(Symbol->VHDLModelString);
+  d->VHDLText->setWordWrap(QTextEdit::NoWrap);
+  d->VerilogText->setText(Symbol->VerilogModelString);
+  d->VerilogText->setWordWrap(QTextEdit::NoWrap);
   d->show();
 }
 
+// ----------------------------------------------------
 void QucsLib::slotSelectLibrary(int Index)
 {
   int Start, End, NameStart, NameEnd;
@@ -228,7 +267,12 @@ void QucsLib::slotSelectLibrary(int Index)
   LibraryComps.clear();
   DefaultSymbol = "";
 
-  QFile file(QucsSettings.LibDir + Library->text(Index) + ".lib");
+  QFile file;
+  if(Index < UserLibCount)  // Is it user library ?
+    file.setName(UserLibDir.absPath() + QDir::separator() + Library->text(Index) + ".lib");
+  else
+    file.setName(QucsSettings.LibDir + Library->text(Index) + ".lib");
+
   if(!file.open(IO_ReadOnly)) {
     QMessageBox::critical(this, tr("Error"),
         tr("Cannot open \"%1\".").arg(
@@ -283,67 +327,72 @@ void QucsLib::slotSelectLibrary(int Index)
   CompList->setSelected(0, true);  // select first item
 }
 
+// ----------------------------------------------------
 void QucsLib::slotSearchComponent()
 {
   SearchDialog *d = new SearchDialog(this);
+  d->setCaption(tr("Search Library Component"));
   if(d->exec() == QDialog::Accepted)
     QMessageBox::information(this, tr("Result"),
                              tr("No appropriate component found."));
 }
 
+// ----------------------------------------------------
 void QucsLib::slotShowComponent(QListBoxItem *Item)
 {
   if(!Item) return;
 
   QStringList::Iterator CompString = LibraryComps.at(CompList->index(Item));
+  QString LibName = (*CompString).section('\n', 0, 0);
   CompDescr->setText("Name: " + Item->text());
-  CompDescr->append("Library: " + (*CompString).section('\n', 0, 0));
+  CompDescr->append("Library: " + LibName);
   CompDescr->append("----------------------------");
 
-  int Start, End;
-  Start = (*CompString).find("<Description>");
-  if(Start > 0) {
-    Start += 13;
-    End = (*CompString).find("</Description>", Start);
-    if(End < 0) {
-      QMessageBox::critical(this, tr("Error"), tr("Library is corrupt."));
-      return;
-    }
-    CompDescr->append(
-      (*CompString).mid(Start, End-Start).replace(QRegExp("\\n\\x20+"), "\n").remove(0, 1));
-  }
+  if(Library->currentItem() < UserLibCount)
+    LibName = UserLibDir.absPath() + QDir::separator() + LibName;
 
+  QString content;
+  if(!getSection("Description", *CompString, content))
+    return;
+  CompDescr->append(content);
 
-  Start = (*CompString).find("<Model>");
-  if(Start > 0) {
-    Start += 7;
-    End = (*CompString).find("</Model>", Start);
-    if(End < 0) {
-      QMessageBox::critical(this, tr("Error"), tr("Library is corrupt."));
-      return;
-    }
-    ModelString =
-      (*CompString).mid(Start, End-Start).replace(QRegExp("\\n\\x20+"), "\n").remove(0, 1);
+  if(!getSection("Model", *CompString, content))
+    return;
+  Symbol->ModelString = content;
+  if(Symbol->ModelString.contains('\n') < 2)
+    Symbol->createSymbol(LibName, Item->text());
 
-    if(ModelString.contains('\n') < 2)
-      Symbol->createSymbol(ModelString,
-                           (*CompString).section('\n', 0, 0), Item->text());
-  }
+  if(!getSection("VHDLModel", *CompString, content))
+    return;
+  Symbol->VHDLModelString = content;
 
+  if(!getSection("VerilogModel", *CompString, content))
+    return;
+  Symbol->VerilogModelString = content;
 
-  Start = (*CompString).find("<Symbol>");
+  if(!getSection("Symbol", *CompString, content))
+    return;
+  if(!content.isEmpty())
+    Symbol->setSymbol(content, LibName, Item->text());
+  else if(!DefaultSymbol.isEmpty())   // has library a default symbol ?
+    Symbol->setSymbol(DefaultSymbol, LibName, Item->text());
+}
+
+// ----------------------------------------------------
+bool QucsLib::getSection(QString section, QString &list, QString &content)
+{
+  int Start, End;
+  Start = list.find("<"+section+">");
+  content = "";
   if(Start > 0) {
-    Start += 8;
-    End = (*CompString).find("</Symbol>", Start);
+    Start += section.length()+2;
+    End = list.find("</"+section+">", Start);
     if(End < 0) {
       QMessageBox::critical(this, tr("Error"), tr("Library is corrupt."));
-      return;
+      return false;
     }
-    Symbol->setSymbol((*CompString).mid(Start, End-Start),
-                      (*CompString).section('\n', 0, 0), Item->text());
+    content = list.mid(Start, End-Start);
+    content.replace(QRegExp("\\n\\x20+"), "\n").remove(0, 1);
   }
-  else if(!DefaultSymbol.isEmpty()) {  // has library a default symbol ?
-    Symbol->setSymbol(DefaultSymbol,
-                      (*CompString).section('\n', 0, 0), Item->text());
-  }
+  return true;
 }
