@@ -22,28 +22,18 @@ Capacitor::Capacitor()
 {
   Description = QObject::tr("capacitor");
 
-  Lines.append(new Line( -4,-11, -4, 11,QPen(QPen::darkBlue,4)));
-  Lines.append(new Line(  4,-11,  4, 11,QPen(QPen::darkBlue,4)));
-  Lines.append(new Line(-30,  0, -4,  0,QPen(QPen::darkBlue,2)));
-  Lines.append(new Line(  4,  0, 30,  0,QPen(QPen::darkBlue,2)));
-
-  Ports.append(new Port(-30,  0));
-  Ports.append(new Port( 30,  0));
-
-  x1 = -30; y1 = -13;
-  x2 =  30; y2 =  13;
+  Props.append(new Property("C", "1 pF", true,
+		QObject::tr("capacitance in Farad")));
+  Props.append(new Property("V", "", false,
+		QObject::tr("initial voltage for transient simulation")));
+  Props.append(new Property("Symbol", "neutral", false,
+	QObject::tr("schematic symbol")+" [neutral, polar]"));
 
+  createSymbol();
   tx = x1+4;
   ty = y2+4;
   Model = "C";
   Name  = "C";
-
-  Props.append(new Property("C", "1 pF", true,
-		QObject::tr("capacitance in Farad")));
-}
-
-Capacitor::~Capacitor()
-{
 }
 
 Component* Capacitor::newOne()
@@ -59,3 +49,26 @@ Element* Capacitor::info(QString& Name, char* &BitmapFile, bool getNewOne)
   if(getNewOne)  return new Capacitor();
   return 0;
 }
+
+void Capacitor::createSymbol()
+{
+  if(Props.getLast()->Value.at(0) == 'n') {
+    Lines.append(new Line( -4,-11, -4, 11,QPen(QPen::darkBlue,4)));
+    Lines.append(new Line(  4,-11,  4, 11,QPen(QPen::darkBlue,4)));
+  }
+  else {
+    Lines.append(new Line(-11, -5,-11,-11,QPen(QPen::red,1)));
+    Lines.append(new Line(-14, -8, -8, -8,QPen(QPen::red,1)));
+    Lines.append(new Line( -4,-11, -4, 11,QPen(QPen::darkBlue,3)));
+    Arcs.append(new Arc(4,-12, 20, 24, 16*122, 16*116,QPen(QPen::darkBlue,3)));
+  }
+
+  Lines.append(new Line(-30,  0, -4,  0,QPen(QPen::darkBlue,2)));
+  Lines.append(new Line(  4,  0, 30,  0,QPen(QPen::darkBlue,2)));
+
+  Ports.append(new Port(-30,  0));
+  Ports.append(new Port( 30,  0));
+
+  x1 = -30; y1 = -13;
+  x2 =  30; y2 =  13;
+}
