@@ -22,7 +22,7 @@ CCVS::CCVS()
 {
   Description = QObject::tr("current controlled voltage source");
 
-  Arcs.append(new Arc(0,-11, 23, 23,  0, 16*360,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc(0,-11, 22, 22,  0, 16*360,QPen(QPen::darkBlue,2)));
 
   Lines.append(new Line(-30,-30,-12,-30,QPen(QPen::darkBlue,2)));
   Lines.append(new Line(-30, 30,-12, 30,QPen(QPen::darkBlue,2)));
@@ -60,7 +60,7 @@ CCVS::CCVS()
 
   Props.append(new Property("G", "1 Ohm", true,
 		QObject::tr("forward transfer factor")));
-  Props.append(new Property("T", "0", true, QObject::tr("delay time")));
+  Props.append(new Property("T", "0", false, QObject::tr("delay time")));
 }
 
 CCVS::~CCVS()
