@@ -51,6 +51,8 @@ Component::Component()
 
   cx = 0;
   cy = 0;
+  tx = 0;
+  ty = 0;
 
   Arcs.setAutoDelete(true);
   Lines.setAutoDelete(true);
@@ -187,7 +189,7 @@ void Component::paint(ViewPainter *p)
   QFont newFont = f;
 
   if(Model.at(0) == '.') {   // is simulation component (dc, ac, ...)
-    newFont.setPointSizeFloat(float(p->Scale) * QucsSettings.largeFontSize);
+    newFont.setPointSizeFloat(p->Scale * Texts.getFirst()->Size);
     newFont.setWeight(QFont::DemiBold);
     p->Painter->setFont(newFont);
     p->map(cx, cy, x, y);
@@ -254,6 +256,8 @@ void Component::paint(ViewPainter *p)
                    p->DX + float(cx+pt->x) * p->Scale,
                    p->DY + float(cy+pt->y) * p->Scale));
       newFont.setPointSizeFloat(p->Scale * pt->Size);
+      newFont.setOverline(pt->over);
+      newFont.setUnderline(pt->under);
       p->Painter->setFont(newFont);
       p->Painter->setPen(pt->Color);
       p->Painter->drawText(0, 0, 0, 0, Qt::DontClip, pt->s);
@@ -351,20 +355,16 @@ void Component::paintScheme(QPainter *p)
 
 // -------------------------------------------------------
 // For output on a printer device.
-void Component::print(ViewPainter *p)
+void Component::print(ViewPainter *p, float FontScale)
 {
-  Arc *pa;
-  for(pa = Arcs.first(); pa != 0; pa = Arcs.next()) {
-    pa->w -= 1;   // to look nice after printing, arcs have to be
-    pa->h -= 1;   // made smaller
-  }
+  Text *pt;
+  for(pt = Texts.first(); pt != 0; pt = Texts.next())
+    pt->Size *= FontScale;
 
   paint(p);
 
-  for(pa = Arcs.first(); pa != 0; pa = Arcs.next()) {
-    pa->w += 1;   // back to old size
-    pa->h += 1;
-  }
+  for(pt = Texts.first(); pt != 0; pt = Texts.next())
+    pt->Size /= FontScale;
 }
 
 // -------------------------------------------------------
@@ -395,7 +395,7 @@ void Component::rotate()
   for(Arc *p3 = Arcs.first(); p3 != 0; p3 = Arcs.next()) {
     tmp = -p3->x;
     p3->x = p3->y;
-    p3->y = tmp - p3->w +1; // +1 is beauty correction
+    p3->y = tmp - p3->w;
     tmp = p3->w;
     p3->w = p3->h;
     p3->h = tmp;
@@ -483,7 +483,7 @@ void Component::mirrorX()
 
   // mirror all arcs
   for(Arc *p3 = Arcs.first(); p3 != 0; p3 = Arcs.next()) {
-    p3->y = -p3->y - p3->h + 1;   // +1 is beauty correction
+    p3->y = -p3->y - p3->h;
     if(p3->angle > 16*180) p3->angle -= 16*360;
     p3->angle  = -p3->angle;    // mirror
     p3->angle -= p3->arclen;    // go back to end of arc
@@ -543,7 +543,7 @@ void Component::mirrorY()
 
   // mirror all arcs
   for(Arc *p3 = Arcs.first(); p3 != 0; p3 = Arcs.next()) {
-    p3->x = -p3->x - p3->w + 1;   // +1 is beauty correction
+    p3->x = -p3->x - p3->w;
     p3->angle = 16*180 - p3->angle - p3->arclen;  // mirror
     if(p3->angle < 0) p3->angle += 16*360;   // angle has to be > 0
   }
@@ -590,34 +590,33 @@ void Component::mirrorY()
 }
 
 // -------------------------------------------------------
-QString Component::NetList()
+QString Component::netlist()
 {
   QString s = Model+":"+Name;
 
   // output all node names
   for(Port *p1 = Ports.first(); p1 != 0; p1 = Ports.next())
     s += " "+p1->Connection->Name;   // node names
-  if(Model.at(0) == '_') {    // add port ? (e.g. BJT without substrate)
-    if(Model == "_BJT")
-      s += " "+Ports.at(1)->Connection->Name;
-    else
-      s += " "+Ports.at(2)->Connection->Name;
-    s.remove(0,1);  // remove leading '_'
-  }
-  else if(Model == "MVIA")
-    s += " gnd";   // add ground node
 
   // output all properties
   for(Property *p2 = Props.first(); p2 != 0; p2 = Props.next())
     if(p2->Name != "Symbol")
       s += " "+p2->Name+"=\""+p2->Value+"\"";
 
-  return s;
+  return s + '\n';
 }
 
 // -------------------------------------------------------
-QString Component::getShortenNetlist()
+QString Component::getNetlist()
 {
+  switch(isActive) {
+    case COMP_IS_ACTIVE:
+      return netlist();
+    case COMP_IS_OPEN:
+      return QString("");
+  }
+
+  // Component is shortened.
   int z=0;
   QString s;
   QString Node1 = Ports.first()->Connection->Name;
@@ -628,21 +627,55 @@ QString Component::getShortenNetlist()
 }
 
 // -------------------------------------------------------
-QString Component::VHDL_Code(int)
+QString Component::verilogCode(int)
 {
   return QString("");   // no digital model
 }
 
 // -------------------------------------------------------
-QString Component::getShortenVHDL()
+QString Component::get_Verilog_Code(int NumPorts)
 {
-  QString s;
-  QString Node1 = Ports.first()->Connection->Name;
-  for(Port *pp = Ports.next(); pp != 0; pp = Ports.next())
-    s += "  " + pp->Connection->Name + " <= " + Node1 + ";\n";
+  switch(isActive) {
+    case COMP_IS_OPEN:
+      return QString("");
+    case COMP_IS_ACTIVE:
+      return verilogCode(NumPorts);
+  }
+
+  // Component is shortened.
+  Port *p = Ports.first();
+  QString Node1 = p->Connection->Name;
+  QString s = "";
+  for(p = Ports.next(); p != 0; p = Ports.next())
+    s += "  assign " + p->Connection->Name + " = " + Node1 + ";\n";
   return s;
 }
 
+// -------------------------------------------------------
+QString Component::vhdlCode(int)
+{
+  return QString("");   // no digital model
+}
+
+// -------------------------------------------------------
+QString Component::get_VHDL_Code(int NumPorts)
+{
+  switch(isActive) {
+    case COMP_IS_OPEN:
+      return QString("");
+    case COMP_IS_ACTIVE:
+      return vhdlCode(NumPorts);
+  }
+
+  // Component is shortened.
+  // This puts the signal of the second port onto the first port.
+  // This is locigally correct for the inverter only, but makes
+  // some sense for the gates (OR, AND etc.).
+  // Has anyone a better idea?
+  QString Node1 = Ports.first()->Connection->Name;
+  return "  " + Node1 + " <= " + Ports.next()->Connection->Name + ";\n";
+}
+
 // -------------------------------------------------------
 QString Component::save()
 {
@@ -733,8 +766,20 @@ bool Component::load(const QString& _s)
   tx = ttx; ty = tty; // restore text position (was changed by rotate/mirror)
 
   unsigned int z=0, counts = s.contains('"');
+  if(Model == "Sub")
+    tmp = 2;   // first property (File) already exists
+  else if(Model == "Lib")
+    tmp = 3;
+  else if(Model == "EDD")
+    tmp = 5;
+  else tmp = counts + 1;    // "+1" because "counts" could be zero
+
+  for(; tmp<=(int)counts/2; tmp++)
+    Props.append(new Property("p", "", true, " "));
+
   // load all properties
-  for(Property *p1 = Props.first(); p1 != 0; p1 = Props.next()) {
+  Property *p1;
+  for(p1 = Props.first(); p1 != 0; p1 = Props.next()) {
     z++;
     n = s.section('"',z,z);    // property value
     z++;
@@ -742,16 +787,35 @@ bool Component::load(const QString& _s)
     if(z > counts) {
       if(p1->Description.isEmpty())
         Props.remove();    // remove if allocated in vain
+
+      if(counts < 58)     // backward compatible
+        if(Model == "Diode") {
+          counts >>= 1;
+          p1 = Props.at(counts-1);
+          for(; p1 != 0; p1 = Props.current()) {
+            if(counts-- < 19)
+              break;
+
+            n = Props.prev()->Value;
+            p1->Value = n;
+          }
+
+          p1 = Props.at(17);
+          p1->Value = Props.at(11)->Value;
+          Props.current()->Value = "0";
+        }
+
       return true;
     }
 
+    if(Model != "EDD")
     if(p1->Description.isEmpty()) {  // unknown number of properties ?
       p1->Name = n.section('=',0,0);
       n = n.section('=',1,1);
       // allocate memory for a new property (e.g. for equations)
       if(Props.count() < (counts>>1)) {
-	Props.insert(z >> 1, new Property("y", "1", true));
-	Props.prev();
+        Props.insert(z >> 1, new Property("y", "1", true));
+        Props.prev();
       }
     }
     if(z == 6)  if(counts == 6)     // backward compatible
@@ -762,9 +826,7 @@ bool Component::load(const QString& _s)
     p1->Value = n;
 
     n  = s.section('"',z,z);    // display
-    if(n.toInt(&ok) == 1) p1->display = true;
-    else p1->display = false;
-    if(!ok) return false;
+    p1->display = (n.at(1) == '1');
   }
 
   return true;
@@ -774,7 +836,7 @@ bool Component::load(const QString& _s)
 // ***  The following functions are used to load the schematic symbol
 // ***  from file. (e.g. subcircuit, library component)
 
-int Component::analyseLine(const QString& Row)
+int Component::analyseLine(const QString& Row, int numProps)
 {
   QPen Pen;
   QBrush Brush;
@@ -833,6 +895,32 @@ int Component::analyseLine(const QString& Row)
     ty = i2;
     Name = Row.section(' ',3,3);
     if(Name.isEmpty())  Name = "SUB";
+
+    i1 = 1;
+    Property *pp = Props.at(numProps-1);
+    for(;;) {
+      s = Row.section('"', i1,i1);
+      if(s.isEmpty())  break;
+
+      pp = Props.next();
+      if(pp == 0) {
+        pp = new Property();
+        Props.append(pp);
+
+        pp->display = (s.at(0) == '1');
+        pp->Value = s.section('=', 2,2);
+      }
+
+      pp->Name  = s.section('=', 1,1);
+      pp->Description = s.section('=', 3,3);
+      if(pp->Description.isEmpty())
+        pp->Description = " ";
+
+      i1 += 2;
+    }
+
+    while(pp != Props.last())
+      Props.remove();
     return 0;   // do not count IDs
   }
   else if(s == "Arrow") {
@@ -1037,6 +1125,35 @@ bool Component::getBrush(const QString& s, QBrush& Brush, int i)
   return true;
 }
 
+// ---------------------------------------------------------------------
+void Component::copyComponent(Component *pc)
+{
+  Type = pc->Type;
+  x1 = pc->x1;
+  y1 = pc->y1;
+  x2 = pc->x2;
+  y2 = pc->y2;
+
+  Model = pc->Model;
+  Name  = pc->Name;
+  showName = pc->showName;
+  Description = pc->Description;
+
+  isActive = pc->isActive;
+  rotated  = pc->rotated;
+  mirroredX = pc->mirroredX;
+  tx = pc->tx;
+  ty = pc->ty;
+
+  Props  = pc->Props;
+  Ports  = pc->Ports;
+  Lines  = pc->Lines;
+  Arcs   = pc->Arcs;
+  Rects  = pc->Rects;
+  Ellips = pc->Ellips;
+  Texts  = pc->Texts;
+}
+
 
 // ***********************************************************************
 // ********                                                       ********
@@ -1097,7 +1214,7 @@ GateComponent::GateComponent()
 }
 
 // -------------------------------------------------------
-QString GateComponent::NetList()
+QString GateComponent::netlist()
 {
   QString s = Model+":"+Name;
 
@@ -1109,12 +1226,12 @@ QString GateComponent::NetList()
   Property *p = Props.at(1);
   s += " " + p->Name + "=\"" + p->Value + "\"";
   p = Props.next();
-  s += " " + p->Name + "=\"" + p->Value + "\"";
+  s += " " + p->Name + "=\"" + p->Value + "\"\n";
   return s;
 }
 
 // -------------------------------------------------------
-QString GateComponent::VHDL_Code(int NumPorts)
+QString GateComponent::vhdlCode(int NumPorts)
 {
   Port *pp = Ports.first();
   QString s = "  " + pp->Connection->Name + " <= ";  // output port
@@ -1143,7 +1260,71 @@ QString GateComponent::VHDL_Code(int NumPorts)
       s += " after " + t;
     }
 
-  s += ';';
+  s += ";\n";
+  return s;
+}
+
+// -------------------------------------------------------
+QString GateComponent::verilogCode(int NumPorts)
+{
+  bool synthesize = true;
+  Port *pp = Ports.first();
+  QString s("");
+
+  if(synthesize) {
+    QString op = Model.lower();
+    if(op == "and" || op == "nand")
+      op = "&";
+    else if (op == "or" || op == "nor")
+      op = "|";
+    else if (op == "xor")
+      op = "^";
+    else if (op == "xnor")
+      op = "^~";
+
+    s = "  assign ";
+
+    if(NumPorts <= 0)  // no truth table simulation ?
+      if(strtod(Props.at(2)->Value.latin1(), 0) != 0.0) {  // delay time
+	QString t = Props.current()->Value;
+	if(!Verilog_Time(t, Name))
+	  return t;    // time has not VHDL format
+	s += "#" + t + " ";
+      }
+    s += pp->Connection->Name + " = ";  // output port
+    if(Model.at(0) == 'N') s += "~(";
+
+    pp = Ports.next();
+    s += pp->Connection->Name;   // first input port
+
+    // output all input ports with node names
+    for(pp = Ports.next(); pp != 0; pp = Ports.next())
+      s += " " + op + " " + pp->Connection->Name;
+
+    if(Model.at(0) == 'N') s += ")";
+    s += ";\n";
+  }
+  else {
+    s = "  " + Model.lower();
+ 
+    if(NumPorts <= 0)  // no truth table simulation ?
+      if(strtod(Props.at(2)->Value.latin1(), 0) != 0.0) {  // delay time
+	QString t = Props.current()->Value;
+	if(!Verilog_Time(t, Name))
+	  return t;    // time has not VHDL format
+	s += " #" + t;
+      }
+    s += " " + Name + " (" + pp->Connection->Name;  // output port
+
+    pp = Ports.next();
+    s += ", " + pp->Connection->Name;   // first input port
+
+    // output all input ports with node names
+    for(pp = Ports.next(); pp != 0; pp = Ports.next())
+      s += ", " + pp->Connection->Name;
+
+    s += ");\n";
+  }
   return s;
 }
 
@@ -1208,7 +1389,7 @@ void GateComponent::createSymbol()
         Lines.append(new Line(-5, 3, 5, 3,QPen(QPen::darkBlue,1)));
       }
       else {
-        Arcs.append(new Arc(-6,-6, 12, 12, 0, 16*360,QPen(QPen::darkBlue,1)));
+        Arcs.append(new Arc(-5,-5, 10, 10, 0, 16*360,QPen(QPen::darkBlue,1)));
         Lines.append(new Line( 0,-5, 0, 5,QPen(QPen::darkBlue,1)));
       }
     }
@@ -1291,6 +1472,7 @@ Component* getComponentFromName(QString& Line)
         else if(cstr == "Inoise") c = new Noise_ii();
         else if(cstr == "Vnoise") c = new Noise_iv();
         else if(cstr == "nv") c = new Logical_Inv();
+        else if(cstr == "exp") c = new iExp();
         break;
   case 'J' : if(cstr == "FET") c = new JFET();
 	else if(cstr == "KFF") c = new JK_FlipFlop();
@@ -1305,9 +1487,13 @@ Component* getComponentFromName(QString& Line)
         else if(cstr == "rect") c = new vRect();
         else if(cstr == "Vnoise") c = new Noise_vv();
         else if(cstr == "HDL") c = new VHDL_File();
+        else if(cstr == "erilog") c = new Verilog_File();
+        else if(cstr == "exp") c = new vExp();
         break;
   case 'T' : if(cstr == "r") c = new Transformer();
         else if(cstr == "LIN") c = new TLine();
+        else if(cstr == "LIN4P") c = new TLine_4Port();
+        else if(cstr == "WIST") c = new TwistedPair();
         break;
   case 's' : if(cstr == "Tr") c = new symTrafo();
         break;
@@ -1333,6 +1519,7 @@ Component* getComponentFromName(QString& Line)
 	break;
   case 'B' : if(cstr == "iasT") c = new BiasT();
         else if(cstr == "JT") c = new BJTsub();
+        else if(cstr == "OND") c = new BondWire();
         break;
   case 'A' : if(cstr == "ttenuator") c = new Attenuator();
         else if(cstr == "mp") c = new Amplifier();
@@ -1354,6 +1541,7 @@ Component* getComponentFromName(QString& Line)
 	else if(cstr == "VIA") c = new MSvia();
 	break;
   case 'E' : if(cstr == "qn") c = new Equation();
+	else if(cstr == "DD") c = new EqnDefined();
         break;
   case 'O' : if(cstr == "pAmp") c = new OpAmp();
         else if(cstr == "R") c = new Logical_OR();
@@ -1368,6 +1556,7 @@ Component* getComponentFromName(QString& Line)
         else if(cstr == "HB") c = new HB_Sim();
         else if(cstr == "SW") c = new Param_Sweep();
         else if(cstr == "Digi") c = new Digi_Sim();
+        else if(cstr == "Opt") c = new Optimize_Sim();
         break;
   case '_' : if(cstr == "BJT") c = new BJT();
 	else if(cstr == "MOSFET") c = new MOSFET();
@@ -1375,6 +1564,10 @@ Component* getComponentFromName(QString& Line)
   case 'X' : if(cstr == "OR") c = new Logical_XOR();
         else if(cstr == "NOR") c = new Logical_XNOR();
         break;
+  case 'h' : if(cstr == "icumL2p1") c = new hicumL2p1();
+        break;
+  case 'H' : if(cstr == "BT_X") c = new HBT_X();
+        break;
   }
   if(!c) {
     QMessageBox::critical(0, QObject::tr("Error"),
