@@ -26,7 +26,6 @@
 #include <qhgroupbox.h>
 #include <qvalidator.h>
 #include <qtabwidget.h>
-#include <qmessagebox.h>
 #include <qfiledialog.h>
 
 #include <math.h>
@@ -46,8 +45,10 @@ ComponentDialog::ComponentDialog(Component *c, Schematic *d)
   QWidget *myParent = this;
   ValInteger = new QIntValidator(1, 1000000, this);
 
-  Expr.setPattern("[^\"=]+");  // valid expression for property 'edit'
+  Expr.setPattern("[^\"=]*");  // valid expression for property 'edit'
   Validator = new QRegExpValidator(Expr, this);
+  Expr.setPattern("[^\"]*");   // valid expression for property 'edit'
+  Validator2 = new QRegExpValidator(Expr, this);
   Expr.setPattern("[\\w_]+");  // valid expression for property 'NameEdit'
   ValRestrict = new QRegExpValidator(Expr, this);
 
@@ -276,7 +277,7 @@ ComponentDialog::ComponentDialog(Component *c, Schematic *d)
 
   edit = new QLineEdit(v1);
   edit->setMinimumWidth(150);
-  edit->setValidator(Validator);
+  edit->setValidator(Validator2);
   connect(edit, SIGNAL(returnPressed()), SLOT(slotApplyProperty()));
 
   // hide, because it only replaces 'edit' in some cases
@@ -353,6 +354,7 @@ ComponentDialog::~ComponentDialog()
 {
   delete all;
   delete Validator;
+  delete Validator2;
   delete ValRestrict;
   delete ValInteger;
 }
@@ -747,7 +749,8 @@ void ComponentDialog::slotBrowseFile()
   QString s = QFileDialog::getOpenFileName(QucsWorkDir.path(),
           tr("All Files")+" (*.*);;"+tr("Touchstone files")+" (*.s?p);;"+
           tr("SPICE files")+" (*.cir *.spi);;"+
-          tr("VHDL files")+" (*.vhdl *.vhd)",
+          tr("VHDL files")+" (*.vhdl *.vhd);;"+
+          tr("Verilog files")+" (*.v)",
           this, "", tr("Select a file"));
   if(!s.isEmpty()) {
     // snip path if file in current directory
