@@ -32,8 +32,9 @@ DC_Sim::DC_Sim()
   if (a < 8 || s.length() - b < 8) b = -1;
   if (b != -1) s[b] = '\n';
 
-  Texts.append(new Text(0, 0, s.left(b)));
-  if (b != -1) Texts.append(new Text(0, 0, s.mid(b+1)));
+  Texts.append(new Text(0, 0, s.left(b), QPen::darkBlue, QucsSettings.largeFontSize));
+  if (b != -1)
+    Texts.append(new Text(0, 0, s.mid(b+1), QPen::darkBlue, QucsSettings.largeFontSize));
 
   x1 = -10; y1 = -9;
   x2 = x1+128; y2 = y1+41;
@@ -57,14 +58,14 @@ DC_Sim::DC_Sim()
   Props.append(new Property("MaxIter", "150", false,
 		QObject::tr("maximum number of iterations until error")));
   Props.append(new Property("saveAll", "no", false,
-		QObject::tr("save subcircuit nodes into dataset")+
-		" [yes, no]"));
+	QObject::tr("save subcircuit nodes into dataset")+
+	" [yes, no]"));
   Props.append(new Property("convHelper", "none", false,
-		QObject::tr("preferred convergence algorithm")+
-		" [none, gMinStepping, SteepestDescent, LineSearch, Attenuation, SourceStepping]"));
+	QObject::tr("preferred convergence algorithm")+
+	" [none, gMinStepping, SteepestDescent, LineSearch, Attenuation, SourceStepping]"));
   Props.append(new Property("Solver", "CroutLU", false,
 	QObject::tr("method for solving the circuit matrix")+
-	" [CroutLU, DoolittleLU, HouseholderQR, HouseholderLQ]"));
+	" [CroutLU, DoolittleLU, HouseholderQR, HouseholderLQ, GolubSVD]"));
 }
 
 DC_Sim::~DC_Sim()
