@@ -22,9 +22,9 @@ Inductor::Inductor()
 {
   Description = QObject::tr("inductor");
 
-  Arcs.append(new Arc(-18, -6, 13, 13,  0, 16*180,QPen(QPen::darkBlue,2)));
-  Arcs.append(new Arc( -6, -6, 13, 13,  0, 16*180,QPen(QPen::darkBlue,2)));
-  Arcs.append(new Arc(  6, -6, 13, 13,  0, 16*180,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc(-18, -6, 12, 12,  0, 16*180,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc( -6, -6, 12, 12,  0, 16*180,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc(  6, -6, 12, 12,  0, 16*180,QPen(QPen::darkBlue,2)));
   Lines.append(new Line(-30,  0,-18,  0,QPen(QPen::darkBlue,2)));
   Lines.append(new Line( 18,  0, 30,  0,QPen(QPen::darkBlue,2)));
 
@@ -41,6 +41,8 @@ Inductor::Inductor()
 
   Props.append(new Property("L", "1 nH", true,
 		QObject::tr("inductance in Henry")));
+  Props.append(new Property("I", "", false,
+		QObject::tr("initial current for transient simulation")));
 }
 
 Inductor::~Inductor()
