@@ -19,6 +19,7 @@
 
 #include "jk_flipflop.h"
 #include "node.h"
+#include "main.h"
 
 JK_FlipFlop::JK_FlipFlop()
 {
@@ -46,7 +47,7 @@ JK_FlipFlop::JK_FlipFlop()
   Texts.append(new Text(-17, 13, "K", QPen::darkBlue, 12.0));
   Texts.append(new Text(  6,-27, "Q", QPen::darkBlue, 12.0));
   Texts.append(new Text(  6, 13, "Q", QPen::darkBlue, 12.0));
-  Lines.append(new Line(  7, 13, 15, 13,QPen(QPen::darkBlue,1)));
+  Texts.current()->over=true;
   Lines.append(new Line(-20, -4,-12,  0,QPen(QPen::darkBlue,0)));
   Lines.append(new Line(-20,  4,-12,  0,QPen(QPen::darkBlue,0)));
 
@@ -67,12 +68,16 @@ JK_FlipFlop::JK_FlipFlop()
 }
 
 // -------------------------------------------------------
-QString JK_FlipFlop::VHDL_Code(int NumPorts)
+QString JK_FlipFlop::vhdlCode(int NumPorts)
 {
   QString s = ";\n";
   if(NumPorts <= 0)  // no truth table simulation ?
-    if(strtod(Props.getFirst()->Value.latin1(), 0) != 0.0)  // delay time
-      s = " after " + Props.getFirst()->Value + ";\n";
+    if(strtod(Props.getFirst()->Value.latin1(), 0) != 0.0) { // delay time
+      s = Props.getFirst()->Value;
+      if(!VHDL_Time(s, Name))
+        return s;    // time has not VHDL format
+      s = " after " + s + ";\n";
+    }
 
   s = "  " + Name + " : process (" +
       Ports.at(5)->Connection->Name + ", " +
@@ -91,10 +96,47 @@ QString JK_FlipFlop::VHDL_Code(int NumPorts)
       Ports.at(2)->Connection->Name + ")" + s +
       "    end if;\n  end process;\n  " +
       Ports.at(3)->Connection->Name + " <= not " +
-      Ports.at(2)->Connection->Name + ";\n";
+      Ports.at(2)->Connection->Name + ";\n\n";
   return s;
 }
 
+// -------------------------------------------------------
+QString JK_FlipFlop::verilogCode(int NumPorts)
+{
+  QString t = "";
+  if(NumPorts <= 0)  // no truth table simulation ?
+    if(strtod(Props.getFirst()->Value.latin1(), 0) != 0.0) { // delay time
+      t = Props.getFirst()->Value;
+      if(!Verilog_Time(t, Name))
+        return t;    // time has not VHDL format
+      t = "    #" + t + ";\n";
+    }
+  
+  QString l = "";
+
+  QString s = Ports.at(5)->Connection->Name;
+  QString r = Ports.at(6)->Connection->Name;
+  QString j = Ports.at(0)->Connection->Name;
+  QString k = Ports.at(1)->Connection->Name;
+  QString q = Ports.at(2)->Connection->Name;
+  QString b = Ports.at(3)->Connection->Name;
+  QString c = Ports.at(4)->Connection->Name;
+  QString v = "net_reg" + Name + q;
+  
+  l = "\n  // " + Name + " JK-flipflop\n" +
+    "  assign  " + q + " = " + v + ";\n" +
+    "  assign  " + b + " = ~" + q + ";\n" +
+    "  reg     " + v + " = 0;\n" +
+    "  always @ (" + c + " or " + r + " or " + s + ") begin\n" + t +
+    "    if (" + r + ") " + v + " <= 0;\n" +
+    "    else if (" + s + ") " + v + " <= 1;\n" +
+    "    else if (" + c + ")\n" + 
+    "      " + v + " <= (" + j + " && ~" + q + ") || (~" +
+    k + " && " + q + ");\n" +
+    "  end\n\n";
+  return l;
+}
+
 // -------------------------------------------------------
 Component* JK_FlipFlop::newOne()
 {
