@@ -1,6 +1,6 @@
 /***************************************************************************
-                          msvia.cpp  -  description
-                             -------------------
+                               msvia.cpp
+                              -----------
     begin                : Sat Oct 30 2004
     copyright            : (C) 2004 by Michael Margraf
     email                : michael.margraf@alumni.tu-berlin.de
@@ -16,13 +16,14 @@
  ***************************************************************************/
 
 #include "msvia.h"
+#include "node.h"
 
 
 MSvia::MSvia()
 {
   Description = QObject::tr("microstrip via");
 
-  Arcs.append(new Arc(-5,-4, 10, 8, 0, 16*360,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc(-5,-4, 10,  7,  0, 16*360,QPen(QPen::darkBlue,2)));
   Lines.append(new Line(-20,  0, -5,  0,QPen(QPen::darkBlue,2)));
   Lines.append(new Line( -5,  0, -5, 14,QPen(QPen::darkBlue,2)));
   Lines.append(new Line(  5,  0,  5, 14,QPen(QPen::darkBlue,2)));
@@ -52,11 +53,13 @@ MSvia::~MSvia()
 {
 }
 
+// -------------------------------------------------------
 Component* MSvia::newOne()
 {
   return new MSvia();
 }
 
+// -------------------------------------------------------
 Element* MSvia::info(QString& Name, char* &BitmapFile, bool getNewOne)
 {
   Name = QObject::tr("Microstrip Via");
@@ -65,3 +68,18 @@ Element* MSvia::info(QString& Name, char* &BitmapFile, bool getNewOne)
   if(getNewOne)  return new MSvia();
   return 0;
 }
+
+// -------------------------------------------------------
+QString MSvia::netlist()
+{
+  QString s = Model+":"+Name;
+
+  // output node name and add ground node
+  s += " " + Ports.getFirst()->Connection->Name + " gnd";
+
+  // output all properties
+  for(Property *p2 = Props.first(); p2 != 0; p2 = Props.next())
+    s += " "+p2->Name+"=\""+p2->Value+"\"";
+
+  return s + '\n';
+}
