@@ -27,8 +27,9 @@ Param_Sweep::Param_Sweep()
   int a = s.findRev(" ");
   if (a != -1) s[a] = '\n';    // break line
 
-  Texts.append(new Text(0, 0, s.left(a)));
-  if (a != -1) Texts.append(new Text(0, 0, s.mid(a+1)));
+  Texts.append(new Text(0, 0, s.left(a), QPen::darkBlue, QucsSettings.largeFontSize));
+  if (a != -1)
+    Texts.append(new Text(0, 0, s.mid(a+1), QPen::darkBlue, QucsSettings.largeFontSize));
 
   x1 = -10; y1 = -9;
   x2 = x1+104; y2 = y1+59;
