@@ -47,8 +47,8 @@ void SubCirPort::createSymbol()
   x2 =   0; y2 =  8;
 
   if(Props.at(1)->Value.at(0) == 'a') {
-    Arcs.append(new Arc(-25, -6, 13, 13,  0, 16*360,QPen(QPen::darkBlue,2)));
-    Lines.append(new Line(-14,  0,  0,  0,QPen(QPen::darkBlue,2)));
+    Arcs.append(new Arc(-25, -6, 12, 12,  0, 16*360,QPen(QPen::darkBlue,2)));
+    Lines.append(new Line(-13,  0,  0,  0,QPen(QPen::darkBlue,2)));
   }
   else {
     Lines.append(new Line( -9,  0,  0,  0,QPen(QPen::darkBlue,2)));
@@ -99,20 +99,27 @@ Element* SubCirPort::info(QString& Name, char* &BitmapFile, bool getNewOne)
 }
 
 // -------------------------------------------------------
-QString SubCirPort::NetList()
+QString SubCirPort::netlist()
 {
   return QString("");
 }
 
 // -------------------------------------------------------
-QString SubCirPort::VHDL_Code(int)
+QString SubCirPort::vhdlCode(int)
 {
-  if(Props.at(1)->Value != "out")  return QString("");
+  if(Props.at(1)->Value != "out")
+    return QString("");
 
   // Insert dummy buffer to avoid reading from an output port.
   QString s = "  net_out";
   Node *pn = Ports.getFirst()->Connection;
   s += pn->Name + " <= ";
-  s += pn->Name + " or '0';";
+  s += pn->Name + " or '0';\n";
   return s;
 }
+
+// -------------------------------------------------------
+QString SubCirPort::verilogCode(int)
+{
+  return QString("");
+}
