@@ -22,7 +22,7 @@ VCCS::VCCS()
 {
   Description = QObject::tr("voltage controlled current source");
 
-  Arcs.append(new Arc(0,-11, 23, 23,  0, 16*360,QPen(QPen::darkBlue,2)));
+  Arcs.append(new Arc(0,-11, 22, 22,  0, 16*360,QPen(QPen::darkBlue,2)));
   Lines.append(new Line( 11, -7, 11,  7,QPen(QPen::darkBlue,3)));
   Lines.append(new Line( 11,  6, 15,  1,QPen(QPen::darkBlue,3)));
   Lines.append(new Line( 11,  6,  7,  1,QPen(QPen::darkBlue,3)));
@@ -37,9 +37,9 @@ VCCS::VCCS()
   Lines.append(new Line( 11,-30, 11,-11,QPen(QPen::darkBlue,2)));
   Lines.append(new Line( 11, 30, 11, 11,QPen(QPen::darkBlue,2)));
 
-  Lines.append(new Line(-12,-18,-12, 18,QPen(QPen::darkBlue,2)));
-  Lines.append(new Line(-12, 18,-17,  9,QPen(QPen::darkBlue,2)));
-  Lines.append(new Line(-12, 18, -8,  9,QPen(QPen::darkBlue,2)));
+  Lines.append(new Line(-12,-18,-12, 18,QPen(QPen::darkBlue,1)));
+  Lines.append(new Line(-12, 18,-17,  9,QPen(QPen::darkBlue,1)));
+  Lines.append(new Line(-12, 18, -7,  9,QPen(QPen::darkBlue,1)));
 
   Lines.append(new Line(-25,-27, 25,-27,QPen(QPen::darkGray,1)));
   Lines.append(new Line( 25,-27, 25, 27,QPen(QPen::darkGray,1)));
@@ -61,7 +61,7 @@ VCCS::VCCS()
 
   Props.append(new Property("G", "1 S", true,
 		QObject::tr("forward transconductance")));
-  Props.append(new Property("T", "0", true, QObject::tr("delay time")));
+  Props.append(new Property("T", "0", false, QObject::tr("delay time")));
 }
 
 VCCS::~VCCS()
