@@ -22,10 +22,10 @@
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
-
 #if HAVE_IEEEFP_H
 # include <ieeefp.h>
 #endif
+#include <locale.h>
 
 #include <qtextstream.h>
 #include <qmessagebox.h>
@@ -47,13 +47,13 @@
 // strtod() appears to be so sloooooooo under Win32...
 static double strtod_faster (char * pPos, char ** pEnd) {
   *pEnd = pPos;
-  while((**pEnd) && (**pEnd > ' ') && (**pEnd != 'j') && (**pEnd != 'i'))
+  while ((**pEnd) && (**pEnd > ' ') && (**pEnd != 'j') && (**pEnd != 'i'))
     (*pEnd)++;
   if ((**pEnd == 'j') || (**pEnd == 'i'))
     (*pEnd)--;
   QCString str = QCString (pPos, *pEnd - pPos + 1);
   bool ok;
-  double x = str.toDouble(&ok);
+  double x = str.toDouble (&ok);
   if (!ok) *pEnd = pPos;
   return x;
 }
@@ -115,7 +115,7 @@ void Diagram::paint(ViewPainter *p)
   // paint all arcs (1 pixel larger to compensate for strange circle method)
   for(struct Arc *pa = Arcs.first(); pa != 0; pa = Arcs.next()) {
     p->Painter->setPen(pa->style);
-    p->drawArc(cx+pa->x, cy-pa->y, pa->w+1, pa->h+1, pa->angle, pa->arclen);
+    p->drawArc(cx+pa->x, cy-pa->y, pa->w, pa->h, pa->angle, pa->arclen);
   }
 
   Graph *pg;
@@ -145,8 +145,14 @@ void Diagram::paint(ViewPainter *p)
 
 
   if(isSelected) {
+    int x_, y_;
+    float fx_, fy_;
+    p->map(cx, cy-y2, x_, y_);
+    fx_ = float(x2)*p->Scale + 10;
+    fy_ = float(y2)*p->Scale + 10;
+
     p->Painter->setPen(QPen(QPen::darkGray,3));
-    p->drawRect(cx-5, cy-y2-5, x2+10, y2+10);
+    p->Painter->drawRect(x_-5, y_-5, TO_INT(fx_), TO_INT(fy_));
     p->Painter->setPen(QPen(QPen::darkRed,2));
     p->drawResizeRect(cx, cy-y2);  // markers for changing the size
     p->drawResizeRect(cx, cy);
@@ -155,6 +161,12 @@ void Diagram::paint(ViewPainter *p)
   }
 }
 
+// ------------------------------------------------------------
+void Diagram::paintScheme(QPainter *p)
+{
+  p->drawRect(cx, cy-y2, x2, y2);
+}
+
 // ------------------------------------------------------------
 // Put axis labels into the text list.
 void Diagram::createAxisLabels()
@@ -287,27 +299,21 @@ void Diagram::createAxisLabels()
 }
 
 // ------------------------------------------------------------
-void Diagram::paintScheme(QPainter *p)
-{
-  p->drawRect(cx, cy-y2, x2, y2);
-}
-
-// ------------------------------------------------------------
-int Diagram::regionCode(int x, int y)
+int Diagram::regionCode(float x, float y)
 {
   int code=0;   // code for clipping
-  if(x < 0)  code |= 1;
-  else if(x > x2)  code |= 2;
-  if(y < 0)  code |= 4;
-  else if(y > y2)  code |= 8;
+  if(x < 0.0)  code |= 1;
+  else if(int(x) > x2)  code |= 2;
+  if(y < 0.0)  code |= 4;
+  else if(int(y) > y2)  code |= 8;
   return code;
 }
 
 // ------------------------------------------------------------
 // Is virtual. This one is for round diagrams only.
-bool Diagram::insideDiagram(int x, int y)
+bool Diagram::insideDiagram(float x, float y)
 {
-  int R = (x2 >> 1) + 1;  // +1 seems better (graph sometimes little outside)
+  float R = float(x2)/2.0 + 1.0; // +1 seems better (graph sometimes little outside)
   x -= R;
   y -= R;
   return ((x*x + y*y) <= R*R);
@@ -315,11 +321,12 @@ bool Diagram::insideDiagram(int x, int y)
 
 // ------------------------------------------------------------
 // Cohen-Sutherland clipping algorithm
-void Diagram::rectClip(int* &p)
+void Diagram::rectClip(float* &p)
 {
-  int x=0, y=0, dx, dy, code, z=0;
-  int x_1 = *(p-4), y_1 = *(p-3);
-  int x_2 = *(p-2), y_2 = *(p-1);
+  int code, z=0;
+  float x=0, y=0, dx, dy;
+  float x_1 = *(p-4), y_1 = *(p-3);
+  float x_2 = *(p-2), y_2 = *(p-1);
 
   int code1 = regionCode(x_1, y_1);
   int code2 = regionCode(x_2, y_2);
@@ -329,12 +336,9 @@ void Diagram::rectClip(int* &p)
     p++;
     *(p-5) = STROKEEND;
   }
-  if(code1 & code2) {   // line not visible at all ?
-    *(p-4) = x_2;
-    *(p-3) = y_2;
-    p -= 2;
-    return;
-  }
+  if(code1 & code2)   // line not visible at all ?
+    goto endWithHidden;
+
   if(code2 != 0) {
     *p = STROKEEND;
     *(p+1) = x_2;
@@ -349,23 +353,23 @@ void Diagram::rectClip(int* &p)
     if(code1)  code = code1;
     else  code = code2;
 
-    dx = x_2 - x_1;
+    dx = x_2 - x_1;  // dx and dy never equals zero !
     dy = y_2 - y_1;
     if(code & 1) {
       y = y_1 - dy * x_1 / dx;
-      x = 0;
+      x = 0.0;
     }
     else if(code & 2) {
       y = y_1 + dy * (x2-x_1) / dx;
-      x = x2;
+      x = float(x2);
     }
     else if(code & 4) {
       x = x_1 - dx * y_1 / dy;
-      y = 0;
+      y = 0.0;
     }
     else if(code & 8) {
       x = x_1 + dx * (y2-y_1) / dy;
-      y = y2;
+      y = float(y2);
     }
 
     if(code == code1) {
@@ -378,6 +382,8 @@ void Diagram::rectClip(int* &p)
       y_2 = y;
       code2 = regionCode(x, y);
     }
+    if(code1 & code2)
+      goto endWithHidden; // line not visible at all ?
   }
 
   *(p-4) = x_1;
@@ -385,33 +391,38 @@ void Diagram::rectClip(int* &p)
   *(p-2) = x_2;
   *(p-1) = y_2;
   p += z;
+  return;
+
+endWithHidden:
+    *(p-4) = x_2;
+    *(p-3) = y_2;
+    p -= 2;
 }
 
 // ------------------------------------------------------------
 // Clipping for round diagrams (smith, polar, ...)
-void Diagram::clip(int* &p)
+void Diagram::clip(float* &p)
 {
-  int R = x2 >> 1;
-  int x_1 = *(p-4) - R, y_1 = *(p-3) - R;
-  int x_2 = *(p-2) - R, y_2 = *(p-1) - R;
+  float R = float(x2) / 2.0;
+  float x_1 = *(p-4) - R, y_1 = *(p-3) - R;
+  float x_2 = *(p-2) - R, y_2 = *(p-1) - R;
 
-  int dt1 = R*R;   // square of radius
-  int dt2 = dt1 - x_2*x_2 - y_2*y_2;
+  float dt1 = R*R;   // square of radius
+  float dt2 = dt1 - x_2*x_2 - y_2*y_2;
   dt1 -= x_1*x_1 + y_1*y_1;
 
-  if(dt1 >= 0) if(dt2 >= 0)  return;  // line completly inside ?
+  if(dt1 >= 0.0) if(dt2 >= 0.0)  return;  // line completly inside ?
 
-  if(dt1 < 0) if(*(p-5) >= 0) { // is there already a line end flag ?
+  if(dt1 < 0.0) if(*(p-5) >= 0.0) { // is there already a line end flag ?
     p++;
     *(p-5) = STROKEEND;
   }
 
-  int x = x_1-x_2;
-  int y = y_1-y_2;
-  int C = x_1*x + y_1*y;
-  int D = x*x + y*y;
-  float F = float(C);
-  F = F*F + float(dt1)*float(D); // use float because number is easily > 2^32
+  float x = x_1-x_2;
+  float y = y_1-y_2;
+  float C = x_1*x + y_1*y;
+  float D = x*x + y*y;
+  float F = C*C + dt1*D;
 
   x_1 += R;
   y_1 += R;
@@ -425,9 +436,9 @@ void Diagram::clip(int* &p)
   }
 
   int code = 0;
-  R   = int(sqrt(F)+0.5);
+  R   = sqrt(F);
   dt1 = C - R;
-  if((dt1 > 0) && (dt1 < D)) { // intersection outside start/end point ?
+  if((dt1 > 0.0) && (dt1 < D)) { // intersection outside start/end point ?
     *(p-4) = x_1 - x*dt1 / D;
     *(p-3) = y_1 - y*dt1 / D;
     code |= 1;
@@ -438,7 +449,7 @@ void Diagram::clip(int* &p)
   }
 
   dt2 = C + R;
-  if((dt2 > 0) && (dt2 < D)) { // intersection outside start/end point ?
+  if((dt2 > 0.0) && (dt2 < D)) { // intersection outside start/end point ?
     *(p-2) = x_1 - x*dt2 / D;
     *(p-1) = y_1 - y*dt2 / D;
     *p = STROKEEND;
@@ -466,7 +477,8 @@ void Diagram::calcData(Graph *g)
   if(!pz)  return;
   if(g->cPointsX.count() < 1) return;
 
-  int dx, dy, xtmp, ytmp, tmp, i, z, Counter=2;
+  int i, z, tmp, Counter=2;
+  float dx, dy, xtmp, ytmp;
   int Size = ((2*(g->cPointsX.getFirst()->count) + 1) * g->countY) + 10;
   
   if(xAxis.autoScale)  if(yAxis.autoScale)  if(zAxis.autoScale)
@@ -475,9 +487,9 @@ void Diagram::calcData(Graph *g)
   double Dummy = 0.0;  // not used
   double *py = &Dummy;
 
-  int *p = (int*)malloc( Size*sizeof(int) );  // create memory for points
-  int *p_end;
-  g->Points = p_end = p;
+  float *p = (float*)malloc( Size*sizeof(float) ); // create memory for points
+  float *p_end;
+  g->ScrPoints = p_end = p;
   p_end += Size - 9;   // limit of buffer
 
   Axis *pa;
@@ -486,7 +498,7 @@ void Diagram::calcData(Graph *g)
 
   double Stroke=10.0, Space=10.0; // length of strokes and spaces in pixel
   switch(g->Style) {
-    case 0:      // ***** solid line **********************************
+    case GRAPHSTYLE_SOLID: // ***** solid line ****************************
       for(i=g->countY; i>0; i--) {  // every branch of curves
 	px = g->cPointsX.getFirst()->Points;
 	calcCoordinate(px, pz, py, p, p+1, pa);
@@ -498,14 +510,11 @@ void Diagram::calcData(Graph *g)
 	  if(Counter >= 2)   // clipping only if an axis is manual
 	    clip(p);
 	}
-	switch(*(p-3)) {
-	  case STROKEEND:
-	       p -= 3;  // no single point after "no stroke"
-	       break;
-	  case BRANCHEND:
-	       if((*(p-2) < 0) || (*(p-1) < 0))
-	         p -= 2;  // erase last hidden point
-	       break;
+	if(*(p-3) == STROKEEND)
+	  p -= 3;  // no single point after "no stroke"
+	else if(*(p-3) == BRANCHEND) {
+	  if((*(p-2) < 0) || (*(p-1) < 0))
+	    p -= 2;  // erase last hidden point
 	}
 	*(p++) = BRANCHEND;
       }
@@ -519,26 +528,25 @@ for(int zz=0; zz<z; zz+=2)
   qDebug("c: %d/%d", *(p+zz), *(p+zz+1));*/
       return;
 
-    case 1: Stroke = 10.0; Space =  6.0;  break;   // dash line
-    case 2: Stroke =  2.0; Space =  4.0;  break;   // dot line
-    case 3: Stroke = 24.0; Space =  8.0;  break;   // long dash line
-
-    default:  // symbol at each point ************************************
-      if(g->Style == 4)  xtmp = GRAPHSTAR;
-      else if(g->Style == 5)  xtmp = GRAPHCIRCLE;
-      else if(g->Style == 6)  xtmp = GRAPHARROW;
+    case GRAPHSTYLE_DASH:
+      Stroke = 10.0; Space =  6.0;
+      break;
+    case GRAPHSTYLE_DOT:
+      Stroke =  2.0; Space =  4.0;
+      break;
+    case GRAPHSTYLE_LONGDASH:
+      Stroke = 24.0; Space =  8.0;
+      break;
 
+    default:  // symbol (e.g. star) at each point **********************
       for(i=g->countY; i>0; i--) {  // every branch of curves
-	px = g->cPointsX.getFirst()->Points;
-	for(z=g->cPointsX.getFirst()->count; z>0; z--) {  // every point
-	  FIT_MEMORY_SIZE;  // need to enlarge memory block ?
-	  calcCoordinate(px, pz, py, p, p+1, pa);
-	  if(insideDiagram(*p, *(p+1))) {    // within diagram ?
-	    *(p+2) = xtmp;
-	    p += 3;
-	  }
-	}
-	*(p++) = BRANCHEND;
+        px = g->cPointsX.getFirst()->Points;
+        for(z=g->cPointsX.getFirst()->count; z>0; z--) {  // every point
+          calcCoordinate(px, pz, py, p, p+1, pa);
+          if(insideDiagram(*p, *(p+1)))    // within diagram ?
+            p += 2;
+        }
+        *(p++) = BRANCHEND;
       }
       *p = GRAPHEND;
 /*qDebug("\n******");
@@ -576,8 +584,8 @@ for(int zz=0; zz<60; zz+=2)
       while(dist > 0) {   // stroke or space finished ?
 	FIT_MEMORY_SIZE;  // need to enlarge memory block ?
 
-	*(p++) = xtmp - int(dist*cos(alpha) + 0.5); // linearly interpolate
-	*(p++) = ytmp - int(dist*sin(alpha) + 0.5);
+	*(p++) = xtmp - float(dist*cos(alpha)); // linearly interpolate
+	*(p++) = ytmp - float(dist*sin(alpha));
 	if((++Counter) >= 2)  clip(p);
 
         if(Flag == 0) {
@@ -599,14 +607,12 @@ for(int zz=0; zz<60; zz+=2)
       }
 
     } // of x loop
-    switch(*(p-3)) {
-      case STROKEEND:
-           p -= 3;  // no single point after "no stroke"
-           break;
-      case BRANCHEND:
-           if((*(p-2) < 0) || (*(p-1) < 0))
-             p -= 2;  // erase last hidden point
-           break;
+
+    if(*(p-3) == STROKEEND)
+      p -= 3;  // no single point after "no stroke"
+    else if(*(p-3) == BRANCHEND) {
+      if((*(p-2) < 0) || (*(p-1) < 0))
+        p -= 2;  // erase last hidden point
     }
     *(p++) = BRANCHEND;
   } // of y loop
@@ -641,31 +647,115 @@ bool Diagram::getSelected(int x_, int y_)
 // ------------------------------------------------------------
 // Checks if the resize area was clicked. If so return "true" and sets
 // x1/y1 and x2/y2 to the border coordinates to draw a rectangle.
-bool Diagram::ResizeTouched(int x, int y)
+bool Diagram::resizeTouched(float fX, float fY, float len)
 {
-  if(x < cx-5) return false;
-  if(x > cx+x2+5) return false;
-  if(y < cy-y2-5) return false;
-  if(y > cy+5) return false;
+  float fCX = float(cx), fCY = float(cy);
+  float fX2 = float(cx+x2), fY2 = float(cy-y2);
+  if(fX < fCX-len) return false;
+  if(fX > fX2+len) return false;
+  if(fY < fY2-len) return false;
+  if(fY > fCY+len) return false;
 
   State = 0;
-  if(x < cx+5) State = 1;
-  else  if(x <= cx+x2-5) return false;
-  if(y > cy-5) State |= 2;
-  else  if(y >= cy-y2+5) return false;
+  if(fX < fCX+len) State = 1;
+  else  if(fX <= fX2-len) return false;
+  if(fY > fCY-len) State |= 2;
+  else  if(fY >= fY2+len) return false;
 
   return true;
 }
 
+// --------------------------------------------------------------------------
+void Diagram::getAxisLimits(Graph *pg)
+{
+  int z;
+  double x, y, *p;
+  DataX *pD = pg->cPointsX.first();
+  if(pD == 0) return;
+
+  if(Name[0] != 'C') {   // not for location curves
+    p = pD->Points;
+    for(z=pD->count; z>0; z--) { // check x coordinates (1. dimension)
+      x = *(p++);
+      if(finite(x)) {
+	if(x > xAxis.max) xAxis.max = x;
+	if(x < xAxis.min) xAxis.min = x;
+      }
+    }
+  }
+
+  if(Name == "Rect3D") {
+    pD = pg->cPointsX.next();
+    if(pD) {
+      p = pD->Points;
+      for(z=pD->count; z>0; z--) { // check y coordinates (2. dimension)
+	y = *(p++);
+	if(finite(y)) {
+	  if(y > yAxis.max) yAxis.max = y;
+	  if(y < yAxis.min) yAxis.min = y;
+	}
+      }
+    }
+  }
+
+  Axis *pa;
+  if(pg->yAxisNo == 0)  pa = &yAxis;
+  else  pa = &zAxis;
+  (pa->numGraphs)++;    // count graphs
+  p = pg->cPointsY;
+  for(z=pg->countY*pD->count; z>0; z--) {  // check every y coordinate
+    x = *(p++);
+    y = *(p++);
+
+    if(Name[0] != 'C') {
+      if(fabs(y) >= 1e-250) x = sqrt(x*x+y*y);
+      if(finite(x)) {
+	if(x > pa->max) pa->max = x;
+	if(x < pa->min) pa->min = x;
+      }
+    }
+    else {   // location curve needs different treatment
+      if(finite(x)) {
+	if(x > xAxis.max) xAxis.max = x;
+	if(x < xAxis.min) xAxis.min = x;
+      }
+      if(finite(y)) {
+	if(y > pa->max) pa->max = y;
+	if(y < pa->min) pa->min = y;
+      }
+    }
+  }
+}
+
 // --------------------------------------------------------------------------
 void Diagram::loadGraphData(const QString& defaultDataSet)
 {
+  int yNum = yAxis.numGraphs;
+  int zNum = zAxis.numGraphs;
   yAxis.numGraphs = zAxis.numGraphs = 0;
+
+  double xmin = xAxis.min, ymin = yAxis.min, zmin = zAxis.min;
+  double xmax = xAxis.max, ymax = yAxis.max, zmax = zAxis.max;
   yAxis.min = zAxis.min = xAxis.min =  DBL_MAX;
   yAxis.max = zAxis.max = xAxis.max = -DBL_MAX;
 
-  for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next())
-    loadVarData(defaultDataSet, pg);  // load data, determine max/min values
+  int No=0;
+  for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next()) {
+    if(loadVarData(defaultDataSet, pg) != 1)   // load data, determine max/min values
+      No++;
+    else
+      getAxisLimits(pg);
+    pg->lastLoaded = QDateTime::currentDateTime();
+  }
+
+  if(No <= 0) {   // All dataset files unchanged ?
+    yAxis.numGraphs = yNum;  // rebuild scrollbar position
+    zAxis.numGraphs = zNum;
+
+    xAxis.min = xmin; yAxis.min = ymin; zAxis.min = zmin;
+    xAxis.max = xmax; yAxis.max = ymax; zAxis.max = zmax;
+    return;    // -> no update neccessary
+  }
 
   if(xAxis.min > xAxis.max)
     xAxis.min = xAxis.max = 0.0;
@@ -687,61 +777,11 @@ void Diagram::recalcGraphData()
 {
   yAxis.min = zAxis.min = xAxis.min =  DBL_MAX;
   yAxis.max = zAxis.max = xAxis.max = -DBL_MAX;
+  yAxis.numGraphs = zAxis.numGraphs = 0;
 
-  int z;
-  double x, y, *p;
   // get maximum and minimum values
-  for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next()) {
-    DataX *pD = pg->cPointsX.first();
-    if(pD == 0) continue;
-    if(Name[0] != 'C') {   // not for location curves
-      p = pD->Points;
-      for(z=pD->count; z>0; z--) { // check x coordinates (1. dimension)
-        x = *(p++);
-        if(x > xAxis.max) xAxis.max = x;
-        if(x < xAxis.min) xAxis.min = x;
-      }
-    }
-
-    if(Name == "Rect3D") {
-      pD = pg->cPointsX.next();
-      if(pD) {
-        p = pD->Points;
-        for(z=pD->count; z>0; z--) { // check y coordinates (2. dimension)
-          y = *(p++);
-          if(y > yAxis.max) yAxis.max = y;
-          if(y < yAxis.min) yAxis.min = y;
-        }
-      }
-    }
-
-    Axis *pa;
-    if(pg->yAxisNo == 0)  pa = &yAxis;
-    else  pa = &zAxis;
-    p = pg->cPointsY;
-    for(z=pg->countY*pD->count; z>0; z--) {  // check every y coordinate
-      x = *(p++);
-      y = *(p++);
-    
-      if(Name[0] != 'C') {
-        if(fabs(y) >= 1e-250) x = sqrt(x*x+y*y);
-        if(finite(x)) {
-          if(x > pa->max) pa->max = x;
-          if(x < pa->min) pa->min = x;
-        }
-      }
-      else {   // location curve needs different treatment
-        if(finite(x)) {
-          if(x > xAxis.max) xAxis.max = x;
-          if(x < xAxis.min) xAxis.min = x;
-        }
-        if(finite(y)) {
-          if(y > pa->max) pa->max = y;
-          if(y < pa->min) pa->min = y;
-        }
-      }
-    }
-  }
+  for(Graph *pg = Graphs.first(); pg != 0; pg = Graphs.next())
+    getAxisLimits(pg);
 
   if(xAxis.min > xAxis.max) {
     xAxis.min = 0.0;
@@ -770,10 +810,16 @@ void Diagram::updateGraphData()
 
   Graph *pg;
   for(pg = Graphs.first(); pg != 0; pg = Graphs.next()) {
-    if(pg->Points != 0) { free(pg->Points);  pg->Points = 0; }
+    if(pg->ScrPoints != 0) {
+      free(pg->ScrPoints);
+      pg->ScrPoints = 0;
+    }
     if((valid & (pg->yAxisNo+1)) != 0)
       calcData(pg);   // calculate screen coordinates
-    else if(pg->cPointsY) { delete[] pg->cPointsY;  pg->cPointsY = 0; }
+    else if(pg->cPointsY) {
+      delete[] pg->cPointsY;
+      pg->cPointsY = 0;
+    }
   }
 
   createAxisLabels();  // virtual function
@@ -786,27 +832,46 @@ void Diagram::updateGraphData()
 }
 
 // --------------------------------------------------------------------------
-bool Diagram::loadVarData(const QString& fileName, Graph *g)
+int Diagram::loadVarData(const QString& fileName, Graph *g)
 {
-  g->countY = 0;
-  g->cPointsX.clear();
-  if(g->cPointsY) { delete[] g->cPointsY;  g->cPointsY = 0; }
-  if(g->Var.isEmpty()) return false;
-
   QFile file;
   QString Variable;
+  QFileInfo Info(fileName);
+
   int pos = g->Var.find(':');
+//  if(g->Var.right(3) == "].X")  // e.g. stdl[8:0].X
+//    if(pos > g->Var.find('['))
+//      pos = -1;
+
+  /* WORK-AROUND: A bug in SCIM (libscim) which Qt is linked to causes
+     to change the locale to the default. */
+  setlocale (LC_NUMERIC, "C");
+
   if(pos <= 0) {
     file.setName(fileName);
     Variable = g->Var;
   }
   else {
-    QFileInfo Info(fileName);
     file.setName(Info.dirPath()+QDir::separator() + g->Var.left(pos)+".dat");
     Variable = g->Var.mid(pos+1);
   }
 
-  if(!file.open(IO_ReadOnly))  return false;
+  Info.setFile(file);
+  if(g->lastLoaded.isValid())
+    if(g->lastLoaded > Info.lastModified())
+      return 1;    // dataset unchanged -> no update neccessary
+
+  g->countY = 0;
+  g->cPointsX.clear();
+  if(g->cPointsY) { delete[] g->cPointsY;  g->cPointsY = 0; }
+  if(Variable.isEmpty()) return 0;
+
+  if(Variable.right(2) == ".X")
+    if(Name.at(0) != 'T')
+      return 0;  // digital variables only for tabulars and ziming diagram
+
+
+  if(!file.open(IO_ReadOnly))  return 0;
 
   // *****************************************************************
   // To strongly speed up the file read operation the whole file is
@@ -815,9 +880,9 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
   FileContent = file.readAll();
   file.close();
   char *FileString = FileContent.data();
-  if(!FileString)  return false;
+  if(!FileString)  return 0;
   char *pPos = FileString+FileContent.size()-1;
-  if(*pPos > ' ')  if(*pPos != '>')  return false;
+  if(*pPos > ' ')  if(*pPos != '>')  return 0;
   *pPos = 0;
 
 
@@ -838,12 +903,12 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
     pFile = strstr(pFile+4, Variable.latin1());
   }
 
-  if(!pFile)  return false;   // data not found
+  if(!pFile)  return 0;   // data not found
 
   QString Line, tmp;
   pFile += Variable.length();
   pPos = strchr(pFile, '>');
-  if(!pPos)  return false;   // file corrupt
+  if(!pPos)  return 0;   // file corrupt
   *pPos = 0;
   Line = QString(pFile);
   *pPos = '>';
@@ -867,7 +932,7 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
   if(isIndep) {    // create independent variable by myself ?
     counting = Line.toInt(&ok);  // get number of values
     g->cPointsX.append(new DataX("number", 0, counting));
-    if(!ok)  return false;
+    if(!ok)  return 0;
 
     p = new double[counting];  // memory of new independent variable
     g->countY = 1;
@@ -886,12 +951,12 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
     for(DataX *pD = g->cPointsX.last(); pD!=0; pD = g->cPointsX.prev()) {
       pa = &xAxis;
       if(pD == g->cPointsX.getFirst()) {
-	xAxis.min = min_tmp;    // only count first independent variable
-	xAxis.max = max_tmp;
+        xAxis.min = min_tmp;    // only count first independent variable
+        xAxis.max = max_tmp;
       }
       else if(pD == bLast)  pa = &yAxis;   // y axis for Rect3D
       counting = loadIndepVarData(pD->Var, FileString, pa, g);
-      if(counting <= 0)  return false;
+      if(counting <= 0)  return 0;
 
       g->countY *= counting;
     }
@@ -911,11 +976,12 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
   char *pEnd;
   double x, y;
   pPos = pFile;
-  // find first position containing no whitespace
-  while((*pPos) && (*pPos <= ' '))  pPos++;
+
+if(Variable.right(3) != ".X ")
 
   for(int z=counting; z>0; z--) {
     pEnd = 0;
+    while((*pPos) && (*pPos <= ' '))  pPos++; // find start of next number
     x = strtod(pPos, &pEnd);  // real part
     pPos = pEnd + 1;
     if(*pEnd < ' ')   // is there an imaginary part ?
@@ -923,7 +989,7 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
     else {
       if(((*pEnd != '+') && (*pEnd != '-')) || (*pPos != 'j')) {
         delete[] g->cPointsY;  g->cPointsY = 0;
-        return false;
+        return 0;
       }
       *pPos = *pEnd;  // overwrite 'j' with sign
       pEnd = 0;
@@ -950,11 +1016,39 @@ bool Diagram::loadVarData(const QString& fileName, Graph *g)
         if(y < pa->min) pa->min = y;
       }
     }
-
-    while((*pPos) && (*pPos <= ' '))  pPos++; // find start of next number
   }
 
-  return true;
+
+else {  // of "if not digital"
+
+  char *pc = (char*)p;
+  pEnd = pc + 2*(counting-1)*sizeof(double);
+  // for digital variables (e.g. 100ZX0):
+  for(int z=counting; z>0; z--) {
+
+    while((*pPos) && (*pPos <= ' '))  pPos++; // find start of next bit vector
+    if(*pPos == 0) {
+      delete[] g->cPointsY;  g->cPointsY = 0;
+      return 0;
+    }
+
+    while(*pPos > ' ') {    // copy bit vector
+      *(pc++) = *(pPos++);
+      if(pEnd <= pc) {
+        counting = pc - (char*)g->cPointsY;
+        pc = (char*)realloc(g->cPointsY, counting+1024);
+        pEnd = pc;
+        g->cPointsY = (double*)pEnd;
+        pc += counting;
+        pEnd += counting+1020;
+      }
+    }
+    *(pc++) = 0;   // terminate each vector with NULL
+  }
+
+}  // of "if not digital"
+
+  return 2;
 }
 
 // --------------------------------------------------------------------------
@@ -964,6 +1058,11 @@ int Diagram::loadIndepVarData(const QString& Variable,
 {
   bool isIndep = false;
   QString Line, tmp;
+
+  /* WORK-AROUND: A bug in SCIM (libscim) which Qt is linked to causes
+     to change the locale to the default. */
+  setlocale (LC_NUMERIC, "C");
+
   Line = "dep "+Variable+" ";
   // "pFile" is used through-out the whole function and must NOT used
   // for other purposes!
@@ -1338,44 +1437,43 @@ void Diagram::createSmithChart(Axis *Axis, int Mode)
   double rMAXq = Axis->up*Axis->up;
   int    theta, beta, phi, len, m, x, y;
 
+  int R1 = int(x2/Axis->up + 0.5);
   // ....................................................
   // draw arcs with im(z)=const
-
   for(m=1; m<GridY; m++) {
     n_sin = M_PI*double(m)/double(GridY);
     n_cos = cos(n_sin);
     n_sin = sin(n_sin);
-    im = (1-n_cos)/n_sin * pow(Axis->up,0.7); // up^0.7 is beauty correction
-    x  = int((1-im)/Axis->up*dx2);
-    y  = int(im/Axis->up*x2);
+    im = (1.0-n_cos)/n_sin * pow(Axis->up,0.7); // up^0.7 is beauty correction
+    y  = int(im/Axis->up*x2 + 0.5);  // diameter
 
     if(Axis->up <= 1.0) {       // Smith chart with |r|=1
-      beta  = int(16.0*180.0*atan2(n_sin-im,n_cos-1)/M_PI - 0.5);
+      beta  = int(16.0*180.0*atan2(n_sin-im,n_cos-1.0)/M_PI - 0.5);
       if(beta<0) beta += 16*360;
       theta = 16*270-beta;
     }
     else {         // Smith chart with |r|>1
-      im = 1/im;
-      real = (rMAXq+1)/(rMAXq-1);
-      root =  real*real - im*im-1;
-      if(root<0) {       // circle lies completely within the Smith chart ?
-        beta = 0;        // yes, ...
-        theta = 16*360;  // ... draw whole circle
+      im = 1.0/im;
+      real = (rMAXq+1.0)/(rMAXq-1.0);
+      root =  real*real - im*im - 1.0;
+      if(root < 0.0) {  // circle lies completely within the Smith chart ?
+        beta = 0;       // yes, ...
+        theta = 16*360; // ... draw whole circle
       }
       else {
 	// calculate both intersections with most outer circle
 	real1 =  sqrt(root)-real;
 	real2 = -sqrt(root)-real;
 
-	root  = (real1+1)*(real1+1) + im*im;
-	n_cos = (real1*real1 + im*im - 1) / root;
-	n_sin = 2*im / root;
-	beta  = int(16.0*180.0*atan2(n_sin-1/im,n_cos-1)/M_PI);
+	root  = (real1+1.0)*(real1+1.0) + im*im;
+	n_cos = (real1*real1 + im*im - 1.0) / root;
+	n_sin = 2.0*im / root;
+	beta  = int(16.0*180.0*atan2(n_sin-1.0/im,n_cos-1.0)/M_PI);
 	if(beta<0) beta += 16*360;
 
-	root  = (real2+1)*(real2+1) + im*im;
-	n_cos = (real2*real2 + im*im - 1) / root;
-	n_sin = 2*im / root;
+	root  = (real2+1.0)*(real2+1.0) + im*im;
+	n_cos = (real2*real2 + im*im - 1.0) / root;
+	n_sin = 2.0*im / root;
 	theta  = int(16.0*180.0*atan2(n_sin-1/im,n_cos-1)/M_PI);
 	if(theta<0) theta += 16*360;
 	theta = theta - beta;   // arc length
@@ -1384,9 +1482,9 @@ void Diagram::createSmithChart(Axis *Axis, int Mode)
     }
 
     if(Zplane)
-      x += dx2;
+      x = (x2 + R1 - y) >> 1;
     else {
-      x -= dx2;
+      x = (x2 - R1 - y) >> 1;
       beta = 16*180 - beta - theta;  // mirror
       if(beta < 0) beta += 16*360;   // angle has to be > 0
     }
@@ -1406,13 +1504,14 @@ void Diagram::createSmithChart(Axis *Axis, int Mode)
 
   for(m=1; m<GridX; m++) {
     im = m*(Axis->up+1.0)/GridX - Axis->up;
-    x  = int(im/Axis->up*double(dx2) + 0.5);  // center
-    y  = int((1.0-im)/Axis->up*double(dx2));  // diameter
+    y  = int((1.0-im)/Axis->up*double(dx2) + 0.5);  // diameter
 
-    if(Zplane)  x += dx2;
-    else  x = 0;
+    if(Zplane)
+      x = ((x2+R1)>>1) - y;
+    else
+      x = (x2-R1)>>1;
     if(fabs(fabs(im)-1.0) > 0.2)   // if too near to |r|=1, it looks ugly
-      Arcs.append(new struct Arc(x, dx2+(y>>1), y, y, beta, theta, GridPen));
+      Arcs.append(new struct Arc(x, (x2+y)>>1, y, y, beta, theta, GridPen));
 
     if(Axis->up > 1.0) {  // draw arcs on the rigth-handed side ?
       im = 1.0-im;
@@ -1420,14 +1519,14 @@ void Diagram::createSmithChart(Axis *Axis, int Mode)
       if(Zplane)  x += y;
       else  x -= y;
       if(im >= 1.0)
-	Arcs.append(new struct Arc(x, dx2+(y>>1), y, y, beta, theta, GridPen));
+        Arcs.append(new struct Arc(x, (x2+y)>>1, y, y, beta, theta, GridPen));
       else {
-	phi = int(16.0*180.0/M_PI*acos(im));
-	len = 16*180-phi;
-	if(Above && Below)  len += len;
-	else if(Below)  phi = 16*180;
-	if(!Zplane)  phi += 16*180;
-	Arcs.append(new struct Arc(x, dx2+(y>>1), y, y, phi, len, GridPen));
+        phi = int(16.0*180.0/M_PI*acos(im));
+        len = 16*180-phi;
+        if(Above && Below)  len += len;
+        else if(Below)  phi = 16*180;
+        if(!Zplane)  phi += 16*180;
+        Arcs.append(new struct Arc(x, (x2+y)>>1, y, y, phi, len, GridPen));
       }
     }
   }
@@ -1435,15 +1534,13 @@ void Diagram::createSmithChart(Axis *Axis, int Mode)
 
   // ....................................................
   if(Axis->up > 1.0) {  // draw circle with |r|=1 ?
-    x = int(x2/Axis->up + 0.5);
-    Arcs.append(new struct Arc(dx2-(x>>1), dx2+(x>>1), x, x, beta, theta,
-			QPen(QPen::black,0)));
+    x = (x2-R1) >> 1;
+    y = (x2+R1) >> 1;
+    Arcs.append(new struct Arc(x, y, R1, R1, beta, theta, QPen(QPen::black,0)));
 
     // vertical line Re(r)=1 (visible only if |r|>1)
-    x = int(x2/Axis->up + 0.5) >> 1;
+    if(Zplane)  x = y;
     y = int(sqrt(rMAXq-1)/Axis->up*dx2 + 0.5);
-    if(Zplane)  x += dx2;
-    else  x = dx2 - x;
     if(Above)  m = y;
     else  m = 0;
     if(!Below)  y = 0;
