@@ -15,10 +15,21 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "polardiagram.h"
-#include "main.h"
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <math.h>
+#include <float.h>
+#ifdef __MINGW32__
+# define finite(x) _finite(x)
+#endif
+#if HAVE_IEEEFP_H
+# include <ieeefp.h>
+#endif
+
+#include "polardiagram.h"
+#include "main.h"
 
 
 PolarDiagram::PolarDiagram(int _cx, int _cy) : Diagram(_cx, _cy)
@@ -40,12 +51,18 @@ PolarDiagram::~PolarDiagram()
 
 // ------------------------------------------------------------
 void PolarDiagram::calcCoordinate(double* &, double* &yD, double* &,
-				  int *px, int *py, Axis*)
+				  float *px, float *py, Axis*)
 {
   double yr = *(yD++);
   double yi = *(yD++);
-  *px = (x2>>1)+int(yr/yAxis.up*double(x2)/2.0 + 0.5);
-  *py = (y2>>1)+int(yi/yAxis.up*double(y2)/2.0 + 0.5);
+  *px = float((yr/yAxis.up + 1.0)*double(x2)/2.0);
+  *py = float((yi/yAxis.up + 1.0)*double(y2)/2.0);
+
+  if(finite(*px))
+    if(finite(*py))
+      return;
+
+  *px = *py = float(cx) / 2.0;
 }
 
 // --------------------------------------------------------------
