@@ -15,10 +15,21 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "psdiagram.h"
-#include "main.h"
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #include <math.h>
+#include <float.h>
+#ifdef __MINGW32__
+# define finite(x) _finite(x)
+#endif
+#if HAVE_IEEEFP_H
+# include <ieeefp.h>
+#endif
+
+#include "psdiagram.h"
+#include "main.h"
 
 // mixed polar and smith diagram
 PSDiagram::PSDiagram(int _cx, int _cy, bool _polarUp)
@@ -42,12 +53,18 @@ PSDiagram::~PSDiagram()
 
 // ------------------------------------------------------------
 void PSDiagram::calcCoordinate(double* &, double* &yD, double* &,
-			       int *px, int *py, Axis *pa)
+                               float *px, float *py, Axis *pa)
 {
   double yr = *(yD++);
   double yi = *(yD++);
-  *px = (x2>>1)+int(yr/pa->up*double(x2)/2.0 + 0.5);
-  *py = (y2>>1)+int(yi/pa->up*double(y2)/2.0 + 0.5);
+  *px = float((yr/pa->up + 1.0)*double(x2)/2.0);
+  *py = float((yi/pa->up + 1.0)*double(y2)/2.0);
+
+  if(finite(*px))
+    if(finite(*py))
+      return;
+
+  *px = *py = float(cx) / 2.0;
 }
 
 // --------------------------------------------------------------
