@@ -15,10 +15,20 @@
  *                                                                         *
  ***************************************************************************/
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <math.h>
 #include <float.h>
+#ifdef __MINGW32__
+# define finite(x) _finite(x)
+#endif
 #include <limits.h>
+#if HAVE_IEEEFP_H
+# include <ieeefp.h>
+#endif
 
 #include "rect3ddiagram.h"
 #include "main.h"
@@ -110,12 +120,12 @@ int Rect3DDiagram::calcCross(int *Xses, int *Yses)
 
   scaleX = double(x2) / (XMAX_2D - XMIN_2D); // scaling 3D -> 2D transformation
   scaleY = double(y2) / (YMAX_2D - YMIN_2D);
-  xorig  = -int(XMIN_2D * scaleX);   // position of origin
-  yorig  = -int(YMIN_2D * scaleY);
+  xorig  = -XMIN_2D * scaleX;   // position of origin
+  yorig  = -YMIN_2D * scaleY;
 
   for(z=0; z<8; z++) {  // calculate 2D coordinates of all corners
-    *(Xses+z) = int(x2D[z] * scaleX + 0.5) + xorig;
-    *(Yses+z) = int(y2D[z] * scaleY + 0.5) + yorig;
+    *(Xses+z) = int(x2D[z] * scaleX + 0.5 + xorig);
+    *(Yses+z) = int(y2D[z] * scaleY + 0.5 + yorig);
   }
   return Center;
 }
@@ -123,7 +133,7 @@ int Rect3DDiagram::calcCross(int *Xses, int *Yses)
 // ------------------------------------------------------------
 // Is needed for markers.
 void Rect3DDiagram::calcCoordinate(double* &xD, double* &zD, double* &yD,
-				   int *px, int *py, Axis*)
+                                   float *px, float *py, Axis*)
 {
   double x3D = *(zD++);
   double y3D = *(zD++);
@@ -162,8 +172,15 @@ void Rect3DDiagram::calcCoordinate(double* &xD, double* &zD, double* &yD,
   else
     y3D = (*yD - yAxis.low) / (yAxis.up - yAxis.low);
 
-  *px = int(calcX_2D(x3D, y3D, z3D) + 0.5) + xorig;
-  *py = int(calcY_2D(x3D, y3D, z3D) + 0.5) + yorig;
+  *px = float(calcX_2D(x3D, y3D, z3D)) + xorig;
+  *py = float(calcY_2D(x3D, y3D, z3D)) + yorig;
+
+  if(finite(*px))
+    if(finite(*py))
+      return;
+
+  *px = float(xorig);
+  *py = float(yorig);
 }
 
 // ------------------------------------------------------------
@@ -203,8 +220,8 @@ void Rect3DDiagram::calcCoordinate3D(double x, double y, double zr, double zi,
   else
     y = (y - yAxis.low) / (yAxis.up - yAxis.low);
 
-  p->x  = int(calcX_2D(x, y, zr) + 0.5) + xorig;
-  p->y  = int(calcY_2D(x, y, zr) + 0.5) + yorig;
+  p->x  = int(calcX_2D(x, y, zr) + 0.5 + xorig);
+  p->y  = int(calcY_2D(x, y, zr) + 0.5 + yorig);
   p->No = pz->No = p-Mem;
   p->done = 0;
   pz->z = float(calcZ_2D(x, y, zr));
@@ -320,6 +337,19 @@ void Rect3DDiagram::calcLine(tPoint3D* &p, tPoint3D* &MemEnd,
   p->done |= 1;   // mark as already worked on
 }
 
+// --------------------------------------------------------------
+// Compare functions for GNU qsort routine.
+int Rect3DDiagram::comparePoint3D(const void *Point1, const void *Point2)
+{
+  return ((tPoint3D*)Point1)->No - ((tPoint3D*)Point2)->No;
+}
+int Rect3DDiagram::comparePointZ(const void *Point1, const void *Point2)
+{
+  if((((tPointZ*)Point2)->z - ((tPointZ*)Point1)->z) < 0.0f)
+    return -1;
+  return 1;
+}
+
 // --------------------------------------------------------------
 // Removes the invisible parts of the graph.
 void Rect3DDiagram::removeHiddenLines(char *zBuffer, tBound *Bounds)
@@ -364,7 +394,7 @@ void Rect3DDiagram::removeHiddenLines(char *zBuffer, tBound *Bounds)
     
       for(j=dx; j>0; j--) { // x coordinates
         calcCoordinate3D(*(px++), *py, *pz, *(pz+1), pMem++, zp++);
-	pz += 2;
+        pz += 2;
       }
 
       (pMem-1)->done |= 8;  // mark as "last in line"
@@ -372,6 +402,7 @@ void Rect3DDiagram::removeHiddenLines(char *zBuffer, tBound *Bounds)
       if(dy > 0) if((i % dy) == 0)
         py = g->cPointsX.at(1)->Points;
     }
+    (pMem-1)->done |= 512;  // mark as "last point before grid"
 
     // ..........................................
     // copy points for cross lines ("dx", "dy" still unchanged ! )
@@ -447,21 +478,10 @@ void Rect3DDiagram::removeHiddenLines(char *zBuffer, tBound *Bounds)
 
 
   // ..........................................
-  // bubble sort algorithm -> sort z-coordinates (greatest first).
+  // Sort z-coordinates (greatest first).
   // After this the polygons that have the smallest distance to the
   // viewer are on top of the list and thus, will be processed first.
-  tPointZ vPZ;
-  for(i=1; i<=Size; i++) {
-    zp = zMem;
-    for(j=0; j<(Size-i); j++) {
-      if(zp->z < (zp+1)->z) {
-	vPZ = *zp;
-	*zp = *(zp+1);
-	*(zp+1) = vPZ;
-      }
-      zp++;
-    }
-  }
+  qsort(zMem, Size, sizeof(tPointZ), comparePointZ);
 
 #if 0
   qDebug("--------------------------- z sorting");
@@ -524,20 +544,8 @@ void Rect3DDiagram::removeHiddenLines(char *zBuffer, tBound *Bounds)
 
   free(zMem);
 
-  // bubble sort algorithm -> sort "No" (least one first)
-  Size = pMem - Mem;
-  tPoint3D v3D;
-  for(i=1; i<=Size; i++) {
-    p = Mem;
-    for(j=0; j<(Size-i); j++) {
-      if(p->No > (p+1)->No) {
-	v3D = *p;
-	*p = *(p+1);
-	*(p+1) = v3D;
-      }
-      p++;
-    }
-  }
+  // sort "No" (least one first)
+  qsort(Mem, pMem - Mem, sizeof(tPoint3D), comparePoint3D);
 
 #if 0
   qDebug("--------------------------- last sorting %d", pMem-Mem);
@@ -934,14 +942,14 @@ void Rect3DDiagram::calcData(Graph *g)
   double *py;
   if(g->countY > 1)  py = g->cPointsX.at(1)->Points;
 
-  int *p = (int*)malloc( Size*sizeof(int) );  // create memory for points
-  int *p_end;
-  g->Points = p_end = p;
+  float *p = (float*)malloc( Size*sizeof(float) );  // create memory for points
+  float *p_end;
+  g->ScrPoints = p_end = p;
   p_end += Size - 9;   // limit of buffer
 
 
   *(p++) = STROKEEND;
-  int dx=0, dy=0, xtmp=0, ytmp=0;
+  float dx=0.0, dy=0.0, xtmp=0.0, ytmp=0.0;
   double Stroke=10.0, Space=10.0; // length of strokes and spaces in pixel
   switch(g->Style) {
     case 0:      // ***** solid line **********************************
@@ -973,15 +981,17 @@ void Rect3DDiagram::calcData(Graph *g)
       *p = GRAPHEND;
       return;
 
-    case 1: Stroke = 10.0; Space =  6.0;  break;   // dash line
-    case 2: Stroke =  2.0; Space =  4.0;  break;   // dot line
-    case 3: Stroke = 24.0; Space =  8.0;  break;   // long dash line
-
-    default:   // symbol at each point ******************************
-      if(g->Style == 4)  xtmp = GRAPHSTAR;
-      else if(g->Style == 5)  xtmp = GRAPHCIRCLE;
-      else if(g->Style == 6)  xtmp = GRAPHARROW;
+    case GRAPHSTYLE_DASH:
+      Stroke = 10.0; Space =  6.0;
+      break;
+    case GRAPHSTYLE_DOT:
+      Stroke =  2.0; Space =  4.0;
+      break;
+    case GRAPHSTYLE_LONGDASH:
+      Stroke = 24.0; Space =  8.0;
+      break;
 
+    default:  // symbol (e.g. star) at each point **********************
       do {
         while(1) {
           if(pMem->done & 11)   // is grid point ?
@@ -992,16 +1002,14 @@ void Rect3DDiagram::calcData(Graph *g)
               }
               else  break;
 
-          FIT_MEMORY_SIZE;  // need to enlarge memory block ?
           *(p++) = pMem->x;
           *(p++) = pMem->y;
-          *(p++) = xtmp;
           break;
         }
 
-        FIT_MEMORY_SIZE;  // need to enlarge memory block ?
-        if(pMem->done & 8)  *(p++) = BRANCHEND;  // new branch
-      } while(((pMem++)->done & 256) == 0);
+        if(pMem->done & 8)
+          *(p++) = BRANCHEND;  // new branch
+      } while(((pMem++)->done & 512) == 0);
       *p = GRAPHEND;
       return;
   }
@@ -1046,8 +1054,8 @@ void Rect3DDiagram::calcData(Graph *g)
         while(dist > 0) {   // stroke or space finished ?
 	  FIT_MEMORY_SIZE;  // need to enlarge memory block ?
 
-	  *(p++) = xtmp - int(dist*cos(alpha) + 0.5); // linearly interpolate
-	  *(p++) = ytmp - int(dist*sin(alpha) + 0.5);
+	  *(p++) = xtmp - float(dist*cos(alpha)); // linearly interpolate
+	  *(p++) = ytmp - float(dist*sin(alpha));
 
           if(Flag == 0) {
             dist -= Stroke;
@@ -1072,14 +1080,11 @@ void Rect3DDiagram::calcData(Graph *g)
     dy = ytmp;
 
     if(pMem->done & 8) {
-      switch(*(p-3)) {
-        case STROKEEND:
-             p -= 3;  // no single point after "no stroke"
-             break;
-        case BRANCHEND:
-             if((*(p-2) < 0) || (*(p-1) < 0))
-               p -= 2;  // erase last hidden point
-             break;
+      if(*(p-3) == STROKEEND)
+        p -= 3;  // no single point after "no stroke"
+      else if(*(p-3) == BRANCHEND) {
+        if((*(p-2) < 0) || (*(p-1) < 0))
+          p -= 2;  // erase last hidden point
       }
       *(p++) = BRANCHEND;  // new branch
       Counter = 0;
@@ -1116,7 +1121,7 @@ void Rect3DDiagram::createAxisLabels()
 }
 
 // ------------------------------------------------------------
-bool Rect3DDiagram::insideDiagram(int x, int y)
+bool Rect3DDiagram::insideDiagram(float x, float y)
 {
   return (regionCode(x, y) == 0);
 }
