@@ -75,7 +75,7 @@ void TabDiagram::paint(ViewPainter *p)
     Points.setPoints(3, x, dy, (x+dx)>>1, y, dx, dy);
     p->Painter->drawConvexPolygon(Points);
     p->Painter->setPen(QColor(224, 224, 224));
-    p->drawLine(x, dy, (x+dx)>>1, y);
+    p->Painter->drawLine(x, dy, (x+dx)>>1, y);
     p->drawLine(cx-15, by, cx-2, by);
     p->drawLine(cx-15, by, cx-15, by + zAxis.numGraphs);
 
@@ -85,9 +85,9 @@ void TabDiagram::paint(ViewPainter *p)
     Points.setPoints(3, x, y-dy, (x+dx)>>1, y, dx, y-dy);
     p->Painter->drawConvexPolygon(Points);
     p->Painter->setPen(QColor(208, 208, 208));
-    p->drawLine(x, y-dy, (x+dx)>>1, y);
+    p->Painter->drawLine(x, y-dy, (x+dx)>>1, y);
     p->Painter->setPen(QColor(224, 224, 224));
-    p->drawLine(x, y-dy, dx, y-dy);
+    p->Painter->drawLine(x, y-dy, dx, y-dy);
 
     p->Painter->setBrush(QBrush(Qt::NoBrush));
   }
@@ -123,7 +123,14 @@ int TabDiagram::calcDiagram()
   QFontMetrics  metrics(QucsSettings.font);
   int tHeight = metrics.lineSpacing();
   QString Str;
-  int colWidth=0, x=8, y = y2-tHeight-6;
+  int colWidth=0, x=8, y;
+
+  if(y2 < (41 + MIN_SCROLLBAR_SIZE))
+    y2 = 41 + MIN_SCROLLBAR_SIZE;
+
+  if(y2 < (tHeight + 8))
+    y2 = tHeight + 8;
+  y = y2 - tHeight - 6;
 
   // outer frame
   Lines.append(new Line(0, y2, x2, y2, QPen(QPen::black,0)));
@@ -132,6 +139,9 @@ int TabDiagram::calcDiagram()
   Lines.append(new Line(0, 0, x2, 0, QPen(QPen::black,0)));
   Lines.append(new Line(0, y+2, x2, y+2, QPen(QPen::black,2)));
 
+  if(xAxis.limit_min < 0.0)
+    xAxis.limit_min = 0.0;
+
   Graph *firstGraph;
   Graph *g = Graphs.first();
   if(g == 0) {  // no variables specified in diagram ?
@@ -162,8 +172,8 @@ if(g) if(!g->cPointsX.isEmpty()) {
   invisibleCount = counting - y/tHeight;
   if(invisibleCount <= 0)  xAxis.limit_min = 0.0;// height bigger than needed
   else {
-    NumLeft = invisibleCount - int(xAxis.limit_min);
-    if(invisibleCount < int(xAxis.limit_min))
+    NumLeft = invisibleCount - int(xAxis.limit_min + 0.5);
+    if(invisibleCount < int(xAxis.limit_min + 0.5))
       xAxis.limit_min = double(invisibleCount); // adjust limit of scroll bar
   }
 
@@ -172,7 +182,7 @@ if(g) if(!g->cPointsX.isEmpty()) {
     Str = pD->Var;
     colWidth = checkColumnWidth(Str, metrics, colWidth, x, y2);
     if(colWidth < 0)  goto funcEnd;
-    startWriting = int(xAxis.limit_min);  // when to reach visible area
+    startWriting = int(xAxis.limit_min + 0.5);  // when to reach visible area
 
     Texts.append(new Text(x-4, y2-2, Str)); // independent variable
     if(pD->count != 0) {
@@ -211,7 +221,8 @@ if(g) if(!g->cPointsX.isEmpty()) {
 
   firstGraph = g;
   // ................................................
-  for(g = Graphs.first(); g!=0; g = Graphs.next()) {// all dependent variables
+  // all dependent variables
+  for(g = Graphs.first(); g!=0; g = Graphs.next()) {
     y = y2-tHeight-5;
     colWidth = 0;
 
@@ -221,29 +232,51 @@ if(g) if(!g->cPointsX.isEmpty()) {
     Texts.append(new Text(x, y2-2, Str));  // dependent variable
 
 
-    startWriting = int(xAxis.limit_min);  // when to reach visible area
+    startWriting = int(xAxis.limit_min + 0.5); // when to reach visible area
     py = g->cPointsY - 2;
     if(g->cPointsX.getFirst()) {
 
       if(sameDependencies(g, firstGraph)) {
         int z=g->cPointsX.getFirst()->count * g->countY;
         if(z > NumAll)  NumAll = z;
-        for(; z>0; z--) {
-          py += 2;
-          if(startWriting-- > 0) continue; // reached visible area ?
-          if(y < tHeight) break;           // no room for more rows ?
-          switch(g->numMode) {
-            case 0: Str = complexRect(*py, *(py+1), g->Precision); break;
-            case 1: Str = complexDeg (*py, *(py+1), g->Precision); break;
-            case 2: Str = complexRad (*py, *(py+1), g->Precision); break;
+
+        if(g->Var.right(2) != ".X")
+          for(; z>0; z--) {
+            py += 2;
+            if(startWriting-- > 0) continue; // reached visible area ?
+            if(y < tHeight) break;           // no room for more rows ?
+            switch(g->numMode) {
+              case 0: Str = complexRect(*py, *(py+1), g->Precision); break;
+              case 1: Str = complexDeg (*py, *(py+1), g->Precision); break;
+              case 2: Str = complexRad (*py, *(py+1), g->Precision); break;
+            }
+
+            colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
+            if(colWidth < 0)  goto funcEnd;
+
+            Texts.append(new Text(x, y, Str));
+            y -= tHeight;
           }
 
-          colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
-          if(colWidth < 0)  goto funcEnd;
+        else {  // digital data
+          char *pcy = (char*)g->cPointsY;
+          for(; z>0; z--) {
+            if(startWriting-- > 0) {  // reached visible area ?
+              pcy += strlen(pcy) + 1;
+              continue;
+            }
+            if(y < tHeight) break;           // no room for more rows ?
+            Str = QString(pcy);
 
-          Texts.append(new Text(x, y, Str));
-          y -= tHeight;
+            colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
+            if(colWidth < 0)  goto funcEnd;
+
+            Texts.append(new Text(x, y, Str));
+            pcy += strlen(pcy) + 1;
+            y -= tHeight;
+          }
         }
+
         if(z > NumLeft)  NumLeft = z;
       }  // of "if(sameDeps)"
       else {
@@ -268,16 +301,22 @@ funcEnd:
   if(invisibleCount > 0) {  // could all numbers be written ?
     x1 = 18;   // extend the select area to the left
 
+    zAxis.limit_max = double(NumAll);  // number of data (rows) 
+
     // calculate data for painting scroll bar
-    xAxis.numGraphs = int(xAxis.limit_min);
-    NumLeft = NumAll - NumLeft - xAxis.numGraphs;
+    y = int(xAxis.limit_min + 0.5);
+    NumLeft = NumAll - NumLeft - y;
+
+    // position of scroll bar in pixel
+    yAxis.numGraphs = (y2 - 39) * y / NumAll;
 
     // height of scroll bar
     zAxis.numGraphs = (y2 - 39) * NumLeft / NumAll;
-    if(zAxis.numGraphs < 3)  zAxis.numGraphs = 3;
-
-    // position of scroll bar in pixel
-    yAxis.numGraphs = (y2 - 39) * xAxis.numGraphs / NumAll;
+    if(zAxis.numGraphs < MIN_SCROLLBAR_SIZE) {
+      yAxis.numGraphs -= (MIN_SCROLLBAR_SIZE - zAxis.numGraphs + 1)
+                         * y / NumAll;
+      zAxis.numGraphs = MIN_SCROLLBAR_SIZE;
+    }
 
     xAxis.numGraphs = NumLeft;  // number of lines in the diagram
   }
@@ -286,10 +325,10 @@ funcEnd:
 }
 
 // ------------------------------------------------------------
-bool TabDiagram::scroll(int clickPos)
+int TabDiagram::scroll(int clickPos)
 {
-  if(x1 <= 0) return false;   // no scroll bar ?
-  double tmp = xAxis.limit_min;
+  if(x1 <= 0) return 0;   // no scroll bar ?
+  int tmp = int(xAxis.limit_min + 0.5);
 
   int y = cy;
   if(clickPos > (cy-20)) {  // scroll one line down ?
@@ -298,25 +337,42 @@ bool TabDiagram::scroll(int clickPos)
   else {
     y -= y2 - 20;
     if(clickPos < y) {  // scroll bar one line up ?
-      if(xAxis.limit_min <= 0)  return false;
+      if(xAxis.limit_min <= 0.0)  return 0;
       xAxis.limit_min--;
     }
     else {
       y += yAxis.numGraphs;
-      if(clickPos < y) {  // scroll bar one page up ?
+      if(clickPos < y)   // scroll bar one page up ?
         xAxis.limit_min -= double(xAxis.numGraphs);
-        if(xAxis.limit_min < 0.0)  xAxis.limit_min = 0.0;
-      }
       else {
         y += zAxis.numGraphs;
         if(clickPos > y)   // a page down?
           xAxis.limit_min += double(xAxis.numGraphs);
+        else
+          return 2;  // click on position bar
       }
     }
   }
 
   calcDiagram();
-  if(tmp == xAxis.limit_min)  return false;   // did anything change ?
+  if(tmp == int(xAxis.limit_min + 0.5))
+    return 0;   // did anything change ?
+
+  return 1;
+}
+
+// ------------------------------------------------------------
+bool TabDiagram::scrollTo(int initial, int, int dy)
+{
+  int tmp = int(xAxis.limit_min + 0.5);
+  xAxis.limit_min  = double(initial);
+  xAxis.limit_min += double(dy) / double(y2 - 39) * zAxis.limit_max;
+  xAxis.limit_min  = floor(xAxis.limit_min + 0.5);
+
+  calcDiagram();
+  if(tmp == int(xAxis.limit_min + 0.5))
+    return false;   // did anything change ?
+
   return true;
 }
 
