@@ -21,7 +21,7 @@
 #include <math.h>
 
 
-TimingDiagram::TimingDiagram(int _cx, int _cy) : Diagram(_cx, _cy)
+TimingDiagram::TimingDiagram(int _cx, int _cy) : TabDiagram(_cx, _cy)
 {
   x1 = 0;    // no extension to select area
   y1 = 0;
@@ -78,7 +78,7 @@ void TimingDiagram::paint(ViewPainter *p)
     Points.setPoints(3, x, (y+dy)>>1, dx, y, dx, dy);
     p->Painter->drawConvexPolygon(Points);
     p->Painter->setPen(QColor(224, 224, 224));
-    p->drawLine(x, (y+dy)>>1, dx, y);
+    p->Painter->drawLine(x, (y+dy)>>1, dx, y);
     p->drawLine(cx+yAxis.numGraphs, cy+2, bx, cy+2);
     p->drawLine(cx+yAxis.numGraphs, cy+2, cx+yAxis.numGraphs, cy+15);
 
@@ -88,9 +88,9 @@ void TimingDiagram::paint(ViewPainter *p)
     Points.setPoints(3, x, (y+dy)>>1, x-dx, y, x-dx, dy);
     p->Painter->drawConvexPolygon(Points);
     p->Painter->setPen(QColor(208, 208, 208));
-    p->drawLine(x-dx, y, x, (y+dy)>>1);
+    p->Painter->drawLine(x-dx, y, x, (y+dy)>>1);
     p->Painter->setPen(QColor(224, 224, 224));
-    p->drawLine(x-dx, y, x-dx, dy);
+    p->Painter->drawLine(x-dx, y, x-dx, dy);
 
     p->Painter->setBrush(QBrush(Qt::NoBrush));
   }
@@ -118,12 +118,16 @@ int TimingDiagram::calcDiagram()
   QFontMetrics  metrics(QucsSettings.font);
   int tHeight = metrics.lineSpacing();
   QString Str;
-  int colWidth=0, x=4, y = y2-tHeight-6, xStart = 0, z;
+  int colWidth=0, x=4, y, xStart = 0, z;
 
   int NumAll=0;   // how many values per row
   int NumLeft=0;  // how many values could not be written
   int invisibleCount = 0;  // how many values are invisible
   
+  if(y2 < (tHeight + 8))
+    y2 = tHeight + 8;
+  y = y2 - tHeight - 6;
+
   // outer frame
   Lines.append(new Line(0, y2, x2, y2, QPen(QPen::black,0)));
   Lines.append(new Line(0, y2, 0, 0, QPen(QPen::black,0)));
@@ -131,6 +135,9 @@ int TimingDiagram::calcDiagram()
   Lines.append(new Line(0, 0, x2, 0, QPen(QPen::black,0)));
   Lines.append(new Line(0, y+2, x2, y+2, QPen(QPen::black,0)));
 
+  if(xAxis.limit_min < 0.0)
+    xAxis.limit_min = 0.0;
+
   Graph *firstGraph;
   Graph *g = Graphs.first();
   if(g == 0) {  // no variables specified in diagram ?
@@ -156,9 +163,30 @@ int TimingDiagram::calcDiagram()
   }
   firstGraph = g;
 
-if(!g->cPointsX.isEmpty()) {
+
+  // First check the maximum bit number of all vectors.
+  colWidth = 0;
+  for(g = Graphs.first(); g!=0; g = Graphs.next())
+    if(g->cPointsY)
+      if(g->Var.right(2) == ".X") {
+        z = strlen((char*)g->cPointsY);
+        if(z > colWidth)
+          colWidth = z;
+      }
+      else {
+        z = 8;
+        if(z > colWidth)
+          colWidth = z;
+      }
+  int TimeStepWidth = colWidth * metrics.width("X") + 8;
+  if(TimeStepWidth < 40)
+    TimeStepWidth = 40;
+
+
+  colWidth = 0;
+if(!firstGraph->cPointsX.isEmpty()) {
   // ................................................
-  if(g->cPointsX.count() > 1) {
+  if(firstGraph->cPointsX.count() > 1) {
     Str = QObject::tr("wrong dependency");
     colWidth = checkColumnWidth(Str, metrics, colWidth, x, y2);
     if(colWidth >= 0)
@@ -168,7 +196,7 @@ if(!g->cPointsX.isEmpty()) {
 
 
   // first, write name of independent variable
-  DataX *pD = g->cPointsX.getFirst();
+  DataX *pD = firstGraph->cPointsX.getFirst();
   NumAll = pD->count;
   Str = pD->Var;
   colWidth = checkColumnWidth(Str, metrics, colWidth, x, y2);
@@ -176,7 +204,7 @@ if(!g->cPointsX.isEmpty()) {
   Texts.append(new Text(x, y2-2, Str));
   
 
-  y -= 2;
+  y -= 5;
   // write all dependent variable names to get width of first column
   for(g = Graphs.first(); g!=0; g = Graphs.next()) {
     if(y < tHeight)  break;
@@ -184,7 +212,7 @@ if(!g->cPointsX.isEmpty()) {
     colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
     if(colWidth < 0)  return 1;
     Texts.append(new Text(x, y, Str));  // dependent variable
-    y -= tHeight;
+    y -= tHeight + 2;
   }
   x += colWidth + 13;
   xAxis.numGraphs = x -6;
@@ -192,11 +220,11 @@ if(!g->cPointsX.isEmpty()) {
   xStart = x;
 
 
-  invisibleCount = NumAll - (x2-xAxis.numGraphs)/40;
+  invisibleCount = NumAll - (x2-xAxis.numGraphs)/TimeStepWidth;
   if(invisibleCount <= 0)  xAxis.limit_min = 0.0;  // longer than needed
   else {
-    NumLeft = invisibleCount - int(xAxis.limit_min);
-    if(invisibleCount < int(xAxis.limit_min))
+    NumLeft = invisibleCount - int(xAxis.limit_min + 0.5);
+    if(invisibleCount < int(xAxis.limit_min + 0.5))
       xAxis.limit_min = double(invisibleCount); // adjust limit of scroll bar
   }
 
@@ -204,7 +232,7 @@ if(!g->cPointsX.isEmpty()) {
   // write independent variable values (usually time)
   y = y2-tHeight-4;
   px = pD->Points;
-  z = int(xAxis.limit_min);
+  z = int(xAxis.limit_min + 0.5);
   px += z;
   z = pD->count - z;
   for( ; z>0; z--) {
@@ -214,17 +242,18 @@ if(!g->cPointsX.isEmpty()) {
 
     Texts.append(new Text( x, y2-2, Str));
     Lines.append(new Line(x+5, y, x+5, y-3, QPen(QPen::black,0)));
-    x += 40;
+    x += TimeStepWidth;
   }
 
 }  // of "if no data in graphs"
 
 
+  tHeight += 2;
   // ................................................
   // work on all dependent variables
   QPen Pen;
-  int  yLast;
-  y = y2-tHeight-8;
+  int  yLast, yNow;
+  y = y2-tHeight-9;
   for(g = Graphs.first(); g!=0; g = Graphs.next()) {
     if(y < tHeight) {
       // mark lack of space with a small arrow
@@ -237,79 +266,168 @@ if(!g->cPointsX.isEmpty()) {
     x = xStart + 5;
     colWidth = 0;
 
-    if(g->cPointsY) {
-      px = g->cPointsY;
-
-      if(sameDependencies(g, firstGraph)) {
-
-        Pen = QPen(g->Color, g->Thick);  // default is solid line
-        switch(g->Style) {
-          case 1: Pen.setStyle(Qt::DashLine); break;
-          case 2: Pen.setStyle(Qt::DotLine);  break;
-        }
-
-        z = int(xAxis.limit_min);
-        px += 2*z;
-
-        yLast = 0;
-        if(z > 0)  yLast += 2; // vertical line before first value ?
-        if(*(px-yLast) > 0.5)  // high or low ?
-          yLast = 2;
-        else
-          yLast = tHeight - 4;
-
-        z = g->cPointsX.getFirst()->count - z;
-        for( ; z>0; z--) {
-          if(*px > 0.5) {  // high or low ?
-            if(yLast > 2)
-              Lines.append(new Line(x, y-tHeight+4, x, y-2, Pen));
-            yLast = 2;
-          }
-          else {
-            if(yLast < 3)
-              Lines.append(new Line(x, y-tHeight+4, x, y-2, Pen));
-            yLast = tHeight - 4;
-          } 
-
-          if(x+40 >= x2) break;
-          Lines.append(new Line(x, y-yLast, x+40, y-yLast, Pen));
-
-          x += 40;
-          px += 2;
-        }
-      }  // of "if(sameDeps)"
-      else {
-        Str = QObject::tr("wrong dependency");
-        colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
-        if(colWidth < 0)  goto funcEnd;
-        Texts.append(new Text(x, y, Str));
-      }
-    }
-    else {   // no data in graph
+    if(g->cPointsY == 0) {
       Str = QObject::tr("no data");
       colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
       if(colWidth < 0)  goto funcEnd;
       Texts.append(new Text(x, y, Str));
+      y -= tHeight;
+      continue;
     }
+
+    if(!sameDependencies(g, firstGraph)) {
+      Str = QObject::tr("wrong dependency");
+      colWidth = checkColumnWidth(Str, metrics, colWidth, x, y);
+      if(colWidth < 0)  goto funcEnd;
+      Texts.append(new Text(x, y, Str));
+      y -= tHeight;
+      continue;
+    }
+
+    Pen = QPen(g->Color, g->Thick);  // default is solid line
+    switch(g->Style) {
+      case 1: Pen.setStyle(Qt::DashLine); break;
+      case 2: Pen.setStyle(Qt::DotLine);  break;
+    }
+
+    z = int(xAxis.limit_min + 0.5);
+    if(g->Var.right(2) != ".X") {  // not digital variable ?
+      px = g->cPointsY;
+      px += 2 * z;
+      z = g->cPointsX.getFirst()->count - z;
+      yNow = 1 + ((tHeight - 6) >> 1);
+      Lines.append(new Line(x, y-yNow, x+2, y-1, Pen));
+      Lines.append(new Line(x+2, y-tHeight+5, x, y-yNow, Pen));
+      for( ; z>0; z--) {
+        if(x+TimeStepWidth >= x2) break;
+        Lines.append(new Line(x+2, y-1, x+TimeStepWidth-2, y-1, Pen));
+        Lines.append(new Line(x+2, y-tHeight+5, x+TimeStepWidth-2, y-tHeight+5, Pen));
+
+        // output magnitude of (complex) number
+        Texts.append(new Text(x+3, y,
+              QString::number(sqrt((*px)*(*px) + (*(px+1))*(*(px+1))))));
+
+        px += 2;
+        x += TimeStepWidth;
+        Lines.append(new Line(x-2, y-tHeight+5, x+2, y-1, Pen));
+        Lines.append(new Line(x+2, y-tHeight+5, x-2, y-1, Pen));
+      }
+      y -= tHeight;
+      continue;
+    }
+
+
+    // digital variable !!!
+    char *pcx = (char*)g->cPointsY;
+    pcx += z * (strlen(pcx)+1);
+
+    if(strlen(pcx) < 2) {   // vector or single bit ?
+
+      // It is single "bit".
+      yLast = 0;
+      if(z > 0)  yLast += 2; // vertical line before first value ?
+      switch(*(pcx-yLast)) {  // high or low ?
+        case '0':  // low
+          yLast = tHeight - 5;
+          break;
+        case '1':  // high
+          yLast = 1;
+          break;
+        default:
+          yLast = 1 + ((tHeight - 6) >> 1);
+      }
+
+      z = g->cPointsX.getFirst()->count - z;
+      for( ; z>0; z--) {
+
+        switch(*pcx) {
+          case '0':  // low
+            yNow = tHeight - 5;
+            break;
+          case '1':  // high
+            yNow = 1;
+            break;
+          default:
+            yNow = 1 + ((tHeight - 6) >> 1);
+        } 
+
+        if(yLast != yNow)
+          Lines.append(new Line(x, y-yLast, x, y-yNow, Pen));
+        if(x+TimeStepWidth >= x2) break;
+        if((*pcx & 254) == '0')
+          Lines.append(new Line(x, y-yNow, x+TimeStepWidth, y-yNow, Pen));
+        else {
+          Texts.append(new Text(x+(TimeStepWidth>>1)-3, y, QString(pcx)));
+          Lines.append(new Line(x+3, y-1, x+TimeStepWidth-3, y-1, Pen));
+          Lines.append(new Line(x+3, y-tHeight+5, x+TimeStepWidth-3, y-tHeight+5, Pen));
+          Lines.append(new Line(x, y-yNow, x+3, y-1, Pen));
+          Lines.append(new Line(x, y-yNow, x+3, y-tHeight+5, Pen));
+          Lines.append(new Line(x+TimeStepWidth-3, y-1, x+TimeStepWidth, y-yNow, Pen));
+          Lines.append(new Line(x+TimeStepWidth-3, y-tHeight+5, x+TimeStepWidth, y-yNow, Pen));
+        }
+
+        yLast = yNow;
+        x += TimeStepWidth;
+        pcx += 2;
+      }
+
+    }
+    else {  // It is a bit vector !!!
+
+      z = g->cPointsX.getFirst()->count - z;
+      yNow = 1 + ((tHeight - 6) >> 1);
+      Lines.append(new Line(x, y-yNow, x+2, y-1, Pen));
+      Lines.append(new Line(x+2, y-tHeight+5, x, y-yNow, Pen));
+      for( ; z>0; z--) {
+        if(x+TimeStepWidth >= x2) break;
+        Lines.append(new Line(x+2, y-1, x+TimeStepWidth-2, y-1, Pen));
+        Lines.append(new Line(x+2, y-tHeight+5, x+TimeStepWidth-2, y-tHeight+5, Pen));
+
+        Texts.append(new Text(x+3, y, QString(pcx)));
+
+        x += TimeStepWidth;
+        pcx += strlen(pcx) + 1;
+        Lines.append(new Line(x-2, y-tHeight+5, x+2, y-1, Pen));
+        Lines.append(new Line(x+2, y-tHeight+5, x-2, y-1, Pen));
+      }
+    }
+
     y -= tHeight;
-  }
+  }  // of "for(Graphs...)"
 
 
 funcEnd:
   if(invisibleCount > 0) {  // could all values be displayed ?
+    x  = x2 - xAxis.numGraphs - 37;
+    if(x < MIN_SCROLLBAR_SIZE+2) {  // not enough space for scrollbar ?
+      Lines.append(new Line(x2, 0, x2, -17, QPen(QPen::red,0)));
+      Lines.append(new Line(xAxis.numGraphs, -17, x2, -17, QPen(QPen::red,0)));
+      Lines.append(new Line(xAxis.numGraphs, 0, xAxis.numGraphs, -17, QPen(QPen::red,0)));
+      return 1;
+    }
+
     y1 = 18;   // extend the select area to the bottom
-    x  = (x2 - xAxis.numGraphs - 37);
-    z  = int(xAxis.limit_min);
+    z  = int(xAxis.limit_min + 0.5);
     if(NumLeft < 0) NumLeft = 0;
     y  = NumAll - NumLeft - z;
 
-    // length of scroll bar
-    zAxis.numGraphs = x * y / NumAll;
-    if(zAxis.numGraphs < 3)  zAxis.numGraphs = 3;
+    // number of data (times) 
+    zAxis.limit_max = double(firstGraph->cPointsX.getFirst()->count);
 
     // position of scroll bar in pixel
     yAxis.numGraphs = x * z / NumAll + xAxis.numGraphs + 19;
 
+    // length of scroll bar
+    zAxis.numGraphs = x * y / NumAll;
+    if(zAxis.numGraphs < MIN_SCROLLBAR_SIZE) {
+      yAxis.numGraphs -= (MIN_SCROLLBAR_SIZE - zAxis.numGraphs + 1)
+                         * z / NumAll;
+      zAxis.numGraphs = MIN_SCROLLBAR_SIZE;
+
+      x = x2 - 19 - yAxis.numGraphs - zAxis.numGraphs;
+      if(x < 0)  yAxis.numGraphs += x;
+    }
+
     xAxis.limit_max = double(y);
   }
 
@@ -317,10 +435,10 @@ funcEnd:
 }
 
 // ------------------------------------------------------------
-bool TimingDiagram::scroll(int clickPos)
+int TimingDiagram::scroll(int clickPos)
 {
-  if(y1 <= 0) return false;   // no scroll bar ?
-  double tmp = xAxis.limit_min;
+  if(y1 <= 0) return 0;   // no scroll bar ?
+  int tmp = int(xAxis.limit_min + 0.5);
 
   int x = cx;
   if(clickPos > (cx+x2-20)) {  // scroll one value to the right ?
@@ -329,25 +447,42 @@ bool TimingDiagram::scroll(int clickPos)
   else {
     x += xAxis.numGraphs + 20;
     if(clickPos < x) {  // scroll bar one value to the left ?
-      if(xAxis.limit_min <= 0)  return false;
+      if(xAxis.limit_min <= 0.0)  return 0;
       xAxis.limit_min--;
     }
     else {
       x = cx + yAxis.numGraphs;
-      if(clickPos < x) {  // scroll bar one page to the left ?
+      if(clickPos < x)   // scroll bar one page to the left ?
         xAxis.limit_min -= xAxis.limit_max;
-        if(xAxis.limit_min < 0.0)  xAxis.limit_min = 0.0;
-      }
       else {
         x += zAxis.numGraphs;
         if(clickPos > x)   // one page to the right ?
           xAxis.limit_min += xAxis.limit_max;
+        else
+          return 2;  // click on position bar
       }
     }
   }
 
   calcDiagram();
-  if(tmp == xAxis.limit_min)  return false;   // did anything change ?
+  if(tmp == int(xAxis.limit_min+0.5))
+    return 0;   // did anything change ?
+
+  return 1;
+}
+
+// ------------------------------------------------------------
+bool TimingDiagram::scrollTo(int initial, int dx, int)
+{
+  int tmp = int(xAxis.limit_min + 0.5);
+  xAxis.limit_min  = double(initial);
+  xAxis.limit_min += double(dx) / double(x2-xAxis.numGraphs-39) * zAxis.limit_max;
+  xAxis.limit_min  = floor(xAxis.limit_min + 0.5);
+
+  calcDiagram();
+  if(tmp == int(xAxis.limit_min + 0.5))
+    return false;   // did anything change ?
+
   return true;
 }
 
