@@ -34,6 +34,7 @@
 #include "qucs.h"
 #include "textdoc.h"
 #include "schematic.h"
+#include "components/opt_sim.h"
 
 
 SimMessage::SimMessage(QWidget *w, QWidget *parent)
@@ -50,7 +51,8 @@ SimMessage::SimMessage(QWidget *w, QWidget *parent)
   DocName = Doc->DocName;
   DataDisplay = Doc->DataDisplay;
   QFileInfo Info(DocName);
-  DataSet = Info.dirPath() + QDir::separator() + Doc->DataSet;
+  DataSet = QDir::convertSeparators(Info.dirPath()) +
+    QDir::separator() + Doc->DataSet;
   showBias = Doc->showBias;     // save some settings as the document...
   SimOpenDpl = Doc->SimOpenDpl; // ...could be closed during the simulation.
 
@@ -107,10 +109,10 @@ bool SimMessage::startProcess()
   Abort->setText(tr("Abort simulation"));
   Display->setDisabled(true);
 
-  ProgText->insert(tr("Starting new simulation on ")+
-                   QDate::currentDate().toString("ddd dd. MMM yyyy"));
-  ProgText->insert(tr(" at ")+
-                   QTime::currentTime().toString("hh:mm:ss")+"\n\n");
+  QString txt = tr("Starting new simulation on %1 at %2").
+    arg(QDate::currentDate().toString("ddd dd. MMM yyyy")).
+    arg(QTime::currentTime().toString("hh:mm:ss"));
+  ProgText->insert(txt + "\n\n");
 
   SimProcess.blockSignals(false);
   if(SimProcess.isRunning()) {
@@ -120,17 +122,17 @@ bool SimMessage::startProcess()
   }
 
   Collect.clear();  // clear list for NodeSets, SPICE components etc.
+  ProgText->insert(tr("creating netlist... "));
+  NetlistFile.setName(QucsHomeDir.filePath("netlist.txt"));
+   if(!NetlistFile.open(IO_WriteOnly)) {
+    ErrText->insert(tr("ERROR: Cannot write netlist file!"));
+    FinishSimulation(-1);
+    return false;
+  }
+
+  Stream.setDevice(&NetlistFile);
+
   if(!DocWidget->inherits("QTextEdit")) {
-    ProgText->insert(tr("creating netlist... "));
-    NetlistFile.setName(QucsHomeDir.filePath("netlist.txt"));
-    if(!NetlistFile.open(IO_WriteOnly)) {
-      ErrText->insert(tr("ERROR: Cannot write netlist file!"));
-      FinishSimulation(-1);
-      return false;
-    }
-
-    Stream.setDevice(&NetlistFile);
-
     SimPorts =
        ((Schematic*)DocWidget)->prepareNetlist(Stream, Collect, ErrText);
     if(SimPorts < -5) {
@@ -185,13 +187,20 @@ void SimMessage::nextSPICE()
   com << (QucsSettings.BinDir + "qucsconv");
   if(makeSubcircuit)
     com << "-g" << "_ref";
-  com << "-if" << "spice" << "-of" << "qucs" << "-i";
-  if(FileName.find(QDir::separator()) < 0)  // add path ?
-    com << QucsWorkDir.path() + QDir::separator() + FileName;
-  else
-    com << FileName;
+  com << "-if" << "spice" << "-of" << "qucs";
   SimProcess.setArguments(com);
 
+  QFile SpiceFile;
+  if(FileName.find(QDir::separator()) < 0)  // add path ?
+    SpiceFile.setName(QucsWorkDir.path() + QDir::separator() + FileName);
+  else
+    SpiceFile.setName(FileName);
+  if(!SpiceFile.open(IO_ReadOnly)) {
+    ErrText->insert(tr("ERROR: Cannot open SPICE file \"%1\".").arg(FileName));
+    FinishSimulation(-1);
+    return;
+  }
+
 
   if(makeSubcircuit) {
     Stream << "\n.Def:" << properName(FileName) << " ";
@@ -209,7 +218,18 @@ void SimMessage::nextSPICE()
     FinishSimulation(-1);
     return;
   }
+
+  QByteArray SpiceContent = SpiceFile.readAll();
+  SpiceFile.close();
+  SimProcess.writeToStdin(SpiceContent);
+  connect(&SimProcess, SIGNAL(wroteToStdin()), SLOT(slotCloseStdin()));
+}
+
+// ------------------------------------------------------------------------
+void SimMessage::slotCloseStdin()
+{
   SimProcess.closeStdin();
+  disconnect(&SimProcess, SIGNAL(wroteToStdin()), 0, 0);
 }
 
 // ------------------------------------------------------------------------
@@ -236,7 +256,6 @@ void SimMessage::slotReadSpiceNetlist()
   }
 }
 
-
 // ------------------------------------------------------------------------
 void SimMessage::slotFinishSpiceNetlist()
 {
@@ -246,6 +265,18 @@ void SimMessage::slotFinishSpiceNetlist()
   nextSPICE();
 }
 
+// ------------------------------------------------------------------------
+#ifdef __MINGW32__
+#include <windows.h>
+static QString getShortPathName(QString longpath) {
+  const char * lpath = QDir::convertSeparators(longpath).ascii();
+  char spath[2048];
+  int len = GetShortPathNameA(lpath,spath,sizeof(spath)-1);
+  spath[len] = '\0';
+  return QString(spath);
+}
+#endif
+
 // ------------------------------------------------------------------------
 void SimMessage::startSimulator()
 {
@@ -258,39 +289,87 @@ void SimMessage::startSimulator()
   QString SimPath = QDir::convertSeparators (QucsHomeDir.absPath());
 #ifdef __MINGW32__
   QString QucsDigi = "qucsdigi.bat";
+  QString QucsVeri = "qucsveri.bat";
 #else
   QString QucsDigi = "qucsdigi";
+  QString QucsVeri = "qucsveri";
 #endif
+  SimOpt = NULL;
+  bool isVerilog = false;
 
   if(DocWidget->inherits("QTextEdit")) {
-    if(copyFile(DocName, QucsHomeDir.filePath("netlist.txt")) < 0) {
-      ErrText->insert(tr("ERROR: Cannot copy \"%1\" !").arg(DocName));
-      FinishSimulation(-1);
-      return;
-    }
+    // Take VHDL file in memory as it could contain unsaved changes.
+    Stream << ((TextDoc*)DocWidget)->text();
+    NetlistFile.close();
+    ProgText->insert(tr("done.")+"\n");  // of "creating netlist... 
 
+    SimTime = ((TextDoc*)DocWidget)->SimTime;
+#ifdef __MINGW32__
+    CommandLine << getShortPathName(QucsSettings.BinDir + QucsDigi)
+		<< "netlist.txt" << DataSet
+		<< SimTime << getShortPathName(SimPath)
+		<< getShortPathName(QucsSettings.BinDir);
+#else
     CommandLine << QucsSettings.BinDir + QucsDigi << "netlist.txt" << DataSet
-       << ((TextDoc*)DocWidget)->SimTime << QucsHomeDir.absPath()
-       << QucsSettings.BinDir;
+       << SimTime << SimPath << QucsSettings.BinDir;
+#endif
   }
   else {
     // output NodeSets, SPICE simulations etc.
     Stream << Collect.join("\n") << '\n';
+    isVerilog = ((Schematic*)DocWidget)->isVerilog;
     SimTime = ((Schematic*)DocWidget)->createNetlist(Stream, SimPorts);
-    NetlistFile.close();
     if(SimTime.at(0) == '') {
+      NetlistFile.close();
       ErrText->insert(SimTime.mid(1));
       FinishSimulation(-1);
       return;
     }
-    ProgText->insert(tr("done.\n"));  // of "creating netlist... 
+    if (isVerilog) {
+      Stream << "\n"
+	     << "  initial begin\n"
+	     << "    $dumpfile(\"digi.vcd\");\n"
+	     << "    $dumpvars();\n"
+	     << "    #" << SimTime << " $finish;\n"
+	     << "  end\n\n"
+	     << "endmodule // TestBench\n";
+    }
+    NetlistFile.close();
+    ProgText->insert(tr("done.")+"\n");  // of "creating netlist... 
 
-    if(SimPorts < 0)
-      CommandLine << QucsSettings.BinDir + "qucsator" << "-b" << "-g"
-         << "-i" << QucsHomeDir.filePath("netlist.txt") << "-o" << DataSet;
-    else
-      CommandLine << QucsSettings.BinDir + QucsDigi << "netlist.txt"
-         << DataSet << SimTime << SimPath << QucsSettings.BinDir << "-c";
+    if(SimPorts < 0) {
+      if((SimOpt = findOptimization((Schematic*)DocWidget))) {
+	((Optimize_Sim*)SimOpt)->createASCOnetlist();
+	CommandLine << QucsSettings.AscoDir + "asco" << "-qucs" <<
+	  QucsHomeDir.filePath("asco_netlist.txt") << "-o" << "asco_out";
+      }
+      else {
+	CommandLine << QucsSettings.BinDir + "qucsator" << "-b" << "-g"
+           << "-i" << QucsHomeDir.filePath("netlist.txt") << "-o" << DataSet;
+      }
+    } else {
+      if (isVerilog) {
+#ifdef __MINGW32__
+	CommandLine << getShortPathName(QucsSettings.BinDir + QucsVeri)
+		    << "netlist.txt" << DataSet
+		    << SimTime << getShortPathName(SimPath)
+		    << getShortPathName(QucsSettings.BinDir) << "-c";
+#else
+	CommandLine << QucsSettings.BinDir + QucsVeri << "netlist.txt"
+           << DataSet << SimTime << SimPath << QucsSettings.BinDir << "-c";
+#endif
+      } else {
+#ifdef __MINGW32__
+	CommandLine << getShortPathName(QucsSettings.BinDir + QucsDigi)
+		    << "netlist.txt" << DataSet
+		    << SimTime << getShortPathName(SimPath)
+		    << getShortPathName(QucsSettings.BinDir) << "-c";
+#else
+	CommandLine << QucsSettings.BinDir + QucsDigi << "netlist.txt"
+           << DataSet << SimTime << SimPath << QucsSettings.BinDir << "-c";
+#endif
+      }
+    }
   }
 
   SimProcess.setArguments(CommandLine);
@@ -312,6 +391,17 @@ void SimMessage::startSimulator()
   }
 }
 
+// ------------------------------------------------------------------------
+Component * SimMessage::findOptimization(Schematic *Doc) {
+  Component *pc;
+  for(pc=Doc->Components->first(); pc!=0; pc=Doc->Components->next())
+    if(pc->isActive)
+      if(pc->Model == ".Opt")
+	return pc;
+  return NULL;
+}
+
+
 // ------------------------------------------------------------------------
 // Is called when the process sends an output to stdout.
 void SimMessage::slotDisplayMsg()
@@ -399,31 +489,48 @@ void SimMessage::FinishSimulation(int Status)
   QTime t = QTime::currentTime();   // get time
 
   if(Status == 0) {
-    ProgText->insert(tr("\nSimulation ended on ")+
-                     d.toString("ddd dd. MMM yyyy"));
-    ProgText->insert(tr(" at ")+t.toString("hh:mm:ss")+"\n");
-    ProgText->insert(tr("Ready.\n"));
+    QString txt = tr("Simulation ended on %1 at %2").
+      arg(d.toString("ddd dd. MMM yyyy")).
+      arg(t.toString("hh:mm:ss"));
+    ProgText->insert("\n" + txt + "\n" + tr("Ready.") + "\n");
   }
   else {
-    ProgText->insert(tr("\nErrors occured during simulation on ")+
-                     d.toString("ddd dd. MMM yyyy"));
-    ProgText->insert(tr(" at ")+t.toString("hh:mm:ss")+"\n");
-    ProgText->insert(tr("Aborted.\n"));
+    QString txt = tr("Errors occurred during simulation on %1 at %2").
+      arg(d.toString("ddd dd. MMM yyyy")).
+      arg(t.toString("hh:mm:ss"));
+    ProgText->insert("\n" + txt + "\n" + tr("Aborted.") + "\n");
   }
 
   QFile file(QucsHomeDir.filePath("log.txt"));  // save simulator messages
   if(file.open(IO_WriteOnly)) {
     int z;
     QTextStream stream(&file);
-    stream << tr("Output:\n----------\n\n");
+    stream << tr("Output:\n-------") << "\n\n";
     for(z=0; z<=ProgText->paragraphs(); z++)
       stream << ProgText->text(z) << "\n";
-    stream << tr("\n\n\nErrors:\n--------\n\n");
+    stream << "\n\n\n" << tr("Errors:\n-------") << "\n\n";
     for(z=0; z<ErrText->paragraphs(); z++)
       stream << ErrText->text(z) << "\n";
     file.close();
   }
 
+  if(Status == 0) {
+    if(SimOpt) { // save optimization data
+      QFile ifile(QucsHomeDir.filePath("asco_out.dat"));
+      QFile ofile(DataSet);
+      if(ifile.open(IO_ReadOnly)) {
+	if(ofile.open(IO_WriteOnly)) {
+	  QByteArray data = ifile.readAll();
+	  ofile.writeBlock(data);
+	  ofile.close();
+	}
+	ifile.close();
+      }
+      if(((Optimize_Sim*)SimOpt)->loadASCOout())
+	((Schematic*)DocWidget)->setChanged(true,true);
+    }
+  }
+
   emit SimulationEnded(Status, this);
 }
 
@@ -440,39 +547,3 @@ void SimMessage::slotDisplayButton()
   emit displayDataPage(DocName, DataDisplay);
   accept();
 }
-
-// ------------------------------------------------------------------------
-int SimMessage::copyFile(const QString& SrcName, const QString& DestName)
-{
-  char *Buffer = (char*)malloc(0x10000);
-  if(!Buffer) return -10;  // should never happen
-
-  QFile origFile, destFile;
-  origFile.setName(SrcName);
-  destFile.setName(DestName);
-
-  if(!origFile.open(IO_ReadOnly)) {
-    free(Buffer);
-    return -1;
-  }
-
-  if(!destFile.open(IO_WriteOnly)) {
-    origFile.close();
-    free(Buffer);
-    return -2;
-  }
-
-  int Num;
-  do {     // copy data
-    Num = origFile.readBlock(Buffer, 0x10000);
-    if(Num < 0)  break;
-    Num = destFile.writeBlock(Buffer, Num);
-    if(Num < 0)  break;
-  } while(Num == 0x10000);
-
-  origFile.close();
-  destFile.close();
-  free(Buffer);
-  if(Num < 0)  return -3;
-  return 0;
-}
