@@ -15,17 +15,17 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "main.h"
+#include "mnemo.h"
+#include "viewpainter.h"
 #include "graphictext.h"
 #include "graphictextdialog.h"
-#include "main.h"
-#include "viewpainter.h"
-#include "mnemo.h"
 
 #include <qwidget.h>
 #include <qpainter.h>
-#include <qpushbutton.h>
 #include <qlineedit.h>
 #include <qtextedit.h>
+#include <qpushbutton.h>
 
 #include <math.h>
 
@@ -56,7 +56,7 @@ void GraphicText::paint(ViewPainter *p)
   p->Painter->rotate(-Angle);   // automatically enables transformation
 
   int Size = Font.pointSize();
-  Font.setPointSizeFloat( p->Scale * float(Size) );
+  Font.setPointSizeFloat( p->FontScale * float(Size) );
 
   QFont f = p->Painter->font();
   p->Painter->setPen(Color);
@@ -107,7 +107,7 @@ void GraphicText::getCenter(int& x, int &y)
 void GraphicText::setCenter(int x, int y, bool relative)
 {
   if(relative) {  cx += x;  cy += y;  }
-  else {  cx = x;  cy = y;  }
+  else {  cx = x-(x2>>1);  cy = y-(y2>>1);  }
 }
 
 // -----------------------------------------------------------------------
@@ -169,20 +169,9 @@ bool GraphicText::load(const QString& s)
 QString GraphicText::save()
 {
   QString t = Text;
-  t.replace('\\', "\\\\");
-  t.replace('\n', "\\n");
+  convert2ASCII(t);
 
-  int i = 0;
-  QChar ch;
-  char Str[8];
-  while((ch=t.at(i++)) != QChar(0)) {  // convert special characters
-    if(ch > QChar(0x7F)) {
-      sprintf(Str, "\\x%04X", ch.unicode());
-      t.replace(ch, Str);
-    }
-  }
-
-  // the 'Text' property has to be the last within the line !
+  // The 'Text' property has to be the last within the line !
   QString s = Name+QString::number(cx)+" "+QString::number(cy)+" "
 		+ QString::number(Font.pointSize())+" "+Color.name()+" "
 		+ QString::number(Angle) + " \""+t+"\"";
@@ -221,14 +210,19 @@ bool GraphicText::MousePressing()
 // ------------------------------------------------------------------------
 // Checks if the coordinates x/y point to the painting.
 // 5 is the precision the user must point onto the painting.
-bool GraphicText::getSelected(int x, int y)
+bool GraphicText::getSelected(float fX, float fY, float)
 {
-  double phi = M_PI/180.0*double(Angle);
-  double sine = sin(phi), cosine = cos(phi);
-  int _x = int( double(x-cx)*cosine - double(y-cy)*sine );
-  int _y = int( double(y-cy)*cosine + double(x-cx)*sine );
+  double phi  = M_PI/180.0*double(Angle);
+  float  sine = sin(phi), cosine = cos(phi);
+
+  fX -= float(cx);
+  fY -= float(cy);
+  int _x = int( fX*cosine - fY*sine );
+  int _y = int( fY*cosine + fX*sine );
+
+  if(_x >= 0) if(_y >= 0) if(_x <= x2) if(_y <= y2)
+    return true;
 
-  if(_x >= 0) if(_y >= 0) if(_x <= x2) if(_y <= y2) return true;
   return false;
 }
 
