@@ -46,7 +46,6 @@
 #include <qlistbox.h>
 #include <qprinter.h>
 #include <qfiledialog.h>
-#include <qmessagebox.h>
 #include <qpixmap.h>
 #include <qtoolbutton.h>
 #include <qstatusbar.h>
@@ -55,6 +54,8 @@
 #include <qmenubar.h>
 #include <qprocess.h>
 #include <qlineedit.h>
+#include <qstringlist.h>
+#include <qdragobject.h>
 
 #include "main.h"
 #include "qucs.h"
@@ -66,7 +67,7 @@
 #include "components/components.h"
 #include "paintings/paintings.h"
 #include "diagrams/diagrams.h"
-#include "dialogs/messagebox.h"
+#include "dialogs/savedialog.h"
 #include "dialogs/newprojdialog.h"
 #include "dialogs/settingsdialog.h"
 #include "dialogs/digisettingsdialog.h"
@@ -76,6 +77,8 @@
 #include "dialogs/labeldialog.h"
 #include "dialogs/matchdialog.h"
 #include "dialogs/simmessage.h"
+#include "dialogs/vtabwidget.h"
+#include "dialogs/vtabbeddockwidget.h"
 
 extern const char *empty_xpm[];
 
@@ -120,6 +123,7 @@ QucsApp::QucsApp()
     tr("Data Display")+" (*.dpl);;"+
     tr("Qucs Documents")+" (*.sch *.dpl);;"+
     tr("VHDL Sources")+" (*.vhdl *.vhd);;"+
+    tr("Verilog Sources")+" (*.v);;"+
     tr("Any File")+" (*)";
   QucsWorkDir.setPath(QDir::homeDirPath()+QDir::convertSeparators ("/.qucs"));
   QucsHomeDir.setPath(QDir::homeDirPath()+QDir::convertSeparators ("/.qucs"));
@@ -131,16 +135,17 @@ QucsApp::QucsApp()
   initActions();
   initMenuBar();
   initToolBar();
-  viewToolBar->setOn(true);
   initStatusBar();
+  viewToolBar->setOn(true);
   viewStatusBar->setOn(true);
+  viewBrowseDock->setOn(true);
   initCursorMenu();
   HierarchyHistory.setAutoDelete(true);
 
   // default settings of the printer
-  Printer = new QPrinter(QPrinter::PrinterResolution);
-//  Printer->setOrientation(QPrinter::Landscape);
+  Printer = new QPrinter(QPrinter::HighResolution);
 #if defined (QT_VERSION) && QT_VERSION > 0x030200
+  Printer->setOptionEnabled(QPrinter::PrintSelection, true);
   Printer->setOptionEnabled(QPrinter::PrintPageRange, false);
 #endif
   Printer->setColorMode(QPrinter::Color);
@@ -155,9 +160,15 @@ QucsApp::QucsApp()
   switchSchematicDoc(true);  // "untitled" document is schematic
 
   // load documents given as command line arguments
-  for(int z=1; z<qApp->argc(); z++)
-    if(*(qApp->argv()[z]) != '-')
-      gotoPage(qApp->argv()[z]);
+  for(int z=1; z<qApp->argc(); z++) {
+    QString arg = qApp->argv()[z];
+    if(*(arg) != '-') {
+      // allow uri's: file:/home/linuxuser/Desktop/example.sch
+      QString f = QDir::convertSeparators(QUriDrag::uriToLocalFile(arg));
+      if(f.isEmpty()) f = arg;
+      gotoPage(f);
+    }
+  }
 }
 
 QucsApp::~QucsApp()
@@ -177,6 +188,7 @@ void QucsApp::initContentListView()
   ConOthers     = new QListViewItem(Content, tr("Others"));
   ConDatasets   = new QListViewItem(Content, tr("Datasets"));
   ConDisplays   = new QListViewItem(Content, tr("Data Displays"));
+  ConVerilog    = new QListViewItem(Content, tr("Verilog"));
   ConSources    = new QListViewItem(Content, tr("VHDL"));
   ConSchematics = new QListViewItem(Content, tr("Schematics"));
 }
@@ -186,15 +198,16 @@ void QucsApp::initView()
   // set application icon
   setIcon (QPixmap(QucsSettings.BitmapDir + "big.qucs.xpm"));
 
-  QVBox *all = new QVBox(this);   // only to fill the entire view area
-  QSplitter *Hsplit = new QSplitter(QSplitter::Horizontal, all);
-  TabView     = new QTabWidget(Hsplit);  // tabs on the left side
-  DocumentTab = new QTabWidget(Hsplit);  // tab on the right side
+  DocumentTab = new QTabWidget(this);
+  setCentralWidget(DocumentTab);
   connect(DocumentTab,
           SIGNAL(currentChanged(QWidget*)), SLOT(slotChangeView(QWidget*)));
 
-  Hsplit->setResizeMode(TabView, QSplitter::KeepSize);
-  setCentralWidget(all);
+  dock = new VTabbedDockWidget(QDockWindow::InDock, this);
+  TabView = new VTabWidget(VTabInterface::TabLeft,dock);  // tabs on the left side
+  
+  
+  connect(dock, SIGNAL(visibilityChanged(bool)), SLOT(slotToggleDock(bool)));
 
   view = new MouseActions();
 
@@ -219,8 +232,8 @@ void QucsApp::initView()
   connect(ProjDel, SIGNAL(clicked()), SLOT(slotProjDelButt()));
 
   Projects = new QListBox(ProjGroup);
-  TabView->addTab(ProjGroup, tr("Projects"));
-  TabView->setTabToolTip(TabView->page(0),
+  TabView->addPage(ProjGroup, tr("Projects"));
+  TabView->setTabToolTip(TabView->id(ProjGroup),
 			 tr("content of project directory"));
 
   connect(Projects, SIGNAL(doubleClicked(QListBoxItem*)),
@@ -237,27 +250,34 @@ void QucsApp::initView()
   Content->setColumnWidth(0, 150);
 
   initContentListView();
-  TabView->addTab(Content,tr("Content"));
-  TabView->setTabToolTip(TabView->page(1), tr("content of current project"));
+  TabView->addPage(Content,tr("Content"));
+  TabView->setTabToolTip(TabView->id(Content), tr("content of current project"));
 
   connect(Content, SIGNAL(doubleClicked(QListViewItem*)),
 		   SLOT(slotOpenContent(QListViewItem*)));
   connect(Content, SIGNAL(clicked(QListViewItem*)),
 		   SLOT(slotSelectSubcircuit(QListViewItem*)));
+  connect(Content, SIGNAL(expanded(QListViewItem*)),
+		   SLOT(slotExpandContentList(QListViewItem*)));
 
   // ----------------------------------------------------------
   // "Component Tab" of the left QTabWidget
   QVBox *CompGroup  = new QVBox(this);
   CompChoose = new QComboBox(CompGroup);
   CompComps  = new myIconView(CompGroup);
-  TabView->addTab(CompGroup,tr("Components"));
-  TabView->setTabToolTip(TabView->page(2), tr("components and diagrams"));
+  TabView->addPage(CompGroup,tr("Components"));
+  TabView->setTabToolTip(TabView->id(CompGroup), tr("components and diagrams"));
   fillComboBox(true);
 
   slotSetCompView(0);
   connect(CompChoose, SIGNAL(activated(int)), SLOT(slotSetCompView(int)));
   connect(CompComps, SIGNAL(clicked(QIconViewItem*)),
 		     SLOT(slotSelectComponent(QIconViewItem*)));
+  dock->setWidget(TabView);
+  setDockEnabled(dock,DockTop,false);
+  setDockEnabled(dock,DockBottom,false);
+  moveDockWindow(dock,DockLeft);
+  TabView->setCurrentPage(0);
 
   // ............................................
   readProjects(); // reads all projects and inserts them into the ListBox
@@ -302,32 +322,37 @@ pInfoFunc Sources[] =
    &Source_ac::info, &Volt_noise::info, &Ampere_noise::info, &VCCS::info,
    &CCCS::info, &VCVS::info, &CCVS::info, &vPulse::info, &iPulse::info,
    &vRect::info, &iRect::info, &Noise_ii::info, &Noise_vv::info,
-   &Noise_iv::info, &AM_Modulator::info, &PM_Modulator::info, 0};
+   &Noise_iv::info, &AM_Modulator::info, &PM_Modulator::info, &iExp::info,
+   &vExp::info, 0};
+
+pInfoFunc Probes[] =
+  {&iProbe::info, &vProbe::info, 0};
 
 pInfoFunc TransmissionLines[] =
-  {&TLine::info, &CoaxialLine::info, &Substrate::info, &MSline::info,
-   &MScoupled::info, &MScorner::info, &MSmbend::info, &MSstep::info,
-   &MStee::info, &MScross::info, &MSopen::info, &MSgap::info, &MSvia::info,
-   &Coplanar::info, &CPWopen::info, &CPWshort::info, &CPWgap::info,
-   &CPWstep::info, 0};
+  {&TLine::info, &TLine_4Port::info, &TwistedPair::info, &CoaxialLine::info,
+   &Substrate::info, &MSline::info, &MScoupled::info, &MScorner::info,
+   &MSmbend::info, &MSstep::info, &MStee::info, &MScross::info, &MSopen::info,
+   &MSgap::info, &MSvia::info, &Coplanar::info, &CPWopen::info,
+   &CPWshort::info, &CPWgap::info, &CPWstep::info, &BondWire::info, 0};
 
 pInfoFunc nonlinearComps[] =
   {&Diode::info, &BJT::info, &BJT::info_pnp, &BJTsub::info,
    &BJTsub::info_pnp, &JFET::info, &JFET::info_p,
    &MOSFET::info, &MOSFET::info_p, &MOSFET::info_depl,
    &MOSFET_sub::info, &MOSFET_sub::info_p, &MOSFET_sub::info_depl,
-   &OpAmp::info, 0};
+   &OpAmp::info, &hicumL2p1::info, &HBT_X::info, &EqnDefined::info};
 
 pInfoFunc digitalComps[] =
   {&Digi_Source::info, &Logical_Inv::info, &Logical_OR::info,
    &Logical_NOR::info, &Logical_AND::info, &Logical_NAND::info,
    &Logical_XOR::info, &Logical_XNOR::info, &RS_FlipFlop::info,
    &D_FlipFlop::info, &JK_FlipFlop::info, &VHDL_File::info,
-   &Digi_Sim::info, 0};
+   &Verilog_File::info, &Digi_Sim::info, 0};
 
 pInfoFunc Simulations[] =
   {&DC_Sim::info, &TR_Sim::info, &AC_Sim::info, &SP_Sim::info,
-   &HB_Sim::info, &Param_Sweep::info, &Digi_Sim::info, 0};
+   &HB_Sim::info, &Param_Sweep::info, &Digi_Sim::info, &Optimize_Sim::info,
+   0};
 
 pInfoFunc FileComponents[] =
   {&SpiceFile::info, &SParamFile::info1, &SParamFile::info2,
@@ -346,7 +371,7 @@ pInfoFunc Paintings[] =
 
 // Order of the component groups in the ComboBox
 pInfoFunc *ComponentGroups[] =
-  {lumpedComponents, Sources, TransmissionLines, nonlinearComps,
+  {lumpedComponents, Sources, Probes, TransmissionLines, nonlinearComps,
    digitalComps, FileComponents, Simulations, Diagrams, 0};
 
 // ---------------------------------------------------------------
@@ -358,6 +383,7 @@ void QucsApp::fillComboBox(bool setAll)
   if(setAll) {
     CompChoose->insertItem(tr("lumped components"));
     CompChoose->insertItem(tr("sources"));
+    CompChoose->insertItem(tr("probes"));
     CompChoose->insertItem(tr("transmission lines"));
     CompChoose->insertItem(tr("nonlinear components"));
     CompChoose->insertItem(tr("digital components"));
@@ -546,7 +572,7 @@ void QucsApp::slotCMenuDelete()
 }
 
 // ----------------------------------------------------------
-// Deletes all files with that name (and suffix sch, dpl, dat, vhdl).
+// Deletes all files with that name (and suffix sch, dpl, dat, vhdl, v).
 void QucsApp::slotCMenuDelGroup()
 {
   QListViewItem *item = Content->selectedItem();
@@ -558,6 +584,8 @@ void QucsApp::slotCMenuDelGroup()
   QString NameDPL = QucsWorkDir.filePath(s+".dpl");
   QString NameDAT = QucsWorkDir.filePath(s+".dat");
   QString NameDIG = QucsWorkDir.filePath(s+".vhdl");
+  QString NameVHD = QucsWorkDir.filePath(s+".vhd");
+  QString NameVER = QucsWorkDir.filePath(s+".v");
 
   int No=0;
   QucsDoc *d;  // search, if files are open
@@ -577,6 +605,16 @@ void QucsApp::slotCMenuDelGroup()
                    tr("Cannot delete the open file: ")+NameDIG);
       return;
     }
+    if(d->DocName == NameVHD) {
+      QMessageBox::critical(this, tr("Error"),
+                   tr("Cannot delete the open file: ")+NameVHD);
+      return;
+    }
+    if(d->DocName == NameVER) {
+      QMessageBox::critical(this, tr("Error"),
+                   tr("Cannot delete the open file: ")+NameVER);
+      return;
+    }
   }
 
 
@@ -584,12 +622,16 @@ void QucsApp::slotCMenuDelGroup()
   bool DPL_exists = QFile::exists(NameDPL);
   bool DAT_exists = QFile::exists(NameDAT);
   bool DIG_exists = QFile::exists(NameDIG);
+  bool VHD_exists = QFile::exists(NameVHD);
+  bool VER_exists = QFile::exists(NameVER);
 
   QString Str;
   if(SCH_exists)  Str += s+".sch\n";
   if(DPL_exists)  Str += s+".dpl\n";
   if(DAT_exists)  Str += s+".dat\n";
   if(DIG_exists)  Str += s+".vhdl\n";
+  if(VHD_exists)  Str += s+".vhd\n";
+  if(VER_exists)  Str += s+".v\n";
 
   No = QMessageBox::warning(this, tr("Warning"),
 	tr("This will delete the files\n")+Str+tr("permanently! Continue ?"),
@@ -621,6 +663,18 @@ void QucsApp::slotCMenuDelGroup()
 		tr("Cannot delete VHDL source: ")+s+".vhdl");
       return;
     }
+  if(VHD_exists)
+    if(!QFile::remove(NameVHD)) {
+      QMessageBox::critical(this, tr("Error"),
+		tr("Cannot delete VHDL source: ")+s+".vhd");
+      return;
+    }
+  if(VER_exists)
+    if(!QFile::remove(NameVER)) {
+      QMessageBox::critical(this, tr("Error"),
+		tr("Cannot delete Verilog source: ")+s+".v");
+      return;
+    }
 
   // remove items from listview ........
   if(SCH_exists) {
@@ -651,6 +705,20 @@ void QucsApp::slotCMenuDelGroup()
       delete item;
     }
   }
+  if(VHD_exists) {
+    item = Content->findItem(s+".vhd", 0);
+    if(item) {
+      item->parent()->takeItem(item);
+      delete item;
+    }
+  }
+  if(VER_exists) {
+    item = Content->findItem(s+".v", 0);
+    if(item) {
+      item->parent()->takeItem(item);
+      delete item;
+    }
+  }
 }
 
 
@@ -662,13 +730,14 @@ void QucsApp::slotCMenuDelGroup()
 void QucsApp::readProjects()
 {
   QDir ProjDir(QDir::homeDirPath());
-  if(!ProjDir.cd(".qucs")) // work directory exists ?
+  if(!ProjDir.cd(".qucs")) { // work directory exists ?
     if(!ProjDir.mkdir(".qucs")) { // no, then create it
       QMessageBox::warning(this, tr("Warning"),
-			tr("Cannot create work directory !"));
+                   tr("Cannot create work directory !"));
       return;
     }
-  ProjDir.cd(".qucs");
+    ProjDir.cd(".qucs");
+  }
 
   // get all directories
   QStringList PrDirs = ProjDir.entryList("*", QDir::Dirs, QDir::Name);
@@ -697,7 +766,8 @@ void QucsApp::slotProjNewButt()
   QDir projDir(QucsHomeDir.path());
   if(projDir.mkdir(d->ProjName->text()+"_prj")) {
     Projects->insertItem(d->ProjName->text(),0);  // at first position
-    if(d->OpenProj->isChecked()) slotOpenProject(Projects->firstItem());
+    if(d->OpenProj->isChecked())
+      slotOpenProject(Projects->firstItem());
   }
   else QMessageBox::information(this, tr("Info"),
                     tr("Cannot create project directory !"));
@@ -738,11 +808,8 @@ int QucsApp::testFile(const QString& DocName)
     return -3;
   }
 
-  QString s = PACKAGE_VERSION;
-  s.remove('.');
   Line = Line.mid(16, Line.length()-17);
-  Line.remove('.');
-  if(Line > s) {  // wrong version number ? (only backward compatible)
+  if(!checkVersion(Line)) { // wrong version number ?
     file.close();
     return -4;
   }
@@ -762,7 +829,7 @@ int QucsApp::testFile(const QString& DocName)
     }
 
     Line = Line.stripWhiteSpace();
-    s    = Line.section(' ',0,0);    // component type
+    QString s = Line.section(' ',0,0);    // component type
     if(s == "<Port") z++;
   }
   return -5;  // component field not closed
@@ -773,7 +840,20 @@ int QucsApp::testFile(const QString& DocName)
 // content ListView
 void QucsApp::readProjectFiles()
 {
-  initContentListView();
+  // Delete the content files, but don't delete the parent items !!!
+  while(ConSchematics->firstChild())
+    delete ConSchematics->firstChild();
+  while(ConDisplays->firstChild())
+    delete ConDisplays->firstChild();
+  while(ConDatasets->firstChild())
+    delete ConDatasets->firstChild();
+  while(ConSources->firstChild())
+    delete ConSources->firstChild();
+  while(ConVerilog->firstChild())
+    delete ConVerilog->firstChild();
+  while(ConOthers->firstChild())
+    delete ConOthers->firstChild();
+
 
   int n;
   // put all files into "Content"-ListView
@@ -787,7 +867,7 @@ void QucsApp::readProjectFiles()
       if(n >= 0) {
         if(n > 0)
           new QListViewItem(ConSchematics, (*it).ascii(),
-				QString::number(n)+tr("-port"));
+                            QString::number(n)+tr("-port"));
         else new QListViewItem(ConSchematics, (*it).ascii());
       }
     }
@@ -795,18 +875,18 @@ void QucsApp::readProjectFiles()
       new QListViewItem(ConDisplays, (*it).ascii());
     else if(Str == "dat")
       new QListViewItem(ConDatasets, (*it).ascii());
-    else if(Str == "vhdl")
+    else if((Str == "vhdl") || (Str == "vhd"))
       new QListViewItem(ConSources, (*it).ascii());
+    else if(Str == "v")
+      new QListViewItem(ConVerilog, (*it).ascii());
     else
       new QListViewItem(ConOthers, (*it).ascii());
   }
-
-  Content->firstChild()->setOpen(true);  // show schematics
 }
 
 // ----------------------------------------------------------
 // Opens an existing project.
-void QucsApp::OpenProject(const QString& Path, const QString& Name)
+void QucsApp::openProject(const QString& Path, const QString& Name)
 {
   editText->setHidden(true); // disable text edit of component property
 
@@ -825,7 +905,8 @@ void QucsApp::OpenProject(const QString& Path, const QString& Name)
   QucsWorkDir.setPath(ProjDir.path());
 
   Content->setColumnText(0,tr("Content of '")+Name+tr("'")); // column text
-  readProjectFiles();
+  ConSchematics->setOpen(false); // get sure to have it closed
+  ConSchematics->setOpen(true);  // also calls readProjectFiles()
 
   TabView->setCurrentPage(1);   // switch to "Content"-Tab
   ProjName = Name;   // remember the name of project
@@ -851,7 +932,7 @@ void QucsApp::slotMenuOpenProject()
   int i = s.findRev('/');
   if(i > 0) s = s.mid(i+1);   // cut out the last subdirectory
   s.remove("_prj");
-  OpenProject(d->selectedFile(), s);
+  openProject(d->selectedFile(), s);
 }
 
 // ----------------------------------------------------------
@@ -870,7 +951,7 @@ void QucsApp::slotProjOpenButt()
 // Is called when project is double-clicked to open it.
 void QucsApp::slotOpenProject(QListBoxItem *item)
 {
-  OpenProject(QucsHomeDir.filePath(item->text()+"_prj"), item->text());
+  openProject(QucsHomeDir.filePath(item->text()+"_prj"), item->text());
 }
 
 // ----------------------------------------------------------
@@ -895,7 +976,40 @@ void QucsApp::slotMenuCloseProject()
 }
 
 // ----------------------------------------------------------
-bool QucsApp::DeleteProject(const QString& Path, const QString& Name)
+bool QucsApp::deleteDirectoryContent(QDir& Dir)
+{
+  // removes every file, remove("*") does not work
+  QStringList Files = Dir.entryList("*", QDir::Files);  // all files
+  QStringList::iterator it;
+  for(it = Files.begin(); it != Files.end(); it++) {
+     if(!Dir.remove(*it)) {
+       QMessageBox::information(this, tr("Info"),
+				tr("Cannot delete file: ")+(*it));
+       return false;
+     }
+  }
+
+  QDir myDir(Dir);
+  // Remove all directories recursively.
+  Files = Dir.entryList("*", QDir::Dirs);
+  Files.pop_front();  // delete "." from list
+  Files.pop_front();  // delete ".." from list
+  for(it = Files.begin(); it != Files.end(); it++) {
+     myDir.cd(*it);
+     if(!deleteDirectoryContent(myDir))
+       return false;
+     myDir.cdUp();
+     if(!myDir.rmdir(*it)) {
+       QMessageBox::information(this, tr("Info"),
+				tr("Cannot remove directory: ")+(*it));
+       return false;
+    }
+  }
+  return true;
+}
+
+// ----------------------------------------------------------
+bool QucsApp::deleteProject(const QString& Path, const QString& Name)
 {
   editText->setHidden(true); // disable text edit of component property
 
@@ -911,17 +1025,8 @@ bool QucsApp::DeleteProject(const QString& Path, const QString& Name)
      tr("&Yes"), tr("&No"), 0,1,1))  return false;
 
   QDir projDir = QDir(Path);
-  QStringList ProjFiles = projDir.entryList("*", QDir::Files);  // all files
-
-  // removes every file, remove("*") does not work
-  QStringList::iterator it;
-  for(it = ProjFiles.begin(); it != ProjFiles.end(); it++) {
-     if(!projDir.remove(*it)) {
-       QMessageBox::information(this, tr("Info"),
-				tr("Cannot remove project file: ")+(*it));
-       return false;
-     }
-  }
+  if(!deleteDirectoryContent(projDir))
+    return false;
 
   projDir.cdUp();  // leave project directory for deleting
   if(!projDir.rmdir(Name+"_prj")) {
@@ -950,7 +1055,7 @@ void QucsApp::slotMenuDelProject()
   int i = s.findRev('/');
   if(i > 0) s = s.mid(i+1);  // cut out the last subdirectory
   s = s.left(s.length()-4);  // remove "_prj" from name
-  DeleteProject(d->selectedFile(), s);
+  deleteProject(d->selectedFile(), s);
   readProjects();   // re-reads all projects and inserts them into the ListBox
 }
 
@@ -965,7 +1070,7 @@ void QucsApp::slotProjDelButt()
     return;
   }
 
-  if(!DeleteProject(QucsHomeDir.filePath(item->text()+"_prj"),
+  if(!deleteProject(QucsHomeDir.filePath(item->text()+"_prj"),
 	item->text()))  return;
   Projects->removeItem(Projects->currentItem());  // remove from project list
 }
@@ -1031,10 +1136,9 @@ bool QucsApp::gotoPage(const QString& Name)
   slotChangeView(DocumentTab->currentPage());
 
   // if only an untitled document was open -> close it
-  if(DocumentTab->count() == 2)
-    if(getDoc(0)->DocName.isEmpty())
-      if(!getDoc(0)->DocChanged)
-        delete DocumentTab->page(0);
+  if(getDoc(0)->DocName.isEmpty())
+    if(!getDoc(0)->DocChanged)
+      delete DocumentTab->page(0);
 
   view->drawn = false;
   return true;
@@ -1170,8 +1274,10 @@ bool QucsApp::saveAs()
           Content->setSelected(new QListViewItem(ConDisplays, s), true);
         else if(Info.extension(false) == "dat")
           Content->setSelected(new QListViewItem(ConDatasets, s), true);
-        else if(Info.extension(false) == "vhdl")
+        else if((Info.extension(false) == "vhdl") || (Info.extension(false) == "vhd"))
           Content->setSelected(new QListViewItem(ConSources, s), true);
+        else if(Info.extension(false) == "v")
+          Content->setSelected(new QListViewItem(ConVerilog, s), true);
         else
           Content->setSelected(new QListViewItem(ConOthers, s), true);
       }
@@ -1254,39 +1360,28 @@ void QucsApp::slotFileClose()
 // --------------------------------------------------------------
 bool QucsApp::closeAllFiles()
 {
-  int  Result = 0;
-  bool notForAll = true;
-  CloseMessageBox *m = new CloseMessageBox(tr("Closing Qucs document"),
-	tr("This document contains unsaved changes!\n"
-	   "Do you want to save the changes before closing?"),this);
-
-  // close all files and ask to save changed ones
-  QucsDoc *Doc;
-  DocumentTab->setCurrentPage(0);
-  while ((Doc=getDoc()) != 0) {
-    if (Doc->DocChanged) {
-      if(notForAll)  Result = m->exec();
-      switch(Result) {
-	case 1: if(!saveFile(Doc))  return false;  // save button
-		break;
-	case 2: Result = 1;         // save all button
-		notForAll = false;
-		if(!saveFile(Doc))  return false;
-		break;
-	case 4: Result = 3;         // discard all button
-		notForAll = false;
-		break;
-	case 5: return false;       // cancel button
-      }
-    }
-
-    delete Doc;
+  SaveDialog *sd = new SaveDialog(this);
+  sd->setApp(this);
+  for(int i=0; i < DocumentTab->count(); ++i) {
+    QucsDoc *doc = getDoc(i);
+    if(doc->DocChanged)
+      sd->addUnsavedDoc(doc);
   }
+  int Result = SaveDialog::DontSave;
+  if(!sd->isEmpty())
+     Result = sd->exec();
+  delete sd;
+
+  if(Result == SaveDialog::AbortClosing)
+    return false;
+
+  QucsDoc *doc = 0;
+  while((doc = getDoc()) != 0)
+    delete doc;
 
-  delete m;
   switchEditMode(true);   // set schematic edit mode
   return true;
-}
+}   
 
 // --------------------------------------------------------------
 // Is called when another document is selected via the TabBar.
@@ -1390,9 +1485,12 @@ void QucsApp::updatePortNumber(QucsDoc *currDoc, int No)
       }
     }
   }
-  else
+  else if(Info.extension() == "vhdl" || Info.extension() == "vhd") {
     Model = "VHDL";
-
+  }
+  else if(Info.extension() == "v") {
+    Model = "Verilog";
+  }
 
   // update all occurencies of subcircuit in all open documents
   No = 0;
@@ -1422,27 +1520,52 @@ void QucsApp::updatePortNumber(QucsDoc *currDoc, int No)
 
 
 // --------------------------------------------------------------
-void QucsApp::slotFilePrint()
+void QucsApp::printCurrentDocument(bool fitToPage)
 {
   statusBar()->message(tr("Printing..."));
   editText->setHidden(true); // disable text edit of component property
 
-  if(Printer->setup(this))   // printer dialog
-    getDoc()->print(Printer, true);
+  if(DocumentTab->currentPage()->inherits("QTextEdit"))
+    Printer->setOrientation(QPrinter::Portrait);
+  else
+    Printer->setOrientation(QPrinter::Landscape);
+
+  if(Printer->setup(this)) {   // printer dialog
+
+    QPainter Painter(Printer);
+    if(!Painter.device())   // valid device available ?
+      goto Error;
+
+    for(int z=Printer->numCopies(); z>0 ; z--) {
+      if(Printer->aborted())
+        break;
+
+      getDoc()->print(Printer, &Painter,
+              Printer->printRange() == QPrinter::AllPages, fitToPage);
+      if(z > 1)
+        if(!Printer->newPage())
+          goto Error;
+    }
+  }
 
   statusBar()->message(tr("Ready."));
+  return;
+
+Error:
+  statusBar()->message(tr("Printer Error."));
 }
 
 // --------------------------------------------------------------
-void QucsApp::slotFilePrintSelected()
+void QucsApp::slotFilePrint()
 {
-  statusBar()->message(tr("Printing selected..."));
-  editText->setHidden(true); // disable text edit of component property
+  printCurrentDocument(false);
+}
 
-  if(Printer->setup(this))   // printer dialog
-    getDoc()->print(Printer, false);
-
-  statusBar()->message(tr("Ready."));
+// --------------------------------------------------------------
+// Fit printed content to page size.
+void QucsApp::slotFilePrintFit()
+{
+  printCurrentDocument(true);
 }
 
 // --------------------------------------------------------------------
@@ -1469,11 +1592,13 @@ void QucsApp::slotFileQuit()
 // To get all close events.
 void QucsApp::closeEvent(QCloseEvent* Event)
 {
-  Event->ignore();
-  if(closeAllFiles()) {
-    emit signalKillEmAll();   // kill all subprocesses
-    qApp->quit();
-  }
+   if(closeAllFiles()) {
+      emit signalKillEmAll();   // kill all subprocesses
+      Event->accept();
+      qApp->quit();
+   }
+   else
+      Event->ignore();
 }
 
 // --------------------------------------------------------------------
@@ -1585,7 +1710,7 @@ void QucsApp::slotShowOne()
 void QucsApp::slotZoomOut()
 {
   editText->setHidden(true); // disable text edit of component property
-  getDoc()->zoom(0.7f);
+  getDoc()->zoomBy(0.7f);
 }
 
 // -----------------------------------------------------------------------
@@ -1707,7 +1832,7 @@ void QucsApp::slotChangePage(QString& DocName, QString& DataDisplay)
   if(d)
     DocumentTab->setCurrentPage(z-1);
   else {   // no open page found ?
-    if(DataDisplay.section('.',1) != "vhdl")
+    if(DataDisplay.section('.',1).left(3) != "vhd")
       d = new Schematic(this, Name);
     else
       d = new TextDoc(this, Name);
@@ -1784,11 +1909,11 @@ void QucsApp::slotOpenContent(QListViewItem *item)
   QFileInfo Info(QucsWorkDir.filePath(item->text(0)));
   QString Suffix = Info.extension(false);
 
-  if((Suffix == "sch") || (Suffix == "dpl") || (Suffix == "vhdl")) {
+  if((Suffix == "sch") || (Suffix == "dpl") || (Suffix.left(3) == "vhd")) {
     gotoPage(Info.absFilePath());
 
     if(item->text(1).isEmpty())     // is subcircuit ?
-      if(Suffix != "vhdl") return;  // is VHDL subcircuit ?
+      if(Suffix.left(3) != "vhd") return;  // is VHDL subcircuit ?
 
     select->blockSignals(true);  // switch on the 'select' action ...
     select->setOn(true);
@@ -1844,14 +1969,18 @@ void QucsApp::slotSelectSubcircuit(QListViewItem *item)
   }
 
 
-  bool isVHDL = true;
+  bool isVHDL = false;
+  bool isVerilog = false;
   if(item->parent() == 0) return;
   if(item->parent()->text(0) == tr("Schematics")) {
     if(item->text(1).isEmpty())
       return;   // return, if not a subcircuit
-    isVHDL = false;
   }
-  else if(item->parent()->text(0) != tr("VHDL"))
+  else if(item->parent()->text(0) == tr("VHDL"))
+    isVHDL = true;
+  else if(item->parent()->text(0) == tr("Verilog"))
+    isVerilog = true;
+  else
     return;
 
   // delete previously selected elements
@@ -1869,6 +1998,8 @@ void QucsApp::slotSelectSubcircuit(QListViewItem *item)
   Component *Comp;
   if(isVHDL)
     Comp = new VHDL_File();
+  else if(isVerilog)
+    Comp = new Verilog_File();
   else
     Comp = new Subcircuit();
   Comp->Props.first()->Value = item->text(0);
@@ -1884,6 +2015,14 @@ void QucsApp::slotSelectSubcircuit(QListViewItem *item)
   MouseDoubleClickAction = 0;
 }
 
+// ---------------------------------------------------------
+// Is called when one of the Content ListView parents was expanded
+// to show the files. It re-reads all files.
+void QucsApp::slotExpandContentList(QListViewItem*)
+{
+  readProjectFiles();
+}
+
 // ---------------------------------------------------------
 // This function is called if the document type changes, i.e.
 // from schematic to text document or vice versa.
@@ -1915,10 +2054,13 @@ void QucsApp::switchSchematicDoc(bool SchematicMode)
   alignBottom->setEnabled(SchematicMode);
   alignLeft->setEnabled(SchematicMode);
   alignRight->setEnabled(SchematicMode);
+  centerHor->setEnabled(SchematicMode);
+  centerVert->setEnabled(SchematicMode);
   distrHor->setEnabled(SchematicMode);
   distrVert->setEnabled(SchematicMode);
   onGrid->setEnabled(SchematicMode);
   moveText->setEnabled(SchematicMode);
+  filePrintFit->setEnabled(SchematicMode);
   editFind->setEnabled(!SchematicMode);
   editFindAgain->setEnabled(!SchematicMode);
   editRotate->setEnabled(SchematicMode);
@@ -1992,12 +2134,6 @@ void QucsApp::slotSymbolEdit()
   changeSchematicSymbolMode(Doc);
   Doc->becomeCurrent(true);
 
-  // This can only be true when switching to the symbol the first time.
-  if(Doc->UndoSymbol.isEmpty()) {
-    Doc->setChanged(false, true); // "not changed" state, but put on undo stack
-    Doc->UndoSymbol.current()->at(1) = 'i';  // state of being unchanged
-  }
-
   Doc->viewport()->update();
   view->drawn = false;
 }
