@@ -32,15 +32,17 @@
 #include <qiconview.h>
 #include <qtabwidget.h>
 #include <qdragobject.h>
+#include <qpaintdevicemetrics.h>
+#include <qdir.h>
 
 #include "qucs.h"
-#include "schematic.h"
-#include "mouseactions.h"
-#include "viewpainter.h"
-#include "diagrams/diagrams.h"
-#include "paintings/paintings.h"
 #include "main.h"
 #include "node.h"
+#include "schematic.h"
+#include "viewpainter.h"
+#include "mouseactions.h"
+#include "diagrams/diagrams.h"
+#include "paintings/paintings.h"
 
 // just dummies for empty lists
 QPtrList<Wire>      SymbolWires;
@@ -77,7 +79,10 @@ Schematic::Schematic(QucsApp *App_, const QString& Name_)
   UndoSymbol.setAutoDelete(true);
   // The 'i' means state for being unchanged.
   UndoStack.append(new QString(" i\n</>\n</>\n</>\n</>\n"));
+  UndoSymbol.append(new QString(" i\n</>\n</>\n</>\n</>\n"));
 
+  isVerilog = false;
+  creatingLib = false;
   QFileInfo Info(Name_);
   if(App) {
     if(Name_.isEmpty())
@@ -90,6 +95,12 @@ Schematic::Schematic(QucsApp *App_, const QString& Name_)
     // calls indirectly "becomeCurrent"
     App->DocumentTab->setCurrentPage(App->DocumentTab->indexOf(this));
 
+    showFrame = 0;  // don't show
+    Frame_Text0 = tr("Title");
+    Frame_Text1 = tr("Drawn By:");
+    Frame_Text2 = tr("Date:");
+    Frame_Text3 = tr("Revision:");
+
     setVScrollBarMode(QScrollView::AlwaysOn);
     setHScrollBarMode(QScrollView::AlwaysOn);
     viewport()->setPaletteBackgroundColor(QucsSettings.BGColor);
@@ -127,6 +138,45 @@ Schematic::~Schematic()
   }
 }
 
+// ---------------------------------------------------
+bool Schematic::createSubcircuitSymbol()
+{
+  // If the number of ports is not equal, remove or add some.
+  unsigned int countPort = adjustPortNumbers();
+
+  // If a symbol does not yet exist, create one.
+  if(SymbolPaints.count() != countPort)
+    return false;
+
+  int h = 30*((countPort-1)/2) + 10;
+  SymbolPaints.prepend(new ID_Text(-20, h+4));
+
+  SymbolPaints.append(
+     new GraphicLine(-20, -h, 40,  0, QPen(QPen::darkBlue,2)));
+  SymbolPaints.append(
+     new GraphicLine( 20, -h,  0,2*h, QPen(QPen::darkBlue,2)));
+  SymbolPaints.append(
+     new GraphicLine(-20,  h, 40,  0, QPen(QPen::darkBlue,2)));
+  SymbolPaints.append(
+     new GraphicLine(-20, -h,  0,2*h, QPen(QPen::darkBlue,2)));
+
+  unsigned int i=0, y = 10-h;
+  while(i<countPort) {
+    i++;
+    SymbolPaints.append(
+       new GraphicLine(-30, y, 10, 0, QPen(QPen::darkBlue,2)));
+    SymbolPaints.at(i)->setCenter(-30,  y);
+
+    if(i == countPort)  break;
+    i++;
+    SymbolPaints.append(
+       new GraphicLine( 20, y, 10, 0, QPen(QPen::darkBlue,2)));
+    SymbolPaints.at(i)->setCenter(30,  y);
+    y += 60;
+  }
+  return true;
+}
+
 // ---------------------------------------------------
 void Schematic::becomeCurrent(bool update)
 {
@@ -140,38 +190,10 @@ void Schematic::becomeCurrent(bool update)
     Paintings = &SymbolPaints;
     Components = &SymbolComps;
 
-    // If the number of ports is not equal, remove or add some.
-    unsigned int countPort = adjustPortNumbers();
-
-    // If a symbol does not yet exist, create one.
-    if(SymbolPaints.count() == countPort) {
-      int h = 30*((countPort-1)/2) + 10;
-      SymbolPaints.prepend(new ID_Text(-20, h+4));
-
-      SymbolPaints.append(
-	new GraphicLine(-20, -h, 40,  0, QPen(QPen::darkBlue,2)));
-      SymbolPaints.append(
-	new GraphicLine( 20, -h,  0,2*h, QPen(QPen::darkBlue,2)));
-      SymbolPaints.append(
-	new GraphicLine(-20,  h, 40,  0, QPen(QPen::darkBlue,2)));
-      SymbolPaints.append(
-	new GraphicLine(-20, -h,  0,2*h, QPen(QPen::darkBlue,2)));
-
-      unsigned int i=0, y = 10-h;
-      while(i<countPort) {
-	i++;
-	SymbolPaints.append(
-	  new GraphicLine(-30, y, 10, 0, QPen(QPen::darkBlue,2)));
-	SymbolPaints.at(i)->setCenter(-30,  y);
-
-	if(i == countPort)  break;
-	i++;
-	SymbolPaints.append(
-	  new GraphicLine( 20, y, 10, 0, QPen(QPen::darkBlue,2)));
-	SymbolPaints.at(i)->setCenter(30,  y);
-	y += 60;
-      }
+    // if no symbol yet exists -> create one
+    if(createSubcircuitSymbol()) {
       sizeOfAll(UsedX1, UsedY1, UsedX2, UsedY2);
+      setChanged(true, true);
     }
     
     ps = UndoSymbol.current();
@@ -269,6 +291,78 @@ void Schematic::setChanged(bool c, bool fillStack, char Op)
   }
 }
 
+// -----------------------------------------------------------
+void Schematic::paintFrame(ViewPainter *p)
+{
+  // dimensions:  X cm / 2.54 * 144
+  int xall, yall;
+  switch(showFrame) {
+    case 1:  xall = 1020; yall =  765; break;  // DIN A5 landscape
+    case 2:  xall =  765; yall = 1020; break;  // DIN A5 portrait
+    case 3:  xall = 1530; yall = 1020; break;  // DIN A4 landscape
+    case 4:  xall = 1020; yall = 1530; break;  // DIN A4 portrait
+    case 5:  xall = 2295; yall = 1530; break;  // DIN A3 landscape
+    case 6:  xall = 1530; yall = 2295; break;  // DIN A3 portrait
+    default:  return;
+  }
+
+  p->Painter->setPen(QPen(QPen::black,0));
+  int d = p->LineSpacing + int(4.0 * p->Scale);
+  int x1_, y1_, x2_, y2_;
+  p->map(xall, yall, x1_, y1_);
+  x2_ = int(xall * p->Scale) + 1;
+  y2_ = int(yall * p->Scale) + 1;
+  p->Painter->drawRect(x1_, y1_, -x2_, -y2_);
+  p->Painter->drawRect(x1_-d, y1_-d, 2*d-x2_, 2*d-y2_);
+
+  int z;
+  for(z=255; z<xall; z+=255) {
+    p->map(z, 0, x2_, y2_);
+    p->Painter->drawLine(x2_, y2_, x2_, y2_+d);
+    p->Painter->drawLine(x2_, y1_-d, x2_, y1_);
+  }
+  for(z=255; z<yall; z+=255) {
+    p->map(0, z, x2_, y2_);
+    p->Painter->drawLine(x2_, y2_, x2_+d, y2_);
+    p->Painter->drawLine(x1_-d, y2_, x1_, y2_);
+  }
+
+  char Letter[2] = "1";
+  for(z=125; z<xall; z+=255) {
+    p->drawText(Letter, z, 3, 0);
+    p->map(z, yall+3, x2_, y2_);
+    p->Painter->drawText(x2_, y2_-d, 0, 0, Qt::DontClip, Letter);
+    Letter[0]++;
+  }
+  Letter[0] = 'A';
+  for(z=125; z<yall; z+=255) {
+    p->drawText(Letter, 5, z, 0);
+    p->map(xall+5, z, x2_, y2_);
+    p->Painter->drawText(x2_-d, y2_, 0, 0, Qt::DontClip, Letter);
+    Letter[0]++;
+  }
+
+
+  // draw text box with text
+  p->map(xall-340, yall-3, x1_, y1_);
+  p->map(xall-3,   yall-3, x2_, y2_);
+  x1_ -= d;  x2_ -= d;
+  y1_ -= d;  y2_ -= d;
+  d = int(6.0 * p->Scale);
+  z = int(200.0 * p->Scale);
+  y1_ -= p->LineSpacing + d;
+  p->Painter->drawLine(x1_, y1_, x2_, y1_);
+  p->Painter->drawText(x1_+d, y1_+(d>>1), 0, 0, Qt::DontClip, Frame_Text2);
+  p->Painter->drawLine(x1_+z, y1_, x1_+z, y1_ + p->LineSpacing+d);
+  p->Painter->drawText(x1_+d+z, y1_+(d>>1), 0, 0, Qt::DontClip, Frame_Text3);
+  y1_ -= p->LineSpacing + d;
+  p->Painter->drawLine(x1_, y1_, x2_, y1_);
+  p->Painter->drawText(x1_+d, y1_+(d>>1), 0, 0, Qt::DontClip, Frame_Text1);
+  y1_ -= (Frame_Text0.contains('\n')+1) * p->LineSpacing + d;
+  p->Painter->drawRect(x2_, y2_, x1_-x2_-1, y1_-y2_-1);
+  p->Painter->drawText(x1_+d, y1_+(d>>1), 0, 0, Qt::DontClip, Frame_Text0);
+}
+
 // -----------------------------------------------------------
 // Is called when the content (schematic or data display) has to be drawn.
 void Schematic::drawContents(QPainter *p, int, int, int, int)
@@ -277,7 +371,10 @@ void Schematic::drawContents(QPainter *p, int, int, int, int)
   Painter.init(p, Scale, -ViewX1, -ViewY1, contentsX(), contentsY());
 
   paintGrid(&Painter, contentsX(), contentsY(),
-		visibleWidth(), visibleHeight());
+            visibleWidth(), visibleHeight());
+
+  if(!symbolMode)
+    paintFrame(&Painter);
 
   for(Component *pc = Components->first(); pc != 0; pc = Components->next())
     pc->paint(&Painter);
@@ -337,21 +434,19 @@ void Schematic::contentsMousePressEvent(QMouseEvent *Event)
   if(App->MouseReleaseAction == &MouseActions::MReleasePaste)
     return;
   
-  int x = int(float(Event->pos().x())/Scale) + ViewX1;
-  int y = int(float(Event->pos().y())/Scale) + ViewY1;
+  float x = float(Event->pos().x())/Scale + float(ViewX1);
+  float y = float(Event->pos().y())/Scale + float(ViewY1);
 
-  if(Event->button() != Qt::LeftButton) {
-    if(App->MousePressAction == &MouseActions::MPressElement)
-      App->view->MPressElement(this, Event, x, y);
-    else if(App->MousePressAction == &MouseActions::MPressWire2)
-      App->view->MPressWire2(this, Event, x, y);
-    else {
-      App->view->rightPressMenu(this, Event, x, y); // show menu on right mouse button
-      if(App->MouseReleaseAction)
-        (App->view->*(App->MouseReleaseAction))(this, Event); // not called (menu has focus)
-    }
-    return;
-  }
+  if(Event->button() != Qt::LeftButton)
+    if(App->MousePressAction != &MouseActions::MPressElement)
+      if(App->MousePressAction != &MouseActions::MPressWire2) {
+        // show menu on right mouse button
+        App->view->rightPressMenu(this, Event, x, y);
+        if(App->MouseReleaseAction)
+           // Is not called automatically because menu has focus.
+          (App->view->*(App->MouseReleaseAction))(this, Event);
+        return;
+      }
 
   if(App->MousePressAction)
     (App->view->*(App->MousePressAction))(this, Event, x, y);
@@ -372,21 +467,54 @@ void Schematic::contentsMouseDoubleClickEvent(QMouseEvent *Event)
 }
 
 // -----------------------------------------------------------
-void Schematic::print(QPrinter *Printer, bool printAll)
+void Schematic::print(QPrinter*, QPainter *Painter, bool printAll, bool fitToPage)
 {
-  QPainter painter(Printer);
-  if(!painter.device())   // valid device available ?
-    return;
+  QPaintDeviceMetrics pmetrics(Painter->device());
+  float printerDpiX = (float)pmetrics.logicalDpiX();
+  float printerDpiY = (float)pmetrics.logicalDpiY();
+  float printerW = (float)pmetrics.width();
+  float printerH = (float)pmetrics.height();
+  QPaintDeviceMetrics smetrics(QPainter(viewport()).device());
+  float screenDpiX = (float)smetrics.logicalDpiX();
+  float screenDpiY = (float)smetrics.logicalDpiY();
+  float PrintScale = 0.5;
+  sizeOfAll(UsedX1, UsedY1, UsedX2, UsedY2);
+  int marginX = (int)(40 * printerDpiX / screenDpiX);
+  int marginY = (int)(40 * printerDpiY / screenDpiY);
+
+  if(fitToPage) {
+    float ScaleX = float(printerW - 2*marginX) /
+                   float((UsedX2-UsedX1) * printerDpiX) * screenDpiX;
+    float ScaleY = float(printerH - 2*marginY) /
+                   float((UsedY2-UsedY1) * printerDpiY) * screenDpiY;
+    if(ScaleX > ScaleY)
+      PrintScale = ScaleY;
+    else
+      PrintScale = ScaleX;
+  }
+
 
   bool selected;
   ViewPainter p;
-  p.init(&painter, 1.0, 0, 0, UsedX1-20, UsedY1-20);
+  int StartX = UsedX1;
+  int StartY = UsedY1;
+  if(showFrame) {
+    if(UsedX1 > 0)  StartX = 0;
+    if(UsedY1 > 0)  StartY = 0;
+  }
+
+  QFont oldFont = Painter->font();
+  p.init(Painter, PrintScale * printerDpiX / screenDpiX,
+         -StartX, -StartY, -marginX, -marginY, PrintScale);
+
+  if(!symbolMode)
+    paintFrame(&p);
 
   for(Component *pc = Components->first(); pc != 0; pc = Components->next())
     if(pc->isSelected || printAll) {
       selected = pc->isSelected;
       pc->isSelected = false;
-      pc->print(&p);   // paint all selected components
+      pc->print(&p, screenDpiX / printerDpiX);
       pc->isSelected = selected;
     }
 
@@ -410,8 +538,8 @@ void Schematic::print(QPrinter *Printer, bool printAll)
   for(Node *pn = Nodes->first(); pn != 0; pn = Nodes->next()) {
     for(pe = pn->Connections.first(); pe != 0; pe = pn->Connections.next())
       if(pe->isSelected || printAll) {
-	pn->paint(&p); // paint all nodes with selected elements
-	break;
+        pn->paint(&p); // paint all nodes with selected elements
+        break;
       }
     if(pn->Label)
       if(pn->Label->isSelected || printAll) {
@@ -459,6 +587,8 @@ void Schematic::print(QPrinter *Printer, bool printAll)
       pp->paint(&p);   // paint all selected paintings
       pp->isSelected = selected;
     }
+
+  Painter->setFont(oldFont);
 }
 
 // -----------------------------------------------------------
@@ -468,13 +598,30 @@ float Schematic::zoom(float s)
   if(Scale > 10.0) Scale = 10.0f;
   if(Scale < 0.01) Scale = 0.01f;
 
+  // "resizeContents()" performs an immediate repaint. So, set widget
+  // to hidden. This causes some flicker, but it is still nicer.
+  viewport()->setHidden(true);
+//  setHidden(true);
   resizeContents(int(Scale*float(ViewX2 - ViewX1)),
                  int(Scale*float(ViewY2 - ViewY1)));
+//  setHidden(false);
+  viewport()->setHidden(false);
+
   viewport()->update();
   App->view->drawn = false;
   return Scale;
 }
 
+// -----------------------------------------------------------
+float Schematic::zoomBy(float s)
+{
+  zoom(s);
+  s -= 1.0;
+  scrollBy( int(s * float(contentsX()+visibleWidth()/2)),
+            int(s * float(contentsY()+visibleHeight()/2)) );
+  return Scale;
+}
+
 // ---------------------------------------------------
 void Schematic::showAll()
 {
@@ -510,7 +657,14 @@ void Schematic::showNoZoom()
   int x2 = UsedX2;
   int y2 = UsedY2;
 
-//  sizeOfAll(x1, y1, x2, y2);
+  if(x1 > x2) {  // happens e.g. if untitled without changes
+    x1 = 0;
+    x2 = 800;
+  }
+  if(y1 > y2) {
+    y1 = 0;
+    y2 = 800;
+  }
   if(x2==0) if(y2==0) if(x1==0) if(y1==0) x2 = y2 = 800;
 
   ViewX1 = x1-40;
@@ -567,7 +721,7 @@ void Schematic::paintGrid(ViewPainter *p, int cX, int cY, int Width, int Height)
 {
   if(!GridOn) return;
 
-  p->Painter->setPen(QPen(QPen::black,1));
+  p->Painter->setPen(QPen(QPen::black,0));
   int dx = -int(Scale*float(ViewX1)) - cX;
   int dy = -int(Scale*float(ViewY1)) - cY;
   p->Painter->drawLine(-3+dx, dy, 4+dx, dy); // small cross at origin
@@ -986,6 +1140,11 @@ bool Schematic::load()
   if(!loadDocument()) return false;
   lastSaved = QDateTime::currentDateTime();
   UndoStack.clear();
+  UndoSymbol.clear();
+  symbolMode = true;
+  setChanged(false, true); // "not changed" state, but put on undo stack
+  UndoSymbol.current()->at(1) = 'i';
+  symbolMode = false;
   setChanged(false, true); // "not changed" state, but put on undo stack
   UndoStack.current()->at(1) = 'i';  // state of being unchanged
 
@@ -995,9 +1154,11 @@ bool Schematic::load()
   sizeOfAll(UsedX1, UsedY1, UsedX2, UsedY2);
   if(ViewX1 > UsedX1)  ViewX1 = UsedX1;
   if(ViewY1 > UsedY1)  ViewY1 = UsedY1;
-  if(ViewX2 > UsedX2)  ViewX2 = UsedX2;
-  if(ViewY2 > UsedY2)  ViewY2 = UsedY2;
+  if(ViewX2 < UsedX2)  ViewX2 = UsedX2;
+  if(ViewY2 < UsedY2)  ViewY2 = UsedY2;
   zoom(1.0f);
+  setContentsPos(tmpViewX1, tmpViewY1);
+  tmpViewX1 = tmpViewY1 = -200;   // was used as temporary cache
   return true;
 }
 
@@ -1006,7 +1167,8 @@ bool Schematic::load()
 int Schematic::save()
 {
   int result = adjustPortNumbers();// same port number for schematic and symbol
-  saveDocument();
+  if(saveDocument() < 0)
+     return -1;
   lastSaved = QDateTime::currentDateTime();
   if(result >= 0) {
     setChanged(false);
@@ -1017,7 +1179,6 @@ int Schematic::save()
     UndoStack.findRef(ps);  // back to current
 
     ps = UndoSymbol.current();
-    if(!ps)  return result;
     for(p = UndoSymbol.first(); p != 0; p = UndoSymbol.next())
       p->at(1) = ' ';  // state of being changed
     ps->at(1) = 'i';   // state of being unchanged
@@ -1379,10 +1540,12 @@ void Schematic::contentsWheelEvent(QWheelEvent *Event)
   // ...................................................................
   else if(Event->state() & Qt::ControlButton) {  // use mouse wheel to zoom ?
       float Scaling;
-      if(delta < 0) Scaling = float(-delta)/60.0/1.1;
+      if(delta < 0) Scaling = float(delta)/-60.0/1.1;
       else Scaling = 1.1*60.0/float(delta);
-      Scaling = zoom(Scaling);
-//      center(Event->x(), Event->y());
+      zoom(Scaling);
+      Scaling -= 1.0;
+      scrollBy( int(Scaling * float(Event->pos().x())),
+                int(Scaling * float(Event->pos().y())) );
   }
   // ...................................................................
   else {     // scroll vertically !
@@ -1551,10 +1714,16 @@ void Schematic::contentsDropEvent(QDropEvent *Event)
     QStrList List;
     QUriDrag::decode(Event, List);
 
+    // do not close untitled document to avoid segfault
+    QucsDoc *d = QucsMain->getDoc(0);
+    bool changed = d->DocChanged;
+    d->DocChanged = true;
+
     // URI:  file:/home/linuxuser/Desktop/example.sch
     for(unsigned int i=0; i < List.count(); i++)
-      App->gotoPage(QString(List.at(i)).section(':',1));
+      App->gotoPage(QDir::convertSeparators(QUriDrag::uriToLocalFile(List.at(i))));
 
+    d->DocChanged = changed;
     return;
   }
 
@@ -1569,35 +1738,57 @@ void Schematic::contentsDropEvent(QDropEvent *Event)
   if(App->view->selElem) delete App->view->selElem;
   App->view->selElem = 0;  // no component selected
 
-  if(App->view->labeledWire)
-    ((QAction*)App->view->labeledWire)->setOn(true);  // restore old action
+  if(formerAction)
+    formerAction->setOn(true);  // restore old action
 }
 
 // ---------------------------------------------------
 void Schematic::contentsDragEnterEvent(QDragEnterEvent *Event)
 {
+  formerAction = 0;
   dragIsOkay = false;
-  if(Event->provides("text/uri-list"))  // file dragged in ?
+
+  // file dragged in ?
+  if(Event->provides("text/uri-list"))
     if(QUriDrag::canDecode(Event)) {
       dragIsOkay = true;
       Event->accept();
       return;
     }
 
+  // drag library component
+  if(Event->provides("text/plain")) {
+    QString s;
+    if(QTextDrag::decode(Event, s))
+      if(s.left(15) == "QucsComponent:<") {
+        s = s.mid(14);
+        App->view->selElem = getComponentFromName(s);
+        if(App->view->selElem) {
+          Event->accept();
+          return;
+        }
+      }
+    Event->ignore();
+    return;
+  }
 
-  if(Event->format(1) == 0)   // only one MIME type ?
+
+  if(Event->format(1) == 0) {  // only one MIME type ?
+
+    // drag component from listview
     if(Event->provides("application/x-qiconlist")) {
       QIconViewItem *Item = App->CompComps->currentItem();
       if(Item) {
-	App->view->labeledWire = (Wire*)App->activeAction; // misuse variable
-	App->slotSelectComponent(Item);  // also sets drawn=false
-	App->MouseMoveAction = 0;
-	App->MousePressAction = 0;
+        formerAction = App->activeAction;
+        App->slotSelectComponent(Item);  // also sets drawn=false
+        App->MouseMoveAction = 0;
+        App->MousePressAction = 0;
 
-	Event->accept();
-	return;
+        Event->accept();
+        return;
       }
     }
+  }
 
   Event->ignore();
 }
@@ -1615,8 +1806,8 @@ void Schematic::contentsDragLeaveEvent(QDragLeaveEvent*)
         App->view->drawn = false;
       }
 
-  if(App->view->labeledWire)
-    ((QAction*)App->view->labeledWire)->setOn(true);  // restore old action
+  if(formerAction)
+    formerAction->setOn(true);  // restore old action
 }
 
 // ---------------------------------------------------
